@@ -1,17 +1,18 @@
 <?php
 /**
- * $Horde: horde/admin/perms/edit.php,v 1.38.2.6 2007/01/02 13:54:04 jan Exp $
+ * $Horde: horde/admin/perms/edit.php,v 1.38.2.8 2008/01/02 11:30:14 jan Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2005-2007 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
+ * @author Jan Schneider <jan@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/../..');
 require_once HORDE_BASE . '/lib/base.php';
-require_once 'Horde/Menu.php';
 
 if (!Auth::isAdmin()) {
     Horde::authenticationFailureRedirect();
@@ -108,8 +109,7 @@ if ($ui->validateEditForm($info)) {
 
 $title = _("Permissions Administration");
 require HORDE_TEMPLATES . '/common-header.inc';
-require HORDE_TEMPLATES . '/admin/common-header.inc';
-$notification->notify(array('listeners' => 'status'));
+require HORDE_TEMPLATES . '/admin/menu.inc';
 
 /* Render the form and tree. */
 $ui->renderForm('edit.php');
