@@ -1,17 +1,19 @@
 <?php
 /**
- * $Horde: horde/admin/setup/index.php,v 1.28.4.13 2007/01/02 13:54:04 jan Exp $
+ * $Horde: horde/admin/setup/index.php,v 1.28.4.18 2008/05/18 23:32:25 jan Exp $
  *
- * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/../..');
 require_once HORDE_BASE . '/lib/base.php';
 require_once 'Horde/Template.php';
+require_once 'Horde/Form/Renderer.php';
 
 if (!Auth::isAdmin()) {
     Horde::fatal('Forbidden.', __FILE__, __LINE__);
@@ -22,11 +24,7 @@ if (!Auth::isAdmin()) {
  */
 function _getVersion($file)
 {
-    $size = @filesize($file);
-    if (!$size || !is_resource($fp = @fopen($file, 'r'))) {
-        return false;
-    }
-    $data = @fread($fp, $size);
+    $data = @file_get_contents($file);
     if (preg_match('/\$.*?conf\.xml,v (.*?) .*\$/', $data, $match)) {
         return $match[1];
     } else {
@@ -55,7 +53,7 @@ function _uploadFTP($params)
         $path = $registry->get('fileroot', $app) . '/config';
         /* Try to back up the current conf.php. */
         if ($vfs->exists($path, 'conf.php')) {
-            if (($result = $vfs->rename($path, 'conf.php', $path, '/conf.php.bak')) === true) {
+            if (($result = $vfs->rename($path, 'conf.php', $path, '/conf.bak.php')) === true) {
                 $notification->push(_("Successfully saved backup configuration."), 'horde.success');
             } elseif (is_a($result, 'PEAR_Error')) {
                 $notification->push(sprintf(_("Could not save a backup configuation: %s"), $result->getMessage()), 'horde.error');
@@ -77,6 +75,40 @@ function _uploadFTP($params)
     return $no_errors;
 }
 
+/* Check for versions if requested. */
+$versions = array();
+if (Util::getFormData('check_versions')) {
+    require_once 'HTTP/Request.php';
+    require_once 'Horde/DOM.php';
+    $http = new HTTP_Request('http://www.horde.org/versions.php');
+    $result = $http->sendRequest();
+    if (is_a($result, 'PEAR_Error')) {
+        $notification->push($result, 'horde.error');
+    } elseif ($http->getResponseCode() != 200) {
+        $notification->push(_("Unexpected response from server, try again later."), 'horde.error');
+    } else {
+        $dom = Horde_DOM_Document::factory(array('xml' => $http->getResponseBody()));
+        $stable = $dom->get_elements_by_tagname('stable');
+        if (!count($stable) || !$stable[0]->has_child_nodes()) {
+            $notification->push(_("Invalid response from server."), 'horde.error');
+        } else {
+            for ($app = $stable[0]->first_child();
+                 !empty($app);
+                 $app = $app->next_sibling()) {
+                if (!is_a($app, 'domelement') &&
+                    !is_a($app, 'Horde_DOM_Element')) {
+                    continue;
+                }
+                $version = $app->get_elements_by_tagname('version');
+                $url = $app->get_elements_by_tagname('url');
+                $versions[$app->get_attribute('name')] = array(
+                    'version' => $version[0]->get_content(),
+                    'url' => $url[0]->get_content());
+            }
+        }
+    }
+}
+
 /* Set up some icons. */
 $success = Horde::img('alerts/success.png', '', '', $registry->getImageDir('horde'));
 $warning = Horde::img('alerts/warning.png', '', '', $registry->getImageDir('horde'));
@@ -86,6 +118,27 @@ $conf_url = Horde::applicationUrl('admin/setup/config.php');
 $a = $registry->listApps(array('inactive', 'hidden', 'notoolbar', 'active', 'admin'));
 $apps = array();
 $i = -1;
+if (file_exists(HORDE_BASE . '/lib/bundle.php')) {
+    include HORDE_BASE . '/lib/bundle.php';
+    $apps[0] = array('sort' => '00',
+                     'name' => '<strong>' . BUNDLE_FULLNAME . '</strong>',
+                     'icon' => Horde::img($registry->get('icon', 'horde'),
+                                          BUNDLE_FULLNAME, '', ''),
+                     'version' => '<strong>' . BUNDLE_VERSION . '</strong>');
+    if (!empty($versions)) {
+        if (!isset($versions[BUNDLE_NAME])) {
+            $apps[0]['load'] = $warning;
+            $apps[0]['vstatus'] = _("No stable version exists yet.");
+        } elseif (version_compare($versions[BUNDLE_NAME]['version'], BUNDLE_VERSION, '>')) {
+            $apps[0]['load'] = $error;
+            $apps[0]['vstatus'] = Horde::link($versions[BUNDLE_NAME]['url'], sprintf(_("Download %s"), BUNDLE_FULLNAME)) . sprintf(_("A newer version (%s) exists."), $versions[BUNDLE_NAME]['version']) . '</a> ';
+        } else {
+            $apps[0]['load'] = $success;
+            $apps[0]['vstatus'] = _("Application is up-to-date.");
+        }
+    }
+    $i++;
+}
 foreach ($a as $app) {
     /* Skip app if no conf.xml file. */
     $path = $registry->get('fileroot', $app) . '/config';
@@ -100,18 +153,31 @@ foreach ($a as $app) {
     $conf_link = Horde::link($conf_link, sprintf(_("Configure %s"), $app));
     $apps[$i]['sort'] = $registry->get('name', $app) . ' (' . $app . ')';
     $apps[$i]['name'] = $conf_link . $apps[$i]['sort'] . '</a>';
+    $apps[$i]['icon'] = Horde::img($registry->get('icon', $app), $registry->get('name', $app), '', '');
     $apps[$i]['version'] = '';
     if (is_readable($registry->get('fileroot', $app) . '/lib/version.php')) {
-        require $registry->get('fileroot', $app) . '/lib/version.php';
+        require_once $registry->get('fileroot', $app) . '/lib/version.php';
         $version_constant = String::upper($app) . '_VERSION';
         if (defined($version_constant)) {
             $apps[$i]['version'] = constant($version_constant);
+            if (!empty($versions)) {
+                if (!isset($versions[$app])) {
+                    $apps[$i]['load'] = $warning;
+                    $apps[$i]['vstatus'] = _("No stable version exists yet.");
+                } elseif (version_compare(preg_replace('/H3 \((.*)\)/', '$1', $versions[$app]['version']), preg_replace('/H3 \((.*)\)/', '$1', $apps[$i]['version']), '>')) {
+                    $apps[$i]['load'] = $error;
+                    $apps[$i]['vstatus'] = Horde::link($versions[$app]['url'], sprintf(_("Download %s"), $app)) . sprintf(_("A newer version (%s) exists."), $versions[$app]['version']) . '</a> ';
+                } else {
+                    $apps[$i]['load'] = $success;
+                    $apps[$i]['vstatus'] = _("Application is up-to-date.");
+                }
+            }
         }
     }
     if (!file_exists($path . '/conf.php')) {
         /* No conf.php exists. */
         $apps[$i]['conf'] = $conf_link . $error . '</a>';
-        $apps[$i]['status'] = _("Missing configuration. You have to generate it now if you want to use this application.");
+        $apps[$i]['status'] = _("Missing configuration. You must generate it before using this application.");
     } else {
         /* A conf.php exists, get the xml version. */
         if (($xml_ver = _getVersion($path . '/conf.xml')) === false) {
@@ -171,9 +237,8 @@ if (!empty($_SESSION['_config'])) {
     require_once 'Horde/Form.php';
     require_once 'Horde/Variables.php';
 
-    $vars = &Variables::getDefaultVariables();
-    $ftpform = &Horde_Form::singleton('', $vars);
-    $ftpform->setTitle(_("FTP upload of setup"));
+    $vars = Variables::getDefaultVariables();
+    $ftpform = new Horde_Form($vars);
     $ftpform->setButtons(_("Upload"), true);
     $ftpform->addVariable(_("Username"), 'username', 'text', true, false, null, array('', 20));
     $ftpform->addVariable(_("Password"), 'password', 'password', false);
@@ -189,9 +254,7 @@ if (!empty($_SESSION['_config'])) {
         }
     }
     /* Render the form. */
-    require_once 'Horde/Form/Renderer.php';
-    $renderer = &new Horde_Form_Renderer();
-    $ftpform = Util::bufferOutput(array($ftpform, 'renderActive'), $renderer, $vars, 'index.php', 'post');
+    $ftpform = Util::bufferOutput(array($ftpform, 'renderActive'), new Horde_Form_Renderer(), $vars, 'index.php', 'post');
 }
 
 if (file_exists(Horde::getTempDir() . '/horde_setup_upgrade.php')) {
@@ -204,16 +267,18 @@ if (file_exists(Horde::getTempDir() . '/horde_setup_upgrade.php')) {
 }
 
 /* Set up the template. */
-$template = &new Horde_Template();
+$template = new Horde_Template();
+$template->setOption('gettext', true);
+$template->set('versions', !empty($versions), true);
+$template->set('version_action', Horde::applicationUrl('admin/setup/index.php'));
+$template->set('version_input', Util::formInput());
 $template->set('apps', $apps);
 $template->set('actions', $actions, true);
 $template->set('ftpform', $ftpform, true);
-$template->set('notify', Util::bufferOutput(array($notification, 'notify')));
-$template->setOption('gettext', true);
 
 $title = sprintf(_("%s Setup"), $registry->get('name', 'horde'));
 Horde::addScriptFile('stripe.js', 'horde', true);
 require HORDE_TEMPLATES . '/common-header.inc';
-require HORDE_TEMPLATES . '/admin/common-header.inc';
+require HORDE_TEMPLATES . '/admin/menu.inc';
 echo $template->fetch(HORDE_TEMPLATES . '/admin/setup/index.html');
 require HORDE_TEMPLATES . '/common-footer.inc';
