@@ -3,23 +3,38 @@
  * Possible strings from weather.com, just to have them in the source for
  * localization reasons.
  *
- * $Horde: horde/lib/Block/weatherdotcom_strings.php,v 1.1.2.2 2005/08/30 10:14:20 jan Exp $
+ * $Horde: horde/lib/Block/weatherdotcom_strings.php,v 1.1.2.10 2008/07/19 08:57:07 jan Exp $
  */
 
 _("AM Clouds");
+_("AM Dizzle");
+_("AM Fog");
 _("AM Light Rain");
+_("AM Light Snow");
 _("AM Rain");
 _("AM Showers");
+_("AM Snow");
 _("AM Snow Showers");
+_("AM Sun");
+_("AM T-Showers");
+_("AM T-Storms");
 _("Clear");
+_("Clearing Early");
 _("Clearing Late");
 _("Clouds Early");
+_("Clouds Late");
 _("Cloudy");
+_("Drizzle");
 _("Fair");
 _("Few Showers");
 _("Few Snow Showers");
 _("Fog");
+_("Fog Early");
+_("Fog Late");
+_("Foggy");
+_("Freezing Drizzle");
 _("Haze");
+_("Heavy Rain");
 _("Heavy T-Storm");
 _("Isolated T-Storms");
 _("Light Drizzle");
@@ -29,16 +44,25 @@ _("Light Rain Late");
 _("Light Rain Shower");
 _("Light Rain with Thunder");
 _("Light Snow");
+_("Light Snow Early");
+_("Light Snow Late");
 _("Light Snow Shower");
 _("Mostly Clear");
 _("Mostly Cloudy");
+_("Mostly Cloudy and Windy");
 _("Mostly Sunny");
 _("Partly Cloudy");
+_("PM Clouds");
+_("PM Dizzle");
+_("PM Fog");
 _("PM Light Rain");
+_("PM Light Snow");
+_("PM Rain");
 _("PM Showers");
 _("PM Snow");
 _("PM Snow Showers");
 _("PM Sun");
+_("PM T-Showers");
 _("PM T-Storms");
 _("Rain");
 _("Rain Early");
@@ -46,24 +70,32 @@ _("Rain Late");
 _("Rain Shower");
 _("Rain and Snow");
 _("Rain to Snow");
+_("Scattered Showers");
 _("Scattered T-Storms");
 _("Showers");
 _("Showers Early");
 _("Showers Late");
 _("Showers in the Vicinity");
 _("Snow");
+_("Snow Early");
+_("Snow Late");
 _("Snow Shower");
 _("Snow Showers");
 _("Snow Showers Early");
 _("Snow Showers Late");
 _("Sunny");
+_("T-Showers");
+_("T-Showers Early");
+_("T-Showers Late");
 _("T-Storm");
 _("T-Storm and Windy");
 _("T-Storms");
 _("T-Storms Early");
+_("T-Storms Late");
 _("Thunder");
 _("Wind");
 _("Wind Early");
+_("Wind Late");
 _("Low");
 _("Moderate");
 _("High");
