@@ -65,12 +65,12 @@ define('AUTH_REASON_SESSIONIP', 'sessionip');
  * The Auth:: class provides a common abstracted interface into the various
  * backends for the Horde authentication system.
  *
- * $Horde: framework/Auth/Auth.php,v 1.142.10.25 2007/01/02 13:54:06 jan Exp $
+ * $Horde: framework/Auth/Auth.php,v 1.142.10.33 2008/09/22 21:51:56 jan Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Horde 1.3
@@ -90,6 +90,7 @@ class Auth {
                               'remove'        => false,
                               'list'          => false,
                               'groups'        => false,
+                              'admins'        => false,
                               'transparent'   => false);
 
     /**
@@ -133,20 +134,15 @@ class Auth {
      */
     function authenticate($userId, $credentials, $login = true, $realm = null)
     {
-        global $conf;
-
         $auth = false;
         $userId = trim($userId);
 
-        if (!empty($conf['hooks']['preauthenticate'])) {
-            include_once HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_horde_hook_preauthenticate')) {
-                if (!call_user_func('_horde_hook_preauthenticate', $userId, $credentials, $realm)) {
-                    if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
-                        $this->_setAuthError(AUTH_REASON_FAILED);
-                    }
-                    return false;
+        if (!empty($GLOBALS['conf']['hooks']['preauthenticate'])) {
+            if (!Horde::callHook('_horde_hook_preauthenticate', array($userId, $credentials, $realm), 'horde', false)) {
+                if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
+                    $this->_setAuthError(AUTH_REASON_FAILED);
                 }
+                return false;
             }
         }
 
@@ -163,24 +159,13 @@ class Auth {
             if (is_a($authenticated, 'PEAR_Error')) {
                 return false;
             }
-            if (!empty($conf['hooks']['postauthenticate'])) {
-                include_once HORDE_BASE . '/config/hooks.php';
-                if (function_exists('_horde_hook_postauthenticate')) {
-                    if (!call_user_func('_horde_hook_postauthenticate', $userId, $credentials, $realm)) {
-                        if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
-                            $this->_setAuthError(AUTH_REASON_FAILED);
-                        }
-                        return false;
-                    }
-                }
-            }
 
             if ($login) {
-                $this->setAuth($this->_authCredentials['userId'],
-                               $this->_authCredentials['credentials'],
-                               $this->_authCredentials['realm'],
-                               $this->_authCredentials['changeRequested']);
-                $auth = true;
+                $auth = $this->setAuth(
+                    $this->_authCredentials['userId'],
+                    $this->_authCredentials['credentials'],
+                    $this->_authCredentials['realm'],
+                    $this->_authCredentials['changeRequested']);
             } else {
                 if (!$this->_checkSessionIP()) {
                     $this->_setAuthError(AUTH_REASON_SESSIONIP);
@@ -226,7 +211,7 @@ class Auth {
             return String::convertCharset('"' . $plaintext . '"', 'ISO-8859-1', 'UTF-16LE');
 
         case 'sha':
-            $encrypted = base64_encode(mhash(MHASH_SHA1, $plaintext));
+            $encrypted = base64_encode(pack('H*', sha1($plaintext)));
             return ($show_encrypt) ? '{SHA}' . $encrypted : $encrypted;
 
         case 'crypt':
@@ -236,33 +221,33 @@ class Auth {
             return ($show_encrypt ? '{crypt}' : '') . crypt($plaintext, $salt);
 
         case 'md5-base64':
-            $encrypted = base64_encode(mhash(MHASH_MD5, $plaintext));
+            $encrypted = base64_encode(pack('H*', md5($plaintext)));
             return ($show_encrypt) ? '{MD5}' . $encrypted : $encrypted;
 
         case 'ssha':
-            $encrypted = base64_encode(mhash(MHASH_SHA1, $plaintext . $salt) . $salt);
+            $encrypted = base64_encode(pack('H*', sha1($plaintext . $salt)) . $salt);
             return ($show_encrypt) ? '{SSHA}' . $encrypted : $encrypted;
 
         case 'smd5':
-            $encrypted = base64_encode(mhash(MHASH_MD5, $plaintext . $salt) . $salt);
+            $encrypted = base64_encode(pack('H*', md5($plaintext . $salt)) . $salt);
             return ($show_encrypt) ? '{SMD5}' . $encrypted : $encrypted;
 
         case 'aprmd5':
             $length = strlen($plaintext);
             $context = $plaintext . '$apr1$' . $salt;
-            $binary = Auth::_bin(md5($plaintext . $salt . $plaintext));
+            $binary = pack('H*', md5($plaintext . $salt . $plaintext));
 
             for ($i = $length; $i > 0; $i -= 16) {
                 $context .= substr($binary, 0, ($i > 16 ? 16 : $i));
             }
             for ($i = $length; $i > 0; $i >>= 1) {
-                $context .= ($i & 1) ? chr(0) : $plaintext{0};
+                $context .= ($i & 1) ? chr(0) : $plaintext[0];
             }
 
-            $binary = Auth::_bin(md5($context));
+            $binary = pack('H*', md5($context));
 
             for ($i = 0; $i < 1000; $i++) {
-                $new = ($i & 1) ? $plaintext : substr($binary, 0,16);
+                $new = ($i & 1) ? $plaintext : substr($binary, 0, 16);
                 if ($i % 3) {
                     $new .= $salt;
                 }
@@ -270,7 +255,7 @@ class Auth {
                     $new .= $plaintext;
                 }
                 $new .= ($i & 1) ? substr($binary, 0, 16) : $plaintext;
-                $binary = Auth::_bin(md5($new));
+                $binary = pack('H*', md5($new));
             }
 
             $p = array();
@@ -312,7 +297,6 @@ class Auth {
      */
     function getSalt($encryption = 'md5-hex', $seed = '', $plaintext = '')
     {
-        // Encrypt the password.
         switch ($encryption) {
         case 'crypt':
         case 'crypt-des':
@@ -340,14 +324,16 @@ class Auth {
             if ($seed) {
                 return substr(base64_decode(preg_replace('|^{SSHA}|i', '', $seed)), 20);
             } else {
-                return mhash_keygen_s2k(MHASH_SHA1, $plaintext, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
+                $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+                return substr(pack('H*', sha1($salt . $plaintext)), 0, 4);
             }
 
         case 'smd5':
             if ($seed) {
                 return substr(base64_decode(preg_replace('|^{SMD5}|i', '', $seed)), 16);
             } else {
-                return mhash_keygen_s2k(MHASH_MD5, $plaintext, substr(pack('h*', md5(mt_rand())), 0, 8), 4);
+                $salt = substr(pack('h*', md5(mt_rand())), 0, 8);
+                return substr(pack('H*', md5($salt . $plaintext)), 0, 4);
             }
 
         case 'aprmd5':
@@ -359,7 +345,7 @@ class Auth {
             } else {
                 $salt = '';
                 for ($i = 0; $i < 8; $i++) {
-                    $salt .= $APRMD5{rand(0, 63)};
+                    $salt .= $APRMD5[mt_rand(0, 63)];
                 }
                 return $salt;
             }
@@ -450,15 +436,23 @@ class Auth {
     {
         global $registry;
 
+        $errApps = array();
+
         foreach ($registry->listApps(array('notoolbar', 'hidden', 'active', 'admin')) as $app) {
             if ($registry->hasMethod('removeUserData', $app)) {
                 if (is_a($result = $registry->callByPackage($app, 'removeUserData', array($userId)), 'PEAR_Error')) {
-                    return $result;
+                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    $errApps[] = $app;
                 }
             }
         }
 
-        return false;
+        if (count($errApps)) {
+            $err = implode(', ', $errApps);
+            return PEAR::raiseError(sprintf(_("The following applications encountered errors removing user data: %s"), $err));
+        } else {
+            return true;
+        }
     }
 
     /**
@@ -484,7 +478,11 @@ class Auth {
      */
     function exists($userId)
     {
-        return in_array($userId, $this->listUsers());
+        $users = $this->listUsers();
+        if (is_a($users, 'PEAR_Error')) {
+            return $users;
+        }
+        return in_array($userId, $users);
     }
 
     /**
@@ -529,8 +527,7 @@ class Auth {
 
         // Try transparent authentication now.
         $auth = &Auth::singleton($GLOBALS['conf']['auth']['driver']);
-        if ($auth->hasCapability('transparent') &&
-            $auth->transparent()) {
+        if ($auth->hasCapability('transparent') && $auth->transparent()) {
             return Auth::isAuthenticated($realm);
         }
 
@@ -625,7 +622,8 @@ class Auth {
         if (!empty($_SESSION['__auth']) &&
             !empty($_SESSION['__auth']['authenticated'])) {
             require_once 'Horde/Secret.php';
-            $credentials = @unserialize(Secret::read(Secret::getKey('auth'), $_SESSION['__auth']['credentials']));
+            $credentials = Secret::read(Secret::getKey('auth'), $_SESSION['__auth']['credentials']);
+            $credentials = @unserialize($credentials);
         } else {
             return false;
         }
@@ -677,10 +675,19 @@ class Auth {
         $userId = trim($userId);
         $userId = Auth::addHook($userId);
 
+        if (!empty($GLOBALS['conf']['hooks']['postauthenticate'])) {
+            if (!Horde::callHook('_horde_hook_postauthenticate', array($userId, $credentials, $realm), 'horde', false)) {
+                if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
+                    $this->_setAuthError(AUTH_REASON_FAILED);
+                }
+                return false;
+            }
+        }
+
         /* If we're already set with this userId, don't continue. */
         if (isset($_SESSION['__auth']['userId']) &&
             $_SESSION['__auth']['userId'] == $userId) {
-            return;
+            return true;
         }
 
         /* Clear any existing info. */
@@ -764,6 +771,8 @@ class Auth {
                 exit;
             }
         }
+
+        return true;
     }
 
     /**
@@ -801,14 +810,20 @@ class Auth {
      */
     function isAdmin($permission = null, $permlevel = null, $user = null)
     {
-        global $conf;
-
-        if ($user === null) {
+        if (is_null($user)) {
             $user = Auth::getAuth();
         }
 
-        if (@is_array($conf['auth']['admins'])) {
-            if ($user && in_array($user, $conf['auth']['admins'])) {
+        if ($user
+            && @is_array($GLOBALS['conf']['auth']['admins'])
+            && in_array($user, $GLOBALS['conf']['auth']['admins'])) {
+            return true;
+        }
+
+        if ($user) {
+            $auth = &Auth::singleton($GLOBALS['conf']['auth']['driver']);
+            if ($auth->hasCapability('admins')
+                && $auth->_isAdmin($permission, $permlevel, $user)) {
                 return true;
             }
         }
@@ -827,21 +842,21 @@ class Auth {
      * Applies a hook defined by the function _username_hook_frombackend() to
      * the given user name if this function exists and user hooks are enabled.
      *
-     * This method should be called if a backend's user name needs to be
-     * converted to a (unique) Horde user name.
+     * This method should be called if a authentication backend's user name
+     * needs to be converted to a (unique) Horde user name. The backend's user
+     * name is what the user sees and uses, but internally we use the Horde
+     * user name.
      *
-     * @param string $userId  The original user name.
+     * @param string $userId  The authentication backend's user name.
      *
-     * @return string  The user name with the hook applied to it.
+     * @return string  The internal Horde user name.
      */
     function addHook($userId)
     {
-        global $conf;
-
-        if (!empty($conf['hooks']['username'])) {
-            require_once HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_username_hook_frombackend')) {
-                $userId = call_user_func('_username_hook_frombackend', $userId);
+        if (!empty($GLOBALS['conf']['hooks']['username'])) {
+            $newId = Horde::callHook('_username_hook_frombackend', array($userId));
+            if (!is_a($newId, 'PEAR_Error')) {
+                return $newId;
             }
         }
 
@@ -853,20 +868,20 @@ class Auth {
      * the given user name if this function exists and user hooks are enabled.
      *
      * This method should be called if a Horde user name needs to be converted
-     * to a backend's user name or displayed to the user.
+     * to an authentication backend's user name or displayed to the user. The
+     * backend's user name is what the user sees and uses, but internally we
+     * use the Horde user name.
      *
-     * @param string $userId  The Horde user name.
+     * @param string $userId  The internal Horde user name.
      *
-     * @return string  The user name with the hook applied to it.
+     * @return string  The authentication backend's user name.
      */
     function removeHook($userId)
     {
-        global $conf;
-
-        if (!empty($conf['hooks']['username'])) {
-            require_once HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_username_hook_tobackend')) {
-                $userId = call_user_func('_username_hook_tobackend', $userId);
+        if (!empty($GLOBALS['conf']['hooks']['username'])) {
+            $newId = Horde::callHook('_username_hook_tobackend', array($userId));
+            if (!is_a($newId, 'PEAR_Error')) {
+                return $newId;
             }
         }
 
@@ -897,8 +912,7 @@ class Auth {
      */
     function getLoginScreen($app = 'horde', $url = '')
     {
-        global $conf;
-        $auth = &Auth::singleton($conf['auth']['driver']);
+        $auth = &Auth::singleton($GLOBALS['conf']['auth']['driver']);
         return $auth->_getLoginScreen($app, $url);
     }
 
@@ -927,10 +941,8 @@ class Auth {
      */
     function getProvider($driver = null, $params = null)
     {
-        global $conf;
-
         if (is_null($driver)) {
-            $driver = $conf['auth']['driver'];
+            $driver = $GLOBALS['conf']['auth']['driver'];
         }
         if (is_null($params)) {
             $params = Horde::getDriverConfig('auth',
@@ -1033,21 +1045,78 @@ class Auth {
      */
     function addLogoutParameters($url, $reason = null, $msg = null)
     {
+        $params = array('horde_logout_token' => Horde::getRequestToken('horde.logout'));
+
+        if (isset($GLOBALS['registry'])) {
+            $params['app'] = $GLOBALS['registry']->getApp();
+        }
+
         if (is_null($reason)) {
             $reason = Auth::getLogoutReason();
         }
 
         if ($reason) {
-            $url = Util::addParameter($url, AUTH_REASON_PARAM, $reason, false);
+            $params[AUTH_REASON_PARAM] = $reason;
             if ($reason == AUTH_REASON_MESSAGE) {
                 if (is_null($msg)) {
                     $msg = Auth::getLogoutReasonString();
                 }
-                $url = Util::addParameter($url, AUTH_REASON_MSG_PARAM, $msg, false);
+                $params[AUTH_REASON_MSG_PARAM] = $msg;
             }
         }
 
-        return $url;
+        return Util::addParameter($url, $params, null, false);
+    }
+
+    /**
+     * Reads session data to determine if it contains Horde authentication
+     * credentials.
+     *
+     * @since Horde 3.2
+     *
+     * @param string $session_data  The session data.
+     * @param boolean $info         Return session information.  The following
+     *                              information is returned: userid, realm,
+     *                              timestamp, remote_addr, browser.
+     *
+     * @return array  An array of the user's sesion information if
+     *                authenticated or false.  The following information is
+     *                returned: userid, realm, timestamp, remote_addr, browser.
+     */
+    function readSessionData($session_data)
+    {
+        if (empty($session_data)) {
+            return false;
+        }
+
+        $pos = strpos($session_data, '__auth|');
+        if ($pos === false) {
+            return false;
+        }
+
+        $endpos = $pos + 7;
+        $old_error = error_reporting(0);
+
+        while ($endpos !== false) {
+            $endpos = strpos($session_data, '|', $endpos);
+            $data = unserialize(substr($session_data, $pos + 7, $endpos));
+            if (is_array($data)) {
+                error_reporting($old_error);
+                if (empty($data['authenticated'])) {
+                    return false;
+                }
+                return array(
+                    'userid' => $data['userId'],
+                    'realm' => $data['realm'],
+                    'timestamp' => $data['timestamp'],
+                    'remote_addr' => $data['remote_addr'],
+                    'browser' => $data['browser']
+                );
+            }
+            ++$endpos;
+        }
+
+        return false;
     }
 
     /**
@@ -1073,7 +1142,7 @@ class Auth {
      * Authentication stub.
      *
      * @abstract
-     * @access private
+     * @access protected
      *
      * @return boolean  False.
      */
@@ -1082,6 +1151,19 @@ class Auth {
         return false;
     }
 
+    /**
+     * Driver-level admin check stub.
+     *
+     * @abstract
+     * @access protected
+     *
+     * @return boolean  False.
+     */
+    function _isAdmin($permission = null, $permlevel = null, $user = null)
+    {
+        return false;
+    }
+
     /**
      * Sets the error message for an invalid authentication.
      *
@@ -1190,26 +1272,6 @@ class Auth {
         return $aprmd5;
     }
 
-    /**
-     * Converts hexadecimal string to binary data.
-     *
-     * @access private
-     *
-     * @param string $hex  Hex data.
-     *
-     * @return string  Binary data.
-     */
-    function _bin($hex)
-    {
-        $bin = '';
-        $length = strlen($hex);
-        for ($i = 0; $i < $length; $i += 2) {
-            $tmp = sscanf(substr($hex, $i, 2), '%x');
-            $bin .= chr(array_shift($tmp));
-        }
-        return $bin;
-    }
-
     /**
      * Attempts to return a concrete Auth instance based on $driver.
      *
@@ -1224,7 +1286,7 @@ class Auth {
      * @return Auth  The newly created concrete Auth instance, or false on an
      *               error.
      */
-    function &factory($driver, $params = null)
+    function factory($driver, $params = null)
     {
         if (is_array($driver)) {
             $app = $driver[0];
@@ -1232,31 +1294,31 @@ class Auth {
         }
 
         $driver = basename($driver);
-
         if (empty($driver) || ($driver == 'none')) {
-            $auth = &new Auth();
-            return $auth;
+            return new Auth();
         }
 
         if (is_null($params)) {
             $params = Horde::getDriverConfig('auth', $driver);
         }
 
-        ini_set('track_errors', 1);
-        $include_error = '';
-        if (!empty($app)) {
-            @include_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Auth/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/Auth/' . $driver . '.php';
-        }
-        if (isset($php_errormsg)) {
-            $include_error = $php_errormsg;
-        }
-        ini_restore('track_errors');
-
         $class = 'Auth_' . $driver;
+        $include_error = '';
+        if (!class_exists($class)) {
+            $oldTrackErrors = ini_set('track_errors', 1);
+            if (!empty($app)) {
+                include $GLOBALS['registry']->get('fileroot', $app) . '/lib/Auth/' . $driver . '.php';
+            } else {
+                include 'Horde/Auth/' . $driver . '.php';
+            }
+            if (isset($php_errormsg)) {
+                $include_error = $php_errormsg;
+            }
+            ini_set('track_errors', $oldTrackErrors);
+        }
+
         if (class_exists($class)) {
-            $auth = &new $class($params);
+            $auth = new $class($params);
         } else {
             $auth = PEAR::raiseError('Auth Driver (' . $class . ') not found' . ($include_error ? ': ' . $include_error : '') . '.');
         }
@@ -1293,7 +1355,7 @@ class Auth {
 
         $signature = serialize(array($driver, $params));
         if (empty($instances[$signature])) {
-            $instances[$signature] = &Auth::factory($driver, $params);
+            $instances[$signature] = Auth::factory($driver, $params);
         }
 
         return $instances[$signature];
