@@ -11,12 +11,12 @@
  * Parameters:
  *   'blocks'     An array of CIDR masks which are allowed access.
  *
- * $Horde: framework/Auth/Auth/ipbasic.php,v 1.20.10.8 2007/01/02 13:54:07 jan Exp $
+ * $Horde: framework/Auth/Auth/ipbasic.php,v 1.20.10.11 2008/07/03 22:08:25 chuck Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Horde 1.3
@@ -81,8 +81,7 @@ class Auth_ipbasic extends Auth {
         $client = $_SERVER['REMOTE_ADDR'];
         foreach ($this->_params['blocks'] as $cidr) {
             if ($this->_addressWithinCIDR($client, $cidr)) {
-                $this->setAuth($cidr, array('transparent' => 1));
-                return true;
+                return $this->setAuth($cidr, array('transparent' => 1));
             }
         }
 
