@@ -4,28 +4,30 @@
  * authentication system.
  *
  * PAM (Pluggable Authentication Modules) is a flexible mechanism for
- * authenticating users.  It has become the standard authentication system for
+ * authenticating users. It has become the standard authentication system for
  * Linux, Solaris and FreeBSD.
  *
- * This implementation requires Chad Cunningham's pam_auth extension:
+ * This driver relies on the PECL PAM package:
  *
- *      http://www.math.ohio-state.edu/~ccunning/pam_auth/
+ *      http://pecl.php.net/package/PAM
  *
  * Optional parameters:<pre>
  *   'service'  The name of the PAM service to use when authenticating.
  *              DEFAULT: php</pre>
  *
  *
- * $Horde: framework/Auth/Auth/pam.php,v 1.3.10.11 2006/01/01 21:28:07 jan Exp $
+ * $Horde: framework/Auth/Auth/pam.php,v 1.3.10.15 2008/07/13 12:54:25 jan Exp $
  *
- * Copyright 2004-2006 Jon Parise <jon@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
- * @author  Jan Parise <jon@horde.org>
- * @since   Horde 3.0
- * @package Horde_Auth
+ * @deprecated This driver will be removed for Horde 4.0 unless there is a
+ *             maintained PAM extension at that time.
+ * @author     Jan Parise <jon@horde.org>
+ * @since      Horde 3.0
+ * @package    Horde_Auth
  */
 class Auth_pam extends Auth {
 
@@ -50,13 +52,12 @@ class Auth_pam extends Auth {
     function Auth_pam($params = array())
     {
         $this->_params = $params;
+
         if (!empty($params['service'])) {
-            ini_set('pam_auth.servicename', $params['service']);
+            ini_set('pam.servicename', $params['service']);
         }
 
-        if (!extension_loaded('pam_auth')) {
-            dl('pam_auth.so');
-        }
+        Util::loadExtension('pam') || Util::loadExtension('pam_auth');
     }
 
     /**
@@ -72,13 +73,16 @@ class Auth_pam extends Auth {
     function _authenticate($userId, $credentials)
     {
         if (empty($credentials['password'])) {
-            Horde::fatal(_("No password provided for Login authentication."), __FILE__, __LINE__);
+            $this->_setAuthError(AUTH_REASON_BADLOGIN);
+            return false;
         }
 
-        if (!extension_loaded('pam_auth')) {
-            Horde::fatal(_("PAM authentication is not available."), __FILE__, __LINE__);
+        if (!function_exists('pam_auth')) {
+            $this->_setAuthError(AUTH_REASON_MESSAGE, _("PAM authentication is not available."));
+            return false;
         }
 
+        $error = null;
         if (!pam_auth($userId, $credentials['password'], &$error)) {
             $this->_setAuthError(AUTH_REASON_MESSAGE, $error);
             return false;
@@ -86,4 +90,5 @@ class Auth_pam extends Auth {
 
         return true;
     }
+
 }
