@@ -39,15 +39,22 @@
  *   'port'         The port on which to connect to the database.
  *   'tty'          The TTY on which to connect to the database.</pre>
  *
+ * Optional values when using separate read and write servers, for example
+ * in replication settings:<pre>
+ *   'splitread'   Boolean, whether to implement the separation or not.
+ *   'read'        Array containing the parameters which are different for
+ *                 the read database connection, currently supported
+ *                 only 'hostspec' and 'port' parameters.</pre>
+ *
  * The table structure for the Auth system is in
  * scripts/sql/horde_users.sql.
  *
- * $Horde: framework/Auth/Auth/sql.php,v 1.69.10.20 2007/01/02 13:54:07 jan Exp $
+ * $Horde: framework/Auth/Auth/sql.php,v 1.69.10.22 2008/01/02 11:30:19 jan Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Horde 1.3
@@ -75,6 +82,14 @@ class Auth_sql extends Auth {
      */
     var $_db;
 
+    /**
+     * Handle for the current database connection, used for writing. Defaults
+     * to the same handle as $_db if a separate write database is not required.
+     *
+     * @var DB
+     */
+    var $_write_db;
+
     /**
      * Boolean indicating whether or not we're connected to the SQL server.
      *
@@ -104,8 +119,11 @@ class Auth_sql extends Auth {
      */
     function _authenticate($userId, $credentials)
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            $this->_setAuthError(AUTH_REASON_FAILED);
+            return false;
+        }
 
         /* Build the SQL query. */
         $query = sprintf('SELECT * FROM %s WHERE %s = ?',
@@ -117,6 +135,7 @@ class Auth_sql extends Auth {
 
         $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             $this->_setAuthError(AUTH_REASON_FAILED);
             return false;
         }
@@ -162,7 +181,9 @@ class Auth_sql extends Auth {
      */
     function addUser($userId, $credentials)
     {
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Build the SQL query. */
         $query = sprintf('INSERT INTO %s (%s, %s) VALUES (?, ?)',
@@ -177,7 +198,7 @@ class Auth_sql extends Auth {
 
         Horde::logMessage('SQL Query by Auth_sql::addUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -196,8 +217,9 @@ class Auth_sql extends Auth {
      */
     function updateUser($oldID, $newID, $credentials)
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Build the SQL query. */
         $tuple = array();
@@ -224,7 +246,7 @@ class Auth_sql extends Auth {
 
             global $notification;
             if (!empty($notification)) {
-                $notification->push(strftime(_("New password will expire on %x."), $date), 'horde.message');
+                $notification->push(strftime(_("New password will expire on %s."), $date), 'horde.message');
             }
 
             if (empty($this->_params['hard_expiration_window'])) {
@@ -243,13 +265,13 @@ class Auth_sql extends Auth {
         require_once 'Horde/SQL.php';
         $query = sprintf('UPDATE %s SET %s WHERE %s = ?',
                          $this->_params['table'],
-                         Horde_SQL::updateValues($this->_db, $tuple),
+                         Horde_SQL::updateValues($this->_write_db, $tuple),
                          $this->_params['username_field']);
         $values = array($oldID);
 
         Horde::logMessage('SQL Query by Auth_sql:updateUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -262,15 +284,16 @@ class Auth_sql extends Auth {
      * Reset a user's password. Used for example when the user does not
      * remember the existing password.
      *
-     * @param string $user_id  The user id for which to reset the password.
+     * @param string $userId  The user id for which to reset the password.
      *
      * @return mixed  The new password on success or a PEAR_Error object on
      *                failure.
      */
-    function resetPassword($user_id)
+    function resetPassword($userId)
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Get a new random password. */
         $password = Auth::genRandomPassword();
@@ -284,11 +307,11 @@ class Auth_sql extends Auth {
                                                   '',
                                                   $this->_params['encryption'],
                                                   $this->_params['show_encryption']),
-                        $user_id);
+                        $userId);
 
         Horde::logMessage('SQL Query by Auth_sql::resetPassword(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -305,8 +328,9 @@ class Auth_sql extends Auth {
      */
     function removeUser($userId)
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Build the SQL query. */
         $query = sprintf('DELETE FROM %s WHERE %s = ?',
@@ -316,7 +340,7 @@ class Auth_sql extends Auth {
 
         Horde::logMessage('SQL Query by Auth_sql::removeUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -331,8 +355,9 @@ class Auth_sql extends Auth {
      */
     function listUsers()
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Build the SQL query. */
         $query = sprintf('SELECT %s FROM %s',
@@ -345,18 +370,18 @@ class Auth_sql extends Auth {
     }
 
     /**
-     * Checks if a userId exists in the sistem.
+     * Checks if a userId exists in the system.
      *
      * @return boolean  Whether or not the userId already exists.
      */
     function exists($userId)
     {
-        /* _connect() will die with Horde::fatal() upon failure. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            return $result;
+        }
 
         /* Build the SQL query. */
-        $query = sprintf('SELECT %s FROM %s WHERE %s = ?',
-                         $this->_params['username_field'],
+        $query = sprintf('SELECT 1 FROM %s WHERE %s = ?',
                          $this->_params['table'],
                          $this->_params['username_field']);
         $values = array($userId);
@@ -435,19 +460,42 @@ class Auth_sql extends Auth {
 
         /* Connect to the SQL server using the supplied parameters. */
         include_once 'DB.php';
-        $this->_db = &DB::connect($this->_params,
-                                  array('persistent' => !empty($this->_params['persistent'])));
-        if (is_a($this->_db, 'PEAR_Error')) {
-            Horde::fatal(_("Unable to connect to SQL server."), __FILE__, __LINE__);
+        $this->_write_db = &DB::connect($this->_params,
+                                        array('persistent' => !empty($this->_params['persistent'])));
+        if (is_a($this->_write_db, 'PEAR_Error')) {
+            return $this->_write_db;
         }
 
         // Set DB portability options.
-        switch ($this->_db->phptype) {
+        switch ($this->_write_db->phptype) {
         case 'mssql':
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
             break;
         default:
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        }
+
+        /* Check if we need to set up the read DB connection
+         * seperately. */
+        if (!empty($this->_params['splitread'])) {
+            $params = array_merge($this->_params, $this->_params['read']);
+            $this->_db = &DB::connect($params,
+                                      array('persistent' => !empty($params['persistent'])));
+            if (is_a($this->_db, 'PEAR_Error')) {
+                return $this->_db;
+            }
+
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+
+        } else {
+            /* Default to the same DB handle for reads. */
+            $this->_db =& $this->_write_db;
         }
 
         $this->_connected = true;
