@@ -1,15 +1,15 @@
 <?php
 
 require_once 'Horde/Array.php';
+require_once 'Horde/Block/Layout.php';
 
 /**
- * The Horde_Block_Layout_Manager class allows manipulation of
- * Horde_Block layouts.
+ * The Horde_Block_Layout_Manager class allows manipulation of Horde_Block
+ * layouts.
  *
- * $Horde: framework/Block/Block/Layout/Manager.php,v 1.4.2.6 2007/11/22 00:16:14 jan Exp $
+ * $Horde: framework/Block/Block/Layout/Manager.php,v 1.4.2.11 2008/06/15 11:52:01 jan Exp $
  *
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -19,7 +19,7 @@ require_once 'Horde/Array.php';
  * @since   Horde 3.2
  * @package Horde_Block
  */
-class Horde_Block_Layout_Manager {
+class Horde_Block_Layout_Manager extends Horde_Block_Layout {
 
     /**
      * Our Horde_Block_Collection instance.
@@ -81,12 +81,13 @@ class Horde_Block_Layout_Manager {
      * Constructor.
      *
      * @param Horde_Block_Collection $collection
-     * @param array $data
+     * @param array $layout
      */
     function Horde_Block_Layout_Manager(&$collection, $layout = array())
     {
         $this->_collection = &$collection;
         $this->_layout = $layout;
+        $this->_editUrl = Horde::selfUrl();
 
         // Fill the _covered caches and empty rows.
         $rows = count($this->_layout);
@@ -190,11 +191,12 @@ class Horde_Block_Layout_Manager {
     /**
      * Process a modification to the current layout.
      *
-     * @var string $action
-     * @var integer $row
-     * @var integer $col
+     * @param string $action
+     * @param integer $row
+     * @param integer $col
+     * @param string $url
      */
-    function handle($action, $row, $col)
+    function handle($action, $row, $col, $url = null)
     {
         switch ($action) {
         case 'moveUp':
@@ -233,7 +235,7 @@ class Horde_Block_Layout_Manager {
                 // Check permissions.
                 if (Horde::hasPermission('max_blocks') !== true &&
                     Horde::hasPermission('max_blocks') <= $this->count()) {
-                    $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d blocks."), Horde::hasPermission('max_blocks')), ENT_COMPAT, NLS::getCharset());
+                    $message = @htmlspecialchars(sprintf(ngettext("You are not allowed to create more than %d block.", "You are not allowed to create more than %d blocks.", Horde::hasPermission('max_blocks')), Horde::hasPermission('max_blocks')), ENT_COMPAT, NLS::getCharset());
                     if (!empty($conf['hooks']['permsdenied'])) {
                         $message = Horde::callHook('_perms_hook_denied', array('horde:max_blocks'), 'horde', $message);
                     }
@@ -283,8 +285,14 @@ class Horde_Block_Layout_Manager {
         // Make a block the current block for editing.
         case 'edit':
             $this->_currentBlock = array($row, $col);
+            $url = null;
             break;
         }
+
+        if (!empty($url)) {
+            header('Location: ' . Util::addParameter($url, 'unique',
+                                                     md5(microtime()), false));
+        }
     }
 
     /**
@@ -381,7 +389,8 @@ class Horde_Block_Layout_Manager {
     function getBlockInfo($row, $col)
     {
         if (!isset($this->_layout[$row][$col]) ||
-            $this->_layout[$row][$col] == 'empty') {
+            $this->isEmpty($row, $col) ||
+            $this->isCovered($row, $col)) {
             return PEAR::raiseError(_("No block exists at the requested position"), 'horde.error');
         }
 
@@ -529,66 +538,6 @@ class Horde_Block_Layout_Manager {
         return $this->_changed_row === $row && $this->_changed_col === $col;
     }
 
-    /**
-     * Returns if the specified block may be removed.
-     *
-     *
-     * @param integer $row  A layout row.
-     * @param integer $col  A layout column.
-     *
-     * @return boolean  True if this block may be removed.
-     */
-    function isRemovable($row, $col)
-    {
-        global $conf;
-
-        $app = $this->_layout[$row][$col]['app'];
-        $type = $this->_layout[$row][$col]['params']['type'];
-        $block = $app . ':' . $type;
-
-        /* Check if the block is a fixed block. */
-        if (!in_array($block, $conf['portal']['fixed_blocks'])) {
-            return true;
-        }
-
-        /* Check if we have still another block of the same type. */
-        $found = false;
-        foreach ($this->_layout as $cur_row) {
-            foreach ($cur_row as $cur_col) {
-                if (isset($cur_col['app']) &&
-                    $cur_col['app'] == $app &&
-                    $cur_col['params']['type'] == $type) {
-                    if ($found) {
-                        return true;
-                    } else {
-                        $found = true;
-                    }
-                }
-            }
-        }
-
-        return false;
-    }
-
-    /**
-     * Returns an URL triggering an action to a block
-     *
-     * @param string $action  An action to trigger.
-     * @param integer $row  A layout row.
-     * @param integer $col  A layout column.
-     *
-     * @return string  An URL with all necessary parameters.
-     */
-    function getActionUrl($action, $row, $col)
-    {
-        $url = Util::addParameter(Horde::selfUrl(),
-                                  array('col' => $col,
-                                        'row' => $row,
-                                        'action' => $action,
-                                        'nocache' => base_convert(microtime(), 10, 36)));
-        return $url . '#block';
-    }
-
     /**
      * Returns a control (linked arrow) for a certain action on the
      * specified block.
@@ -701,7 +650,8 @@ class Horde_Block_Layout_Manager {
      */
     function getWidth($row, $col)
     {
-        if (!is_array($this->_layout[$row][$col])) {
+        if (!isset($this->_layout[$row][$col]) ||
+            !is_array($this->_layout[$row][$col])) {
             return 1;
         }
         if (!isset($this->_layout[$row][$col]['width'])) {
@@ -722,7 +672,8 @@ class Horde_Block_Layout_Manager {
      */
     function getHeight($row, $col)
     {
-        if (!is_array($this->_layout[$row][$col])) {
+        if (!isset($this->_layout[$row][$col]) ||
+            !is_array($this->_layout[$row][$col])) {
             return 1;
         }
         if (!isset($this->_layout[$row][$col]['height'])) {
@@ -978,6 +929,9 @@ class Horde_Block_Layout_Manager {
         }
 
         for ($i = 0; $i < $width; $i++) {
+            if (!isset($this->_layout[$row][$col + $i])) {
+                continue;
+            }
             $prev = $this->_layout[$row][$col + $i];
             // Move bottom edge
             $this->_layout[$lastrow][$col + $i] = 'covered';
@@ -1061,6 +1015,9 @@ class Horde_Block_Layout_Manager {
 
             $lastcol = $col + $this->getWidth($row, $col) - 1;
             for ($i = 0; $i < $height; $i++) {
+                if (!isset($this->_layout[$row + $i][$col])) {
+                    continue;
+                }
                 $prev = $this->_layout[$row + $i][$col];
                 // Move left hand edge
                 $this->_layout[$row + $i][$col - 1] = $prev;
@@ -1126,6 +1083,9 @@ class Horde_Block_Layout_Manager {
         }
 
         for ($i = 0; $i < $height; $i++) {
+            if (!isset($this->_layout[$row + $i][$col])) {
+                continue;
+            }
             $prev = $this->_layout[$row + $i][$col];
             // Move right hand edge
             $this->_layout[$row + $i][$lastcol] = 'covered';
