@@ -3,14 +3,14 @@
  * The Horde_Compress:: class provides an API for various compression
  * techniques that can be used by Horde applications.
  *
- * $Horde: framework/Compress/Compress.php,v 1.7.12.11 2007/01/02 13:54:11 jan Exp $
+ * $Horde: framework/Compress/Compress.php,v 1.7.12.13 2008/01/02 11:30:26 jan Exp $
  *
- * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Horde 3.0
  * @package Horde_Compress
  */
@@ -36,18 +36,17 @@ class Horde_Compress {
         }
 
         $driver = basename($driver);
-
-        if (!empty($app)) {
-            require_once $app . '/lib/Compress/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/Compress/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/Compress/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/Compress/' . $driver . '.php';
+        $class = 'Horde_Compress_' . $driver;
+        if (!class_exists($class)) {
+            if (!empty($app)) {
+                include_once $app . '/lib/Compress/' . $driver . '.php';
+            } else {
+                include_once 'Horde/Compress/' . $driver . '.php';
+            }
         }
 
-        $class = 'Horde_Compress_' . $driver;
         if (class_exists($class)) {
-            $compress = &new $class($params);
+            $compress = new $class($params);
         } else {
             $compress = false;
         }
