@@ -1,24 +1,31 @@
 <?php
+/**
+ * @package Horde_Framework
+ */
 
-require_once 'Horde/Form.php';
+/**
+ * Horde_DOM
+ */
+include_once 'Horde/DOM.php';
+
+/**
+ * Horde_Form
+ */
+include_once 'Horde/Form.php';
+
+/**
+ * Horde_Form_Renderer
+ */
+include_once 'Horde/Form/Renderer.php';
 
 /**
  * The Config:: package provides a framework for managing the
  * configuration of Horde applications, writing conf.php files from
  * conf.xml source files, generating user interfaces, etc.
  *
- * The Horde_Config_Node DOM classes and subclasses
- * (Horde_Config_Document, Horde_Config_Element) are simply massaged
- * from code written by to emulate PHP4's DOMXML extension with PHP5's
- * DOM support. Only changes are naming and syntactic sugar/Horde
- * coding standards; all credit goes to Alexandre and the code is used
- * with his permission. His original code can be found at:
+ * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.37 2008/09/22 13:49:06 jan Exp $
  *
- *   http://alexandre.alapetite.net/doc-alex/domxml-php4-php5/
- *
- * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.30 2007/01/12 18:28:37 chuck Exp $
- *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -139,7 +146,7 @@ class Horde_Config {
             }
 
             /* Load the DOM object. */
-            $doc = Horde_Config_Document::factory($path . '/conf.xml');
+            $doc = Horde_DOM_Document::factory(array('filename' => $path . '/conf.xml'));
 
             /* Check if there is a CVS version tag and store it. */
             $node = $doc->first_child();
@@ -380,7 +387,7 @@ class Horde_Config {
             switch ($node->tagname) {
             case 'configdescription':
                 if (empty($name)) {
-                    $name = md5(microtime());
+                    $name = md5(uniqid(mt_rand(), true));
                 }
                 $conf[$name] = array('_type' => 'description',
                                      'desc' => Text_Filter::filter($this->_default($curctx, $this->_getNodeOnlyText($node)), 'linkurls', array('callback' => 'Horde::externalUrl')));
@@ -388,7 +395,7 @@ class Horde_Config {
 
             case 'configheader':
                 if (empty($name)) {
-                    $name = md5(microtime());
+                    $name = md5(uniqid(mt_rand(), true));
                 }
                 $conf[$name] = array('_type' => 'header',
                                      'desc' => $this->_default($curctx, $this->_getNodeOnlyText($node)));
@@ -397,9 +404,9 @@ class Horde_Config {
             case 'configswitch':
                 $values = $this->_getSwitchValues($node, $ctx);
                 if ($quote) {
-                    $default = $this->_default($curctx, $this->_getNodeOnlyText($node));
+                    list($default, $isDefault) = $this->__default($curctx, $this->_getNodeOnlyText($node));
                 } else {
-                    $default = $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node));
+                    list($default, $isDefault) = $this->__defaultRaw($curctx, $this->_getNodeOnlyText($node));
                 }
                 if ($default === '') {
                     $default = key($values);
@@ -409,15 +416,16 @@ class Horde_Config {
                 }
                 $conf[$name] = array('desc' => $desc,
                                      'switch' => $values,
-                                     'default' => $default);
+                                     'default' => $default,
+                                     'is_default' => $isDefault);
                 break;
 
             case 'configenum':
                 $values = $this->_getEnumValues($node);
                 if ($quote) {
-                    $default = $this->_default($curctx, $this->_getNodeOnlyText($node));
+                    list($default, $isDefault) = $this->__default($curctx, $this->_getNodeOnlyText($node));
                 } else {
-                    $default = $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node));
+                    list($default, $isDefault) = $this->__defaultRaw($curctx, $this->_getNodeOnlyText($node));
                 }
                 if ($default === '') {
                     $default = key($values);
@@ -430,11 +438,12 @@ class Horde_Config {
                                      'quote' => $quote,
                                      'values' => $values,
                                      'desc' => $desc,
-                                     'default' => $default);
+                                     'default' => $default,
+                                     'is_default' => $isDefault);
                 break;
 
             case 'configlist':
-                $default = $this->_default($curctx, null);
+                list($default, $isDefault) = $this->__default($curctx, null);
                 if ($default === null) {
                     $default = $this->_getNodeOnlyText($node);
                 } elseif (is_array($default)) {
@@ -443,31 +452,36 @@ class Horde_Config {
                 $conf[$name] = array('_type' => 'stringlist',
                                      'required' => $required,
                                      'desc' => $desc,
-                                     'default' => $default);
+                                     'default' => $default,
+                                     'is_default' => $isDefault);
                 break;
 
             case 'configmultienum':
                 $values = $this->_getEnumValues($node);
                 require_once 'Horde/Array.php';
+                list($default, $isDefault) = $this->__default($curctx, explode(',', $this->_getNodeOnlyText($node)));
                 $conf[$name] = array('_type' => 'multienum',
                                      'required' => $required,
                                      'values' => $values,
                                      'desc' => $desc,
-                                     'default' => Horde_Array::valuesToKeys($this->_default($curctx, explode(',', $this->_getNodeOnlyText($node)))));
+                                     'default' => Horde_Array::valuesToKeys($default),
+                                     'is_default' => $isDefault);
                 break;
 
             case 'configpassword':
                 $conf[$name] = array('_type' => 'password',
                                      'required' => $required,
                                      'desc' => $desc,
-                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)));
+                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)),
+                                     'is_default' => $this->_isDefault($curctx, $this->_getNodeOnlyText($node)));
                 break;
 
             case 'configstring':
                 $conf[$name] = array('_type' => 'text',
                                      'required' => $required,
                                      'desc' => $desc,
-                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)));
+                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)),
+                                     'is_default' => $this->_isDefault($curctx, $this->_getNodeOnlyText($node)));
                 if ($conf[$name]['default'] === false) {
                     $conf[$name]['default'] = 'false';
                 } elseif ($conf[$name]['default'] === true) {
@@ -485,7 +499,8 @@ class Horde_Config {
                 $conf[$name] = array('_type' => 'boolean',
                                      'required' => $required,
                                      'desc' => $desc,
-                                     'default' => $this->_default($curctx, $default));
+                                     'default' => $this->_default($curctx, $default),
+                                     'is_default' => $this->_isDefault($curctx, $default));
                 break;
 
             case 'configinteger':
@@ -494,7 +509,8 @@ class Horde_Config {
                                      'required' => $required,
                                      'values' => $values,
                                      'desc' => $desc,
-                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)));
+                                     'default' => $this->_default($curctx, $this->_getNodeOnlyText($node)),
+                                     'is_default' => $this->_isDefault($curctx, $this->_getNodeOnlyText($node)));
                 if ($node->get_attribute('octal') == 'true' &&
                     $conf[$name]['default'] != '') {
                     $conf[$name]['_type'] = 'octal';
@@ -507,7 +523,16 @@ class Horde_Config {
                                      'required' => $required,
                                      'quote' => false,
                                      'desc' => $desc,
-                                     'default' => $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node)));
+                                     'default' => $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node)),
+                                     'is_default' => $this->_isDefaultRaw($curctx, $this->_getNodeOnlyText($node)));
+                break;
+
+            case 'configsecret':
+                $conf[$name] = array('_type' => 'text',
+                                     'required' => true,
+                                     'desc' => $desc,
+                                     'default' => $this->_default($curctx, sha1(uniqid(mt_rand(), true))),
+                                     'is_default' => $this->_isDefault($curctx, $this->_getNodeOnlyText($node)));
                 break;
 
             case 'configsql':
@@ -527,7 +552,7 @@ class Horde_Config {
                 break;
 
             case 'configtab':
-                $key = md5(microtime());
+                $key = md5(uniqid(mt_rand(), true));
                 $conf[$key] = array('tab' => $name,
                                     'desc' => $desc);
                 if ($node->has_child_nodes()) {
@@ -536,7 +561,7 @@ class Horde_Config {
                 break;
 
             case 'configplaceholder':
-                $conf[md5(microtime())] = 'placeholder';
+                $conf[md5(uniqid(mt_rand(), true))] = 'placeholder';
                 break;
 
             default:
@@ -622,7 +647,7 @@ class Horde_Config {
             '_type' => 'text',
             'required' => true,
             'desc' => 'Internally used charset',
-            'default' => $this->_default($ctx . '|charset', 'iso-8859-1'));
+            'default' => $this->_default($ctx . '|charset', 'utf-8'));
 
         $oci8_fields = array(
             'persistent' => $persistent,
@@ -643,6 +668,36 @@ class Horde_Config {
         }
         $oci8_fields['charset'] = $charset;
 
+        $read_hostspec = array(
+            '_type' => 'text',
+            'required' => true,
+            'desc' => 'Read database server/host',
+            'default' => $this->_default($ctx . '|read|hostspec', ''));
+        $read_port = array(
+            '_type' => 'int',
+            'required' => false,
+            'desc' => 'Port the read DB is running on, if non-standard',
+            'default' => $this->_default($ctx . '|read|port', null));
+        $splitread = array(
+            '_type' => 'boolean',
+            'required' => false,
+            'desc' => 'Split reads to a different server?',
+            'default' => $this->_default($ctx . '|splitread', 'false'),
+            'switch' => array(
+                'false' => array(
+                    'desc' => 'Disabled',
+                    'fields' => array()),
+                'true' => array(
+                    'desc' => 'Enabled',
+                    'fields' => array(
+                        'read' => array(
+                            'persistent' => $persistent,
+                            'username' => $username,
+                            'password' => $password,
+                            'protocol' => $mysql_protocol,
+                            'database' => $database,
+                            'charset' => $charset)))));
+
         $custom_fields = array(
             'required' => true,
             'desc' => 'What database backend should we use?',
@@ -745,7 +800,8 @@ class Horde_Config {
                         'password' => $password,
                         'protocol' => $mysql_protocol,
                         'database' => $database,
-                        'charset' => $charset)),
+                        'charset' => $charset,
+                        'splitread' => $splitread)),
                 'mysqli' => array(
                     'desc' => 'MySQL (mysqli)',
                     'fields' => array(
@@ -753,7 +809,8 @@ class Horde_Config {
                         'password' => $password,
                         'protocol' => $mysql_protocol,
                         'database' => $database,
-                        'charset' => $charset)),
+                        'charset' => $charset,
+                        'splitread' => $splitread)),
                 'oci8' => array(
                     'desc' => 'Oracle',
                     'fields' => $oci8_fields),
@@ -800,7 +857,6 @@ class Horde_Config {
                     'desc' => 'PostgreSQL',
                     'fields' => array(
                         'persistent' => $persistent,
-                        'hostspec' => $hostspec,
                         'username' => $username,
                         'password' => $password,
                         'protocol' => $protocol,
@@ -838,9 +894,11 @@ class Horde_Config {
             return $custom_fields;
         }
 
+        list($default, $isDefault) = $this->__default($ctx . '|' . (isset($node) ? $node->get_attribute('switchname') : $switchname), 'horde');
         $config = array(
             'desc' => 'Driver configuration',
-            'default' => $this->_default($ctx . '|' . (isset($node) ? $node->get_attribute('switchname') : $switchname), 'horde'),
+            'default' => $default,
+            'is_default' => $isDefault,
             'switch' => array(
                 'horde' => array(
                     'desc' => 'Horde defaults',
@@ -848,9 +906,6 @@ class Horde_Config {
                 'custom' => array(
                     'desc' => 'Custom parameters',
                     'fields' => array(
-                        md5(microtime()) => array(
-                            '_type' => 'description',
-                            'desc' => 'Any parameters that the storage driver needs.'),
                         'phptype' => $custom_fields))));
 
         if (isset($node) && $node->has_child_nodes()) {
@@ -880,17 +935,18 @@ class Horde_Config {
     function _configVFS($ctx, $node)
     {
         $sql = $this->_configSQL($ctx . '|params');
+        $default = $node->get_attribute('default');
+        $default = (empty($default)) ? 'none' : $default;
+        list($default, $isDefault) = $this->__default($ctx . '|' . $node->get_attribute('switchname'), $default);
         $config =
             array(
                 'desc' => 'What VFS driver should we use?',
-                'default' => $this->_default($ctx . '|' . $node->get_attribute('switchname'), 'horde'),
+                'default' => $default,
+                'is_default' => $isDefault,
                 'switch' => array(
                     'none' => array(
                         'desc' => 'None',
                         'fields' => array()),
-                    'horde' => array(
-                        'desc' => 'Horde defaults',
-                        'fields' => array()),
                     'file' => array(
                         'desc' => 'Files on the local system',
                         'fields' => array(
@@ -905,6 +961,11 @@ class Horde_Config {
                             'params' => array(
                                 'driverconfig' => $sql)))));
 
+        if (isset($node) && $node->get_attribute('baseconfig') != 'true') {
+            $config['switch']['horde'] = array(
+                                          'desc' => 'Horde defaults',
+                                          'fields' => array());
+        }
         $cases = $this->_getSwitchValues($node, $ctx . '|params');
         foreach ($cases as $case => $fields) {
             if (isset($config['switch'][$case])) {
@@ -930,21 +991,64 @@ class Horde_Config {
      *                key or the default value if the key wasn't found.
      */
     function _default($ctx, $default)
+    {
+        list ($ptr,) = $this->__default($ctx, $default);
+        return $ptr;
+    }
+
+    /**
+     * Returns whether a certain value from the current configuration array
+     * exists or a default value will be used.
+     *
+     * @access private
+     *
+     * @param string $ctx     A string representing the key of the
+     *                        configuration array to return.
+     * @param mixed $default  The default value to return if the key wasn't
+     *                        found.
+     *
+     * @return boolean  Whether the default value will be used.
+     */
+    function _isDefault($ctx, $default)
+    {
+        list (,$isDefault) = $this->__default($ctx, $default);
+        return $isDefault;
+    }
+
+    /**
+     * Returns a certain value from the current configuration array or a
+     * default value, if not found, and which of the values have been
+     * returned.
+     *
+     * @access private
+     *
+     * @param string $ctx     A string representing the key of the
+     *                        configuration array to return.
+     * @param mixed $default  The default value to return if the key wasn't
+     *                        found.
+     *
+     * @return array  First element: either the value of the configuration
+     *                array's requested key or the default value if the key
+     *                wasn't found.
+     *                Second element: whether the returned value was the
+     *                default value.
+     */
+    function __default($ctx, $default)
     {
         $ctx = explode('|', $ctx);
         $ptr = $this->_currentConfig;
         for ($i = 0; $i < count($ctx); $i++) {
             if (!isset($ptr[$ctx[$i]])) {
-                return $default;
+                return array($default, true);
             } else {
                 $ptr = $ptr[$ctx[$i]];
             }
         }
         if (is_string($ptr)) {
-            return String::convertCharset($ptr, 'iso-8859-1');
-        } else {
-            return $ptr;
+            $ptr = String::convertCharset($ptr, 'iso-8859-1');
         }
+
+        return array($ptr, false);
     }
 
     /**
@@ -964,13 +1068,59 @@ class Horde_Config {
      *                key or the default value if the key wasn't found.
      */
     function _defaultRaw($ctx, $default)
+    {
+        list ($ptr,) = $this->__defaultRaw($ctx, $default);
+        return $ptr;
+    }
+
+    /**
+     * Returns whether a certain value from the current configuration array
+     * exists or a default value will be used.
+     *
+     * @access private
+     *
+     * @param string $ctx     A string representing the key of the
+     *                        configuration array to return.
+     * @param mixed $default  The default value to return if the key wasn't
+     *                        found.
+     *
+     * @return boolean  Whether the default value will be used.
+     */
+    function _isDefaultRaw($ctx, $default)
+    {
+        list (,$isDefault) = $this->__defaultRaw($ctx, $default);
+        return $isDefault;
+    }
+
+    /**
+     * Returns a certain value from the current configuration file or
+     * a default value, if not found, and which of the values have been
+     * returned.
+     *
+     * It does NOT return the actual value, but the PHP expression as used
+     * in the configuration file.
+     *
+     * @access private
+     *
+     * @param string $ctx     A string representing the key of the
+     *                        configuration array to return.
+     * @param mixed $default  The default value to return if the key wasn't
+     *                        found.
+     *
+     * @return array  First element: either the value of the configuration
+     *                array's requested key or the default value if the key
+     *                wasn't found.
+     *                Second element: whether the returned value was the
+     *                default value.
+     */
+    function __defaultRaw($ctx, $default)
     {
         $ctx = explode('|', $ctx);
         $pattern = '/^\$conf\[\'' . implode("'\]\['", $ctx) . '\'\] = (.*);\r?$/m';
         if (preg_match($pattern, $this->getPHPConfig(), $matches)) {
-            return $matches[1];
+            return array($matches[1], false);
         }
-        return $default;
+        return array($default, true);
     }
 
     /**
@@ -1000,6 +1150,7 @@ class Horde_Config {
     /**
      * Returns an associative array containing all possible values of the
      * specified <configenum> tag.
+     *
      * The keys contain the actual enum values while the values contain their
      * corresponding descriptions.
      *
@@ -1109,7 +1260,10 @@ class Horde_Config {
             $f = array();
             if ($registry->hasMethod('clients/getClientSource')) {
                 $addressbook = $registry->call('clients/getClientSource');
-                $fields = $registry->call('clients/fields', array($addressbook));
+                $fields = $registry->call('clients/clientFields', array($addressbook));
+                if (is_a($fields, 'PEAR_Error')) {
+                    $fields = $registry->call('clients/fields', array($addressbook));
+                }
                 if (!is_a($fields, 'PEAR_Error')) {
                     foreach ($fields as $field) {
                         $f[$field['name']] = $field['label'];
@@ -1118,6 +1272,12 @@ class Horde_Config {
             }
             return $f;
             break;
+
+        case 'list-contact-sources':
+            global $registry;
+            $res = $registry->call('contacts/sources');
+            return $res;
+            break;
         }
 
         return array();
@@ -1176,14 +1336,14 @@ class ConfigForm extends Horde_Form {
      * Constructor.
      *
      * @param Variables &$vars  The Variables object of this form.
-     * @param string $app             The name of the application that this
-     *                                configuration form is for.
+     * @param string $app       The name of the application that this
+     *                          configuration form is for.
      */
     function ConfigForm(&$vars, $app)
     {
         parent::Horde_Form($vars);
 
-        $this->_xmlConfig = &new Horde_Config($app);
+        $this->_xmlConfig = new Horde_Config($app);
         $this->_vars = &$vars;
         $config = $this->_xmlConfig->readXMLConfig();
         $this->addHidden('', 'app', 'text', true);
@@ -1208,7 +1368,6 @@ class ConfigForm extends Horde_Form {
         foreach ($config as $name => $configitem) {
             $prefixedname = empty($prefix) ? $name : $prefix . '|' . $name;
             $varname = str_replace('|', '__', $prefixedname);
-            $description = null;
             if ($configitem == 'placeholder') {
                 continue;
             } elseif (isset($configitem['tab'])) {
@@ -1229,10 +1388,17 @@ class ConfigForm extends Horde_Form {
                         }
                     }
                 }
-                $v = &$this->addVariable($configitem['desc'], $varname, 'enum', true, false, null, array($var_params, $select_option));
+
+                $name = '$conf[' . implode('][', explode('|', $prefixedname)) . ']';
+                $desc = $configitem['desc'];
+
+                $v = &$this->addVariable($name, $varname, 'enum', true, false, $desc, array($var_params, $select_option));
                 if (array_key_exists('default', $configitem)) {
                     $v->setDefault($configitem['default']);
                 }
+                if (!empty($configitem['is_default'])) {
+                    $v->_new = true;
+                }
                 $v_action = &Horde_Form_Action::factory('reload');
                 $v->setAction($v_action);
                 if (isset($selected) && isset($configitem['switch'][$selected])) {
@@ -1241,6 +1407,7 @@ class ConfigForm extends Horde_Form {
             } elseif (isset($configitem['_type'])) {
                 $required = (isset($configitem['required'])) ? $configitem['required'] : true;
                 $type = $configitem['_type'];
+
                 // FIXME: multienum fields can well be required, meaning that
                 // you need to select at least one entry. Changing this before
                 // Horde 4.0 would break a lot of configuration files though.
@@ -1248,19 +1415,32 @@ class ConfigForm extends Horde_Form {
                     $type == 'description') {
                     $required = false;
                 }
+
                 if ($type == 'multienum' || $type == 'enum') {
                     $var_params = array($configitem['values']);
                 } else {
                     $var_params = array();
                 }
-                if ($type == 'php') {
-                    $type = 'text';
-                    $description = 'Enter a valid PHP expression.';
+
+                if ($type == 'header' || $type == 'description') {
+                    $name = $configitem['desc'];
+                    $desc = null;
+                } else {
+                    $name = '$conf[' . implode('][', explode('|', $prefixedname)) . ']';
+                    $desc = $configitem['desc'];
+                    if ($type == 'php') {
+                        $type = 'text';
+                        $desc .= "\nEnter a valid PHP expression.";
+                    }
                 }
-                $v = &$this->addVariable($configitem['desc'], $varname, $type, $required, false, $description, $var_params);
+
+                $v = &$this->addVariable($name, $varname, $type, $required, false, $desc, $var_params);
                 if (isset($configitem['default'])) {
                     $v->setDefault($configitem['default']);
                 }
+                if (!empty($configitem['is_default'])) {
+                    $v->_new = true;
+                }
             } else {
                 $this->_buildVariables($configitem, $prefixedname);
             }
@@ -1268,102 +1448,3 @@ class ConfigForm extends Horde_Form {
     }
 
 }
-
-class Horde_Config_Document extends Horde_Config_Node {
-
-    function factory($file)
-    {
-        if (function_exists('domxml_open_file')) {
-            return domxml_open_file($file);
-        } elseif (version_compare(phpversion(), '5', '>=')) {
-            return $doc = new Horde_Config_Document($file);
-        } else {
-            Horde::fatal('DOM support not present.', __FILE__, __LINE__);
-        }
-    }
-
-    function Horde_Config_Document($filename)
-    {
-        $this->node = new DOMDocument();
-        $this->node->load($filename);
-    }
-
-    function root()
-    {
-        return $root = new Horde_Config_Element($this->node->documentElement);
-    }
-
-}
-
-class Horde_Config_Node {
-
-    var $node;
-
-    function Horde_Config_Node($aDomNode) {$this->node=$aDomNode;}
-
-    function __get($name)
-    {
-        switch ($name) {
-        case 'type':
-            return $this->node->nodeType;
-
-        case 'tagname':
-            return $this->node->tagName;
-
-        case 'content':
-            return $this->node->textContent;
-
-        default:
-            return false;
-        }
-    }
-
-    function child_nodes()
-    {
-        $nodeList = $this->node->childNodes;
-        $nodeSet = array();
-        if (isset($nodeList)) {
-            $i = 0;
-            while ($node = $nodeList->item($i)) {
-                $nodeSet[] = new Horde_Config_Element($node);
-                $i++;
-            }
-        }
-        return $nodeSet;
-    }
-
-    function first_child()
-    {
-        return new Horde_Config_Element($this->node->firstChild);
-    }
-
-    function get_content()
-    {
-        return $this->node->textContent;
-    }
-
-    function has_child_nodes()
-    {
-        return $this->node->hasChildNodes();
-    }
-
-    function next_sibling()
-    {
-        return new Horde_Config_Element($this->node->nextSibling);
-    }
-
-    function node_value()
-    {
-        return $this->node->nodeValue;
-    }
-
-}
-
-class Horde_Config_Element extends Horde_Config_Node {
-
-    function get_attribute($name)
-    {
-        return $this->node->getAttribute($name);
-    }
-
-}
