@@ -6,15 +6,14 @@ require_once 'Horde/Crypt.php';
  * Horde_Crypt_smime:: provides a framework for Horde applications to
  * interact with the OpenSSL library and implement S/MIME.
  *
- * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.14 2007/08/15 08:51:52 selsky Exp $
+ * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.20 2008/12/07 19:36:36 slusarz Exp $
  *
- * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @since   Horde 3.0
  * @package Horde_Crypt
  */
 class Horde_Crypt_smime extends Horde_Crypt {
@@ -196,7 +195,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
             }
         }
 
-        $ob = &new stdClass;
+        $ob = new stdClass;
 
         if (!empty($root_certs)) {
             $result = openssl_pkcs7_verify($input, 0, $output, $root_certs);
@@ -212,8 +211,10 @@ class Horde_Crypt_smime extends Horde_Crypt {
         /* Try again without verfying the signer's cert */
         $result = openssl_pkcs7_verify($input, PKCS7_NOVERIFY, $output);
 
-        if (($result === true) || ($result === -1)) {
+        if ($result === true) {
             $ob->result = PEAR::raiseError(_("Message Verified Successfully but the signer's certificate could not be verified."), 'horde.warning');
+        } elseif ($result == -1) {
+            $ob->result = PEAR::raiseError(_("Verification failed - an unknown error has occurred."), 'horde.error');
         } else {
             $ob->result = PEAR::raiseError(_("Verification failed - this message may have been tampered with."), 'horde.error');
         }
@@ -235,30 +236,26 @@ class Horde_Crypt_smime extends Horde_Crypt {
      */
     function extractSignedContents($data, $sslpath)
     {
-        $pipes_desc = array(
-            0 => array('pipe', 'r'),
-            1 => array('pipe', 'w')
-        );
-
-        $fp = proc_open($sslpath . ' smime -verify -noverify -nochain', $pipes_desc, $pipes);
-        if (!is_resource($fp)) {
-            return PEAR::raiseError(_("OpenSSL error: Could not extract data from signed S/MIME part."), 'horde.error');
+        /* Check for availability of OpenSSL PHP extension. */
+        $openssl = $this->checkForOpenSSL();
+        if (is_a($openssl, 'PEAR_Error')) {
+            return $openssl;
         }
 
-        $output = '';
+        /* Create temp files for input/output. */
+        $input = $this->_createTempFile('horde-smime');
+        $output = $this->_createTempFile('horde-smime');
 
-        /* $pipes[0] => writeable handle connected to child stdin
-           $pipes[1] => readable handle connected to child stdout */
-        fwrite($pipes[0], $data);
-        fclose($pipes[0]);
+        /* Write text to file. */
+        file_put_contents($input, $data);
+        unset($data);
 
-        while (!feof($pipes[1])) {
-            $output .= fgets($pipes[1], 1024);
-        }
-        fclose($pipes[1]);
-        proc_close($fp);
+        exec($sslpath . ' smime -verify -noverify -nochain -in ' . $input . ' -out ' . $output);
 
-        return $output;
+        $ret = file_get_contents($output);
+        return $ret
+            ? $ret
+            : PEAR::raiseError(_("OpenSSL error: Could not extract data from signed S/MIME part."), 'horde.error');
     }
 
     /**
@@ -286,7 +283,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
         $smime_sign->transferDecodeContents();
         $smime_sign->setTransferEncoding('base64');
 
-        $smime_part = &new MIME_Part('multipart/signed');
+        $smime_part = new MIME_Part('multipart/signed');
         $smime_part->setContents('This is a cryptographically signed message in MIME format.' . "\n");
         $smime_part->addPart($mime_part);
         $smime_part->addPart($smime_sign);
@@ -316,11 +313,15 @@ class Horde_Crypt_smime extends Horde_Crypt {
             return $message;
         }
 
+        /* Get charset for mime part description. */
+        $charset = NLS::getEmailCharset();
+
         /* Break the result into its components */
         $mime_message = MIME_Structure::parseTextMIMEMessage($message);
 
         $smime_part = $mime_message->getBasePart();
-        $smime_part->setDescription(_("S/MIME Encrypted Message"));
+        $smime_part->setCharset($charset);
+        $smime_part->setDescription(String::convertCharset(_("S/MIME Encrypted Message"), NLS::getCharset(), $charset));
         $smime_part->transferDecodeContents();
         $smime_part->setTransferEncoding('base64');
         $smime_part->setDisposition('inline');
@@ -747,9 +748,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
 
         $cert_details = array();
         $cert_details['fingerprints']['md5'] = md5($raw_cert);
-        if (Util::extensionExists('mhash')) {
-            $cert_details['fingerprints']['sha1'] = bin2hex(mhash(MHASH_SHA1, $raw_cert));
-        }
+        $cert_details['fingerprints']['sha1'] = sha1($raw_cert);
 
         $cert_details['certificate']['extensions'] = array();
         $cert_details['certificate']['version'] = $cert_data[1][0][1][0][1] + 1;
@@ -821,10 +820,12 @@ class Horde_Crypt_smime extends Horde_Crypt {
         }
 
         $i = 9;
-        while (isset($cert_data[1][0][1][$i])) {
+
+        while (isset($cert_data[1][0][1][$i]) &&
+               is_array($cert_data[1][0][1][$i][1])) {
             $oid = $cert_data[1][0][1][$i][1][0][1];
             $cert_details['certificate']['extensions'][$oid] = $cert_data[1][0][1][$i][1][1];
-            $i++;
+            ++$i;
         }
 
         foreach ($cert_details['certificate']['extensions'] as $oid => $val) {
@@ -1238,12 +1239,17 @@ class Horde_Crypt_smime extends Horde_Crypt {
      *
      * @param string $key  The public key.
      *
-     * @return string  Returns the email address, or null if none is found.
+     * @return mixed Returns the first email address found, or null if
+     * there are none.
      */
     function getEmailFromKey($key)
     {
         $key_info = openssl_x509_parse($key);
-        if (is_array($key_info) && isset($key_info['subject'])) {
+        if (!is_array($key_info)) {
+            return null;
+        }
+
+        if (isset($key_info['subject'])) {
             if (isset($key_info['subject']['Email'])) {
                 return $key_info['subject']['Email'];
             } elseif (isset($key_info['subject']['emailAddress'])) {
@@ -1251,6 +1257,20 @@ class Horde_Crypt_smime extends Horde_Crypt {
             }
         }
 
+        // Check subjectAltName per http://www.ietf.org/rfc/rfc3850.txt
+        if (isset($key_info['extensions']['subjectAltName'])) {
+            $names = preg_split('/\s*,\s*/', $key_info['extensions']['subjectAltName'], -1, PREG_SPLIT_NO_EMPTY);
+            foreach ($names as $name) {
+                if (strpos($name, ':') === false) {
+                    continue;
+                }
+                list($kind, $value) = explode(':', $name, 2);
+                if (String::lower($kind) == 'email') {
+                    return $value;
+                }
+            }
+        }
+
         return null;
     }
 
@@ -1292,7 +1312,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
         $input = $this->_createTempFile('horde-smime');
         $output = $this->_createTempFile('horde-smime');
 
-        $ob = &new stdClass;
+        $ob = new stdClass;
 
         /* Write text to file */
         $fp = fopen($input, 'w+');
@@ -1323,7 +1343,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
             return PEAR::raiseError(_("Password incorrect"), 'horde.error');
         }
 
-        /* Extract the public key next. */
+        /* Extract the client public key next. */
         $cmdline = $sslpath . ' pkcs12 -in ' . $input . ' -out ' . $output . ' -nokeys -clcerts';
         if (isset($params['password'])) {
             $cmdline .= ' -passin stdin';
@@ -1335,7 +1355,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
         }
         $ob->public = trim(file_get_contents($output));
 
-        /* Extract the public key next. */
+        /* Extract the CA public key next. */
         $cmdline = $sslpath . ' pkcs12 -in ' . $input . ' -out ' . $output . ' -nokeys -cacerts';
         if (isset($params['password'])) {
             $cmdline .= ' -passin stdin';
