@@ -2,9 +2,9 @@
 /**
  * Horde_Data implementation for tab-separated data (TSV).
  *
- * $Horde: framework/Data/Data/tsv.php,v 1.23.10.6 2005/07/03 04:39:57 selsky Exp $
+ * $Horde: framework/Data/Data/tsv.php,v 1.23.10.16 2008/03/31 17:38:15 jan Exp $
  *
- * Copyright 1999-2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -19,8 +19,20 @@ class Horde_Data_tsv extends Horde_Data {
     var $_extension = 'tsv';
     var $_contentType = 'text/tab-separated-values';
 
+    /**
+     * Convert data file contents to list of data records.
+     *
+     * @param string $contents   Data file contents.
+     * @param boolean $header    True if a header row is present.
+     * @param string $delimiter  Field delimiter.
+     *
+     * @return array  List of data records.
+     */
     function importData($contents, $header = false, $delimiter = "\t")
     {
+        if ($_SESSION['import_data']['format'] == 'pine') {
+            $contents = preg_replace('/\n +/', '', $contents);
+        }
         $contents = explode("\n", $contents);
         $data = array();
         if ($header) {
@@ -133,7 +145,7 @@ class Horde_Data_tsv extends Horde_Data {
                 foreach ($_SESSION['import_data']['data'] as $row) {
                     $hash = array();
                     if ($format == 'mulberry') {
-                        if (preg_match("/^Grp:/", $row[0])) {
+                        if (preg_match("/^Grp:/", $row[0]) || empty($row[1])) {
                             continue;
                         }
                         $row[1] = preg_replace('/^([^,"]+),\s*(.*)$/', '$2 $1', $row[1]);
@@ -147,6 +159,17 @@ class Horde_Data_tsv extends Horde_Data {
                             continue;
                         }
                         $row[1] = preg_replace('/^([^,"]+),\s*(.*)$/', '$2 $1', $row[1]);
+                        /* Address can be a full RFC822 address */
+                        require_once 'Horde/MIME.php';
+                        $addr_arr = MIME::parseAddressList($row[2]);
+                        if (is_a($addr_arr, 'PEAR_Error') ||
+                            empty($addr_arr[0]->mailbox)) {
+                            continue;
+                        }
+                        $row[2] = $addr_arr[0]->mailbox . '@' . $addr_arr[0]->host;
+                        if (empty($row[1]) && !empty($addr_arr[0]->personal)) {
+                            $row[1] = $addr_arr[0]->personal;
+                        }
                         foreach ($dataKeys as $key) {
                             if (array_key_exists($key, $row)) {
                                 $hash[$key] = $row[$key];
