@@ -1,30 +1,30 @@
 <?php
-
 /**
- * List every object in an array, similar to PEAR/html/menu.php.
+ * @package Horde_DataTree
+ *
+ * $Horde: framework/DataTree/DataTree.php,v 1.151.2.23 2008/06/06 04:17:20 chuck Exp $
  */
+
+/** List every object in an array, similar to PEAR/html/menu.php. */
 define('DATATREE_FORMAT_TREE', 1);
 
-/**
- * Get a full list - an array of keys.
- */
+/** Get a full list - an array of keys. */
 define('DATATREE_FORMAT_FLAT', 2);
 
-/**
- * The root element (top-level parent) of each DataTree group.
- */
+/** The root element (top-level parent) of each DataTree group. */
 define('DATATREE_ROOT', -1);
 
-/**
- * Build a normal select query.
- */
+/** Build a normal select query. */
 define('DATATREE_BUILD_SELECT', 0);
 
-/**
- * Build a count only query.
- */
+/** Build a count only query. */
 define('DATATREE_BUILD_COUNT', 1);
 
+/** Build an attribute only query. */
+define('DATATREE_BUILD_VALUES', 2);
+
+define('DATATREE_BUILD_VALUES_COUNT', 3);
+
 /**
  * The DataTree:: class provides a common abstracted interface into the
  * various backends for the Horde DataTree system.
@@ -36,24 +36,20 @@ define('DATATREE_BUILD_COUNT', 1);
  * Required values for $params:<pre>
  *   'group' -- Define each group of objects we want to build.</pre>
  *
- * $Horde: framework/DataTree/DataTree.php,v 1.151.2.19 2007/01/02 13:54:13 jan Exp $
+ * Copyright 1999-2003 The Horde Project (http://www.horde.org/)
  *
- * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
- *
- * @author  Stephane Huther <shuther@bigfoot.com>
+ * @author  Stephane Huther <shuther1@free.fr>
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @since   Horde 2.1
  * @package Horde_DataTree
  */
 class DataTree {
 
     /**
-     * Array of all data: indexed by id. The format is: array(id =>
-     * 'name' => name, 'parent' => parent).
+     * Array of all data: indexed by id. The format is:
+     *   array(id => 'name' => name, 'parent' => parent).
      *
      * @var array
      */
@@ -89,7 +85,7 @@ class DataTree {
      *                       We always need 'group', a string that defines the
      *                       prefix for each set of hierarchical data.
      */
-    function DataTree($params = null)
+    function DataTree($params = array())
     {
         $this->_params = $params;
     }
@@ -110,7 +106,9 @@ class DataTree {
      * Removes an object.
      *
      * @param string $object  The object to remove.
-     * @param $force          Force removal of every child object?
+     * @param boolean $force  Force removal of every child object?
+     *
+     * @return TODO
      */
     function remove($object, $force = false)
     {
@@ -125,14 +123,13 @@ class DataTree {
         $children = $this->getNumberOfChildren($object);
         if ($children) {
             /* TODO: remove children if $force == true */
-            return PEAR::raiseError(sprintf(_("Cannot remove, %d children exist."), count($children)));
+            return PEAR::raiseError(sprintf(ngettext("Cannot remove, %d child exists.", "Cannot remove, %d children exist.", count($children)), count($children)));
         }
 
         $id = $this->getId($object);
         $pid = $this->getParent($object);
         $order = $this->_data[$id]['order'];
-        unset($this->_data[$id]);
-        unset($this->_nameMap[$id]);
+        unset($this->_data[$id], $this->_nameMap[$id]);
 
         // Shift down the order positions.
         $this->_reorder($pid, $order);
@@ -143,7 +140,11 @@ class DataTree {
     /**
      * Removes all DataTree objects owned by a certain user.
      *
+     * @abstract
+     *
      * @param string $user  A user name.
+     *
+     * @return TODO
      */
     function removeUserData($user)
     {
@@ -155,6 +156,8 @@ class DataTree {
      *
      * @param mixed $object      The object to move.
      * @param string $newparent  The new parent object. Defaults to the root.
+     *
+     * @return mixed  True on success, PEAR_Error on error.
      */
     function move($object, $newparent = null)
     {
@@ -182,6 +185,8 @@ class DataTree {
      *
      * @param mixed  $old_object       The old object.
      * @param string $new_object_name  The new object name.
+     *
+     * @return mixed  True on success, PEAR_Error on error.
      */
     function rename($old_object, $new_object_name)
     {
@@ -207,6 +212,8 @@ class DataTree {
     /**
      * Changes the order of the children of an object.
      *
+     * @abstract
+     *
      * @param string $parent  The full id path of the parent object.
      * @param mixed $order    If an array it specifies the new positions for
      *                        all child objects.
@@ -215,6 +222,8 @@ class DataTree {
      *                        $cid is not specified, the position gets deleted,
      *                        causing the following positions to shift up.
      * @param integer $cid    See $order.
+     *
+     * @return TODO
      */
     function reorder($parents, $order = null, $cid = null)
     {
@@ -240,17 +249,18 @@ class DataTree {
             if (is_null($cid)) {
                 // No id given so shuffle down.
                 foreach ($this->_data as $c_key => $c_val) {
-                    if ($this->_data[$c_key]['parent'] == $pid
-                        && $this->_data[$c_key]['order'] > $order) {
-                        $this->_data[$c_key]['order']--;
+                    if ($this->_data[$c_key]['parent'] == $pid &&
+                        $this->_data[$c_key]['order'] > $order) {
+                        --$this->_data[$c_key]['order'];
                     }
                 }
             } else {
                 // We have an id so shuffle up.
                 foreach ($this->_data as $c_key => $c_val) {
-                    if ($c_key != $cid  && $this->_data[$c_key]['parent'] == $pid
-                        && $this->_data[$c_key]['order'] >= $order) {
-                        $this->_data[$c_key]['order']++;
+                    if ($c_key != $cid &&
+                        $this->_data[$c_key]['parent'] == $pid &&
+                        $this->_data[$c_key]['order'] >= $order) {
+                        ++$this->_data[$c_key]['order'];
                     }
                 }
             }
@@ -269,6 +279,8 @@ class DataTree {
      *
      * @param integer $id     The datatree object id to change.
      * @param integer $order  The new order.
+     *
+     * @return TODO
      */
     function setOrder($id, $order)
     {
@@ -280,11 +292,11 @@ class DataTree {
      *
      * @param array $attributes  The set of attributes that contain the class
      *                           information. Defaults to DataTreeObject.
+     *
+     * @return TODO
      */
     function _defineObjectClass($attributes)
     {
-        global $registry;
-
         $class = 'DataTreeObject';
         if (!is_array($attributes)) {
             return $class;
@@ -292,12 +304,14 @@ class DataTree {
 
         foreach ($attributes as $attr) {
             if ($attr['name'] == 'DataTree') {
-                if ($attr['key'] == 'objectClass') {
+                switch ($attr['key']) {
+                case 'objectClass':
                     $class = $attr['value'];
                     break;
-                } elseif ($attr['key'] == 'objectType') {
+
+                case 'objectType':
                     $result = explode('/', $attr['value']);
-                    $class = $registry->callByPackage($result[0], 'defineClass', array('type' => $result[1]));
+                    $class = $GLOBALS['registry']->callByPackage($result[0], 'defineClass', array('type' => $result[1]));
                     break;
                 }
             }
@@ -316,6 +330,8 @@ class DataTree {
      *                        into the attributes table to determine the
      *                        subclass to use. If none is found it uses
      *                        DataTreeObject.
+     *
+     * @return TODO
      */
     function &getObject($object, $class = 'DataTreeObject')
     {
@@ -330,9 +346,47 @@ class DataTree {
             return $error;
         }
 
+        return $this->_getObject($this->getId($object), $object, $class);
+    }
+
+    /**
+     * Returns a DataTreeObject (or subclass) object of the data in the
+     * object with the ID $id.
+     *
+     * @param integer $id    An object id.
+     * @param string $class  Subclass of DataTreeObject to use. Defaults to
+     *                       DataTreeObject. Null forces the driver to look
+     *                       into the attributes table to determine the
+     *                       subclass to use. If none is found it uses
+     *                       DataTreeObject.
+     *
+     * @return TODO
+     */
+    function &getObjectById($id, $class = 'DataTreeObject')
+    {
+        if (empty($id)) {
+            $object = PEAR::raiseError('No id requested.');
+            return $object;
+        }
+
+        $result = $this->_loadById($id);
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
+        return $this->_getObject($id, $this->getName($id), $class);
+    }
+
+    /**
+     * Helper function for getObject() and getObjectById().
+     *
+     * @access private
+     */
+    function &_getObject($id, $name, $class)
+    {
         $use_attributes = is_null($class) || is_callable(array($class, '_fromAttributes'));
         if ($use_attributes) {
-            $attributes = $this->getAttributes($this->getId($object));
+            $attributes = $this->getAttributes($id);
             if (is_a($attributes, 'PEAR_Error')) {
                 return $attributes;
             }
@@ -347,63 +401,9 @@ class DataTree {
             return $error;
         }
 
-        $dataOb = &new $class($object);
+        $dataOb = new $class($name);
         $dataOb->setDataTree($this);
-        /* If the class has a _fromAttributes method, load data from
-         * the attributes backend. */
-        if ($use_attributes) {
-            $dataOb->_fromAttributes($attributes);
-        } else {
-            /* Otherwise load it from the old data storage field. */
-            $dataOb->setData($this->getData($this->getId($object)));
-        }
 
-        $dataOb->setOrder($this->getOrder($object));
-        return $dataOb;
-    }
-
-    /**
-     * Returns a DataTreeObject (or subclass) object of the data in the
-     * object with the ID $id.
-     *
-     * @param integer $id    An object id.
-     * @param string $class  Subclass of DataTreeObject to use. Defaults to
-     *                       DataTreeObject. Null forces the driver to look
-     *                       into the attributes table to determine the
-     *                       subclass to use. If none is found it uses
-     *                       DataTreeObject.
-     */
-    function &getObjectById($id, $class = 'DataTreeObject')
-    {
-        if (empty($id)) {
-            $object = PEAR::raiseError('No id requested.');
-            return $object;
-        }
-
-        $result = $this->_loadById($id);
-        if (is_a($result, 'PEAR_Error')) {
-            return $result;
-        }
-
-        $use_attributes = is_null($class) || is_callable(array($class, '_fromAttributes'));
-        if ($use_attributes) {
-            $attributes = $this->getAttributes($id);
-            if (is_a($attributes, 'PEAR_Error')) {
-                return $attributes;
-            }
-
-            if (is_null($class)) {
-                $class = $this->_defineObjectClass($attributes);
-            }
-        }
-
-        if (!class_exists($class)) {
-            return PEAR::raiseError($class . ' not found.');
-        }
-
-        $name = $this->getName($id);
-        $dataOb = &new $class($name);
-        $dataOb->setDataTree($this);
         /* If the class has a _fromAttributes method, load data from
          * the attributes backend. */
         if ($use_attributes) {
@@ -428,6 +428,8 @@ class DataTree {
      *                       into the attributes table to determine the
      *                       subclass to use. If none is found it uses
      *                       DataTreeObject.
+     *
+     * @return TODO
      */
     function &getObjects($ids, $class = 'DataTreeObject')
     {
@@ -437,7 +439,7 @@ class DataTree {
         }
 
         $defineClass = is_null($class);
-        $attributes = is_null($class) || is_callable(array($class, '_fromAttributes'));
+        $attributes = $defineClass || is_callable(array($class, '_fromAttributes'));
 
         if ($attributes) {
             $data = $this->getAttributes($ids);
@@ -457,7 +459,7 @@ class DataTree {
                     return PEAR::raiseError($class . ' not found.');
                 }
 
-                $objects[$name] = &new $class($name);
+                $objects[$name] = new $class($name);
                 $objects[$name]->setDataTree($this);
                 if ($attributes) {
                     $objects[$name]->_fromAttributes($data[$id]);
@@ -474,24 +476,24 @@ class DataTree {
     /**
      * Export a list of objects.
      *
-     * @param constant $format      Format of the export
-     * @param string   $startleaf   The name of the leaf from which we start
-     *                              the export tree.
-     * @param boolean  $reload      Re-load the requested chunk? Defaults to
-     *                              false (only what is currently loaded).
-     * @param string   $rootname    The label to use for the root element.
-     *                              Defaults to DATATREE_ROOT.
-     * @param integer  $maxdepth    The maximum number of levels to return.
-     *                              Defaults to DATATREE_ROOT, which is no
-     *                              limit.
-     * @param boolean  $loadTree    Load a tree starting at $root, or just the
-     *                              requested level and direct parents?
-     *                              Defaults to single level.
-     * @param string   $sortby_name Attribute name to use for sorting.
-     * @param string   $sortby_key  Attribute key to use for sorting.
-     * @param integer  $direction   Sort direction:
-     *                               0 - ascending
-     *                               1 - descending
+     * @param constant $format       Format of the export
+     * @param string   $startleaf    The name of the leaf from which we start
+     *                               the export tree.
+     * @param boolean  $reload       Re-load the requested chunk? Defaults to
+     *                               false (only what is currently loaded).
+     * @param string   $rootname     The label to use for the root element.
+     *                               Defaults to DATATREE_ROOT.
+     * @param integer  $maxdepth     The maximum number of levels to return.
+     *                               Defaults to DATATREE_ROOT, which is no
+     *                               limit.
+     * @param boolean  $loadTree     Load a tree starting at $root, or just the
+     *                               requested level and direct parents?
+     *                               Defaults to single level.
+     * @param string   $sortby_name  Attribute name to use for sorting.
+     * @param string   $sortby_key   Attribute key to use for sorting.
+     * @param integer  $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      *
      * @return mixed  The tree representation of the objects, or a PEAR_Error
      *                on failure.
@@ -500,13 +502,14 @@ class DataTree {
                  $rootname = DATATREE_ROOT, $maxdepth = -1, $loadTree = false,
                  $sortby_name = null, $sortby_key = null, $direction = 0)
     {
+        $out = array();
+
         /* Set sorting hash */
-        if (!is_null($sortby_name) || !is_null($sortby_name)) {
+        if (!is_null($sortby_name)) {
             $this->_sortHash = DataTree::sortHash($startleaf, $sortby_name, $sortby_key, $direction);
         }
 
         $this->_load($startleaf, $loadTree, $reload, $sortby_name, $sortby_key, $direction);
-        $out = array();
 
         switch ($format) {
         case DATATREE_FORMAT_TREE:
@@ -535,6 +538,7 @@ class DataTree {
         if (!is_null($this->_sortHash)) {
             /* Reset sorting hash. */
             $this->_sortHash = null;
+
             /* Reverse since the attribute sorting combined with tree up-ward
              * sorting produces a reversed object order. */
             $out = array_reverse($out, true);
@@ -569,7 +573,7 @@ class DataTree {
      *                              0 - ascending
      *                              1 - descending
      *
-     * @return string The sort hash
+     * @return string  The sort hash.
      */
     function sortHash($root, $sortby_name = null, $sortby_key = null,
                       $direction = 0)
@@ -622,7 +626,9 @@ class DataTree {
     /**
      * Returns a list of all groups (root nodes) of the data tree.
      *
-     * @return array  The the group IDs
+     * @abstract
+     *
+     * @return mixed  The group IDs or PEAR_Error on error.
      */
     function getGroups()
     {
@@ -635,6 +641,8 @@ class DataTree {
      * @abstract
      *
      * @param integer $cid  The object id to fetch, or an array of object ids.
+     *
+     * @return TODO
      */
     function getData($cid)
     {
@@ -645,8 +653,10 @@ class DataTree {
      * Import a list of objects. Used by drivers to populate the internal
      * $_data array.
      *
-     * @param array   $data     The data to import.
-     * @param string  $charset  The charset to convert the object name from.
+     * @param array $data      The data to import.
+     * @param string $charset  The charset to convert the object name from.
+     *
+     * @return TODO
      */
     function set($data, $charset = null)
     {
@@ -703,7 +713,7 @@ class DataTree {
      *
      * @param string $leaf  Name of the parent from which to start.
      *
-     * @return array
+     * @return array  TODO
      */
     function _extractOneLevel($leaf = DATATREE_ROOT)
     {
@@ -714,11 +724,7 @@ class DataTree {
             }
         }
 
-        if (is_null($this->_sortHash)) {
-            uasort($out, array($this, '_cmp'));
-        } else {
-            uasort($out, array($this, '_cmpSorted'));
-        }
+        uasort($out, array($this, (is_null($this->_sortHash)) ? '_cmp' : '_cmpSorted'));
 
         return $out;
     }
@@ -727,16 +733,21 @@ class DataTree {
      * Extract all levels of data, starting from a given parent
      * leaf in the datatree.
      *
-     * @param array $out         This is an iterating function, so $out is
-     *                           passed by reference to contain the result.
-     * @param string  $parent    The name of the parent from which to begin.
-     * @param integer $maxdepth  Max of levels of depth to check.
+     * @access private
      *
      * @note If nothing is returned that means there is no child, but
      * don't forget to add the parent if any subsequent operations are
      * required!
+     *
+     * @param array $out         This is an iterating function, so $out is
+     *                           passed by reference to contain the result.
+     * @param string $parent     The name of the parent from which to begin.
+     * @param integer $maxdepth  Max of levels of depth to check.
+     *
+     * @return TODO
      */
-    function _extractAllLevelTree(&$out, $parent = DATATREE_ROOT, $maxdepth = -1)
+    function _extractAllLevelTree(&$out, $parent = DATATREE_ROOT,
+                                  $maxdepth = -1)
     {
         if ($maxdepth == 0) {
             return false;
@@ -745,7 +756,7 @@ class DataTree {
         $out[$parent] = true;
 
         $k = $this->_extractOneLevel($parent);
-        foreach ($k as $object => $v) {
+        foreach (array_keys($k) as $object) {
             if (!is_array($out[$parent])) {
                 $out[$parent] = array();
             }
@@ -760,12 +771,17 @@ class DataTree {
      *
      * Returned array format: array(parent => array(child => true))
      *
+     * @access private
+     *
      * @param array $out         This is an iterating function, so $out is
      *                           passed by reference to contain the result.
-     * @param string  $parent    The name of the parent from which to begin.
+     * @param string $parent     The name of the parent from which to begin.
      * @param integer $maxdepth  Max number of levels of depth to check.
+     *
+     * @return TODO
      */
-    function _extractAllLevelList(&$out, $parent = DATATREE_ROOT, $maxdepth = -1)
+    function _extractAllLevelList(&$out, $parent = DATATREE_ROOT,
+                                  $maxdepth = -1)
     {
         if ($maxdepth == 0) {
             return false;
@@ -777,8 +793,7 @@ class DataTree {
             $out[$parent] = $this->getName($parent);
         }
 
-        $k = array_keys($this->_extractOneLevel($parent));
-        foreach ($k as $object) {
+        foreach (array_keys($this->_extractOneLevel($parent)) as $object) {
             $out[$object] = $this->getName($object);
             $this->_extractAllLevelList($out, $object, $maxdepth - 1);
         }
@@ -791,7 +806,7 @@ class DataTree {
      *                      path elements, or the object name for which
      *                      to look up the parent's ID.
      *
-     * @return integer  The unique ID of the parent.
+     * @return mixed  The unique ID of the parent or PEAR_Error on error.
      */
     function getParent($child)
     {
@@ -802,8 +817,7 @@ class DataTree {
         if (is_a($id, 'PEAR_Error')) {
             return $id;
         }
-
-        return $this->_data[$id]['parent'];
+        return $this->getParentById($id);
     }
 
     /**
@@ -811,7 +825,7 @@ class DataTree {
      *
      * @param integer $childId  Get the parent of this object.
      *
-     * @return integer  The unique ID of the parent.
+     * @return mixed  The unique ID of the parent or PEAR_Error on error.
      */
     function getParentById($childId)
     {
@@ -825,11 +839,11 @@ class DataTree {
      * Get a list of parents all the way up to the root object for
      * $child.
      *
-     * @param mixed   $child   The name of the child
+     * @param mixed $child     The name of the child
      * @param boolean $getids  If true, return parent IDs; otherwise, return
      *                         names.
      *
-     * @return array  [child] [parent] in a tree format.
+     * @return mixed  [child] [parent] in a tree format or PEAR_Error.
      */
     function getParents($child, $getids = false)
     {
@@ -838,11 +852,7 @@ class DataTree {
             return PEAR::raiseError('Parents not found: ' . $pid->getMessage());
         }
         $pname = $this->getName($pid);
-        if ($getids) {
-            $parents = array($pid => true);
-        } else {
-            $parents = array($pname => true);
-        }
+        $parents = ($getids) ? array($pid => true) : array($pname => true);
 
         if ($pid != DATATREE_ROOT) {
             if ($getids) {
@@ -921,11 +931,13 @@ class DataTree {
         }
         $this->_load($parent);
         $out = $this->_extractOneLevel($this->getId($parent));
+
         return is_array($out) ? count($out) : 0;
     }
 
     /**
-     * Check if an object exists or not. The root element DATATREE_ROOT always exists.
+     * Check if an object exists or not. The root element DATATREE_ROOT always
+     * exists.
      *
      * @param mixed $object  The name of the object.
      *
@@ -936,6 +948,7 @@ class DataTree {
         if (empty($object)) {
             return false;
         }
+
         if (is_a($object, 'DataTreeObject')) {
             $object = $object->getName();
         } elseif (is_array($object)) {
@@ -946,14 +959,12 @@ class DataTree {
             return true;
         }
 
-        $idMap = array_flip($this->_nameMap);
-        if (isset($idMap[$object])) {
+        if (array_search($object, $this->_nameMap) !== false) {
             return true;
         }
 
-        $this->_load($object);
-        $idMap = array_flip($this->_nameMap);
-        return isset($idMap[$object]);
+        // Consult the backend directly.
+        return $this->_exists($object);
     }
 
     /**
@@ -961,7 +972,7 @@ class DataTree {
      *
      * @param integer $id  The id for which to look up the name.
      *
-     * @return string
+     * @return string  TODO
      */
     function getName($id)
     {
@@ -980,14 +991,8 @@ class DataTree {
             return $this->_nameMap[$id];
         }
 
-        /* Not found in name map, try loading this id into the name
-         * map. */
-        $this->_loadById($id);
-
-        /* If id loaded return the name, otherwise return null. */
-        return isset($this->_nameMap[$id]) ?
-            $this->_nameMap[$id] :
-            null;
+        /* Not found in name map, consult the backend. */
+        return $this->_getName($id);
     }
 
     /**
@@ -1015,15 +1020,17 @@ class DataTree {
             return DATATREE_ROOT;
         }
 
-        /* Check if the name actually exists, if not return a PEAR
-         * error. */
-        if (!$this->exists($name)) {
-            return PEAR::raiseError($name . ' does not exist');
+        /* Flip the name map to look up the id using the name as key. */
+        if (($id = array_search($name, $this->_nameMap)) !== false) {
+            return $id;
         }
 
-        /* Flip the name map to look up the id using the name as key. */
-        $idMap = array_flip($this->_nameMap);
-        return $idMap[$name];
+        /* Not found in name map, consult the backend. */
+        $id = $this->_getId($name);
+        if (is_null($id)) {
+            return PEAR::raiseError($name . ' does not exist');
+        }
+        return $id;
     }
 
     /**
@@ -1042,6 +1049,7 @@ class DataTree {
         if (is_a($id, 'PEAR_Error')) {
             return $id;
         }
+        $this->_loadById($id);
 
         return isset($this->_data[$id]['order']) ?
             $this->_data[$id]['order'] :
@@ -1098,6 +1106,8 @@ class DataTree {
      * @param integer $direction   Sort direction:
      *                             0 - ascending
      *                             1 - descending
+     *
+     * @return array TODO
      */
     function getSortedTree($root, $loadTree = false, $sortby_name = null,
                            $sortby_key = null, $direction = 0)
@@ -1116,6 +1126,8 @@ class DataTree {
      *                             be used as object name. Used in situations
      *                             where there is no available unique input for
      *                             object name.
+     *
+     * @return TODO
      */
     function add($object, $id_as_name = false)
     {
@@ -1125,12 +1137,16 @@ class DataTree {
     /**
      * Add an object.
      *
+     * @private
+     *
      * @param string   $name  The short object name.
      * @param integer  $id    The new object's unique ID.
      * @param integer  $pid   The unique ID of the object's parent.
      * @param integer  $order The ordering data for the object.
      *
      * @access protected
+     *
+     * @return TODO
      */
     function _add($name, $id, $pid, $order = '')
     {
@@ -1175,6 +1191,8 @@ class DataTree {
      *                            children of $parent? Defaults to all levels.
      * @param string  $restrict   Only return attributes with the same
      *                            attribute_name or attribute_id.
+     *
+     * @return TODO
      */
     function countByAttributes($criteria, $parent = DATATREE_ROOT,
                                $allLevels = true, $restrict = 'name')
@@ -1202,6 +1220,8 @@ class DataTree {
      * @param integer $direction    Sort direction:
      *                                0 - ascending
      *                                1 - descending
+     *
+     * @return TODO
      */
     function getByAttributes($criteria, $parent = DATATREE_ROOT,
                              $allLevels = true, $restrict = 'name', $from = 0,
@@ -1239,6 +1259,8 @@ class DataTree {
      * @abstract
      *
      * @param DataTree $object  A DataTree object.
+     *
+     * @return TODO
      */
     function updateData($object)
     {
@@ -1252,6 +1274,8 @@ class DataTree {
      * You never call this function; it's used in uasort() calls. Do
      * NOT use usort(); you'll lose key => value associations.
      *
+     * @private
+     *
      * @param array $a  The first object
      * @param array $b  The second object
      *
@@ -1278,6 +1302,8 @@ class DataTree {
      *
      * @since Horde 3.1
      *
+     * @private
+     *
      * @param array $a  The first object
      * @param array $b  The second object
      *
@@ -1287,11 +1313,7 @@ class DataTree {
      */
     function _cmpSorted($a, $b)
     {
-        if ($a['sorter'][$this->_sortHash] < $b['sorter'][$this->_sortHash]) {
-            return 1;
-        } else {
-            return 0;
-        }
+        return intval($a['sorter'][$this->_sortHash] < $b['sorter'][$this->_sortHash]);
     }
 
     /**
@@ -1325,11 +1347,16 @@ class DataTree {
         include_once 'Horde/DataTree/' . $driver . '.php';
         $class = 'DataTree_' . $driver;
         if (class_exists($class)) {
-            $dt = &new $class($params);
+            $dt = new $class($params);
+            $result = $dt->_init();
+            if (is_a($result, 'PEAR_Error')) {
+                include_once 'Horde/DataTree/null.php';
+                $dt = new DataTree_null($params);
+            }
         } else {
             $dt = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
- 
+
         return $dt;
     }
 
@@ -1357,10 +1384,7 @@ class DataTree {
      */
     function &singleton($driver, $params = null)
     {
-        static $instances;
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         if (is_null($params)) {
             $params = Horde::getDriverConfig('datatree', $driver);
@@ -1380,7 +1404,7 @@ class DataTree {
  * Class that can be extended to save arbitrary information as part of a stored
  * object.
  *
- * @author  Stephane Huther <shuther@bigfoot.com>
+ * @author  Stephane Huther <shuther1@free.fr>
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Horde 2.1
  * @package Horde_DataTree
@@ -1550,7 +1574,7 @@ class DataTreeObject {
         if (!$autocreate || $this->datatree->exists($name)) {
             $child = &$this->datatree->getObject($name);
         } else {
-            $child = &new DataTreeObject($name);
+            $child = new DataTreeObject($name);
             $child->setDataTree($this->datatree);
             $this->datatree->add($child);
         }
