@@ -3,8 +3,8 @@
  * The Horde_Form_Action class provides an API for adding actions to
  * Horde_Form variables.
  *
- * $Horde: framework/Form/Form/Action.php,v 1.19.10.7 2007/01/02 13:54:17 jan Exp $
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * $Horde: framework/Form/Form/Action.php,v 1.19.10.10 2008/06/06 04:17:20 chuck Exp $
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -66,10 +66,10 @@ class Horde_Form_Action {
      * Attempts to return a concrete Horde_Form_Action instance
      * based on $form.
      *
-     * @param mixed $form    The type of concrete Horde_Form_Action subclass
-     *                       to return. If $form is an array, then we will look
-     *                       in $form[0]/lib/Form/Action/ for the subclass
-     *                       implementation named $form[1].php.
+     * @param mixed $action  The type of concrete Horde_Form_Action subclass
+     *                       to return. If $action is an array, then we will look
+     *                       in $action[0]/lib/Form/Action/ for the subclass
+     *                       implementation named $action[1].php.
      * @param array $params  A hash containing any additional configuration a
      *                       form might need.
      *
@@ -84,21 +84,17 @@ class Horde_Form_Action {
         }
 
         $action = basename($action);
-        if (empty($action) || (strcmp($action, 'none') == 0)) {
-            return PEAR::raiseError('Cannot instantiate abstract class Horde_Form_Action.');
-        }
-
-        if (!empty($app)) {
-            require_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Form/Action/' . $action . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/Action/' . $action . '.php')) {
-            require_once dirname(__FILE__) . '/Action/' . $action . '.php';
-        } else {
-            require_once 'Horde/Form/Action/' . $action . '.php';
-        }
-
         $class = 'Horde_Form_Action_' . $action;
+        if (!class_exists($class)) {
+            if (!empty($app)) {
+                include_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Form/Action/' . $action . '.php';
+            } else {
+                include_once 'Horde/Form/Action/' . $action . '.php';
+            }
+        }
+
         if (class_exists($class)) {
-            $instance = &new $class($params);
+            $instance = new $class($params);
         } else {
             $instance = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
@@ -130,10 +126,7 @@ class Horde_Form_Action {
      */
     function &singleton($action, $params = null)
     {
-        static $instances;
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         $signature = serialize(array($action, $params));
         if (!isset($instances[$signature])) {
