@@ -4,9 +4,9 @@
  * form with the current (not the original) value after the form element
  * that the action is attached to is modified.
  *
- * $Horde: framework/Form/Form/Action/reload.php,v 1.7.10.5 2007/01/02 13:54:19 jan Exp $
+ * $Horde: framework/Form/Form/Action/reload.php,v 1.7.10.7 2008/01/02 11:30:30 jan Exp $
  *
- * Copyright 2003-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -20,8 +20,10 @@ class Horde_Form_Action_reload extends Horde_Form_Action {
 
     function getActionScript($form, $renderer, $varname)
     {
-        return 'if (this.value) { document.' . $form->getName() . '.formname.value=\'\';' .
-            'document.' . $form->getName() . '.submit() }';
+        Horde::addScriptFile('prototype.js', 'horde', true);
+        Horde::addScriptFile('effects.js', 'horde', true);
+        Horde::addScriptFile('redbox.js', 'horde', true);
+        return 'if (this.value) { document.' . $form->getName() . '.formname.value=\'\'; RedBox.loading(); document.' . $form->getName() . '.submit() }';
     }
 
 }
