@@ -4,9 +4,9 @@
  * form after the form element that the action is attached to is
  * modified.
  *
- * $Horde: framework/Form/Form/Action/submit.php,v 1.10.12.2 2005/04/21 11:26:40 selsky Exp $
+ * $Horde: framework/Form/Form/Action/submit.php,v 1.10.12.7 2008/01/02 11:30:30 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -20,7 +20,10 @@ class Horde_Form_Action_submit extends Horde_Form_Action {
 
     function getActionScript($form, $renderer, $varname)
     {
-        return 'document.' . $form->getName() . '.submit()';
+        Horde::addScriptFile('prototype.js', 'horde', true);
+        Horde::addScriptFile('effects.js', 'horde', true);
+        Horde::addScriptFile('redbox.js', 'horde', true);
+        return 'RedBox.loading(); document.' . $form->getName() . '.submit()';
     }
 
 }
