@@ -4,9 +4,9 @@
  * the value of one Horde_Form variable as the variable the action is
  * attached to is updated.
  *
- * $Horde: framework/Form/Form/Action/updatefield.php,v 1.5.12.5 2007/01/02 13:54:19 jan Exp $
+ * $Horde: framework/Form/Form/Action/updatefield.php,v 1.5.12.8 2008/01/02 11:30:30 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -54,7 +54,7 @@ function updateField<?php echo $this->id() ?>()
 {
     var target = document.getElementById('<?php echo $this->getTarget() ?>');
     if (target) {
-        target.value = <?php echo implode(' + ', str_replace("\n", "\\n", $args)) ?>;
+        target.value = (<?php echo implode(' + ', str_replace("\n", "\\n", $args)) ?>).replace(/(^ +| +$)/, '').replace(/ +/g, ' ');
     }
 }<?php
         $this->_printJavaScriptEnd();
