@@ -17,12 +17,12 @@
  *               'imap/ssl/novalidate-cert' (for a self-signed certificate).
  *               DEFAULT: 'imap'</pre>
  *
- * $Horde: framework/IMAP/IMAP/ACL/rfc2086.php,v 1.6.8.17 2007/01/17 03:16:01 selsky Exp $
+ * $Horde: framework/IMAP/IMAP/ACL/rfc2086.php,v 1.6.8.19 2008/01/02 11:30:34 jan Exp $
  *
- * Copyright 2003-2007 Chris Hastie <imp@oak-wood.co.uk>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chris Hastie <imp@oak-wood.co.uk>
  * @since   Horde 3.0
@@ -93,6 +93,18 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
              'd' => _("Delete and purge messages"),
              'a' => _("Administer - set permissions for other users")
         );
+
+        $this->_rightsListTitles = array(
+            'l' => _("List"),
+            'r' => _("Read"),
+            's' => _("Mark (Seen)"),
+            'w' => _("Mark (Other)"),
+            'i' => _("Insert"),
+            'p' => _("Post"),
+            'c' => _("Create Folder"),
+            'd' => _("Delete/purge"),
+            'a' => _("Administer")
+        );
     }
 
     function _connect()
@@ -109,12 +121,15 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
                 $this->_imap = PEAR::raiseError(imap_last_error());
             }
         }
+
         return !is_a($this->_imap, 'PEAR_Error');
     }
 
     /**
      * Sets the ACL on an IMAP server.
      *
+     * @todo Cleanup for PHP 5.
+     *
      * @param string $folder      The folder on which to edit the ACL.
      * @param string $share_user  The user to grant rights to.
      * @param array $acl          An array, the keys of which are the rights to
@@ -125,9 +140,8 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
      */
     function createACL($folder, $share_user, $acl)
     {
-        if (!$this->_connect()) {
+        if (!$this->_connect())
             return $this->_imap;
-        }
 
         $acl_str = '';
         if (!empty($acl)) {
@@ -192,7 +206,6 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
     {
         $capabilities = null;
         $server = $this->_params['hostspec'];
-        $sub_proto = null;
 
         if (preg_match('|^[^/]+/ssl|', $this->_params['protocol'])) {
             $server = 'ssl://' . $server;
@@ -200,43 +213,40 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
 
         $imap = fsockopen($server, $this->_params['port'], $errno, $errstr, 30);
 
-        if (!$imap) {
+        if (!$imap)
             return PEAR::raiseError(_("Could not retrieve server's capabilities") . ' - ' . ($errno ? _("Connection failed: ") . $errno . ' : ' . $errstr : _("Connection failed.")));
-            return null;
-        } else {
-            $response = fgets($imap, 4096);
-            if (preg_match('/^\*\s+OK/', $response)) {
-                fputs($imap, "x CAPABILITY\r\n");
-                $response = trim(fgets($imap, 1024));
-                if (preg_match('/^\*\s+CAPABILITY/', $response)) {
-                    $response_array = explode(' ', $response);
-                    foreach ($response_array as $var) {
-                        if (strpos($var, '=') !== false) {
-                            $var2 = explode('=', $var, 2);
-                            $capability = String::lower($var2[0]);
-                            /* We need to make sure this array element exists and is not a
-                             * scalar (1) when we want to set a qualified capability. */
-                            if (!isset($capabilities[$capability]) ||
-                                !is_array($capabilities[$capability])) {
-                                $capabilities[$capability] = array();
-                            }
-                            $capabilities[$capability][String::lower($var2[1])] = 1;
-                        } else {
-                            $capabilities[String::lower($var)] = 1;
-                        }
-                    }
 
-                } else {
-                    fclose ($imap);
-                    return PEAR::raiseError(_("Could not retrieve server's capabilities") . ' - ' . _("Unexpected response from server to: ") . '\'x CAPABILITY\' : ' . $response);
-                }
-            } else {
-                fclose ($imap);
-                return PEAR::raiseError(_("Could not retrieve server's capabilities") . ' - ' . _("Unexpected response from server on connection: ") . $response);
-            }
+        $response = fgets($imap, 4096);
+        if (!preg_match('/^\*\s+OK/', $response)) {
             fclose ($imap);
+            return PEAR::raiseError(_("Could not retrieve server's capabilities") . ' - ' . _("Unexpected response from server on connection: ") . $response);
         }
 
+        fputs($imap, "x CAPABILITY\r\n");
+        $response = trim(fgets($imap, 1024));
+        if (!preg_match('/^\*\s+CAPABILITY/', $response)) {
+            fclose ($imap);
+            return PEAR::raiseError(_("Could not retrieve server's capabilities") . ' - ' . _("Unexpected response from server to: ") . '\'x CAPABILITY\' : ' . $response);
+        }
+
+        $response_array = explode(' ', $response);
+        foreach ($response_array as $var) {
+            if (strpos($var, '=') !== false) {
+                $var2 = explode('=', $var, 2);
+                $capability = String::lower($var2[0]);
+                /* We need to make sure this array element exists and is not a
+                 * scalar (1) when we want to set a qualified capability. */
+                if (!isset($capabilities[$capability]) ||
+                    !is_array($capabilities[$capability])) {
+                    $capabilities[$capability] = array();
+                }
+                $capabilities[$capability][String::lower($var2[1])] = 1;
+            } else {
+                $capabilities[String::lower($var)] = 1;
+            }
+        }
+        fclose ($imap);
+
         return $capabilities;
     }
 
@@ -283,6 +293,7 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
      * password to the IMAP server as plain text!!
      *
      * @access private
+     * @todo Cleanup for PHP 5.
      *
      * @param string folder    The folder to get the ACL for.
      * @param string authMech  The authorisation mechanism to use. One of
@@ -331,12 +342,9 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
             return $returnACL;
         }
 
-        global $notification;
-
         $have_sasl = false;
         $returnACL = array();
         $server = $this->_params['hostspec'];
-        $sub_proto = null;
         $txid = 0;
 
         /* Silence warnings during check if Auth_SASL module is installed. */
@@ -358,94 +366,87 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
 
         $imap = fsockopen($server, $this->_params['port'], $errno, $errstr, 30);
 
-        if (!$imap) {
+        if (!$imap)
             return PEAR::raiseError(_("Could not retrieve ACL")
                 . ' - ' . ($errno ? _("Connection failed: ") . $errno.' : ' . $errstr : _("Connection failed.")));
-        } else {
-            $response = fgets($imap, 4096);
-            if (preg_match('/^\*\s+OK/', $response)) {
 
-                /* login using the preferred mechanism default to login if
-                   Auth_SASL is not installed. */
-                if ($have_sasl && ($authMech == 'cram-md5')) {
-                    $login = Auth_SASL::factory('crammd5');
+        $response = fgets($imap, 4096);
+        if (!preg_match('/^\*\s+OK/', $response)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server on connection: ") . $response);
+        }
 
-                    fputs($imap, "$txid AUTHENTICATE CRAM-MD5\r\n");
-                    $challenge = explode(' ', trim(fgets($imap, 1024)));
+        /* login using the preferred mechanism default to login if
+           Auth_SASL is not installed. */
+        if ($have_sasl && ($authMech == 'cram-md5')) {
+            $login = Auth_SASL::factory('crammd5');
 
-                    $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]));
-                    fputs($imap, base64_encode($response) . "\r\n");
+            fputs($imap, "$txid AUTHENTICATE CRAM-MD5\r\n");
+            $challenge = explode(' ', trim(fgets($imap, 1024)));
 
-                } elseif($have_sasl && ($authMech == 'digest-md5')) {
-                    $login = Auth_SASL::factory('digestmd5');
+            $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]));
+            fputs($imap, base64_encode($response) . "\r\n");
+        } elseif ($have_sasl && ($authMech == 'digest-md5')) {
+            $login = Auth_SASL::factory('digestmd5');
 
-                    fputs($imap, "$txid AUTHENTICATE DIGEST-MD5\r\n");
-                    $challenge = explode(' ', trim(fgets($imap, 1024)));
+            fputs($imap, "$txid AUTHENTICATE DIGEST-MD5\r\n");
+            $challenge = explode(' ', trim(fgets($imap, 1024)));
 
-                    $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]),
-                        $_SESSION['imp']['server'], $_SESSION['imp']['base_protocol']);
+            $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]),
+                $_SESSION['imp']['server'], $_SESSION['imp']['base_protocol']);
 
-                    fputs($imap, base64_encode($response) . "\r\n");
-                    $response = explode(' ', trim(fgets($imap, 1024)));
-                    $response = base64_decode($response[1]);
-                    if (strpos($response, 'rspauth=') === false) {
-                        fclose($imap);
-                        return PEAR::raiseError(_("Could not retrieve ACL")
-                            . ' - ' . _("Unexpected response from server to: ") . 'Digest-MD5 response', 'horde.warning');
-                    }
-                    fputs($imap, "\r\n");
-
-                } else {
-                    if (preg_match('/\W/', $pass)) {
-                        $pass = addslashes($pass);
-                        $pass = '"' . $pass . '"';
-                    }
-                    fputs($imap, "$txid LOGIN " . $_SESSION['imp']['user'] . ' ' . $pass . "\r\n");
-
-                }
-                $response = trim(fgets($imap, 1024));
-                if (preg_match("/^$txid\sOK/", $response)) {
-                    $txid++;
-                    fputs($imap, "$txid GETACL " . $folder . "\r\n");
-                    $response = trim(fgets($imap, 4096));
-                    if (preg_match('/^\*\s+ACL\s+(.*)/i', $response, $matches)) {
-                        $res_arr = $this->_atomise($matches[1]);
-                        $res_folder = array_shift($res_arr);
-                        $is_key = 1;
-                        $key = null;
-                        foreach ($res_arr as $var) {
-                            if ($is_key) {
-                                $key = $var;
-                                $is_key = 0;
-                            } else {
-                                $perms = preg_split('//', $var, -1, PREG_SPLIT_NO_EMPTY);
-                                foreach ($perms as $p_key => $p_var) {
-                                    $returnACL[$key][$p_var] = 1;
-                                }
-                                $is_key = 1;
-                            }
-                        }
-                    } else {
-                        fclose($imap);
-                        return PEAR::raiseError(_("Could not retrieve ACL")
-                            . ' - ' . _("Unexpected response from server to: ") . "'$txid GETACL' : " .$response);
-
-                    }
-                } else {
-                    fclose($imap);
-                    return PEAR::raiseError(_("Could not retrieve ACL")
-                        . ' - ' . _("Unexpected response from server to: ") . 'login : ' . $response);
-
-                }
-            } else {
+            fputs($imap, base64_encode($response) . "\r\n");
+            $response = explode(' ', trim(fgets($imap, 1024)));
+            $response = base64_decode($response[1]);
+            if (strpos($response, 'rspauth=') === false) {
                 fclose($imap);
                 return PEAR::raiseError(_("Could not retrieve ACL")
-                    . ' - ' . _("Unexpected response from server on connection: ") . $response);
-
+                    . ' - ' . _("Unexpected response from server to: ") . 'Digest-MD5 response', 'horde.warning');
             }
-            fclose($imap);
+            fputs($imap, "\r\n");
+        } else {
+            if (preg_match('/\W/', $pass)) {
+                $pass = addslashes($pass);
+                $pass = '"' . $pass . '"';
+            }
+            fputs($imap, "$txid LOGIN " . $_SESSION['imp']['user'] . ' ' . $pass . "\r\n");
         }
 
+        $response = trim(fgets($imap, 1024));
+        if (!preg_match("/^$txid\sOK/", $response)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server to: ") . 'login : ' . $response);
+        }
+
+        $txid++;
+        fputs($imap, "$txid GETACL " . $folder . "\r\n");
+        $response = trim(fgets($imap, 4096));
+        if (!preg_match('/^\*\s+ACL\s+(.*)/i', $response, $matches)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server to: ") . "'$txid GETACL' : " .$response);
+        }
+
+        $res_arr = $this->_atomise($matches[1]);
+        $res_folder = array_shift($res_arr);
+        $is_key = 1;
+        $key = null;
+        foreach ($res_arr as $var) {
+            if ($is_key) {
+                $key = $var;
+                $is_key = 0;
+            } else {
+                $perms = preg_split('//', $var, -1, PREG_SPLIT_NO_EMPTY);
+                foreach ($perms as $p_key => $p_var) {
+                    $returnACL[$key][$p_var] = 1;
+                }
+                $is_key = 1;
+            }
+        }
+        fclose($imap);
+
         return $returnACL;
     }
 
@@ -463,7 +464,127 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
         /* We can't establish if the user is in a group with the 'a'
            privilege, so just return true and leave the decision to the
            server */
-        return true;
+        if (strcmp($this->_params['username'], $user) != 0)
+            return true;
+
+        if (isset($this->_caps['auth']['digest-md5'])) {
+            return $this->_canEdit($folder, 'digest-md5');
+        } elseif (isset($this->_caps['auth']['cram-md5'])) {
+            return $this->_canEdit($folder, 'cram-md5');
+        } else {
+            return $this->_canEdit($folder, 'login');
+        }
+    }
+
+    /**
+     * Can current user edit ACL for a folder.
+     *
+     * NB: if Auth_SASL is not installed this function will send the users
+     * password to the IMAP server as plain text!!
+     *
+     * @access private
+     *
+     * @param string folder    The folder to get the ACL for.
+     * @param string authMech  The authorization mechanism to use. One of
+     *                         cram-md5, digest-md5 or login.
+     *
+     * @return boolean  True if current user has permission to edit the ACL on
+     *                  $folder
+     */
+    function _canEdit($folder, $authMech)
+    {
+        $have_sasl = false;
+        $server = $this->_params['hostspec'];
+        $txid = 0;
+
+        /* Silence warnings during check if Auth_SASL module is installed. */
+        if (@include_once 'Auth/SASL.php') {
+            $have_sasl = true;
+        }
+
+        $pass = $this->_params['password'];
+
+        if (preg_match('|^[^/]+/ssl|', $this->_params['protocol'])) {
+            $server = 'ssl://' . $server;
+        }
+
+        /* Quote the folder string if it contains non alpha-numeric
+           characters. */
+        if (preg_match('/\W/',$folder)) {
+            $folder = '"' . $folder . '"';
+        }
+
+        $imap = fsockopen($server, $this->_params['port'], $errno, $errstr, 30);
+
+        if (!$imap)
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . ($errno ? _("Connection failed: ") . $errno.' : ' . $errstr : _("Connection failed.")));
+
+        $response = fgets($imap, 4096);
+        if (!preg_match('/^\*\s+OK/', $response)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server on connection: ") . $response);
+        }
+
+        /* login using the preferred mechanism default to login if
+           Auth_SASL is not installed. */
+        if ($have_sasl && ($authMech == 'cram-md5')) {
+            $login = Auth_SASL::factory('crammd5');
+
+            fputs($imap, "$txid AUTHENTICATE CRAM-MD5\r\n");
+            $challenge = explode(' ', trim(fgets($imap, 1024)));
+
+            $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]));
+            fputs($imap, base64_encode($response) . "\r\n");
+        } elseif ($have_sasl && ($authMech == 'digest-md5')) {
+            $login = Auth_SASL::factory('digestmd5');
+
+            fputs($imap, "$txid AUTHENTICATE DIGEST-MD5\r\n");
+            $challenge = explode(' ', trim(fgets($imap, 1024)));
+
+            $response = $login->getResponse($_SESSION['imp']['user'], $pass, base64_decode($challenge[1]),
+                $_SESSION['imp']['server'], $_SESSION['imp']['base_protocol']);
+
+            fputs($imap, base64_encode($response) . "\r\n");
+            $response = explode(' ', trim(fgets($imap, 1024)));
+            $response = base64_decode($response[1]);
+            if (strpos($response, 'rspauth=') === false) {
+                fclose($imap);
+                return PEAR::raiseError(_("Could not retrieve ACL")
+                    . ' - ' . _("Unexpected response from server to: ") . 'Digest-MD5 response', 'horde.warning');
+            }
+            fputs($imap, "\r\n");
+        } else {
+            if (preg_match('/\W/', $pass)) {
+                $pass = addslashes($pass);
+                $pass = '"' . $pass . '"';
+            }
+            fputs($imap, "$txid LOGIN " . $_SESSION['imp']['user'] . ' ' . $pass . "\r\n");
+        }
+
+        $response = trim(fgets($imap, 1024));
+        if (!preg_match("/^$txid\sOK/", $response)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server to: ") . 'login : ' . $response);
+        }
+
+        $txid++;
+        fputs($imap, "$txid MYRIGHTS " . $folder . "\r\n");
+        $response = trim(fgets($imap, 4096));
+        if (!preg_match('/^\*\s+MYRIGHTS\s+(.*)/i', $response, $matches)) {
+            fclose($imap);
+            return PEAR::raiseError(_("Could not retrieve ACL")
+                . ' - ' . _("Unexpected response from server to: ") . "'$txid MYRIGHTS' : " .$response);
+        }
+
+        $res_arr = $this->_atomise($matches[1]);
+        $res_folder = array_shift($res_arr);
+        $res_rights = array_shift($res_arr);
+        fclose($imap);
+
+        return (strpos($res_rights, 'a') !== false);
     }
 
     /**
