@@ -11,9 +11,12 @@
  *   LATT_HASCHILDREN (long) 0x20 = 32
  *   LATT_HASNOCHILDREN (long) 0x40 = 64
  * ...but these constant names do not appear in PHP until PHP 4.3.5 and 5.0.
+ *
+ * Also, no need to define LATT_REFERRAL at the present time since we don't use
+ * it anywhere.
  */
 if (!defined('LATT_HASCHILDREN')) {
-    @define('LATT_REFERRAL', 16);
+//    @define('LATT_REFERRAL', 16);
     @define('LATT_HASCHILDREN', 32);
     @define('LATT_HASNOCHILDREN', 64);
 }
@@ -27,6 +30,7 @@ define('IMAPTREE_ELT_IS_DISCOVERED', 1024);
 define('IMAPTREE_ELT_IS_POLLED', 2048);
 define('IMAPTREE_ELT_NEED_SORT', 4096);
 // '8192' is used by IMP 4.x so do not use it here
+// TODO: Renumber to 128 in Horde 4.0
 define('IMAPTREE_ELT_NAMESPACE', 16384);
 
 /* The isOpen() expanded mode constants. */
@@ -56,12 +60,9 @@ define('IMAPTREE_BASE_ELT', null);
  * functions to iterate through this tree and query information about
  * individual mailboxes/folders.
  *
- * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48.2.44 2007/01/02 13:54:22 jan Exp $
+ * $Horde: framework/IMAP/IMAP/Tree.php,v 1.48.2.46 2008/01/02 11:30:34 jan Exp $
  *
- * Copyright 2000-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2000-2007 Jon Parise <jon@horde.org>
- * Copyright 2000-2007 Anil Madhavapeddy <avsm@horde.org>
- * Copyright 2003-2007 Michael Slusarz <slusarz@horde.org>
+ * Copyright 2000-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -219,6 +220,7 @@ class IMAP_Tree {
      * @var array
      */
     var $_namespaces;
+
     /**
      * Does the IMAP server support the children extension?
      *
@@ -453,6 +455,7 @@ class IMAP_Tree {
         $this->_tree[IMAPTREE_BASE_ELT] = array('a' => IMAPTREE_ELT_IS_DISCOVERED);
 
         /* Check to make sure all namespace elements are in the tree. */
+        /* TODO: Remove namespaces availability check in Horde 4.0 */
         if ($this->_namespaces) {
             foreach ($this->_namespaces as $key => $val) {
                 if ($val['type'] != 'personal') {
@@ -582,7 +585,7 @@ class IMAP_Tree {
         if ($mask & IMAPTREE_NEXT_SHOWSUB) {
             $this->_showunsub = false;
         }
-       
+
         if ($this->_activeElt($curr) &&
             (($mask & IMAPTREE_NEXT_SHOWCLOSED) || $this->isOpen($curr)) &&
             ($this->_currparent != $curr['v'])) {
@@ -1493,6 +1496,7 @@ class IMAP_Tree {
                 $this->_subscribed['INBOX'] = 1;
             }
 
+            /* TODO: Remove namespaces availability check in Horde 4.0 */
             if ($this->_namespaces) {
                 foreach ($this->_namespaces as $val) {
                     $tmp = @imap_lsub($this->_getStream(), $this->_server, $val['name'] . '*');
@@ -1526,6 +1530,7 @@ class IMAP_Tree {
             $all_list = array();
 
             /* Get list of all mailboxes. */
+            /* TODO: Remove namespaces availability check in Horde 4.0 */
             if ($this->_namespaces) {
                 foreach ($this->_namespaces as $val) {
                     $tmp = @imap_list($this->_getStream(), $this->_server, $val['name'] . '*');
@@ -1652,6 +1657,8 @@ class IMAP_Tree {
      * THIS METHOD MUST BE DEFINED IN ALL SUBCLASSES.
      *
      * @abstract
+     * @todo Deprecate in Horde 4.0 - just have the subclass define a $_stream
+     *       variable.
      *
      * @access private
      *
@@ -1803,6 +1810,7 @@ class IMAP_Tree {
             return $namespaceCache[$mailbox];
         }
 
+        /* TODO: Remove namespaces availability check in Horde 4.0 */
         if (!empty($this->_namespaces)) {
             foreach ($this->_namespaces as $key => $val) {
                 if ((($mailbox . $val['delimiter']) == $key) ||
