@@ -6,9 +6,9 @@ require_once 'Horde/Util.php';
  * This class defines the Horde_Image:: API, and also provides some
  * utility functions, such as generating highlights of a color.
  *
- * $Horde: framework/Image/Image.php,v 1.39.10.14 2007/01/02 13:54:23 jan Exp $
+ * $Horde: framework/Image/Image.php,v 1.39.10.17 2008/06/06 04:17:20 chuck Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -82,6 +82,13 @@ class Horde_Image {
      */
     var $_tmpdir;
 
+    /**
+     * Array containing available Effects
+     *
+     * @var array
+     */
+    var $_loadedEffects = array();
+
     /**
      * Constructor.
      *
@@ -311,12 +318,16 @@ class Horde_Image {
      */
     function getDimensions()
     {
-        $tmp = $this->toFile();
-        $details = @getimagesize($tmp);
-        unlink($tmp);
+        // Check if we know it already
+        if ($this->_width == 0 && $this->_height == 0) {
+            $tmp = $this->toFile();
+            $details = @getimagesize($tmp);
+            list($this->_width, $this->_height) = $details;
+            unlink($tmp);
+        }
 
-        return array('width' => $details[0],
-                     'height' => $details[1]);
+        return array('width' => $this->_width,
+                     'height' => $this->_height);
     }
 
     /**
@@ -359,6 +370,16 @@ class Horde_Image {
         }
     }
 
+    /**
+     * Ouputs image data to file.  If $data is false, outputs current
+     * image data after performing any pending operations on the data.
+     * If $data contains raw image data, outputs that data to file without
+     * regard for $this->_data
+     *
+     * @param mixed  String of binary image data | false
+     *
+     * @return string  Path to temporary file.
+     */
     function toFile($data = false)
     {
         $tmp = Util::getTempFile('img', false, $this->_tmpdir);
@@ -441,6 +462,75 @@ class Horde_Image {
         return $pts;
     }
 
+
+    /**
+     * Attempts to apply requested effect to this image.  If the
+     * effect cannot be found a PEAR_Error is returned.
+     *
+     * @param string $type    The type of effect to apply.
+     * @param array $params   Any parameters for the effect.
+     *
+     * @return mixed  true on success | PEAR_Error on failure.
+     */
+    function addEffect($type, $params)
+    {
+        require_once 'Horde/Image/Effect.php';
+
+        // PHP4 always returns lcase from get_class()
+        $class = str_replace('horde_image_', '', String::lower(get_class($this)));
+        $effect = Horde_Image_Effect::factory($type, $class, $params);
+        if (is_a($effect, 'PEAR_Error')) {
+            return $effect;
+        }
+        $effect->_setImageObject($this);
+        return $effect->apply();
+    }
+
+    /**
+     * Load a list of available effects for this driver.
+     */
+    function getLoadedEffects()
+    {
+
+        if (empty($this->_loadedEffects)) {
+            $class = str_replace('horde_image_', '', String::lower(get_class($this)));
+
+            // First, load the driver-agnostic Effects.
+            $path = dirname(__FILE__) . '/Image/Effect/';
+            if (is_dir($path)) {
+                if ($handle = opendir($path)) {
+                    while (($file = readdir($handle)) !== false) {
+                        if (substr($file, -4, 4) == '.php') {
+                            $this->_loadedEffects[] = substr($file, 0, strlen($file) - 4);
+                        }
+                    }
+                }
+            }
+
+            // Driver specific effects.
+            $path = $path . $class;
+            if (is_dir($path)) {
+                if ($handle = opendir($path)) {
+                    while (($file = readdir($handle)) !== false) {
+                        if (substr($file, -4, 4) == '.php') {
+                            $this->_loadedEffects[] = substr($file, 0, strlen($file) - 4);
+                        }
+                    }
+                }
+            }
+        }
+        return $this->_loadedEffects;
+    }
+
+    /**
+     * Apply any effects in the effect queue.
+     */
+    function applyEffects()
+    {
+        $this->raw();
+    }
+
+
     /**
      * Attempts to return a concrete Horde_Image instance based on $driver.
      *
@@ -453,8 +543,7 @@ class Horde_Image {
      * @param array $params  A hash containing any additional configuration or
      *                       connection parameters a subclass might need.
      *
-     * @return Horde_Image|boolean  The newly created concrete Horde_Image
-     *                              instance, or false on an error.
+     * @return mixed  Horde_Image object | PEAR_Error
      */
     function &factory($driver, $params = array())
     {
@@ -462,16 +551,17 @@ class Horde_Image {
             list($app, $driver) = $driver;
         }
 
-        if (!empty($app)) {
-            require_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Image/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/Image/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/Image/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/Image/' . $driver . '.php';
-        }
         $class = 'Horde_Image_' . $driver;
+        if (!class_exists($class)) {
+            if (!empty($app)) {
+                include_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Image/' . $driver . '.php';
+            } else {
+                include_once 'Horde/Image/' . $driver . '.php';
+            }
+        }
+
         if (class_exists($class)) {
-            $image = &new $class($params);
+            $image = new $class($params);
         } else {
             $image = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
@@ -503,10 +593,7 @@ class Horde_Image {
      */
     function &singleton($driver, $params = array())
     {
-        static $instances;
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         $signature = serialize(array($driver, $params));
         if (!isset($instances[$signature])) {
