@@ -7,9 +7,9 @@ require_once dirname(__FILE__) . '/../Image.php';
  * extension. It mainly provides some utility functions, such as the
  * ability to make pixels, for now.
  *
- * $Horde: framework/Image/Image/gd.php,v 1.48.10.17 2007/01/03 00:07:32 jan Exp $
+ * $Horde: framework/Image/Image/gd.php,v 1.48.10.19 2008/01/02 11:30:35 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -33,7 +33,6 @@ class Horde_Image_gd extends Horde_Image {
                                'grayscale',
                                'sepia',
                                'yellowize',
-                               'watermark',
                                'canvas',
                          );
 
@@ -68,7 +67,12 @@ class Horde_Image_gd extends Horde_Image {
 
         if (!empty($params['width'])) {
             $this->_im = &$this->_create($this->_width, $this->_height);
-            $this->_call('imageFill', array($this->_im, 0, 0, $this->allocateColor($this->_background)));
+            if (is_a($this->_im, 'PEAR_Error')) {
+                return $this->_im;
+            }
+            if (is_resource($this->_im)) {
+                $this->_call('imageFill', array($this->_im, 0, 0, $this->_allocateColor($this->_background)));
+            }
         }
     }
 
@@ -97,6 +101,9 @@ class Horde_Image_gd extends Horde_Image {
      */
     function raw($convert = false)
     {
+        if (!is_resource($this->_im)) {
+            return '';
+        }
         return Util::bufferOutput('image' . $this->_type, $this->_im);
     }
 
@@ -122,11 +129,17 @@ class Horde_Image_gd extends Horde_Image {
     {
         if (is_a($this->_im, 'PEAR_Error')) {
             return $this->_im;
-        } elseif (is_resource($this->_im)) {
-            return array('width' => $this->_call('imageSX', array($this->_im)),
-                         'height' => $this->_call('imageSY', array($this->_im)));
+        } elseif (is_resource($this->_im) &&
+                  $this->_width == 0 &&
+                  $this->_height ==0) {
+
+            $this->_width = $this->_call('imageSX', array($this->_im));
+            $this->_height = $this->_call('imageSY', array($this->_im));
+            return array('width' => $this->_width,
+                         'height' => $this->_height);
         } else {
-            return array('width' => 0, 'height' => 0);
+            return array('width' => $this->_width,
+                         'height' => $this->_height);
         }
     }
 
@@ -138,7 +151,7 @@ class Horde_Image_gd extends Horde_Image {
      *
      * @return integer  The resource of the color that can be passed to GD.
      */
-    function allocateColor($name)
+    function _allocateColor($name)
     {
         static $colors = array();
 
@@ -258,12 +271,14 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $width      The new width.
      * @param integer $height     The new height.
      * @param boolean $ratio      Maintain original aspect ratio.
+     *
+     * @return PEAR_Error on failure
      */
     function resize($width, $height, $ratio = true)
     {
-        /* Abort if we're asked to divide by zero, or truncate the
-         * image completely in either direction. */
-        if (!$width || !$height) {
+        /* Abort if we're asked to divide by zero, truncate the image
+         * completely in either direction, or there is no image data. */
+        if (!$width || !$height || !is_resource($this->_im)) {
             return;
         }
 
@@ -277,6 +292,11 @@ class Horde_Image_gd extends Horde_Image {
 
         $im = $this->_im;
         $this->_im = &$this->_create($width, $height);
+
+        /* Reset geometry since it will change */
+        $this->_width = 0;
+        $this->_height = 0;
+
         if (is_a($this->_im, 'PEAR_Error')) {
             return $this->_im;
         }
@@ -303,6 +323,8 @@ class Horde_Image_gd extends Horde_Image {
         if (is_a($this->_im, 'PEAR_Error')) {
             return $this->_im;
         }
+        $this->_width = 0;
+        $this->_height = 0;
         return $this->_call('imageCopy', array($this->_im, $im, 0, 0, $x1, $y1, $x2 - $x1, $y2 - $y1));
     }
 
@@ -319,11 +341,14 @@ class Horde_Image_gd extends Horde_Image {
             return;
         }
 
-        $background = $this->allocateColor($background);
+        $background = $this->_allocateColor($background);
         if (is_a($background, 'PEAR_Error')) {
             return $background;
         }
 
+        $this->_width = 0;
+        $this->_height = 0;
+
         switch ($angle) {
         case '90':
             $x = $this->_call('imageSX', array($this->_im));
@@ -509,100 +534,6 @@ class Horde_Image_gd extends Horde_Image {
         }
     }
 
-    function watermark($text, $halign = 'right', $valign = 'bottom', $font = 'small')
-    {
-        $color = $this->_call('imageColorClosest', array($this->_im, 255, 255, 255));
-        if (is_a($color, 'PEAR_Error')) {
-            return $color;
-        }
-        $shadow = $this->_call('imageColorClosest', array($this->_im, 0, 0, 0));
-        if (is_a($shadow, 'PEAR_Error')) {
-            return $shadow;
-        }
-
-        // Shadow offset in pixels.
-        $drop = 1;
-
-        // Maximum text width.
-        $maxwidth = 200;
-
-        // Amount of space to leave between the text and the image
-        // border.
-        $padding = 10;
-
-        $f = $this->getFont($font);
-        $fontwidth = $this->_call('imageFontWidth', array($f));
-        if (is_a($fontwidth, 'PEAR_Error')) {
-            return $fontwidth;
-        }
-        $fontheight = $this->_call('imageFontHeight', array($f));
-        if (is_a($fontheight, 'PEAR_Error')) {
-            return $fontheight;
-        }
-
-        // So that shadow is not off the image with right align and
-        // bottom valign.
-        $margin = floor($padding + $drop) / 2;
-
-        if ($maxwidth) {
-            $maxcharsperline = floor(($maxwidth - ($margin * 2)) / $fontwidth);
-            $text = wordwrap($text, $maxcharsperline, "\n", 1);
-        }
-
-        // Split $text into individual lines.
-        $lines = explode("\n", $text);
-
-        switch ($valign) {
-        case 'center':
-            $y = ($this->_call('imageSY', array($this->_im)) - ($fontheight * count($lines))) / 2;
-            break;
-
-        case 'bottom':
-            $y = $this->_call('imageSY', array($this->_im)) - (($fontheight * count($lines)) + $margin);
-            break;
-
-        default:
-            $y = $margin;
-            break;
-        }
-
-        switch ($halign) {
-        case 'right':
-            foreach ($lines as $line) {
-                if (is_a($result = $this->_call('imageString', array($this->_im, $f, ($this->_call('imageSX', array($this->_im)) - $fontwidth * strlen($line)) - $margin + $drop, ($y + $drop), $line, $shadow)), 'PEAR_Error')) {
-                    return $result;
-                }
-                $result = $this->_call('imageString', array($this->_im, $f, ($this->_call('imageSX', array($this->_im)) - $fontwidth * strlen($line)) - $margin, $y, $line, $color));
-                $y += $fontheight;
-            }
-            break;
-
-        case 'center':
-            foreach ($lines as $line) {
-                if (is_a($result = $this->_call('imageString', array($this->_im, $f, floor(($this->_call('imageSX', array($this->_im)) - $fontwidth * strlen($line)) / 2) + $drop, ($y + $drop), $line, $shadow)), 'PEAR_Error')) {
-                    return $result;
-                }
-                $result = $this->_call('imageString', array($this->_im, $f, floor(($this->_call('imageSX', array($this->_im)) - $fontwidth * strlen($line)) / 2), $y, $line, $color));
-                $y += $fontheight;
-            }
-            break;
-
-        default:
-            foreach ($lines as $line) {
-                if (is_a($result = $this->_call('imageString', array($this->_im, $f, $margin + $drop, ($y + $drop), $line, $shadow)), 'PEAR_Error')) {
-                    return $result;
-                }
-                $result = $this->_call('imageString', array($this->_im, $f, $margin, $y, $line, $color));
-                $y += $fontheight;
-            }
-            break;
-        }
-
-        if (is_a($result, 'PEAR_Error')) {
-            return $result;
-        }
-    }
-
     /**
      * Draws a text string on the image in a specified location, with
      * the specified style information.
@@ -618,9 +549,9 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $direction  An integer that specifies the orientation of
      *                            the text.
      */
-    function text($string, $x, $y, $font = 'monospace', $color = 'black', $direction = 0)
+    function text($string, $x, $y, $font = 'monospace', $color = 'black', $direction = 0, $fontsize = 'small')
     {
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
@@ -650,7 +581,7 @@ class Horde_Image_gd extends Horde_Image {
      */
     function circle($x, $y, $r, $color, $fill = null)
     {
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
@@ -658,7 +589,7 @@ class Horde_Image_gd extends Horde_Image {
             $result = $this->_call('imageEllipse', array($this->_im, $x, $y, $r * 2, $r * 2, $c));
         } else {
             if ($fill !== $color) {
-                $fillColor = $this->allocateColor($fill);
+                $fillColor = $this->_allocateColor($fill);
                 if (is_a($fillColor, 'PEAR_Error')) {
                     return $fillColor;
                 }
@@ -692,7 +623,7 @@ class Horde_Image_gd extends Horde_Image {
         }
 
         if ($fill != 'none') {
-            $f = $this->allocateColor($fill);
+            $f = $this->_allocateColor($fill);
             if (is_a($f, 'PEAR_Error')) {
                 return $f;
             }
@@ -702,7 +633,7 @@ class Horde_Image_gd extends Horde_Image {
         }
 
         if ($fill == 'none' || $fill != $color) {
-            $c = $this->allocateColor($color);
+            $c = $this->_allocateColor($color);
             if (is_a($c, 'PEAR_Error')) {
                 return $c;
             }
@@ -725,7 +656,7 @@ class Horde_Image_gd extends Horde_Image {
     function rectangle($x, $y, $width, $height, $color = 'black', $fill = 'none')
     {
         if ($fill != 'none') {
-            $f = $this->allocateColor($fill);
+            $f = $this->_allocateColor($fill);
             if (is_a($f, 'PEAR_Error')) {
                 return $f;
             }
@@ -735,7 +666,7 @@ class Horde_Image_gd extends Horde_Image {
         }
 
         if ($fill == 'none' || $fill != $color) {
-            $c = $this->allocateColor($color);
+            $c = $this->_allocateColor($color);
             if (is_a($c, 'PEAR_Error')) {
                 return $c;
             }
@@ -763,7 +694,7 @@ class Horde_Image_gd extends Horde_Image {
             return $this->rectangle($x, $y, $width, $height, $color, $fill);
         }
 
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
@@ -829,7 +760,7 @@ class Horde_Image_gd extends Horde_Image {
         }
 
         if ($fill != 'none') {
-            $f = $this->allocateColor($fill);
+            $f = $this->_allocateColor($fill);
             if (is_a($f, 'PEAR_Error')) {
                 return $f;
             }
@@ -851,7 +782,7 @@ class Horde_Image_gd extends Horde_Image {
      */
     function line($x1, $y1, $x2, $y2, $color = 'black', $width = 1)
     {
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
@@ -901,11 +832,11 @@ class Horde_Image_gd extends Horde_Image {
      */
     function dashedLine($x0, $y0, $x1, $y1, $color = 'black', $width = 1, $dash_length = 2, $dash_space = 2)
     {
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
-        $w = $this->allocateColor('white');
+        $w = $this->_allocateColor('white');
         if (is_a($w, 'PEAR_Error')) {
             return $w;
         }
@@ -966,7 +897,7 @@ class Horde_Image_gd extends Horde_Image {
      */
     function arc($x, $y, $r, $start, $end, $color = 'black', $fill = null)
     {
-        $c = $this->allocateColor($color);
+        $c = $this->_allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
             return $c;
         }
@@ -974,7 +905,7 @@ class Horde_Image_gd extends Horde_Image {
             $result = $this->_call('imageArc', array($this->_im, $x, $y, $r * 2, $r * 2, $start, $end, $c));
         } else {
             if ($fill !== $color) {
-                $f = $this->allocateColor($fill);
+                $f = $this->_allocateColor($fill);
                 if (is_a($f, 'PEAR_Error')) {
                     return $f;
                 }
@@ -989,6 +920,122 @@ class Horde_Image_gd extends Horde_Image {
         return $result;
     }
 
+    /**
+     * Applies the specified mask to the image.
+     *
+     * @param resource $gdimg_mask  The gd image resource representing the mask
+     *
+     * @return mixed  true | PEAR_Error
+     */
+    function _applyMask($gdimg_mask) {
+        $imgX = round($this->_call('imageSX', array($this->_im)));
+        $imgY = round($this->_call('imageSY', array($this->_im)));
+
+        $gdimg_mask_resized = $this->_create($imgX, $imgY);
+        if (!is_a($gdimg_mask_resized, 'PEAR_Error')) {
+            $result = $this->_call('imageCopyResampled',
+                                   array($gdimg_mask_resized,
+                                         $gdimg_mask,
+                                         0, 0, 0, 0,
+                                         $imgX,
+                                         $imgY,
+                                         $this->_call('imageSX', array($gdimg_mask)),
+                                         $this->_call('imageSY', array($gdimg_mask))));
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+
+            $gdimg_mask_blendtemp = $this->_create($imgX, $imgY);
+            if (is_a($gdimg_mask_blendtemp, 'PEAR_Error')) {
+                return $gdimg_mask_blendtemp;
+            }
+            $mbtX = $this->_call('imageSX', array($gdimg_mask_blendtemp));
+            $mbtY = $this->_call('imageSY', array($gdimg_mask_blendtemp));
+
+            if (!is_a($gdimg_mask_blendtemp, 'PEAR_Error')) {
+                $color_background = $this->_call('imageColorAllocate',
+                                                 array($gdimg_mask_blendtemp, 0, 0, 0));
+
+                $this->_call('imageFilledRectangle', array($gdimg_mask_blendtemp,
+                                                           0,
+                                                           0,
+                                                           $mbtX,
+                                                           $mbtY,
+                                                           $color_background));
+
+                $this->_call('imageAlphaBlending',
+                             array($gdimg_mask_blendtemp, false));
+
+                $this->_call('imageSaveAlpha',
+                             array($gdimg_mask_blendtemp, true));
+
+                for ($x = 0; $x < $imgX; $x++) {
+                    for ($y = 0; $y < $imgY; $y++) {
+                        $colorat = $this->_call('imageColorAt',
+                                                 array($this->_im, $x, $y));
+
+                        if (is_a($colorat, 'PEAR_Error')) {
+                            return $colorat;
+                        }
+
+                        $realPixel = $this->_call('imageColorsForIndex',
+                                                  array($this->_im, $colorat));
+
+                        $colorat = $this->_call('imageColorAt',
+                                                array($gdimg_mask_resized, $x, $y));
+
+                        if (is_a($colorat, 'PEAR_Error')) {
+                            return $colorat;
+                        }
+
+                        $maskPixel = $this->_grayscalePixel($this->_call('imageColorsForIndex',
+                                                                         array($gdimg_mask_resized, $colorat)));
+
+                        $maskAlpha = 127 - (floor($maskPixel['red'] / 2) * (1 - ($realPixel['alpha'] / 127)));
+                        $newcolor = $this->_allocateColorAlpha($gdimg_mask_blendtemp,
+                                                               $realPixel['red'],
+                                                               $realPixel['green'],
+                                                               $realPixel['blue'],
+                                                               intval($maskAlpha));
+                        $this->_call('imageSetPixel',
+                                     array($gdimg_mask_blendtemp, $x, $y, $newcolor));
+                    }
+                }
+                $this->_call('imageAlphaBlending', array($this->_im, false));
+                $this->_call('imageSaveAlpha', array($this->_im, true));
+                $this->_call('imageCopy', array($this->_im,
+                                                $gdimg_mask_blendtemp,
+                                                0, 0, 0, 0,
+                                                $mbtX,
+                                                $mbtY));
+
+                $this->_call('imageDestroy', array($gdimg_mask_blendtemp));
+
+            } else {
+                return $gdimg_mask_blendtemp; // PEAR_Error
+            }
+            $this->_call('imageDestroy', array($gdimg_mask_resized));
+        } else {
+            return $gdimg_mask_resized; // PEAR_Error
+        }
+        return true;
+    }
+
+    /**
+     * @TODO
+     */
+    function _grayscaleValue($r, $g, $b) {
+        return round(($r * 0.30) + ($g * 0.59) + ($b * 0.11));
+    }
+
+    /**
+     * @TODO
+     */
+    function _grayscalePixel($OriginalPixel) {
+        $gray = $this->_grayscaleValue($OriginalPixel['red'], $OriginalPixel['green'], $OriginalPixel['blue']);
+        return array('red'=>$gray, 'green'=>$gray, 'blue'=>$gray);
+    }
+
     /**
      * Creates an image of the given size.
      * If possible the function returns a true color image.
@@ -996,8 +1043,8 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $width   The image width.
      * @param integer $height  The image height.
      *
-     * @return resource|object PEAR Error  The image handler or a PEAR_Error on
-     *                                     error.
+     * @return resource|object PEAR Error  The image handler or a PEAR_Error
+     *                                     on error.
      */
     function &_create($width, $height)
     {
@@ -1008,6 +1055,17 @@ class Horde_Image_gd extends Horde_Image {
         return $result;
     }
 
+    function _allocateColorAlpha($gdimg_hexcolorallocate, $r, $g, $b , $alpha = false) {
+        $result = $this->_call('imageColorAllocateAlpha',
+                                array($gdimg_hexcolorallocate, $r, $g, $b, intval($alpha)));
+        if (is_a($result, 'PEAR_Error')) {
+            $result = $this->_call('imageColorAllocate',
+                                   array($gdimg_hexcolorallocate, $r, $g, $b));
+        }
+        return $result;
+    }
+
+
     /**
      * Wraps a call to a function of the gd extension.
      * If the call produces an error, a PEAR_Error is returned, the function
