@@ -5,16 +5,22 @@ require_once 'Horde/Image.php';
 /**
  * This class implements the Horde_Image:: API for ImageMagick.
  *
- * $Horde: framework/Image/Image/im.php,v 1.34.10.13 2007/01/02 13:54:23 jan Exp $
+ * Additional parameters to the constructor exclusive to this driver:
+ * <pre>
+ * 'type' - What type of image should be generated.
+ * </pre>
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * $Horde: framework/Image/Image/im.php,v 1.34.10.24 2008/10/23 03:20:43 mrubinsk Exp $
+ *
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
+ * @author  Michael J. Rubinsky <mrubinsk@horde.org>
+ *
  * @since   Horde 3.0
  * @package Horde_Image
  */
@@ -43,12 +49,21 @@ class Horde_Image_im extends Horde_Image {
     var $_type = 'png';
 
     /**
-     * Operations to be performed.
+     * Operations to be performed. These are added before the source filename
+     * is specified on the command line.
      *
      * @var array
      */
     var $_operations = array();
 
+    /**
+     * Operations to be added after the source filename is specified on the
+     * command line.
+     *
+     * @var array
+     */
+    var $_postSrcOperations = array();
+
     /**
      * Current stroke color; cached so we don't issue more -stroke commands
      * than necessary.
@@ -73,6 +88,20 @@ class Horde_Image_im extends Horde_Image {
      */
     var $_fillColor = null;
 
+    /**
+     * Reference to an Horde_Image_ImagickProxy object.
+     *
+     * @var Imagick
+     */
+    var $_imagick = null;
+
+    /**
+     * TODO
+     *
+     * @var array
+     */
+    var $_toClean = array();
+
     /**
      * Constructor.
      */
@@ -84,16 +113,67 @@ class Horde_Image_im extends Horde_Image {
             $this->_type = $params['type'];
         }
 
-        // Make sure we start with a white background to be consistent
-        // with other drivers.
-        if (!empty($params['background'])) {
-            $bg = $params['background'];
-        } else {
-            $bg = 'white';
+        // Imagick library doesn't play nice with outputting data for 0x0
+        // images, so use 1x1 for our default if needed.
+        if (Util::loadExtension('imagick')) {
+            ini_set('imagick.locale_fix', 1);
+            require_once 'Horde/Image/imagick.php';
+            $this->_width = max(array($this->_width, 1));
+            $this->_height = max(array($this->_height, 1));
+
+            // Use a proxy for the Imagick calls to keep exception catching
+            // code out of PHP4 compliant code.
+            $this->_imagick = new Horde_Image_ImagickProxy($this->_width,
+                                                           $this->_height,
+                                                           $this->_background,
+                                                           $this->_type);
+            $this->_data = $this->_imagick->getImageBlob();
         }
-        $this->rectangle(0, 0, $this->_width, $this->_height, $bg, $bg);
+
+        $this->rectangle(0, 0, $this->_width, $this->_height, $this->_background, $this->_background);
     }
 
+
+    /**
+     * Load the image data from a string. Need to override this method
+     * in order to load the imagick object if we need to.
+     *
+     * @param string $id          An arbitrary id for the image.
+     * @param string $image_data  The data to use for the image.
+     */
+    function loadString($id, $image_data)
+    {
+        parent::loadString($id, $image_data);
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->clear();
+            $this->_imagick->readImageBlob($image_data);
+            $this->_imagick->setFormat($this->_type);
+            $this->_imagick->setIteratorIndex(0);
+        }
+    }
+
+    /**
+     * Load the image data from a file. Need to override this method
+     * in order to load the imagick object if we need to.
+     *
+     * @param string $filename  The full path and filename to the file to load
+     *                          the image data from. The filename will also be
+     *                          used for the image id.
+     *
+     * @return mixed  PEAR Error if file does not exist or could not be loaded
+     *                otherwise NULL if successful or already loaded.
+     */
+    function loadFile($filename)
+    {
+        parent::loadFile($filename);
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->clear();
+            $this->_imagick->readImageBlob($this->_data);
+            $this->_imagick->setIteratorIndex(0);
+        }
+    }
+
+
     /**
      * Return the content type for this image.
      *
@@ -117,11 +197,17 @@ class Horde_Image_im extends Horde_Image {
     {
         global $conf;
 
+        // Make sure _data is sync'd with imagick object
+        if (!is_null($this->_imagick)) {
+            $this->_data = $this->_imagick->getImageBlob();
+        }
+
         if (!empty($this->_data)) {
-            // If there are no operations, and we already have data,
-            // don't bother writing out files, just return the current
-            // data.
-            if (!$convert && !count($this->_operations)) {
+            // If there are no operations, and we already have data, don't
+            // bother writing out files, just return the current data.
+            if (!$convert &&
+                !count($this->_operations) &&
+                !count($this->_postSrcOperations)) {
                 return $this->_data;
             }
 
@@ -134,15 +220,36 @@ class Horde_Image_im extends Horde_Image {
             fclose($fp);
         }
 
-        $tmpout = Util::getTempFile('img', false, $this->_tmpdir);
+        // Perform convert command if needed
+        if (count($this->_operations) || count($this->_postSrcOperations)) {
+            $tmpout = Util::getTempFile('img', false, $this->_tmpdir);
 
-        $command  = $conf['image']['convert'];
-        $command .= ' ' . implode(' ', $this->_operations);
-        $command .= ' "' . $tmpin . '" +profile "*" ' . $this->_type . ':"' . $tmpout . '" 2>&1';
+            $command = $conf['image']['convert']
+                . ' ' . implode(' ', $this->_operations)
+                . ' "' . $tmpin . '"\'[0]\' '
+                . implode(' ', $this->_postSrcOperations)
+                . ' +profile "*" ' . $this->_type . ':"' . $tmpout . '" 2>&1';
+            Horde::logMessage('convert command executed by Horde_Image_im::raw(): ' . $command, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            exec($command, $output, $retval);
+            if ($retval) {
+                Horde::logMessage('Error running command: ' . $command . "\n" . implode("\n", $output), __FILE__, __LINE__, PEAR_LOG_WARNING);
+            }
 
-        exec($command, $output, $retval);
+            /* Empty the operations queue */
+            $this->_operations = array();
+            $this->_postSrcOperations = array();
 
-        $this->_data = file_get_contents($tmpout);
+            /* Load the result */
+            $this->_data = file_get_contents($tmpout);
+
+            // Keep imagick object in sync if we need to.
+            // @TODO: Might be able to stop doing this once all operations
+            // are doable through imagick api.
+            if (!is_null($this->_imagick)) {
+                $this->_imagick->clear();
+                $this->_imagick->readImageBlob($this->_data);
+            }
+        }
 
         @unlink($tmpin);
         @unlink($tmpout);
@@ -157,6 +264,12 @@ class Horde_Image_im extends Horde_Image {
     {
         parent::reset();
         $this->_operations = array();
+        $this->_postSrcOperations = array();
+        if (is_object($this->_imagick)) {
+            $this->_imagick->clear();
+        }
+        $this->_width = 0;
+        $this->_height = 0;
     }
 
     /**
@@ -168,25 +281,72 @@ class Horde_Image_im extends Horde_Image {
      */
     function resize($width, $height, $ratio = true)
     {
-        if ($ratio) {
-            $this->_operations[] = "-size {$width}x{$height} -resize {$width}x{$height}";
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->thumbnailImage($width, $height, $ratio);
         } else {
-            $this->_operations[] = "-size {$width}x{$height} -resize {$width}x{$height}!";
+            if ($ratio) {
+                $this->_operations[] = "-size {$width}x{$height} -resize {$width}x{$height}";
+            } else {
+                $this->_operations[] = "-size {$width}x{$height} -resize {$width}x{$height}!";
+            }
+        }
+        // Reset the width and height instance variables since after resize
+        // we don't know the *exact* dimensions yet (especially if we maintained
+        // aspect ratio.
+        // Refresh the data
+        $this->raw();
+        $this->_width = 0;
+        $this->_height = 0;
+    }
+
+    /**
+     * More efficient way of getting size if using imagick library.
+     * *ALWAYS* use getDimensions() to get image geometry...instance
+     * variables only cache geometry until it changes, then they go
+     * to zero.
+     *
+     */
+    function getDimensions()
+    {
+        if (!is_null($this->_imagick)) {
+            if ($this->_height == 0 && $this->_width == 0) {
+                $size = $this->_imagick->getImageGeometry();
+                if (is_a($size, 'PEAR_Error')) {
+                    return $size;
+                }
+                $this->_height = $size['height'];
+                $this->_width = $size['width'];
+            }
+            return array('width' => $this->_width,
+                         'height' => $this->_height);
+        } else {
+            return parent::getDimensions();
         }
     }
 
     /**
      * Crop the current image.
      *
-     * @param integer $x1  The top left corner of the cropped image.
-     * @param integer $y1  The top right corner of the cropped image.
-     * @param integer $x2  The bottom left corner of the cropped image.
-     * @param integer $y2  The bottom right corner of the cropped image.
+     * @param integer $x1  x for the top left corner
+     * @param integer $y1  y for the top left corner
+     * @param integer $x2  x for the bottom right corner of the cropped image.
+     * @param integer $y2  y for the bottom right corner of the cropped image.
      */
     function crop($x1, $y1, $x2, $y2)
     {
-        $line = ($x2 - $x1) . 'x' . ($y2 - $y1) . '+' . $x1 . '+' . $y2;
-        $this->_operations[] = '-crop ' . $line;
+        if (!is_null($this->_imagick)) {
+            $result = $this->_imagick->cropImage($x2 - $x1, $y2 - $y1, $x1, $y1);
+            $this->_imagick->setImagePage(0, 0, 0, 0);
+        } else {
+            $line = ($x2 - $x1) . 'x' . ($y2 - $y1) . '+' . $x1 . '+' . $y1;
+            $this->_operations[] = '-crop ' . $line . ' +repage';
+            $result = true;
+        }
+        // Reset width/height since these might change
+        $this->raw();
+        $this->_width = 0;
+        $this->_height = 0;
+        return $result;
     }
 
     /**
@@ -198,7 +358,16 @@ class Horde_Image_im extends Horde_Image {
      */
     function rotate($angle, $background = 'white')
     {
-        $this->_operations[] = "-rotate {$angle} -background $background";
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->rotateImage($background, $angle);
+        } else {
+            $this->raw();
+            $this->_operations[] = "-background $background -rotate {$angle}";
+            $this->raw();
+        }
+        // Reset width/height since these might have changed
+        $this->_width = 0;
+        $this->_height = 0;
     }
 
     /**
@@ -206,7 +375,11 @@ class Horde_Image_im extends Horde_Image {
      */
     function flip()
     {
-        $this->_operations[] = '-flip';
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->flipImage();
+        } else {
+            $this->_operations[] = '-flip';
+        }
     }
 
     /**
@@ -214,7 +387,11 @@ class Horde_Image_im extends Horde_Image {
      */
     function mirror()
     {
-        $this->_operations[] = '-flop';
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->flopImage();
+        } else {
+            $this->_operations[] = '-flop';
+        }
     }
 
     /**
@@ -222,18 +399,28 @@ class Horde_Image_im extends Horde_Image {
      */
     function grayscale()
     {
-        $this->_operations[] = '-colorspace GRAY';
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->setImageColorSpace(constant('Imagick::COLORSPACE_GRAY'));
+        } else {
+            $this->_postSrcOperations[] = '-colorspace GRAY';
+        }
     }
 
     /**
      * Sepia filter.
+     *
+     * @param integer $threshold  Extent of sepia effect.
      */
-    function sepia()
+    function sepia($threshold =  85)
     {
-        $this->_operations[] = '-modulate 110 -colorspace GRAY -colors 256 -gamma 1.25/1/0.66';
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->sepiaToneImage($threshold);
+        } else {
+            $this->_operations[] = '-sepia-tone ' . $threshold . '%';
+        }
     }
 
-    /**
+     /**
      * Draws a text string on the image in a specified location, with
      * the specified style information.
      *
@@ -243,15 +430,19 @@ class Horde_Image_im extends Horde_Image {
      * @param string  $font       The font identifier you want to use for the text.
      * @param string  $color      The color that you want the text displayed in.
      * @param integer $direction  An integer that specifies the orientation of the text.
+     * @param string  $fontsize   Size of the font (small, medium, large, giant)
      */
-    function text($string, $x, $y, $font = '_sans', $color = 'black', $direction = 0)
+    function text($string, $x, $y, $font = 'ariel', $color = 'black', $direction = 0, $fontsize = 'small')
     {
-        $this->setStrokeColor($color);
-        $this->setStrokeWidth(1);
-
-        $string = addslashes('"' . $string . '"');
-        $y = $y + 12;
-        $this->_operations[] = "-draw \"text $x,$y $string\"";
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->text($string, $x, $y, $font, $color, $direction,
+                                         $fontsize);
+        } else {
+            $this->setFillColor($color);
+            $string = addslashes('"' . $string . '"');
+            $fontsize = $this->_getFontSize($fontsize);
+            $this->_operations[] = "-font $font -pointsize $fontsize -gravity northwest -draw \"text $x,$y $string\"";
+        }
     }
 
     /**
@@ -265,11 +456,14 @@ class Horde_Image_im extends Horde_Image {
      */
     function circle($x, $y, $r, $color, $fill = 'none')
     {
-        $this->setStrokeColor($color);
-        $this->setFillColor($fill);
-
-        $xMax = $x + $r;
-        $this->_operations[] = "-draw \"circle $x,$y $xMax,$y\"";
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->circle($x, $y, $r, $color, $fill);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setFillColor($fill);
+            $xMax = $x + $r;
+            $this->_operations[] = "-draw \"circle $x,$y $xMax,$y\"";
+        }
     }
 
     /**
@@ -282,14 +476,18 @@ class Horde_Image_im extends Horde_Image {
      */
     function polygon($verts, $color, $fill = 'none')
     {
-        $this->setStrokeColor($color);
-        $this->setFillColor($fill);
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->polygon($verts, $color, $fill);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setFillColor($fill);
 
-        $command = '';
-        foreach ($verts as $vert) {
-            $command .= sprintf(' %d,%d', $vert['x'], $vert['y']);
+            $command = '';
+            foreach ($verts as $vert) {
+                $command .= sprintf(' %d,%d', $vert['x'], $vert['y']);
+            }
+            $this->_operations[] = "-draw \"polygon $command\"";
         }
-        $this->_operations[] = "-draw \"polygon $command\"";
     }
 
     /**
@@ -304,12 +502,15 @@ class Horde_Image_im extends Horde_Image {
      */
     function rectangle($x, $y, $width, $height, $color, $fill = 'none')
     {
-        $this->setStrokeColor($color);
-        $this->setFillColor($fill);
-
-        $xMax = $x + $width;
-        $yMax = $y + $height;
-        $this->_operations[] = "-draw \"rectangle $x,$y $xMax,$yMax\"";
+        if (!is_null($this->_imagick)) {
+            $this->_imagick->rectangle($x, $y, $width, $height, $color, $fill);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setFillColor($fill);
+            $xMax = $x + $width;
+            $yMax = $y + $height;
+            $this->_operations[] = "-draw \"rectangle $x,$y $xMax,$yMax\"";
+        }
     }
 
     /**
@@ -345,9 +546,13 @@ class Horde_Image_im extends Horde_Image {
      */
     function line($x0, $y0, $x1, $y1, $color = 'black', $width = 1)
     {
-        $this->setStrokeColor($color);
-        $this->setStrokeWidth($width);
-        $this->_operations[] = "-draw \"line $x0,$y0 $x1,$y1\"";
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->line($x0, $y0, $x1, $y1, $color, $width);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setStrokeWidth($width);
+            $this->_operations[] = "-draw \"line $x0,$y0 $x1,$y1\"";
+        }
     }
 
     /**
@@ -364,9 +569,15 @@ class Horde_Image_im extends Horde_Image {
      */
     function dashedLine($x0, $y0, $x1, $y1, $color = 'black', $width = 1, $dash_length = 2, $dash_space = 2)
     {
-        $this->setStrokeColor($color);
-        $this->setStrokeWidth($width);
-        $this->_operations[] = "-draw \"line $x0,$y0 $x1,$y1\"";
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->dashedLine($x0, $y0, $x1, $y1, $color,
+                                               $width, $dash_length,
+                                               $dash_space);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setStrokeWidth($width);
+            $this->_operations[] = "-draw \"line $x0,$y0 $x1,$y1\"";
+        }
     }
 
     /**
@@ -380,15 +591,19 @@ class Horde_Image_im extends Horde_Image {
      */
     function polyline($verts, $color, $width = 1)
     {
-        $this->setStrokeColor($color);
-        $this->setStrokeWidth($width);
-        $this->setFillColor('none');
+        if (!is_null($this->_imagick)) {
+            return $this->_imagick->polyline($verts, $color, $width);
+        } else {
+            $this->setStrokeColor($color);
+            $this->setStrokeWidth($width);
+            $this->setFillColor('none');
 
-        $command = '';
-        foreach ($verts as $vert) {
-            $command .= sprintf(' %d,%d', $vert['x'], $vert['y']);
+            $command = '';
+            foreach ($verts as $vert) {
+                $command .= sprintf(' %d,%d', $vert['x'], $vert['y']);
+            }
+            $this->_operations[] = "-draw \"polyline $command\"";
         }
-        $this->_operations[] = "-draw \"polyline $command\"";
     }
 
     /**
@@ -506,4 +721,56 @@ class Horde_Image_im extends Horde_Image {
         }
     }
 
+    function applyEffects()
+    {
+        $this->raw();
+        foreach ($this->_toClean as $tempfile) {
+            @unlink($tempfile);
+        }
+    }
+
+
+    /**
+     * Return point size for font
+     */
+    function _getFontSize($fontsize)
+    {
+        switch ($fontsize) {
+        case 'medium':
+            $point = 18;
+            break;
+        case 'large':
+            $point = 24;
+            break;
+        case 'giant':
+            $point = 30;
+            break;
+        default:
+            $point = 12;
+        }
+        return $point;
+    }
+
+
+    function _getIMVersion()
+    {
+        static $version = null;
+        if (!is_array($version)) {
+            if (!is_null($this->_imagick)) {
+                $output = $this->_imagick->getVersion();
+                $output[0] = $output['versionString'];
+            } else {
+                $commandline = $GLOBALS['conf']['image']['convert'] . ' --version';
+                exec($commandline, $output, $retval);
+            }
+            if (preg_match('/([0-9])\.([0-9])\.([0-9])/', $output[0], $matches)) {
+                $version = $matches;
+                return $matches;
+            } else {
+               return false;
+            }
+        }
+        return $version;
+    }
+
 }
