@@ -6,9 +6,9 @@ require_once 'XML/SVG.php';
 /**
  * This class implements the Horde_Image:: API for SVG.
  *
- * $Horde: framework/Image/Image/svg.php,v 1.31.10.7 2007/01/02 13:54:23 jan Exp $
+ * $Horde: framework/Image/Image/svg.php,v 1.31.10.9 2008/01/02 11:30:37 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -31,8 +31,8 @@ class Horde_Image_svg extends Horde_Image {
     function Horde_Image_svg($params)
     {
         parent::Horde_Image($params);
-        $this->_svg = &new XML_SVG_Document(array('width' => $this->_width,
-                                                  'height' => $this->_height));
+        $this->_svg = new XML_SVG_Document(array('width' => $this->_width,
+                                                 'height' => $this->_height));
     }
 
     function getContentType()
@@ -67,15 +67,15 @@ class Horde_Image_svg extends Horde_Image {
     function _createSymbol($s, $id)
     {
         $s->setParam('id', $id);
-        $defs = &new XML_SVG_Defs();
+        $defs = new XML_SVG_Defs();
         $defs->addChild($s);
         $this->_svg->addChild($defs);
     }
 
     function _createDropShadow($id = 'dropShadow')
     {
-        $defs = &new XML_SVG_Defs();
-        $filter = &new XML_SVG_Filter(array('id' => $id));
+        $defs = new XML_SVG_Defs();
+        $filter = new XML_SVG_Filter(array('id' => $id));
         $filter->addPrimitive('GaussianBlur', array('in' => 'SourceAlpha',
                                                     'stdDeviation' => 2,
                                                     'result' => 'blur'));
@@ -83,7 +83,7 @@ class Horde_Image_svg extends Horde_Image {
                                               'dx' => 4,
                                               'dy' => 4,
                                               'result' => 'offsetBlur'));
-        $merge = &new XML_SVG_FilterPrimitive('Merge');
+        $merge = new XML_SVG_FilterPrimitive('Merge');
         $merge->addMergeNode('offsetBlur');
         $merge->addMergeNode('SourceGraphic');
 
