@@ -6,9 +6,9 @@ require_once dirname(__FILE__) . '/../Image.php';
  * This class implements the Horde_Image:: API for SWF, using the PHP
  * Ming extension.
  *
- * $Horde: framework/Image/Image/swf.php,v 1.24.10.7 2007/01/02 13:54:23 jan Exp $
+ * $Horde: framework/Image/Image/swf.php,v 1.24.10.9 2008/01/02 11:30:37 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -37,7 +37,7 @@ class Horde_Image_swf extends Horde_Image {
     {
         parent::Horde_Image($params);
 
-        $this->_movie = &new SWFMovie();
+        $this->_movie = new SWFMovie();
         $this->_movie->setDimension($this->_width, $this->_height);
 
         // FIXME: honor the 'background' parameter here.
@@ -123,10 +123,10 @@ class Horde_Image_swf extends Horde_Image {
     {
         $color = $this->allocateColor($color);
 
-        if (substr(PHP_OS, 0, 3) == 'WIN') {
-            $text = &new SWFTextField(SWFTEXTFIELD_NOEDIT);
+        if (!strncasecmp(PHP_OS, 'WIN', 3)) {
+            $text = new SWFTextField(SWFTEXTFIELD_NOEDIT);
         } else {
-            $text = &new SWFText();
+            $text = new SWFText();
         }
         $text->setColor($color['red'], $color['green'], $color['blue'], $color['alpha']);
         $text->addString($string);
@@ -150,7 +150,7 @@ class Horde_Image_swf extends Horde_Image {
      */
     function circle($x, $y, $r, $color, $fill = 'none')
     {
-        $s = &new SWFShape();
+        $s = new SWFShape();
         $color = $this->allocateColor($color);
         $s->setLine(1, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 
@@ -190,7 +190,7 @@ class Horde_Image_swf extends Horde_Image {
         $color = $this->allocateColor($color);
 
         if (is_array($color) && is_array($verts) && (sizeof($verts) > 2)) {
-            $shape = &new SWFShape();
+            $shape = new SWFShape();
             $shape->setLine(1, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 
             if ($fill != 'none') {
@@ -257,7 +257,7 @@ class Horde_Image_swf extends Horde_Image {
             return $this->rectangle($x, $y, $width, $height, $color, $fill);
         }
 
-        $s = &new SWFShape();
+        $s = new SWFShape();
         $color = $this->allocateColor($color);
         $s->setLine(1, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 
@@ -345,7 +345,7 @@ class Horde_Image_swf extends Horde_Image {
         $color = $this->allocateColor($color);
 
         if (is_array($color)) {
-            $shape = &new SWFShape();
+            $shape = new SWFShape();
             $shape->setLine($width, $color['red'], $color['green'], $color['blue'], $color['alpha']);
             $shape->movePenTo($x1, $y1);
             $shape->drawLineTo($x2, $y2);
@@ -403,7 +403,7 @@ class Horde_Image_swf extends Horde_Image {
     {
         $color = $this->allocateColor($color);
 
-        $shape = &new SWFShape();
+        $shape = new SWFShape();
         $shape->setLine($width, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 
         $first_done = false;
@@ -431,7 +431,7 @@ class Horde_Image_swf extends Horde_Image {
      */
     function arc($x, $y, $r, $start, $end, $color = 'black', $fill = 'none')
     {
-        $s = &new SWFShape();
+        $s = new SWFShape();
         $color = $this->allocateColor($color);
         $s->setLine(1, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 
@@ -491,7 +491,7 @@ class Horde_Image_swf extends Horde_Image {
      */
     function gradientRectangle($x, $y, $width, $height, $color = 'black', $fill1 = 'black', $fill2 = 'white')
     {
-        $s = &new SWFShape();
+        $s = new SWFShape();
 
         if ($color != 'none') {
             $color = $this->allocateColor($color);
@@ -500,7 +500,7 @@ class Horde_Image_swf extends Horde_Image {
 
         $fill1 = $this->allocateColor($fill1);
         $fill2 = $this->allocateColor($fill2);
-        $gradient = &new SWFGradient();
+        $gradient = new SWFGradient();
         $gradient->addEntry(0.0, $fill1['red'], $fill1['green'], $fill1['blue'], $fill1['alpha']);
         $gradient->addEntry(1.0, $fill2['red'], $fill2['green'], $fill2['blue'], $fill2['alpha']);
 
