@@ -1,6 +1,7 @@
 <?php
 
 require_once dirname(__FILE__) . '/Message.php';
+require_once dirname(__FILE__) . '/Structure.php';
 
 /**
  * The name of the URL parameter that holds the MIME_Contents cache
@@ -28,15 +29,14 @@ define('MIME_CONTENTS_DISPLAY_TYPE_BOTH', 2);
  * The MIME_Contents:: class contains functions related to handling the output
  * of MIME content.
  *
- * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.37 2007/01/02 13:54:24 jan Exp $
+ * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.44 2008/10/06 16:02:09 jan Exp $
  *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (GPL). If you did not
- * receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME
  */
 class MIME_Contents {
@@ -55,6 +55,15 @@ class MIME_Contents {
      */
     var $_cachemessage;
 
+    /**
+     * The MIME part id of the message body.
+     *
+     * @since Horde 3.2
+     *
+     * @var integer
+     */
+    var $_body_id;
+
     /**
      * The attachments list.
      *
@@ -69,6 +78,15 @@ class MIME_Contents {
      */
     var $_parts = array();
 
+    /**
+     * List of all downloadable parts.
+     *
+     * @since Horde 3.2
+     *
+     * @var array
+     */
+    var $_downloads = null;
+
     /**
      * The summary parts list.
      *
@@ -155,6 +173,8 @@ class MIME_Contents {
      */
     function MIME_Contents($messageOb, $viewID = array(), $contents = array())
     {
+        $ptr = array(&$messageOb);
+        MIME_Structure::addMultipartInfo($ptr);
         $this->_message = $messageOb;
         $this->_cachemessage = Util::cloneObject($messageOb);
         $this->_viewID = $viewID;
@@ -188,7 +208,7 @@ class MIME_Contents {
      */
     function getBodyPart($id)
     {
-        if (($part = &$this->getMIMEPart($id))) {
+        if (($part = $this->getMIMEPart($id))) {
             return $part->getContents();
         } else {
             return '';
@@ -214,8 +234,7 @@ class MIME_Contents {
      */
     function &getMIMEPart($id)
     {
-        $part = $this->_message->getPart($id);
-        return $part;
+        return $this->_message->getPart($id);
     }
 
     /**
@@ -239,12 +258,11 @@ class MIME_Contents {
      *
      * @return MIME_Part  The MIME_Part.
      */
-    function &getRawMIMEPart($id, $all = false)
+    function getRawMIMEPart($id, $all = false)
     {
-        $mime_part = &$this->getMIMEPart($id);
+        $mime_part = $this->getMIMEPart($id);
         if (!is_a($mime_part, 'MIME_Part')) {
-            $mime_part = null;
-            return $mime_part;
+            return null;
         }
 
         /* If all text is requested, change the ID now. */
@@ -272,12 +290,12 @@ class MIME_Contents {
      */
     function &getDecodedMIMEPart($id, $all = false)
     {
-        if (($mime_part = &$this->getRawMIMEPart($id, $all))) {
+        if (($mime_part = $this->getRawMIMEPart($id, $all))) {
             $mime_part->transferDecodeContents();
-            // Noop.
         } else {
             $mime_part = null;
         }
+
         return $mime_part;
     }
 
@@ -313,31 +331,30 @@ class MIME_Contents {
         $msgCount = count($this->_parts);
         $partDisplayed = false;
 
-        if ($oneframe) {
-            $msg .= '<style type="text/css">.onefr {padding:1px; margin-bottom:1px; border:1px dashed #00c; background:#eef; color:#006}</style>' . "\n";
-        }
-
         // TODO: Temporary hack to display header info for a message with one
         // MIME part that cannot be displayed inline.
         if (!$msgCount || ($msgCount == 1 && !reset($this->_parts))) {
             $this->setSummary($this->_message, 'part');
             $msgCount = 1;
         }
-            
-        foreach ($this->_parts as $key => $value) {
+
+        ksort($this->_parts);
+
+        reset($this->_parts);
+        while (list($key, $value) = each($this->_parts)) {
             if (isset($this->_summary[$key]) &&
                 ((($msgCount == 1) && empty($value)) || ($msgCount > 1))) {
-                $msg .= '<tr><td>';
+                $msg .= '<tr><td><table class="mimePartInfo">';
                 if ($oneframe) {
-                    $msg .= '<table class="onefr">';
                     $summary = $this->_summary[$key];
                     $summary = array_merge(array_splice($summary, 0, 1), array_splice($summary, 1));
                     $msg .= $this->_arrayToTableRow($summary);
                 } else {
-                    $msg .= '<table>';
                     $msg .= $this->_arrayToTableRow($this->_summary[$key]);
                 }
                 $msg .= '</table></td></tr>';
+            } elseif ($partDisplayed && !empty($value)) {
+                $msg .= '<tr><td class="linedRow"></td></tr>';
             }
             if (!empty($value)) {
                 $msg .= '<tr><td class="text">' . $value . '</td></tr>';
@@ -346,7 +363,7 @@ class MIME_Contents {
         }
 
         if (!$partDisplayed) {
-            $msg .= '<tr><td class="text" align="left"><em>' . _("There are no parts that can be displayed inline.") . '</em></td></tr>';
+            $msg .= '<tr><td class="text"><table class="mimeStatusMessage"><tr><td>' . _("There are no parts that can be displayed inline.") . '</td></tr></table></td></tr>';
         }
 
         return $msg;
@@ -367,7 +384,7 @@ class MIME_Contents {
 
         foreach ($array as $elem) {
             if (!empty($elem)) {
-                $text .= "<td>$elem&nbsp;</td>\n";
+                $text .= "<td>$elem</td>\n";
             }
         }
 
@@ -415,7 +432,7 @@ class MIME_Contents {
      */
     function canDisplayInline(&$mime_part)
     {
-        $viewer = &$this->getMIMEViewer($mime_part);
+        $viewer = $this->getMIMEViewer($mime_part);
         if (!$viewer) {
             return false;
         }
@@ -441,19 +458,22 @@ class MIME_Contents {
      *
      * @return MIME_Viewer  The MIME_Viewer object, or false on error.
      */
-    function &getMIMEViewer(&$mime_part)
+    function &getMIMEViewer(&$mime_part, $mime_type = null)
     {
         /* Make sure we have a MIME_Part to process. */
         if (empty($mime_part)) {
-            $result = false;
-            return $result;
+            $ret = false;
+            return $ret;
         }
 
-        require_once dirname(__FILE__) . '/Viewer.php';
+        if (empty($mime_type)) {
+            $mime_type = $mime_part->getType();
+        }
 
-        $key = $mime_part->getUniqueID() . '|' . $mime_part->getType();
+        $key = $mime_part->getUniqueID() . '|' . $mime_type;
         if (!isset($this->_viewerCache[$key])) {
-            $this->_viewerCache[$key] = &MIME_Viewer::factory($mime_part);
+            require_once dirname(__FILE__) . '/Viewer.php';
+            $this->_viewerCache[$key] = MIME_Viewer::factory($mime_part, $mime_type);
         }
 
         return $this->_viewerCache[$key];
@@ -470,7 +490,7 @@ class MIME_Contents {
      */
     function getMIMEViewerType(&$mime_part)
     {
-        if (($viewer = &$this->getMIMEViewer($mime_part))) {
+        if (($viewer = $this->getMIMEViewer($mime_part))) {
             return $viewer->getType();
         } else {
             return false;
@@ -490,18 +510,19 @@ class MIME_Contents {
      */
     function _getMIMEKey(&$mime_part, $override = true)
     {
-        $id = $this->getMIMEKeyOverride();
-
-        if ($override && !is_null($id)) {
-            return $id;
-        } else {
-            $id = $mime_part->getMIMEId();
-            if (is_null($id)) {
-                return false;
-            } else {
+        if ($override) {
+            $id = $this->getMIMEKeyOverride();
+            if (!is_null($id)) {
                 return $id;
             }
         }
+
+        $id = $mime_part->getMIMEId();
+        if (is_null($id)) {
+            return false;
+        } else {
+            return $id;
+        }
     }
 
     /**
@@ -683,19 +704,18 @@ class MIME_Contents {
             if (!$this->_attach822) {
                 $this->_parts[$key] = $msg;
             }
+
             /* Some MIME_Viewers set the summary by themelves, so only
              * add to attachment/inline lists if nothing has been set
              * as of yet. */
             if ((($mime_part->getType() != 'multipart/mixed') ||
                  !empty($msg)) &&
                 !empty($key) &&
-                !$this->getIndex($key, 'summary')) {
-                $this->setSummary($mime_part, 'part');
-                if ($this->_attach822 &&
-                    (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
-                    ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
-                    $this->setSummary($mime_part, 'attachment');
-                }
+                !$this->getIndex($key, 'summary') &&
+                $this->_attach822 &&
+                (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
+                 ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
+                $this->setSummary($mime_part, 'attachment');
             }
         }
 
@@ -711,7 +731,9 @@ class MIME_Contents {
                 $increment_822 = true;
             }
 
-            foreach ($mime_part->getParts() as $part) {
+            $parts = $mime_part->getParts();
+            reset($parts);
+            while (list(,$part) = each($parts)) {
                 $msg .= $this->buildMessagePart($part);
             }
 
@@ -723,6 +745,141 @@ class MIME_Contents {
         return $msg;
     }
 
+    /**
+     * Generate the list of MIME IDs to use for download all.
+     *
+     * @since Horde 3.2
+     *
+     * @return array  The list of MIME IDs that should be downloaded when
+     *                downloading all attachments.
+     */
+    function getDownloadAllList()
+    {
+        if (is_array($this->_downloads)) {
+            return $this->_downloads;
+        }
+
+        $this->_downloads = array();
+        $bodyid = $this->findBody();
+
+        /* Here is what we consider 'downloadable':
+         * All parts not 'multipart/*' and 'message/*' except for
+         *  'message/rfc822'
+         * All parts that are not PGP or S/MIME signature information
+         * NOT the body part (if one exists)
+         * Parts that are either marked a 'attachment' or have a filename. */
+        foreach ($this->_message->contentTypeMap() as $key => $val) {
+            if ($key === $bodyid) {
+                continue;
+            }
+
+            $mime_part = $this->getMIMEPart($key);
+
+            if (strpos($val, 'message/') === 0) {
+                if (strpos($val, 'message/rfc822') === 0) {
+                    $this->_downloads[] = $key;
+                }
+            } elseif ((($mime_part->getDisposition() == 'attachment') ||
+                       $mime_part->getContentTypeParameter('name')) &&
+                      ($val != 'application/applefile') &&
+                      (strpos($val, 'multipart/') === false) &&
+                      (strpos($val, 'application/x-pkcs7-signature') === false) &&
+                      (strpos($val, 'application/pkcs7-signature') === false)) {
+                $this->_downloads[] = $key;
+            }
+        }
+
+        return $this->_downloads;
+    }
+
+    /**
+     * Returns a list of attachments and their contents.
+     *
+     * @since Horde 3.2
+     *
+     * @return array  List of hashes with the keys 'name' and 'data'.
+     */
+    function getAttachmentContents()
+    {
+        $attachments = array();
+        foreach ($this->getDownloadAllList() as $attachment) {
+            $part = &$this->_message->getPart($attachment);
+            $part->transferDecodeContents();
+            $part_name = $part->getName(true, true);
+            if (empty($part_name)) {
+                $part_name = MIME_DEFAULT_DESCRIPTION;
+            }
+            $attachments[] = array('name' => $part_name,
+                                   'data' => $part->getContents());
+        }
+        return $attachments;
+    }
+
+    /**
+     * Finds the main "body" text part (if any) in a message.
+     * "Body" data is the first text part in the base MIME part.
+     *
+     * @since Horde 3.2
+     *
+     * @param string $subtype  Specifically search for this subtype.
+     *
+     * @return string  The MIME ID of the main "body" part.
+     */
+    function findBody($subtype = null)
+    {
+        if (isset($this->_body_id) && ($subtype === null)) {
+            return $this->_body_id;
+        }
+
+        /* Look for potential body parts. */
+        $part = $this->_message->getBasePart();
+        $primary_type = $part->getPrimaryType();
+        if (($primary_type == MIME::type(TYPEMULTIPART)) ||
+            ($primary_type == MIME::type(TYPETEXT))) {
+            $body_id = $this->_findBody($part, $subtype);
+            if ($subtype !== null) {
+                $this->_body_id = $body_id;
+            }
+            return $body_id;
+        }
+
+        return null;
+    }
+
+    /**
+     * Processes a MIME Part and looks for "body" data.
+     *
+     * @since Horde 3.2
+     *
+     * @access private
+     *
+     * @return string  The MIME ID of the main "body" part.
+     */
+    function _findBody($mime_part, $subtype)
+    {
+        if (intval($mime_part->getMIMEId()) < 2 ||
+            $mime_part->getInformation('alternative') === 0) {
+            if ($mime_part->getPrimaryType() == MIME::type(TYPEMULTIPART)) {
+                $parts = $mime_part->getParts();
+                while ($part = array_shift($parts)) {
+                    if (($partid = $this->_findBody($part, $subtype))) {
+                        return $partid;
+                    }
+                }
+            } elseif ($mime_part->getPrimaryType() == MIME::type(TYPETEXT)) {
+                if ($mime_part->getDisposition() != 'attachment' &&
+                    (($subtype === null) ||
+                     ($subtype == $mime_part->getSubType())) &&
+                    ($mime_part->getBytes() ||
+                     $this->getBodyPart($mime_part->getMIMEId()))) {
+                    return $mime_part->getMIMEId();
+                }
+            }
+        }
+
+        return null;
+    }
+
     /**
      * Are we viewing this page as an attachment through view.php?
      * This method can also be called via MIME_Contents::viewAsAttachment().
@@ -787,93 +944,77 @@ class MIME_Contents {
     {
         $attachment = ($mime_part->getDisposition() == 'attachment');
         $bytes = $mime_part->getBytes();
-        $description = htmlspecialchars($mime_part->getDescription(true, true));
+        $size = $mime_part->getSize(true);
+        $description = $mime_part->getDescription(true, true);
         $summary = array();
-        $type = htmlspecialchars($mime_part->getType());
-        $viewer = &$this->getMIMEViewer($mime_part);
+        $mime_type = $mime_part->getType();
+
+        /* If the MIME Type is application/octet-stream or application/base64,
+           try to use the file extension to determine the actual MIME type. */
+        $param_array = array();
+        if ($this->_links &&
+            !empty($size) &&
+            ($mime_type == 'application/octet-stream') ||
+            ($mime_type == 'application/base64')) {
+            require_once dirname(__FILE__) . '/Magic.php';
+            $mime_type = MIME_Magic::filenameToMIME(MIME::decode($mime_part->getName()));
+            $param_array['ctype'] = $mime_type;
+        }
+
+        $viewer = &$this->getMIMEViewer($mime_part, $mime_type);
         if (!$viewer) {
             return $summary;
         }
 
-        /* Return if we still didn't find a native mime viewer
-         * for a guessed mime type. */
-        if ($guess && is_a($viewer, 'MIME_Viewer_default')) {
-            return $summary;
-        }
-
-        /* Get the MIME id index. */
-        $id = $this->_getMIMEKey($mime_part);
-
         /* Icon column. */
-        $summary[] = Horde::img($viewer->getIcon($type), null, null, '');
+        $summary[] = Horde::img($viewer->getIcon($mime_type), '', array('title' => $mime_type), '');
 
         /* Number column. */
+        $id = $this->_getMIMEKey($mime_part);
         if (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH) &&
             !is_null($this->_summaryType)) {
-            $summary[] = '<a id="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ?'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
+            $summary[] = '<a id="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ? 'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
         } else {
             $summary[] = $id;
         }
 
         /* Name/text part column. */
+        $description = htmlspecialchars($description);
         if (!$this->_links ||
             (!$attachment && empty($bytes))  ||
             !isset($this->_viewID['view']) ||
             is_a($viewer, 'MIME_Viewer_default')) {
             $summary[] = $description;
         } else {
-            $param_array = array();
-            if ($guess === true) {
-                $param_array['ctype'] = $type;
-            }
-            $summary[] = $this->linkViewJS($mime_part, $this->_viewID['view'], $description, null, null, $param_array);
+            $summary[] = $this->linkViewJS($mime_part, $this->_viewID['view'], $description, sprintf(_("View %s [%s]"), $description, $mime_type), null, $param_array);
         }
 
-        /* MIME type column. */
-        if ($guess) {
-            $summary[] = '<em>' . sprintf(_("View as %s"), $type) . '</em>';
-        } else {
-            $summary[] = '[' . $type . ']';
-        }
-
-        /* Size Column. */
-        $size = $mime_part->getSize(true);
+        /* Size. */
         if (!empty($bytes) &&
             ($mime_part->getCurrentEncoding() == 'base64')) {
             /* From RFC 2045 [6.8]: "...the encoded data are consistently
                only about 33 percent larger than the unencoded data." */
             $size = number_format(max((($bytes * 0.75) / 1024), 1));
-            $summary[] = '<acronym title="' . _("Approximate Size") . '">' . sprintf(_("%s KB"), $size) . '</acronym>';
+            if ($size > 1024) {
+                $size = sprintf(_("%s MB"), number_format(max(($size / 1024), 1)));
+            } else {
+                $size = sprintf(_("%s KB"), $size);
+            }
         } else {
-            $summary[] = sprintf(_("%s KB"), $size);
+            if ($size > 1024) {
+                $size = sprintf(_("%s MB"), number_format(max(($size / 1024), 1)));
+            } else {
+                $size = sprintf(_("%s KB"), $size);
+            }
         }
 
         /* Download column. */
         if (!$this->_links ||
             is_null($size) ||
             !isset($this->_viewID['download'])) {
-            $summary[] = null;
+            $summary[] = '<span class="download">' . $size . '</span>';
         } else {
-            $summary[] = $this->linkView($mime_part, $this->_viewID['download'], Horde::img('download.png', _("Download"), null, $GLOBALS['registry']->getImageDir('horde')), array('jstext' => sprintf(_("Download %s"), $mime_part->getDescription(true, true))), true);
-        }
-
-        /* If the MIME Type is application/octet-stream, try to use
-           the file extension to determine the actual MIME type. */
-        if ($this->_links &&
-            !$guess &&
-            !is_null($size) &&
-            ($type == 'application/octet-stream') ||
-            ($type == 'application/base64')) {
-            require_once dirname(__FILE__) . '/Magic.php';
-            $new_type = MIME_Magic::filenameToMIME(MIME::decode($mime_part->getName()));
-            if ($new_type != 'application/octet-stream') {
-                $temp_part = $mime_part;
-                $temp_part->setType($new_type);
-                $summary = array_map(
-                    create_function('$a, $b', 'return empty($b) ? $a : $a . \'<br />\' . $b;'),
-                    $summary,
-                    $this->partSummary($temp_part, true));
-            }
+            $summary[] = $this->linkView($mime_part, $this->_viewID['download'], $size, array('class' => 'download', 'jstext' => sprintf(_("Download %s"), $description)), true);
         }
 
         return $summary;
@@ -913,9 +1054,12 @@ class MIME_Contents {
     /**
      * Generate a link to the view.php page.
      *
+     * Important: the calling code has to make sure that the $text parameter
+     * is properly escaped!
+     *
      * @param MIME_Part &$mime_part  The MIME_Part object to view.
      * @param integer $actionID      The actionID value.
-     * @param string $text           The link text.
+     * @param string $text           The ESCAPED link text.
      * @param array $params          A list of additional parameters.
      *   'class'       -  The CSS class to use.
      *   'jstext'      -  The JS text to use.
@@ -950,9 +1094,12 @@ class MIME_Contents {
     /**
      * Generate a javascript link to the view.php page.
      *
+     * Important: the calling code has to make sure that the $text parameter
+     * is properly escaped!
+     *
      * @param MIME_Part &$mime_part  The MIME_Part object to view.
      * @param integer $actionID      The ActionID to perform.
-     * @param string $text           The link text.
+     * @param string $text           The ESCAPED link text.
      * @param string $jstext         The Javascript link text.
      * @param string $css            The CSS class to use.
      * @param array $params          A list of any additional parameters that
@@ -1003,6 +1150,10 @@ class MIME_Contents {
     function formatStatusMsg($msg, $img = null, $printable = true,
                              $class = null)
     {
+        if (empty($msg)) {
+            return '';
+        }
+
         if (!is_array($msg)) {
             $msg = array($msg);
         }
@@ -1023,9 +1174,7 @@ class MIME_Contents {
                 $text .= '<tr><td>' . $val . '</td></tr>' . "\n";
             }
         } else {
-            $text .= '<tr><td valign="middle">' . $img . '&nbsp;</td>';
-            $text .= '<td>';
-
+            $text .= '<tr><td class="mimeStatusIcon">' . $img . '</td><td>';
             if (count($msg) == 1) {
                 $text .= $msg[0];
             } else {
@@ -1049,7 +1198,7 @@ class MIME_Contents {
      */
     function &getBaseObjectPtr()
     {
-        if (is_null($this->_base)) {
+        if ($this->_base === null) {
             return $this;
         } else {
             return $this->_base;
@@ -1082,7 +1231,15 @@ class MIME_Contents {
      */
     function _createCacheID()
     {
-        return md5(microtime());
+        // Use Auth class, if available, to add uniqueness as it prevents us
+        // from having to call mt_rand(), a relatively expensive call.
+        if (class_exists('Auth')) {
+            $entropy = Auth::getAuth();
+        }
+        if (empty($entropy)) {
+            $entropy = mt_rand();
+        }
+        return md5(microtime() . $entropy . getmypid());
     }
 
     /**
@@ -1092,9 +1249,6 @@ class MIME_Contents {
      */
     function _addCacheShutdown()
     {
-        require_once 'Horde/SessionObjects.php';
-        $cache = &Horde_SessionObjects::singleton();
-
         /* Don't save the MIME_Viewer cached objects since there is no easy
            way to regenerate them on cache reload. */
         $this->_viewerCache = array();
@@ -1102,7 +1256,15 @@ class MIME_Contents {
         /* Copy the MIME_Message cache object to the _message variable. */
         $this->_message = $this->_cachemessage;
 
-        $cache->store($this->_sessionCacheID, $this);
+        if (!empty($GLOBALS['conf']['cache']['driver'])) {
+            require_once 'Horde/Cache.php';
+            $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
+            $cache->set($this->_sessionCacheID, @serialize($this));
+        } else {
+            require_once 'Horde/SessionObjects.php';
+            $cache = &Horde_SessionObjects::singleton();
+            $cache->overwrite($this->_sessionCacheID, $this);
+        }
     }
 
     /**
@@ -1123,9 +1285,16 @@ class MIME_Contents {
             $cacheid = Util::getFormData(MIME_CONTENTS_CACHE);
         }
 
-        require_once 'Horde/SessionObjects.php';
-        $cache = &Horde_SessionObjects::singleton();
-        $contents = &$cache->query($cacheid);
+        if (!empty($GLOBALS['conf']['cache']['driver'])) {
+            require_once 'Horde/Cache.php';
+            $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
+            $contents = @unserialize($cache->get($cacheid, 0));
+        } else {
+            require_once 'Horde/SessionObjects.php';
+            $cache = &Horde_SessionObjects::singleton();
+            $contents = &$cache->query($cacheid);
+        }
+
         return $contents;
     }
 
