@@ -2,7 +2,7 @@
 /**
  * The description of Horde to use in the 'User-Agent:' header.
  */
-define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.1');
+define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.2');
 
 /**
  * The MIME_Headers:: class contains generic functions related to
@@ -12,15 +12,14 @@ define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.1');
  * variable $GLOBALS['mime_headers']['default_charset'] (defaults to US-ASCII
  * per RFC 2045).
  *
- * $Horde: framework/MIME/MIME/Headers.php,v 1.29.10.22 2007/01/02 13:54:25 jan Exp $
+ * $Horde: framework/MIME/MIME/Headers.php,v 1.29.10.27 2008/07/03 06:37:26 slusarz Exp $
  *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME
  */
 class MIME_Headers {
@@ -53,13 +52,6 @@ class MIME_Headers {
      */
     var $_headerObject;
 
-    /**
-     * The internal flags array.
-     *
-     * @var array
-     */
-    var $_flags = array();
-
     /**
      * The User-Agent string to use.
      * THIS VALUE SHOULD BE OVERRIDEN BY ALL SUBCLASSES.
@@ -89,7 +81,7 @@ class MIME_Headers {
     /**
      * Constructor.
      *
-     * @param integer $index  The message index to parse headers.
+     * @param integer $index  The message index to parse headers (DEPRECATED).
      */
     function MIME_Headers($index = null)
     {
@@ -116,8 +108,9 @@ class MIME_Headers {
     {
         if (!is_null($this->_index) && empty($this->_headerText)) {
             $this->_headerText = @imap_fetchheader($this->_getStream(), $this->_index, FT_UID);
-            if (!empty($GLOBALS['mime_headers']['default_charset'])) {
-                $this->_headerText = String::convertCharset($this->_headerText, $GLOBALS['mime_headers']['default_charset']);
+            require_once 'Horde/MIME.php';
+            if (MIME::is8bit($this->_headerText)) {
+                $this->_headerText = String::convertCharset($this->_headerText, !empty($GLOBALS['mime_headers']['default_charset']) ? $GLOBALS['mime_headers']['default_charset'] : 'US-ASCII');
             }
         }
 
@@ -133,30 +126,14 @@ class MIME_Headers {
      */
     function getAllHeaders($decode = true)
     {
-        require_once 'Horde/MIME/Structure.php';
-
         if (!is_null($this->_index) && empty($this->_allHeaders)) {
+            require_once 'Horde/MIME/Structure.php';
             $this->_allHeaders = MIME_Structure::parseMIMEHeaders($this->getHeaderText(), $decode);
         }
 
         return $this->_allHeaders;
     }
 
-    /**
-     * Return the header object from imap_headerinfo().
-     *
-     * @return stdClass  See imap_headerinfo().
-     */
-    function getHeaderObject()
-    {
-        if (!is_null($this->_index) && empty($this->_headerObject)) {
-            $stream = $this->_getStream();
-            $this->_headerObject = @imap_headerinfo($stream, @imap_msgno($stream, $this->_index));
-        }
-
-        return $this->_headerObject;
-    }
-
     /**
      * Build the header array.
      *
@@ -164,59 +141,10 @@ class MIME_Headers {
      */
     function buildHeaders($decode = true)
     {
-        if (!empty($this->_headers)) {
-            return;
-        }
-
-        /* Parse through the list of all headers. */
-        foreach ($this->getAllHeaders($decode) as $key => $val) {
-            $this->addHeader($key, $val);
-        }
-    }
-
-    /**
-     * Build the flags array.
-     */
-    function buildFlags()
-    {
-        if (!empty($this->_flags)) {
-            return;
-        }
-
-        /* Get the IMAP header object. */
-        $ob = $this->getHeaderObject();
-        if (!is_object($ob)) {
-            return;
-        }
-
-        /* Unseen flag */
-        if (($ob->Unseen == 'U') || ($ob->Recent == 'N')) {
-            $this->_flags['unseen'] = true;
-        }
-
-        /* Recent flag */
-        if (($ob->Recent == 'N') || ($ob->Recent == 'R')) {
-            $this->_flags['recent'] = true;
-        }
-
-        /* Answered flag */
-        if ($ob->Answered == 'A') {
-            $this->_flags['answered'] = true;
-        }
-
-        /* Draft flag */
-        if (isset($ob->Draft) && ($ob->Draft == 'X')) {
-            $this->_flags['draft'] = true;
-        }
-
-        /* Flagged flag */
-        if ($ob->Flagged == 'F') {
-            $this->_flags['flagged'] = true;
-        }
-
-        /* Deleted flag */
-        if ($ob->Deleted == 'D') {
-            $this->_flags['deleted'] = true;
+        if (empty($this->_headers)) {
+            foreach ($this->getAllHeaders($decode) as $key => $val) {
+                $this->addHeader($key, $val);
+            }
         }
     }
 
@@ -252,7 +180,6 @@ class MIME_Headers {
      */
     function toString()
     {
-        $eol = $this->getEOL();
         $text = '';
 
         foreach ($this->_headers as $ob) {
@@ -260,11 +187,11 @@ class MIME_Headers {
                 $ob['value'] = array($ob['value']);
             }
             foreach ($ob['value'] as $entry) {
-                $text .= $ob['header'] . ': ' . $entry . $eol;
+                $text .= $ob['header'] . ': ' . $entry . $this->_eol;
             }
         }
 
-        return $text . $eol;
+        return $text . $this->_eol;
     }
 
     /**
@@ -304,7 +231,7 @@ class MIME_Headers {
         } else {
             $server_name = 'unknown';
         }
-        $received .= 'by ' . $server_name . ' (Horde MIME library) with HTTP; ';
+        $received .= 'by ' . $server_name . ' (Horde Framework) with HTTP; ';
 
         $received .= date('r');
 
@@ -316,7 +243,7 @@ class MIME_Headers {
      */
     function addMessageIdHeader()
     {
-        require_once dirname(__FILE__) . '/../MIME.php';
+        require_once 'Horde/MIME.php';
         $this->addHeader('Message-ID', MIME::generateMessageID());
     }
 
@@ -329,7 +256,7 @@ class MIME_Headers {
      */
     function addResentHeaders($from, $to)
     {
-        require_once dirname(__FILE__) . '/../MIME.php';
+        require_once 'Horde/MIME.php';
 
         /* We don't set Resent-Sender, Resent-Cc, or Resent-Bcc. */
         $this->addHeader('Resent-Date', date('r'));
@@ -338,17 +265,6 @@ class MIME_Headers {
         $this->addHeader('Resent-Message-ID', MIME::generateMessageID());
     }
 
-    /**
-     * Generate delivery receipt headers.
-     *
-     * @param string $to  The address the receipt should be mailed to.
-     */
-    function addDeliveryReceiptHeaders($to)
-    {
-        /* This is old sendmail (pre-8.7) behavior. */
-        $this->addHeader('Return-Receipt-To', $to);
-    }
-
     /**
      * Generate the user agent description header.
      */
@@ -530,6 +446,7 @@ class MIME_Headers {
      * <pre>
      * Object elements:
      * 'address'   -  Full address
+     * 'display'   -  A displayable version of the address (Horde 3.2.1+)
      * 'host'      -  Host name
      * 'inner'     -  Trimmed, bare address
      * 'personal'  -  Personal string
@@ -545,15 +462,7 @@ class MIME_Headers {
 
         foreach ($obs as $ob) {
             /* Ensure we're working with initialized values. */
-            if (isset($ob->personal)) {
-                $ob->personal = MIME::decode($ob->personal);
-                if ((substr($ob->personal, 0, 1) == '"') &&
-                    (substr($ob->personal, -1) == '"')) {
-                    $ob->personal = substr($ob->personal, 1, -1);
-                }
-            } else {
-                $ob->personal = '';
-            }
+             $ob->personal = (isset($ob->personal)) ? stripslashes(trim(MIME::decode($ob->personal), '"')) : '';
 
             if (isset($ob->mailbox)) {
                 /* Don't process invalid addresses. */
@@ -569,11 +478,14 @@ class MIME_Headers {
                 $ob->host = '';
             }
 
+            $inner = MIME::trimEmailAddress(MIME::rfc822WriteAddress($ob->mailbox, $ob->host, ''));
+
             /* Generate the new object. */
             $newOb = &new stdClass;
             $newOb->address = MIME::addrObject2String($ob, array('undisclosed-recipients@', 'Undisclosed recipients@'));
+            $newOb->display = (empty($ob->personal) ? '' : $ob->personal . ' <') . $inner . (empty($ob->personal) ? '' : '>');
             $newOb->host = $ob->host;
-            $newOb->inner = MIME::trimEmailAddress(MIME::rfc822WriteAddress($ob->mailbox, $ob->host, ''));
+            $newOb->inner = $inner;
             $newOb->personal = $ob->personal;
 
             $retArray[] = &$newOb;
@@ -609,13 +521,7 @@ class MIME_Headers {
      */
     function listHeadersExist()
     {
-        foreach ($this->listHeaders() as $val => $str) {
-            if (isset($this->_headers[$val])) {
-                return true;
-            }
-        }
-
-        return false;
+        return (bool) count(array_intersect(array_keys($this->listHeaders()), array_keys($this->_headers)));
     }
 
     /**
@@ -638,24 +544,6 @@ class MIME_Headers {
         return $this->_eol;
     }
 
-    /**
-     * Returns the flag status.
-     *
-     * @param string $flag  Is this flag set?
-     *                      Flags: recent, unseen, answered, draft, important,
-     *                             deleted
-     *
-     * @return boolean  True if the flag has been set, false if not.
-     */
-    function getFlag($flag)
-    {
-        if (!empty($this->_flags[String::lower($flag)])) {
-            return true;
-        } else {
-            return false;
-        }
-    }
-
     /**
      * Get the primary from address (first address in the From: header).
      *
@@ -668,7 +556,6 @@ class MIME_Headers {
         }
 
         require_once 'Horde/MIME.php';
-
         return trim(MIME::trimEmailAddress(MIME::rfc822WriteAddress($ob[0]->mailbox, (isset($ob[0]->host)) ? $ob[0]->host : '', '')));
     }
 
@@ -697,9 +584,9 @@ class MIME_Headers {
         if (isset($ob->$field)) {
             $data = $ob->$field;
             if (!empty($decode) && !is_object($data) && !is_array($data)) {
-                include_once 'Horde/MIME.php';
-                if (!empty($GLOBALS['mime_headers']['default_charset'])) {
-                    $data = String::convertCharset($data, $GLOBALS['mime_headers']['default_charset']);
+                require_once 'Horde/MIME.php';
+                if (MIME::is8bit($this->_headerText)) {
+                    $data = String::convertCharset($data, !empty($GLOBALS['mime_headers']['default_charset']) ? $GLOBALS['mime_headers']['default_charset'] : 'US-ASCII');
                 }
                 $data = MIME::decode($data);
             }
@@ -708,4 +595,112 @@ class MIME_Headers {
         return (is_string($data)) ? strtr($data, "\t", ' ') : $data;
     }
 
+    /* Deprecated functions. */
+
+    /**
+     * The internal flags array.
+     *
+     * @deprecated since Horde 3.2
+     * @var array
+     */
+    var $_flags = array();
+
+    /**
+     * Build the flags array.
+     *
+     * @deprecated since Horde 3.2
+     */
+    function buildFlags()
+    {
+        if (!empty($this->_flags)) {
+            return;
+        }
+
+        /* Get the IMAP header object. */
+        $ob = $this->getHeaderObject();
+        if (!is_object($ob)) {
+            return;
+        }
+
+        /* Unseen flag */
+        if (($ob->Unseen == 'U') || ($ob->Recent == 'N')) {
+            $this->_flags['unseen'] = true;
+        }
+
+        /* Recent flag */
+        if (($ob->Recent == 'N') || ($ob->Recent == 'R')) {
+            $this->_flags['recent'] = true;
+        }
+
+        /* Answered flag */
+        if ($ob->Answered == 'A') {
+            $this->_flags['answered'] = true;
+        }
+
+        /* Draft flag */
+        if (isset($ob->Draft) && ($ob->Draft == 'X')) {
+            $this->_flags['draft'] = true;
+        }
+
+        /* Flagged flag */
+        if ($ob->Flagged == 'F') {
+            $this->_flags['flagged'] = true;
+        }
+
+        /* Deleted flag */
+        if ($ob->Deleted == 'D') {
+            $this->_flags['deleted'] = true;
+        }
+    }
+
+    /**
+     * Returns the flag status.
+     *
+     * @deprecated since Horde 3.2
+     *
+     * @param string $flag  Is this flag set?
+     *                      Flags: recent, unseen, answered, draft, important,
+     *                             deleted
+     *
+     * @return boolean  True if the flag has been set, false if not.
+     */
+    function getFlag($flag)
+    {
+        if (!empty($this->_flags[String::lower($flag)])) {
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Return the header object from imap_headerinfo().
+     *
+     * @deprecated since Horde 3.2
+     *
+     * @return stdClass  See imap_headerinfo().
+     */
+    function getHeaderObject()
+    {
+        if (!is_null($this->_index) && empty($this->_headerObject)) {
+            $stream = $this->_getStream();
+            $this->_headerObject = @imap_headerinfo($stream, @imap_msgno($stream, $this->_index));
+        }
+
+        return $this->_headerObject;
+    }
+
+    /**
+     * Generate delivery receipt headers.
+     *
+     * @deprecated since Horde 3.2
+     *
+     * @param string $to  The address the receipt should be mailed to.
+     */
+    function addDeliveryReceiptHeaders($to)
+    {
+        /* This is old sendmail (pre-8.7) behavior. */
+        $this->addHeader('Return-Receipt-To', $to);
+    }
+
 }
