@@ -40,17 +40,15 @@ define('MIME_DEFAULT_ENCODING', '7bit');
  * The MIME_Part:: class provides a wrapper around MIME parts and methods
  * for dealing with them.
  *
- * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.8 2005/07/11 17:10:15 jan Exp $
+ * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.25 2008/12/03 06:48:58 slusarz Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 1.3
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME
  */
 class MIME_Part {
@@ -261,8 +259,10 @@ class MIME_Part {
 
     /**
      * Set the name of this part.
+     * TODO: MIME encode here instead of in header() - add a charset
+     * parameter.
      *
-     * @param string $name The name to set.
+     * @param string $name  The name to set.
      */
     function setName($name)
     {
@@ -619,20 +619,37 @@ class MIME_Part {
      */
     function getPart($id)
     {
-        $orig = null;
         $mimeid = $this->getMIMEId();
 
+        /* This will convert '#.0' to simply '#', which is how the part is
+         * internally stored. */
+        $search_id = $id;
+        if (($str = strrchr($id, '.')) &&
+            ($str == '.0')) {
+            $search_id = substr($search_id, 0, -2);
+        }
+
         /* Return this part if:
            1) There is only one part (e.g. the MIME ID is 0, or the
               MIME ID is 1 and there are no subparts.
            2) $id matches this parts MIME ID. */
-        if (($id == 0) ||
-            (($id == 1) && !count($this->_parts)) ||
-            (!empty($mimeid) && ($id == $mimeid))) {
-            return $this;
+        if (($search_id == 0) ||
+            (($search_id == 1) && !count($this->_parts)) ||
+            (!empty($mimeid) && ($search_id == $mimeid))) {
+            $part = $this;
+        } else {
+            $part = $this->_partFind($id, $this->_parts);
         }
 
-        return $this->_partFind($id, $this->_parts);
+        if ($part &&
+            ($search_id != $id) &&
+            ($part->getType() == 'message/rfc822')) {
+            $ret_part = Util::cloneObject($part);
+            $ret_part->_parts = array();
+            return $ret_part;
+        }
+
+        return $part;
     }
 
     /**
@@ -855,7 +872,6 @@ class MIME_Part {
     {
         $eol = $this->getEOL();
         $ptype = $this->getPrimaryType();
-        $stype = $this->getSubType();
 
         /* Get the character set for this part. */
         $charset = $this->getCharset();
@@ -868,7 +884,16 @@ class MIME_Part {
             if ($key == 'charset') {
                 continue;
             }
-            $ctype .= '; ' . $key . '="' . MIME::encode($value, $charset) . '"';
+            $encode_2231 = MIME::encodeRFC2231($key, $value, $charset);
+            /* Try to work around non RFC 2231-compliant MUAs by sending both
+             * a RFC 2047-like parameter name and then the correct RFC 2231
+             * parameter.  See:
+             *   http://lists.horde.org/archives/dev/Week-of-Mon-20040426/014240.html */
+            if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
+                (strpos($encode_2231, '*=') !== false)) {
+                $ctype .= '; ' . $key . '="' . MIME::encode($value, $charset) . '"';
+            }
+            $ctype .= '; ' . $encode_2231;
         }
         $headers['Content-Type'] = MIME::wrapHeaders('Content-Type', $ctype, $eol);
 
@@ -877,15 +902,6 @@ class MIME_Part {
             $headers['Content-Description'] = MIME::wrapHeaders('Content-Description', MIME::encode($descrip, $charset), $eol);
         }
 
-        /* RFC 2045 [4] - message/rfc822 and message/partial require the
-           MIME-Version header only if they themselves claim to be MIME
-           compliant. */
-        if (($ptype == 'message') &&
-            (($stype == 'rfc822') || ($stype == 'partial')) &&
-            (strpos($this->_contents, 'MIME-Version: 1.0') !== false)) {
-            $headers['MIME-Version'] = '1.0';
-        }
-
         /* message/* parts require no additional header information. */
         if ($ptype == 'message') {
             return $headers;
@@ -899,7 +915,13 @@ class MIME_Part {
 
             /* Add any disposition parameter information, if available. */
             if (!empty($name)) {
-                $disp .= '; ' . 'filename="' . MIME::encode($name, $charset) . '"';
+                $encode_2231 = MIME::encodeRFC2231('filename', $name, $charset);
+                /* Same broken RFC 2231 workaround as above. */
+                if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
+                    (strpos($encode_2231, '*=') !== false)) {
+                    $disp .= '; filename="' . MIME::encode($name, $charset) . '"';
+                }
+                $disp .= '; ' . $encode_2231;
             }
 
             $headers['Content-Disposition'] = MIME::wrapHeaders('Content-Disposition', $disp, $eol);
@@ -959,7 +981,8 @@ class MIME_Part {
             if (!strlen($this->_contents)) {
                 $text .= 'This message is in MIME format.' . $eol;
             }
-            foreach ($this->getParts() as $part) {
+            reset($this->_parts);
+            while (list(,$part) = each($this->_parts)) {
                 $text .= $eol . '--' . $boundary . $eol;
                 $oldEOL = $part->getEOL();
                 $part->setEOL($eol);
@@ -1022,36 +1045,38 @@ class MIME_Part {
                base64 or q-p, switch it to 8bit instead.
                RFC 2046 [5.2.2, 5.2.3, 5.2.4] - All other message/* messages
                only allow 7bit encodings. */
-            if ($this->getSubType() == 'rfc822') {
-                $encoding = '8bit';
-            } else {
-                $encoding = '7bit';
-            }
+            $encoding = ($this->getSubType() == 'rfc822') ? '8bit' : '7bit';
             break;
 
         case 'text':
+            $eol = $this->getEOL();
             if (MIME::is8bit($text)) {
-                if ($this->_encode7bit) {
-                    $encoding = 'quoted-printable';
-                } else {
-                    $encoding = '8bit';
-                }
+                $encoding = ($this->_encode7bit) ? 'quoted-printable' : '8bit';
+            } elseif (preg_match("/(?:" . $eol . "|^)[^" . $eol . "]{999,}(?:" . $eol . "|$)/", $this->_contents)) {
+                /* If the text is longer than 998 characters between
+                 * linebreaks, use quoted-printable encoding to ensure the
+                 * text will not be chopped (i.e. by sendmail if being sent
+                 * as mail text). */
+                $encoding = 'quoted-printable';
             }
             break;
 
         default:
-            /* By default, if there is 8bit data, we MUST either 8bit or
-               base64 encode contents. */
             if (MIME::is8bit($text)) {
-                if ($this->_encode7bit) {
-                    $encoding = 'base64';
-                } else {
-                    $encoding = '8bit';
-                }
+                $encoding = ($this->_encode7bit) ? 'base64' : '8bit';
             }
             break;
         }
 
+        /* Need to do one last check for binary data if encoding is 7bit or
+         * 8bit.  If the message contains a NULL character at all, the message
+         * MUST be in binary format. RFC 2046 [2.7, 2.8, 2.9]. Q-P and base64
+         * can handle binary data fine so no need to switch those encodings. */
+        if ((($encoding == '8bit') || ($encoding == '7bit')) &&
+            preg_match('/\x00/', $text)) {
+            $encoding = ($this->_encode7bit) ? 'base64' : 'binary';
+        }
+
         return $encoding;
     }
 
@@ -1097,7 +1122,8 @@ class MIME_Part {
         /* Quoted-Printable Encoding: See RFC 2045, section 6.7 */
         case 'quoted-printable':
             $output = MIME::quotedPrintableEncode($this->_contents, $eol);
-            if (($eollength = String::length($eol))) {
+            if (($eollength = String::length($eol)) &&
+                (substr($output, $eollength * -1) == $eol)) {
                 return substr($output, 0, $eollength * -1);
             } else {
                 return $output;
@@ -1142,9 +1168,14 @@ class MIME_Part {
         case 'uuencode':
         case 'x-uuencode':
         case 'x-uue':
-            require_once 'Mail/mimeDecode.php';
-            $files = &Mail_mimeDecode::uudecode($this->_contents);
-            $message = $files[0]['filedata'];
+            if (function_exists('convert_uudecode')) {
+                $message = convert_uuencode($this->_contents);
+            } else {
+                // TODO: Remove once PHP5 is required
+                require_once 'Mail/mimeDecode.php';
+                $files = &Mail_mimeDecode::uudecode($this->_contents);
+                $message = $files[0]['filedata'];
+            }
             $this->_flags['lastTransferDecode'] = '8bit';
             break;
 
@@ -1186,28 +1217,40 @@ class MIME_Part {
         $eol = $this->getEOL();
         $retvalue = false;
 
-        foreach (explode($eol, $this->_contents) as $line) {
-            $pos = strpos($line, '--' . $boundary);
-            if (($pos === false) && isset($part_ptr)) {
-                $data[] = $line;
-            } elseif ($pos === 0) {
-                $retvalue = true;
-                if (isset($part_ptr)) {
-                    $this->_parts[$part_ptr]->setContents(implode($eol, $data));
-                    $this->_parts[$part_ptr]->splitContents();
-                    next($this->_parts);
-                } else {
-                    reset($this->_parts);
-                }
-                if (isset($data)) {
-                    unset($data);
-                }
-                $data = array();
-                $part_ptr = key($this->_parts);
+        $boundary = '--' . $boundary;
+        if (substr($this->_contents, 0, strlen($boundary)) == $boundary) {
+            $pos1 = 0;
+        } else {
+            $pos1 = strpos($this->_contents, $eol . $boundary);
+        }
+        if ($pos1 === false) {
+            return false;
+        }
+        $pos1 = strpos($this->_contents, $eol, $pos1 + 1);
+        if ($pos1 === false) {
+            return false;
+        }
+        $pos1 += strlen($eol);
+
+        reset($this->_parts);
+        $part_ptr = key($this->_parts);
+
+        while ($pos2 = strpos($this->_contents, $eol . $boundary, $pos1)) {
+            $this->_parts[$part_ptr]->setContents(substr($this->_contents, $pos1, $pos2 - $pos1));
+            $this->_parts[$part_ptr]->splitContents();
+            next($this->_parts);
+            $part_ptr = key($this->_parts);
+            if (is_null($part_ptr)) {
+                return false;
             }
+            $pos1 = strpos($this->_contents, $eol, $pos2 + 1);
+            if ($pos1 === false) {
+                return true;
+            }
+            $pos1 += strlen($eol);
         }
 
-        return $retvalue;
+        return true;
     }
 
     /**
@@ -1254,7 +1297,8 @@ class MIME_Part {
         if (empty($this->_flags['contentsSet']) && $this->_bytes) {
             $bytes = $this->_bytes;
         } elseif ($this->getPrimaryType() == 'multipart') {
-            foreach ($this->getParts() as $part) {
+            reset($this->_parts);
+            while (list(,$part) = each($this->_parts)) {
                 /* Skip multipart entries (since this may result in double
                    counting). */
                 if ($part->getPrimaryType() != 'multipart') {
@@ -1340,6 +1384,16 @@ class MIME_Part {
         return $this->_contentid;
     }
 
+    /**
+     * Returns the Content-ID for this part.
+     *
+     * @return string  The Content-ID for this part.
+     */
+    function getContentID()
+    {
+        return $this->_contentid;
+    }
+
     /**
      * Alter the MIME ID of this part.
      *
