@@ -3,15 +3,14 @@
  * The MIME_Viewer_rfc822 class renders out messages from the
  * message/rfc822 content type.
  *
- * $Horde: framework/MIME/MIME/Viewer/rfc822.php,v 1.8.10.10 2007/03/01 07:10:30 slusarz Exp $
+ * $Horde: framework/MIME/MIME/Viewer/rfc822.php,v 1.8.10.14 2008/06/03 18:08:51 slusarz Exp $
  *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME_Viewer
  */
 class MIME_Viewer_rfc822 extends MIME_Viewer {
@@ -25,13 +24,18 @@ class MIME_Viewer_rfc822 extends MIME_Viewer {
      */
     function render($params = array())
     {
-        $part = $this->mime_part;
-        $part->transferDecodeContents();
+        if (!$this->mime_part) {
+            require_once 'Horde/MIME/Contents.php';
+            $contents = new MIME_Contents(new MIME_Part());
+            return $contents->formatStatusMsg(_("There was an error displaying this message part"));
+        }
 
+        $part =& Util::cloneObject($this->mime_part);
+        $part->transferDecodeContents();
         $text = $part->getContents();
         if (!$text) {
             require_once 'Horde/MIME/Contents.php';
-            $contents = &new MIME_Contents(new MIME_Part());
+            $contents = new MIME_Contents(new MIME_Part());
             return $contents->formatStatusMsg(_("There was an error displaying this message part"));
         } else {
             return $text;
@@ -47,13 +51,18 @@ class MIME_Viewer_rfc822 extends MIME_Viewer {
      */
     function renderAttachmentInfo($params = array())
     {
-        $msg = '';
+        if (!$this->mime_part) {
+            return '';
+        }
 
         /* Get the text of the part.  Since we need to look for the end of
          * the headers by searching for the CRLFCRLF sequence, use
          * getCanonicalContents() to make sure we are getting the text with
          * CRLF's. */
         $text = $this->mime_part->getCanonicalContents();
+        if (empty($text)) {
+            return '';
+        }
 
         /* Search for the end of the header text (CRLFCRLF). */
         $text = substr($text, 0, strpos($text, "\r\n\r\n"));
@@ -62,17 +71,14 @@ class MIME_Viewer_rfc822 extends MIME_Viewer {
         require_once 'Horde/MIME/Structure.php';
         $headers = MIME_Structure::parseMIMEHeaders($text, true, true);
 
-        require_once 'Horde/MIME/Contents.php';
-        $contents = &new MIME_Contents(new MIME_Part());
-        $msg = $contents->formatStatusMsg(_("The following are the headers for this message/rfc822 message."), Horde::img('info_icon.png', _("Info"), 'height="16" width="16"', $GLOBALS['registry']->getImageDir('horde')), false);
-
-        $msg .= '<span class="fixed">';
-
         $header_array = array(
             'date' => _("Date"),
-            'subject' => _("Subject"),
             'from' => _("From"),
-            'to' => _("To")
+            'to' => _("To"),
+            'cc' => _("Cc"),
+            'bcc' => _("Bcc"),
+            'reply-to' => _("Reply-To"),
+            'subject' => _("Subject")
         );
         $header_output = array();
 
@@ -86,9 +92,7 @@ class MIME_Viewer_rfc822 extends MIME_Viewer {
         }
 
         require_once 'Horde/Text/Filter.php';
-        $msg .= implode("<br />\n", Text_Filter::filter($header_output, 'emails')) . '</span>';
-
-        return $msg;
+        return '<div class="mimeHeaders">' . Text_Filter::filter(implode("<br />\n", $header_output), 'emails') . '</div>';
     }
 
     /**
