@@ -9,6 +9,7 @@ class Horde_Mobile_card extends Horde_Mobile_element {
     var $_form;
     var $_elements = array();
     var $_softkeys = array();
+    var $_linksetAdded = false;
 
     /**
      * Constructor
@@ -30,13 +31,10 @@ class Horde_Mobile_card extends Horde_Mobile_element {
 
     function &add(&$element)
     {
-        static $linksetAdded;
-
         if (!is_a($element, 'Horde_Mobile_element')) {
             $error = PEAR::raiseError('Invalid element.');
             return $error;
         } elseif (is_a($element, 'Horde_Mobile_text') ||
-                  is_a($element, 'Horde_Mobile_table') ||
                   is_a($element, 'Horde_Mobile_image') ||
                   is_a($element, 'Horde_Mobile_link') ||
                   is_a($element, 'Horde_Mobile_phone') ||
@@ -53,13 +51,13 @@ class Horde_Mobile_card extends Horde_Mobile_element {
             $this->_elements[] = &$element;
             $this->_form = &$element;
         } elseif (is_a($element, 'Horde_Mobile_linkset')) {
-            if (!empty($linksetAdded)) {
+            if ($this->_linksetAdded) {
                 $error = PEAR::raiseError('Cards may only contain one Linkset element.');
                 return $error;
             }
-            $block = &new Horde_Mobile_block($element);
-            $this->_elements[] = &$block;
-            $linksetAdded = true;
+
+            $this->_elements[] = &$element;
+            $this->_linksetAdded = true;
         } else {
             $error = PEAR::raiseError('This element must be inside an appropriate container element.');
             return $error;
@@ -89,7 +87,7 @@ class Horde_Mobile_card extends Horde_Mobile_element {
  * renderer object
  *
  * Do not overstuff Horde_Mobile objects. Remember that a lot of WAP
- * clients can not handle more than about 1400 byte of compiled data.
+ * clients cannot handle more than about 1400 bytes of compiled data.
  *
  * Examples:
  *
@@ -105,9 +103,9 @@ class Horde_Mobile_card extends Horde_Mobile_element {
  *
  * $myPage->render();
  *
- * $Horde: framework/Mobile/Mobile.php,v 1.32.10.5 2005/07/16 17:54:31 jan Exp $
+ * $Horde: framework/Mobile/Mobile.php,v 1.32.10.13 2008/01/02 11:30:44 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -210,6 +208,9 @@ class Horde_Mobile extends Horde_Mobile_card {
  */
 class Horde_Mobile_element {
 
+    /**
+     * @TODO Remove in Horde 4.0
+     */
     function Horde_Mobile_element()
     {
     }
@@ -305,6 +306,7 @@ class Horde_Mobile_formElement extends Horde_Mobile_element {
  * @see Horde_Mobile_text
  * @see Horde_Mobile_image
  * @see Horde_Mobile_table
+ * @see Horde_Mobile_dl
  * @see Horde_Mobile_input
  * @see Horde_Mobile_textarea
  * @see Horde_Mobile_select
@@ -349,7 +351,6 @@ class Horde_Mobile_form extends Horde_Mobile_element {
             $this->_elements[] = &$formElement;
         } elseif (is_a($formElement, 'Horde_Mobile_formElement') ||
                   is_a($formElement, 'Horde_Mobile_text') ||
-                  is_a($formElement, 'Horde_Mobile_table') ||
                   is_a($formElement, 'Horde_Mobile_rule') ||
                   is_a($formElement, 'Horde_Mobile_image')) {
             $block = &new Horde_Mobile_block($formElement);
@@ -425,6 +426,34 @@ class Horde_Mobile_form extends Horde_Mobile_element {
                             }
                         }
                         break;
+
+                    case 'horde_mobile_dl':
+                        foreach ($bval->_dts as $dt) {
+                            foreach ($dt->_dds as $dd) {
+                                switch (strtolower(get_class($dd))) {
+                                case 'horde_mobile_checkbox':
+                                    if ($dd->isChecked()) {
+                                        $defaults[] = array('name'  => $bval->get('name'),
+                                                            'value' => $bval->get('value'));
+                                    }
+                                    break;
+
+                                case 'horde_mobile_input':
+                                case 'horde_mobile_textarea':
+                                case 'horde_mobile_select':
+                                case 'horde_mobile_radio':
+                                    $defaults[] = array('name'  => $bval->get('name'),
+                                                        'value' => $bval->get('value'));
+                                    break;
+
+                                case 'horde_mobile_hidden':
+                                    $defaults[] = array('name'  => $bval->get('name'),
+                                                        'value' => $bval->get('value'));
+                                    break;
+                                }
+                            }
+                        }
+                        break;
                     }
                 }
                 break;
@@ -468,6 +497,23 @@ class Horde_Mobile_form extends Horde_Mobile_element {
                             }
                         }
                         break;
+
+                    case 'horde_mobile_dl':
+                        foreach ($bval->_dts as $dt) {
+                            foreach ($dt->_dds as $dd) {
+                                switch (strtolower(get_class($dd))) {
+                                case 'horde_mobile_input':
+                                case 'horde_mobile_hidden':
+                                case 'horde_mobile_textarea':
+                                case 'horde_mobile_select':
+                                case 'horde_mobile_checkbox':
+                                case 'horde_mobile_radio':
+                                    $getvars[] = $dd->get('name');
+                                    break;
+                                }
+                            }
+                        }
+                        break;
                     }
                 }
                 break;
@@ -485,7 +531,7 @@ class Horde_Mobile_form extends Horde_Mobile_element {
  *
  * Examples:
  *
- * $block = new Horde_Mobile_block("Hello WAP!");
+ * $block = new Horde_Mobile_block("Hello World");
  * $text = new Horde_Mobile_text("Welcome to Horde_Mobile", 'b');
  * $block->add($text);
  *
@@ -510,7 +556,9 @@ class Horde_Mobile_block extends Horde_Mobile_element {
                 $this->add($elements);
             } else {
                 foreach ($elements as $element) {
-                    $this->add($element);
+                    if ($this->allows($element)) {
+                        $this->_elements[] = $element;
+                    }
                 }
             }
         }
@@ -520,14 +568,7 @@ class Horde_Mobile_block extends Horde_Mobile_element {
     {
         if (!is_a($element, 'Horde_Mobile_element')) {
             $error = PEAR::raiseError('Invalid element.');
-        } elseif (is_a($element, 'Horde_Mobile_text') ||
-                  is_a($element, 'Horde_Mobile_table') ||
-                  is_a($element, 'Horde_Mobile_image') ||
-                  is_a($element, 'Horde_Mobile_formElement') ||
-                  is_a($element, 'Horde_Mobile_link') ||
-                  is_a($element, 'Horde_Mobile_linkset') ||
-                  is_a($element, 'Horde_Mobile_phone') ||
-                  is_a($element, 'Horde_Mobile_rule')) {
+        } elseif ($this->allows($element)) {
             $this->_elements[] = &$element;
             return $element;
         } else {
@@ -536,6 +577,19 @@ class Horde_Mobile_block extends Horde_Mobile_element {
         return $error;
     }
 
+    function allows($element)
+    {
+        return (is_a($element, 'Horde_Mobile_text') ||
+                is_a($element, 'Horde_Mobile_table') ||
+                is_a($element, 'Horde_Mobile_dl') ||
+                is_a($element, 'Horde_Mobile_image') ||
+                is_a($element, 'Horde_Mobile_formElement') ||
+                is_a($element, 'Horde_Mobile_link') ||
+                is_a($element, 'Horde_Mobile_linkset') ||
+                is_a($element, 'Horde_Mobile_phone') ||
+                is_a($element, 'Horde_Mobile_rule'));
+    }
+
 }
 
 /**
@@ -544,7 +598,7 @@ class Horde_Mobile_block extends Horde_Mobile_element {
  *
  * Examples:
  *
- * $myText1 = new Horde_Mobile_text("Hello WAP!");
+ * $myText1 = new Horde_Mobile_text("Hello World");
  * $myText2 = new Horde_Mobile_text("Welcome to Horde_Mobile", 'b');
  * $myText3 = new Horde_Mobile_text("Good Morning", array('b', 'big'));
  *
@@ -584,6 +638,11 @@ class Horde_Mobile_text extends Horde_Mobile_element {
  * This class allows to insert bitmap images into a Horde_Mobile_block,
  * Horde_Mobile_form or Horde_Mobile_row object.
  *
+ * Examples:
+ *
+ * $image = new Horde_Mobile_image('/path/to/image.wbmp',
+ *                                 array('height' => 100, 'width' => 100));
+ *
  * @see Horde_Mobile_block
  * @see Horde_Mobile_form
  * @see Horde_Mobile_row
@@ -591,6 +650,29 @@ class Horde_Mobile_text extends Horde_Mobile_element {
  * @package Horde_Mobile
  */
 class Horde_Mobile_image extends Horde_Mobile_element {
+
+    var $_src = '';
+    var $_attributes = array();
+
+    /**
+     * Constructor
+     * @param string $src        The source location of the image.
+     * @param array $attributes  Image attributes. Any of:
+     *   'align'
+     *   'alt'
+     *   'height'
+     *   'hspace'
+     *   'vspace'
+     *   'width'
+     *   'class'
+     *   'id'
+     */
+    function Horde_Mobile_image($src, $attributes = array())
+    {
+        $this->_src = $src;
+        $this->_attributes = $attributes;
+    }
+
 }
 
 /**
@@ -619,13 +701,23 @@ class Horde_Mobile_image extends Horde_Mobile_element {
  *
  * @package Horde_Mobile
  */
-class Horde_Mobile_table extends Horde_Mobile_element {
+class Horde_Mobile_table extends Horde_Mobile_block {
 
     var $_rows = array();
     var $_border = null;
     var $_padding = null;
     var $_spacing = null;
 
+    /**
+     * Constructor.  Need because Horde_Mobile_block requires a reference
+     * parameter and PHP 4 does not allow default arguments.
+     *
+     * @TODO Remove in Horde 4.0
+     */
+    function Horde_Mobile_table()
+    {
+    }
+
     /**
      * Adds a Horde_Mobile_row object to Horde_Mobile_table.
      *
@@ -705,6 +797,126 @@ class Horde_Mobile_row extends Horde_Mobile_element {
 
 }
 
+/**
+ * This class allows to insert definition lists into a Horde_Mobile or
+ * Horde_Mobile_form object.
+ *
+ * Examples:
+ *
+ * $myDl = new Horde_Mobile_dl();
+ *
+ * $dt1 = new Horde_Mobile_dt();
+ * $dt1->add($image1);
+ * $dt1->add($text1);
+ * $myDl->add($dt1);
+ *
+ * $dt2 = new Horde_Mobile_dt();
+ * $dt2->add($image2);
+ * $dt2->add($text2);
+ * $myDl->add($dt2);
+ *
+ * $myDeck->add($myDl);
+ *
+ * @see Horde_Mobile
+ * @see Horde_Mobile_form
+ * @see Horde_Mobile_dt
+ *
+ * @package Horde_Mobile
+ */
+class Horde_Mobile_dl extends Horde_Mobile_block {
+
+    var $_dts = array();
+
+    /**
+     * Constructor.  Need because Horde_Mobile_block requires a reference
+     * parameter and PHP 4 does not allow default arguments.
+     *
+     * @TODO Remove in Horde 4.0
+     */
+    function Horde_Mobile_dl()
+    {
+    }
+
+    /**
+     * Adds a Horde_Mobile_dt object to Horde_Mobile_dl.
+     *
+     * @param Horde_Mobile_dt $dt The dl object to add.
+     */
+    function &add(&$dt)
+    {
+        if (!is_a($dt, 'Horde_Mobile_dt')) {
+            $error = PEAR::raiseError('Must be Horde_Mobile_dt objects.');
+            return $error;
+        }
+
+        $this->_dts[] = &$dt;
+        return $dt;
+    }
+
+}
+
+/**
+ * This class defines the terms of a Horde_Mobile_dl object.
+ *
+ * Examples:
+ *
+ * $image1 = new Horde_Mobile_image("my_image.wbmp", "my_image.png", ":-)");
+ * $text1 = new Horde_Mobile_text("my text");
+ * $dt1 = new Horde_Mobile_dt();
+ * $dt1->add($image1);
+ * $dt1->add();
+ * $dt1->add($text1);
+ *
+ * @see Horde_Mobile_dl
+ * @see Horde_Mobile_text
+ * @see Horde_Mobile_image
+ * @see Horde_Mobile_link
+ *
+ * @package Horde_Mobile
+ */
+class Horde_Mobile_dt extends Horde_Mobile_element {
+
+    var $_dds = array();
+
+    /**
+     * Adds a definition term element to a Horde_Mobile_dt object.
+     *
+     * @param Horde_Mobile_Element $ddElement  Can be a Horde_Mobile_text
+     *                                         object, a Horde_Mobile_image
+     *                                         object, a Horde_Mobile_link
+     *                                         object or null. The latter
+     *                                         results in an empty dd.
+     *
+     * @return Horde_Mobile_Element
+     */
+    function &add($ddElement = null)
+    {
+        if (is_object($ddElement)) {
+            if (!is_a($ddElement, 'Horde_Mobile_text') &&
+                !is_a($ddElement, 'Horde_Mobile_link') &&
+                !is_a($ddElement, 'Horde_Mobile_image')) {
+                $error = PEAR::raiseError('Description can only contain text, links or images.');
+                return $error;
+            }
+            $this->_dds[]= &$ddElement;
+            return $ddElement;
+        } elseif (!is_null($ddElement)) {
+            $t = &new Horde_Mobile_text($ddElement);
+            $this->_dds[] = &$t;
+            return $t;
+        } else {
+            $this->_dds[] = &$ddElement;
+            return $ddElement;
+        }
+    }
+
+    function getDdsCount()
+    {
+        return count($this->_dds);
+    }
+
+}
+
 /**
  * This class provides a text input field in a Horde_Mobile_form object.
  *
@@ -816,6 +1028,7 @@ class Horde_Mobile_select extends Horde_Mobile_formElement {
 
     var $_type;
     var $_options = array();
+    var $_htmlchars = true;
 
     /**
      * Constructor
@@ -826,12 +1039,18 @@ class Horde_Mobile_select extends Horde_Mobile_formElement {
      *                      'popup': popup the whole selection list
      *                      'spin':  rotate options on a WAP device screen (OW
      *                               1.3 GUI only).
+     * @param string $label Describes your input field on the surfer's
+     *                      screen/display.
+     *
+     * @param string $htmlchars Are the options already encoded for output?
      */
-    function Horde_Mobile_select($name, $type = 'popup')
+    function Horde_Mobile_select($name, $type = 'popup', $label = '', $htmlchars = false)
     {
         $this->_name = $name;
         $this->_type = $type;
+        $this->_label = $label;
         $this->_value = null;
+        $this->_htmlchars = $htmlchars;
     }
 
     /**
