@@ -3,9 +3,9 @@
  * Horde_Mobile_Renderer:: framework for mobile device markup
  * renderers.
  *
- * $Horde: framework/Mobile/Mobile/Renderer.php,v 1.15.10.9 2007/01/02 13:54:27 jan Exp $
+ * $Horde: framework/Mobile/Mobile/Renderer.php,v 1.15.10.12 2008/03/11 01:49:50 slusarz Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -21,7 +21,7 @@ class Horde_Mobile_Renderer extends Horde_Mobile {
     function Horde_Mobile_Renderer($browser = null)
     {
         if (is_null($browser)) {
-            $this->_browser = &new Browser();
+            $this->_browser = new Browser();
         } else {
             $this->_browser = $browser;
         }
@@ -70,6 +70,29 @@ class Horde_Mobile_Renderer extends Horde_Mobile {
         }
     }
 
+    function _renderDl(&$dl)
+    {
+        foreach ($dl->_dts as $dt) {
+            $this->_renderDt($dt);
+        }
+    }
+
+    function _renderDt(&$dt)
+    {
+        $i = 0;
+        foreach ($dt->_dds as $dd) {
+            echo $out = ($i == 0) ? '<dt>' : '<dd>';
+
+            // Call create function for each ddelement that is a
+            // Horde_Mobile object.
+            if (!is_null($dd)) {
+                $this->renderElement($dd);
+            }
+
+            echo $out = ($i++ == 0) ? '</dt>' : '</dd>';
+        }
+    }
+
     function _renderTable(&$table)
     {
         foreach ($table->_rows as $row) {
@@ -108,16 +131,13 @@ class Horde_Mobile_Renderer extends Horde_Mobile {
     function &factory($type, $browser = null, $params = array())
     {
         $type = basename($type);
-
-        if (@file_exists(dirname(__FILE__) . '/Renderer/' . $type . '.php')) {
-            include_once dirname(__FILE__) . '/Renderer/' . $type . '.php';
-        } else {
-            @include_once 'Horde/Mobile/Renderer/' . $type . '.php';
+        $class = 'Horde_Mobile_Renderer_' . $type;
+        if (!class_exists($class)) {
+            include_once 'Horde/Mobile/Renderer/' . $type . '.php';
         }
 
-        $class = 'Horde_Mobile_Renderer_' . $type;
         if (class_exists($class)) {
-            $renderer = &new $class($browser, $params);
+            $renderer = new $class($browser, $params);
         } else {
             $renderer = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
