@@ -1,15 +1,19 @@
 <?php
+/**
+ * $Horde: framework/NLS/NLS.php,v 1.82.4.21 2008/08/26 16:17:32 jan Exp $
+ *
+ * @package Horde_NLS
+ */
+
+/** String */
+require_once 'Horde/String.php';
+
 /**
  * The NLS:: class provides Native Language Support. This includes common
  * methods for handling language detection and selection, timezones, and
  * hostname->country lookups.
  *
- * $Horde: framework/NLS/NLS.php,v 1.82.4.6 2005/07/03 05:52:01 selsky Exp $
- *
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Jan Schneider <jan@horde.org>
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -17,7 +21,7 @@
  * @author  Jon Parise <jon@horde.org>
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Horde 3.0
  * @package Horde_NLS
  */
@@ -35,13 +39,13 @@ class NLS {
         $lang = Util::getFormData('new_lang');
 
         /* First, check if language pref is locked and, if so, set it to its
-           value */
+         * value */
         if (isset($prefs) && $prefs->isLocked('language')) {
             $language = $prefs->getValue('language');
         /* Check if the user selected a language from the login screen */
-        } elseif (!empty($lang)) {
+        } elseif (!empty($lang) && NLS::isValid($lang)) {
             $language = $lang;
-        /* Check if we have a language set in a cookie */
+        /* Check if we have a language set in the session */
         } elseif (isset($_SESSION['horde_language'])) {
             $language = $_SESSION['horde_language'];
         /* Use site-wide default, if one is defined */
@@ -61,10 +65,19 @@ class NLS {
                     $language = $lang;
                     break;
                 }
-                /* In case no full match, save best guess based on prefix */
-                if (!isset($partial_lang) &&
-                    NLS::isValid(NLS::_map(substr($lang, 0, 2)))) {
-                    $partial_lang = NLS::_map(substr($lang, 0, 2));
+
+                /* In case there's no full match, save our best guess. Try
+                 * ll_LL, followed by just ll. */
+                if (!isset($partial_lang)) {
+                    $ll_LL = String::lower(substr($lang, 0, 2)) . '_' . String::upper(substr($lang, 0, 2));
+                    if (NLS::isValid($ll_LL)) {
+                        $partial_lan = $ll_LL;
+                    } else {
+                        $ll = NLS::_map(substr($lang, 0, 2));
+                        if (NLS::isValid($ll))  {
+                            $partial_lang = $ll;
+                        }
+                    }
                 }
             }
         }
@@ -84,29 +97,33 @@ class NLS {
     /**
      * Sets the language.
      *
-     * @param string $lang  The language abbriviation.
+     * @param string $lang  The language abbreviation.
      */
     function setLang($lang = null)
     {
-        include_once HORDE_BASE . '/config/nls.php';
+        Horde::loadConfiguration('nls.php', null, 'horde');
 
         if (empty($lang) || !NLS::isValid($lang)) {
             $lang = NLS::select();
         }
 
-        if (isset($GLOBALS['language']) && $GLOBALS['language'] == $lang) {
-            return;
-        }
-
-        $GLOBALS['language'] = $lang;
         $_SESSION['horde_language'] = $lang;
 
+        if (isset($GLOBALS['language'])) {
+            if ($GLOBALS['language'] == $lang) {
+                return;
+            } else {
+                $GLOBALS['registry']->clearCache();
+            }
+        }
+        $GLOBALS['language'] = $lang;
+
         /* First try language with the current charset. */
         $lang_charset = $lang . '.' . NLS::getCharset();
         if ($lang_charset != setlocale(LC_ALL, $lang_charset)) {
             /* Next try language with its default charset. */
             global $nls;
-            $charset = !empty($nls['charsets'][$lang]) ? $nls['charsets'][$lang] : $nls['defaults']['charset'];
+            $charset = !empty($nls['charsets'][$lang]) ? $nls['charsets'][$lang] : 'ISO-8859-1';
             $lang_charset = $lang . '.' . $charset;
             NLS::_cachedCharset(0, $charset);
             if ($lang_charset != setlocale(LC_ALL, $lang_charset)) {
@@ -115,6 +132,7 @@ class NLS {
                 setlocale(LC_ALL, $lang_charset);
             }
         }
+
         @putenv('LANG=' . $lang_charset);
         @putenv('LANGUAGE=' . $lang_charset);
     }
@@ -142,6 +160,32 @@ class NLS {
         }
     }
 
+    /**
+     * Sets the language and reloads the whole NLS environment.
+     *
+     * When setting the language, the gettext catalogs have to be reloaded
+     * too, charsets have to be updated etc. This method takes care of all
+     * this.
+     *
+     * @since Horde 3.2
+     *
+     * @param string $language  The new language.
+     * @param string $app       The application for reloading the gettext
+     *                          catalog. The current application if empty.
+     */
+    function setLanguageEnvironment($language = null, $app = null)
+    {
+        if (empty($app)) {
+            $app = $GLOBALS['registry']->getApp();
+        }
+        NLS::setLang($language);
+        NLS::setTextdomain(
+            $app,
+            $GLOBALS['registry']->get('fileroot', $app) . '/locale',
+            NLS::getCharset());
+        String::setDefaultCharset(NLS::getCharset());
+    }
+
     /**
      * Determines whether the supplied language is valid.
      *
@@ -166,11 +210,8 @@ class NLS {
      *
      * @return string  The mapped language code.
      */
-
     function _map($language)
     {
-        require_once 'Horde/String.php';
-
         $aliases = &$GLOBALS['nls']['aliases'];
 
         // Translate the $language to get broader matches.
@@ -213,7 +254,7 @@ class NLS {
         }
 
         if ($original) {
-            $charset = empty($nls['charsets'][$language]) ? $nls['defaults']['charset'] : $nls['charsets'][$language];
+            $charset = empty($nls['charsets'][$language]) ? 'ISO-8859-1' : $nls['charsets'][$language];
         } else {
             require_once 'Horde/Browser.php';
             $browser = &Browser::singleton();
@@ -233,7 +274,6 @@ class NLS {
         return $charset;
     }
 
-
     /**
      * Returns the current charset of the environment
      *
@@ -260,7 +300,7 @@ class NLS {
             }
         }
 
-        return (!empty($nls['charsets'][$language])) ? $nls['charsets'][$language] : $nls['defaults']['charset'];
+        return (!empty($nls['charsets'][$language])) ? $nls['charsets'][$language] : 'ISO-8859-1';
     }
 
     /**
@@ -277,11 +317,7 @@ class NLS {
      */
     function _cachedCharset($index, $charset = null)
     {
-        static $cache;
-
-        if (!isset($cache)) {
-            $cache = array();
-        }
+        static $cache = array();
 
         if ($charset == null) {
             return isset($cache[$index]) ? $cache[$index] : null;
@@ -305,7 +341,7 @@ class NLS {
             return $charset;
         }
         return isset($nls['emails'][$language]) ? $nls['emails'][$language] :
-               (isset($nls['charsets'][$language]) ? $nls['charsets'][$language] : $nls['defaults']['charset']);
+               (isset($nls['charsets'][$language]) ? $nls['charsets'][$language] : 'ISO-8859-1');
     }
 
     /**
@@ -343,6 +379,51 @@ class NLS {
         return $valid;
     }
 
+    /**
+     * Sets the charset.
+     *
+     * In general, the applied charset is automatically determined by browser
+     * language and browser capabilities and there's no need to manually call
+     * setCharset. However for headless (RPC) operations the charset may be
+     * set manually to ensure correct character conversion in the backend.
+     *
+     * @param string $charset  If specified, this charset will be stored in the
+     *                         given cache slot.
+     * @param integer $index   The ID of a cache slot. 0 for the UI charset, 1
+     *                         for the translation charset and 2 for the
+     *                         external charset. Defaults to 0: this is the
+     *                         charset returned by getCharset and used for
+     *                         conversion.
+     */
+    function setCharset($charset, $index = 0)
+    {
+        NLS::_cachedCharset($index, $charset);
+    }
+
+    /**
+     * Sets the charset and reloads the whole NLS environment.
+     *
+     * When setting the charset, the gettext catalogs have to be reloaded too,
+     * to match the new charset, among other things. This method takes care of
+     * all this.
+     *
+     * @since Horde 3.2
+     *
+     * @param string $charset  The new charset.
+     */
+    function setCharsetEnvironment($charset)
+    {
+        unset($GLOBALS['language']);
+        NLS::setCharset($charset);
+        NLS::setLang();
+        $app = $GLOBALS['registry']->getApp();
+        NLS::setTextdomain(
+            $app,
+            $GLOBALS['registry']->get('fileroot', $app) . '/locale',
+            NLS::getCharset());
+        String::setDefaultCharset(NLS::getCharset());
+    }
+
     /**
      * Sets the current timezone, if available.
      */
@@ -374,8 +455,32 @@ class NLS {
     }
 
     /**
-     * Get country information from a hostname or IP address.
+     * Get the language info returned by nl_langinfo(), but cache it, to
+     * avoid repeated calls.
      *
+     * @since Horde 3.1
+     *
+     * @param const $item  The langinfo item to return.
+     *
+     * @return array  The results of nl_langinfo().
+     */
+    function getLangInfo($item)
+    {
+        if (!function_exists('nl_langinfo')) {
+            return false;
+        }
+
+        static $nl_info = array();
+
+        if (!isset($nl_info[$item])) {
+            $nl_info[$item] = nl_langinfo($item);
+        }
+
+        return $nl_info[$item];
+    }
+
+    /**
+     * Get country information from a hostname or IP address.
      *
      * @param string $host  The hostname or IP address.
      *
@@ -386,8 +491,6 @@ class NLS {
      */
     function getCountryByHost($host)
     {
-        global $conf;
-
         /* List of generic domains that we know is not in the country TLD
            list. See: http://www.iana.org/gtld/gtld.htm */
         $generic = array(
@@ -416,16 +519,9 @@ class NLS {
         }
 
         /* Try GeoIP lookup next. */
-        if (!empty($conf['geoip']['datafile'])) {
-            require_once 'Horde/NLS/GeoIP.php';
-            $geoip = &NLS_GeoIP::singleton($conf['geoip']['datafile']);
-            $id = $geoip->countryIdByName($checkHost);
-            if (!empty($id)) {
-                return array('code' => String::lower($GLOBALS['GEOIP_COUNTRY_CODES'][$id]), 'name' => $GLOBALS['GEOIP_COUNTRY_NAMES'][$id]);
-            }
-        }
-
-        return false;
+        require_once 'Horde/NLS/GeoIP.php';
+        $geoip = &NLS_GeoIP::singleton(!empty($GLOBALS['conf']['geoip']['datafile']) ? $GLOBALS['conf']['geoip']['datafile'] : null);
+        return $geoip->getCountryInfo($checkHost);
     }
 
     /**
@@ -443,7 +539,7 @@ class NLS {
         if ($data !== false) {
             $img = $data['code'] . '.png';
             if (file_exists($registry->get('themesfs', 'horde') . '/graphics/flags/' . $img)) {
-                return Horde::img($img, $data['name'], '', $registry->getImageDir('horde') . '/flags');
+                return Horde::img($img, $data['name'], array('title' => $data['name']), $registry->getImageDir('horde') . '/flags');
             } else {
                 return '[' . $data['name'] . ']';
             }
@@ -461,15 +557,17 @@ class NLS {
      *                corresponding country name. If empty will return an
      *                array of all the country codes and their names.
      */
-    function &getCountryISO($code = '')
+    function getCountryISO($code = '')
     {
-        static $countries = array();
-        if (empty($countries)) {
-            require_once 'Horde/NLS/countries.php';
+        static $countries;
+        if (!isset($countries)) {
+            include 'Horde/NLS/countries.php';
         }
+
         if (empty($code)) {
             return $countries;
-        } elseif (isset($countries[$code])) {
+        }
+        if (isset($countries[$code])) {
             return $countries[$code];
         }
         return false;
