@@ -3,7 +3,7 @@
  * Horde optimized interface to the MaxMind IP Address->Country
  * listing.
  *
- * $Horde: framework/NLS/NLS/GeoIP.php,v 1.10.10.4 2005/07/03 05:52:01 selsky Exp $
+ * $Horde: framework/NLS/NLS/GeoIP.php,v 1.10.10.10 2008/01/02 11:30:44 jan Exp $
  *
  * Based on PHP geoip.inc library by MaxMind LLC:
  *   http://www.maxmind.com/download/geoip/api/php/
@@ -12,7 +12,7 @@
  * 2002 by jim winstead <jimw@apache.org>
  *
  * Copyright (C) 2003 MaxMind LLC
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @since   Horde 3.0
  * @package Horde_NLS
  */
@@ -181,7 +181,7 @@ class NLS_GeoIP {
     function _open()
     {
         /* Return if we already have an object. */
-        if (!empty($this->_gi)) {
+        if (!empty($this->_filehandle)) {
             return true;
         }
 
@@ -191,6 +191,9 @@ class NLS_GeoIP {
         }
 
         $this->_filehandle = fopen($this->_datafile, 'rb');
+        if (!$this->_filehandle) {
+            return false;
+        }
 
         $filepos = ftell($this->_filehandle);
         fseek($this->_filehandle, -3, SEEK_END);
@@ -207,6 +210,33 @@ class NLS_GeoIP {
         return true;
     }
 
+    /**
+     * Returns the country ID and Name for a given hostname.
+     *
+     * @since Horde 3.2
+     *
+     * @param string $name  The hostname.
+     *
+     * @return mixed  An array with 'code' as the country code and 'name' as
+     *                the country name, or false if not found.
+     */
+    function getCountryInfo($name)
+    {
+        if (Util::extensionExists('geoip')) {
+            $id = @geoip_country_code_by_name($name);
+            $cname = @geoip_country_name_by_name($name);
+            return (!empty($id) && !empty($cname)) ?
+                array('code' => String::lower($id), 'name' => $cname):
+                false;
+        }
+
+        $id = $this->countryIdByName($name);
+        if (!empty($id)) {
+            return array('code' => String::lower($GLOBALS['GEOIP_COUNTRY_CODES'][$id]), 'name' => $GLOBALS['GEOIP_COUNTRY_NAMES'][$id]);
+        }
+        return false;
+    }
+
     /**
      * Returns the country ID for a hostname.
      *
@@ -220,7 +250,7 @@ class NLS_GeoIP {
             return false;
         }
         $addr = gethostbyname($name);
-        if (!$addr || $addr == $name) {
+        if (!$addr || ($addr == $name)) {
             return false;
         }
         return $this->countryIdByAddr($addr);
