@@ -1,13 +1,20 @@
 <?php
+/**
+ * @package Horde_Notification
+ *
+ * $Horde: framework/Notification/Notification.php,v 1.46.2.16 2008/07/03 21:55:50 chuck Exp $
+ *
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
+ */
+
+/** Notification_Event */
+require_once dirname(__FILE__) . '/Notification/Event.php';
+
 /**
  * The Notification:: class provides a subject-observer pattern for
  * raising and showing messages of different types and to different
  * listeners.
  *
- * $Horde: framework/Notification/Notification.php,v 1.46.2.12 2007/01/02 13:54:33 jan Exp $
- *
- * Copyright 2001-2007 Jan Schneider <jan@horde.org>
- *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
@@ -31,6 +38,13 @@ class Notification {
      */
     var $_stack = 'hordeMessageStacks';
 
+    /**
+     * A Horde_Alarm instance.
+     *
+     * @var Horde_Alarm
+     */
+    var $_alarm;
+
     /**
      * Returns a reference to the global Notification object, only
      * creating it if it doesn't already exist.
@@ -47,7 +61,7 @@ class Notification {
         static $notification = array();
 
         if (!isset($notification[$stack])) {
-            $notification[$stack] = new Notification($stack);
+            $notification[$stack] = &new Notification($stack);
         }
 
         return $notification[$stack];
@@ -69,24 +83,29 @@ class Notification {
         if (!isset($_SESSION[$this->_stack])) {
             $_SESSION[$this->_stack] = array();
         }
+
+        if (!empty($GLOBALS['conf']['alarms']['driver'])) {
+            require_once 'Horde/Alarm.php';
+            $this->_alarm = Horde_Alarm::factory();
+        }
     }
 
     /**
      * Registers a listener with the notification object and includes
      * the necessary library file dynamically.
      *
-     * @param string $driver  The name of the listener to attach. These names
-     *                        must be unique; further listeners with the same
-     *                        name will be ignored.
-     * @param array $params   A hash containing any additional configuration or
-     *                        connection parameters a listener driver might
-     *                        need.
-     * @param string $class   The class name from which the driver was
-     *                        instantiated if not the default one. If given
-     *                        you have to include the library file containing
-     *                        this class yourself. This is useful if you want
-     *                        the listener driver to be overriden by an
-     *                        application's implementation.
+     * @param string $listener The name of the listener to attach. These names
+     *                         must be unique; further listeners with the same
+     *                         name will be ignored.
+     * @param array $params    A hash containing any additional configuration or
+     *                         connection parameters a listener driver might
+     *                         need.
+     * @param string $class    The class name from which the driver was
+     *                         instantiated if not the default one. If given
+     *                         you have to include the library file containing
+     *                         this class yourself. This is useful if you want
+     *                         the listener driver to be overriden by an
+     *                         application's implementation.
      */
     function &attach($listener, $params = array(), $class = null)
     {
@@ -96,11 +115,11 @@ class Notification {
         }
 
         if (is_null($class)) {
-            @require_once dirname(__FILE__) . '/Notification/Listener/' . $listener . '.php';
+            include_once dirname(__FILE__) . '/Notification/Listener/' . $listener . '.php';
             $class = 'Notification_Listener_' . $listener;
         }
         if (class_exists($class)) {
-            $this->_listeners[$listener] = &new $class($params);
+            $this->_listeners[$listener] = new $class($params);
             if (!isset($_SESSION[$this->_stack][$listener])) {
                 $_SESSION[$this->_stack][$listener] = array();
             }
@@ -148,8 +167,7 @@ class Notification {
             !is_a($event, 'PEAR_Error')) {
             /* Transparently create a Notification_Event object and
              * set the message attribute. */
-            require_once dirname(__FILE__) . '/Notification/Event.php';
-            $event = &new Notification_Event($event);
+            $event = new Notification_Event($event);
         }
         if (is_a($event, 'PEAR_Error')) {
             if (!isset($type)) {
@@ -186,8 +204,15 @@ class Notification {
         } elseif (!is_array($options['listeners'])) {
             $options['listeners'] = array($options['listeners']);
         }
+
+        if ($this->_alarm && in_array('status', $options['listeners'])) {
+            $this->_alarm->notify(Auth::getAuth());
+        }
+
         foreach ($options['listeners'] as $listener) {
-            $this->_listeners[$listener]->notify($_SESSION[$this->_stack][$this->_listeners[$listener]->getName()], $options);
+            if (isset($this->_listeners[$listener])) {
+                $this->_listeners[$listener]->notify($_SESSION[$this->_stack][$this->_listeners[$listener]->getName()], $options);
+            }
         }
     }
 
