@@ -3,9 +3,9 @@
  * The Notification_Listener:: class provides functionality for displaying
  * messages from the message stack as a status line.
  *
- * $Horde: framework/Notification/Notification/Listener.php,v 1.16.2.5 2005/07/03 05:11:47 selsky Exp $
+ * $Horde: framework/Notification/Notification/Listener.php,v 1.16.2.13 2008/01/02 11:30:46 jan Exp $
  *
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -85,27 +85,40 @@ class Notification_Listener {
         } else {
             require_once dirname(__FILE__) . '/Event.php';
             $ob = @unserialize($message['event']);
-            if (!method_exists($ob, 'getMessage')) {
+            if (!is_callable(array($ob, 'getMessage'))) {
                 if (isset($ob->_message)) {
-                    $ob = &new Notification_Event($ob->_message);
+                    $ob = new Notification_Event($ob->_message);
                 }
             }
         }
 
         /* If we've failed to create a valid Notification_Event object
          * (or subclass object) so far, return a PEAR_Error. */
-        if (!method_exists($ob, 'getMessage')) {
+        if (!is_callable(array($ob, 'getMessage'))) {
             $ob = PEAR::raiseError('Unable to decode message event: ' . $message['event']);
         }
 
         /* Specially handle PEAR_Error objects and add userinfo if
          * it's there. */
-        if (method_exists($ob, 'getUserInfo')) {
+        if (is_callable(array($ob, 'getUserInfo'))) {
             $userinfo = $ob->getUserInfo();
             if ($userinfo) {
                 if (is_array($userinfo)) {
-                    $userinfo = @implode(', ', $userinfo);
+                    $userinfo_elts = array();
+                    foreach ($userinfo as $userinfo_elt) {
+                        if (is_scalar($userinfo_elt)) {
+                            $userinfo_elts[] = $userinfo_elt;
+                        } elseif (is_object($userinfo_elt)) {
+                            if (is_callable(array($userinfo_elt, '__toString'))) {
+                                $userinfo_elts[] = $userinfo_elt->__toString();
+                            } elseif (is_callable(array($userinfo_elt, 'getMessage'))) {
+                                $userinfo_elts[] = $userinfo_elt->getMessage();
+                            }
+                        }
+                    }
+                    $userinfo = implode(', ', $userinfo_elts);
                 }
+
                 $ob->_message = $ob->getMessage() . ' : ' . $userinfo;
             }
         }
