@@ -3,9 +3,9 @@
  * The Perms_UI:: class provides UI methods for the Horde permissions
  * system.
  *
- * $Horde: framework/Perms/Perms/UI.php,v 1.27.2.9 2007/01/02 13:54:34 jan Exp $
+ * $Horde: framework/Perms/Perms/UI.php,v 1.27.2.22 2008/07/03 19:04:24 bklang Exp $
  *
- * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -59,10 +59,11 @@ class Perms_UI {
      *
      * @return string  The html showing the permissions as a Horde_Tree.
      */
-    function renderTree($current = DATATREE_ROOT)
+    function renderTree($current = PERMS_ROOT)
     {
         global $registry;
 
+        require_once 'Horde/Array.php';
         require_once 'Horde/Tree.php';
 
         /* Get the perms tree. */
@@ -71,10 +72,12 @@ class Perms_UI {
         $icondir = array('icondir' => $GLOBALS['registry']->getImageDir());
         $perms_node = $icondir + array('icon' => 'perms.png');
         $add = Horde::applicationUrl('admin/perms/addchild.php');
+        $add_img = Horde::img('add_perm.png', _("Add Permission"));
         $edit = Horde::applicationUrl('admin/perms/edit.php');
         $delete = Horde::applicationUrl('admin/perms/delete.php');
         $edit_img = Horde::img('edit.png', _("Edit Permission"));
         $delete_img = Horde::img('delete.png', _("Delete Permission"));
+        $blank_img = Horde::img('blank.gif', '', array('width' => 16, 'height' => 16));
 
         /* Set up the tree. */
         $tree = &Horde_Tree::singleton('perms_ui', 'javascript');
@@ -82,27 +85,71 @@ class Perms_UI {
         $tree->setHeader(array(array('width' => '50%')));
 
         foreach ($nodes as $perm_id => $node) {
-            $node_class = ($current == $perm_id) ? array('class' => 'selected') : array();
-            if ($perm_id == DATATREE_ROOT) {
-                $add_img = Horde::img('perms.png', _("Add New Permission"));
-                $add_link = Horde::link(Util::addParameter($add, 'perm_id', $perm_id), _("Add New Permission")) . $add_img . '</a>';
-                $base_node_params = $icondir + array('icon' => 'administration.png');
+            $node_class = ($current == $perm_id)
+                ? array('class' => 'selected')
+                : array();
+            if ($perm_id == PERMS_ROOT) {
+                $add_link = Horde::link(
+                    Util::addParameter($add, 'perm_id', $perm_id),
+                    _("Add New Permission"))
+                    . $add_img . '</a>';
+                $base_node_params = $icondir +
+                    array('icon' => 'administration.png');
 
-                $tree->addNode($perm_id, null, _("All Permissions"), 0, true, $base_node_params + $node_class, array($add_link));
+                $tree->addNode($perm_id, null, _("All Permissions"), 0, true,
+                               $base_node_params + $node_class,
+                               array($add_link));
             } else {
-                $parent_id = $this->_perms->_datatree->getParent($node);
+                $parent_id = $this->_perms->getParent($node);
                 if (is_a($parent_id, 'PEAR_Error')) {
                     Horde::fatal($parent_id, __FILE__, __LINE__);
                 }
 
-                $add_img = Horde::img('perms.png', _("Add Child Permission"));
-                $add_link = Horde::link(Util::addParameter($add, 'perm_id', $perm_id), _("Add Child Permission")) . $add_img . '</a>';
-                $edit_link = Horde::link(Util::addParameter($edit, 'perm_id', $perm_id), _("Edit Permission")) . $edit_img . '</a>';
-                $delete_link = Horde::link(Util::addParameter($delete, 'perm_id', $perm_id), _("Delete Permission")) . $delete_img . '</a>';
-                $perms_extra = array($add_link, $edit_link, $delete_link);
+                $perms_extra = array();
+                $parents = explode(':', $node);
+
+                if (!in_array($parents[0], $GLOBALS['registry']->listApps())) {
+                    // This backend has permissions for an application that is
+                    // not installed.  Perhaps the application has been removed
+                    // or the backend is shared with other Horde installations.
+                    // Skip this app and do not include it in the tree.
+                    continue;
+                }
+
+                $app_perms = $this->_perms->getApplicationPermissions($parents[0]);
+                if (is_a($app_perms, 'PEAR_Error')) {
+                    $GLOBALS['notification']->push($app_perms);
+                    continue;
+                }
+                if (isset($app_perms['tree']) &&
+                    is_array(Horde_Array::getElement($app_perms['tree'], $parents))) {
+                    $add_link = Horde::link(
+                        Util::addParameter($add, 'perm_id', $perm_id),
+                        _("Add Child Permission"))
+                        . $add_img . '</a>';
+                    $perms_extra[] = $add_link;
+                } else {
+                    $perms_extra[] = $blank_img;
+                }
+
+                $edit_link = Horde::link(
+                    Util::addParameter($edit, 'perm_id', $perm_id),
+                    _("Edit Permission"))
+                    . $edit_img . '</a>';
+                $perms_extra[] = $edit_link;
+                $delete_link = Horde::link(
+                    Util::addParameter($delete, 'perm_id', $perm_id),
+                    _("Delete Permission"))
+                    . $delete_img . '</a>';
+                $perms_extra[] = $delete_link;
                 $name = $this->_perms->getTitle($node);
 
-                $tree->addNode($perm_id, $parent_id, $name, substr_count($node, ':') + 1, false, $perms_node + $node_class, $perms_extra);
+                $expanded = isset($nodes[$current]) &&
+                    strpos($nodes[$current], $node) === 0 &&
+                    $nodes[$current] != $node;
+                $tree->addNode($perm_id, $parent_id, $name,
+                               substr_count($node, ':') + 1, $expanded,
+                               $perms_node + $node_class, $perms_extra);
             }
         }
 
@@ -145,7 +192,7 @@ class Perms_UI {
         $this->_formInit();
 
         $this->_form->setTitle(sprintf(_("Add a child permission to \"%s\""), $this->_perms->getTitle($permission->getName())));
-        $this->_form->setButtons(_("Add"), true);
+        $this->_form->setButtons(_("Add"));
         $this->_vars->set('perm_id', $this->_perms->getPermissionId($permission));
         $this->_form->addHidden('', 'perm_id', 'text', false);
 
@@ -185,7 +232,7 @@ class Perms_UI {
     /**
      * Create a permission editing form.
      *
-     * @param DataTreeObject_Permission $permission
+     * @param Horde_Permission $permission
      */
     function setupEditForm($permission)
     {
@@ -291,7 +338,7 @@ class Perms_UI {
                 if (is_array($new_users)) {
                     $u_n = Util::getFormData('u_n');
                     $u_n = empty($u_n['u']) ? null : $u_n['u'];
-                    $user_html = '<select name="u_n[u]"><option>' . _("-- select --") . '</option>';
+                    $user_html = '<select name="u_n[u]"><option value="">' . _("-- select --") . '</option>';
                     foreach ($new_users as $new_user) {
                         $user_html .= '<option value="' . $new_user . '"';
                         $user_html .= $u_n == $new_user ? ' selected="selected"' : '';
@@ -315,6 +362,11 @@ class Perms_UI {
         require_once 'Horde/Group.php';
         $groups = &Group::singleton();
         $group_list = $groups->listGroups();
+        if (is_a($group_list, 'PEAR_Error')) {
+            $GLOBALS['notification']->push($group_list);
+            $group_list = array();
+        }
+
         if (!empty($group_list)) {
             /* There is an available list of groups so set up an array which
              * the matrix field type will recognise to set up an enum box for
@@ -345,7 +397,7 @@ class Perms_UI {
                 if (is_array($new_groups)) {
                     $g_n = Util::getFormData('g_n');
                     $g_n = empty($g_n['g']) ? null : $g_n['g'];
-                    $group_html = '<select name="g_n[g]"><option>' . _("-- select --") . '</option>';
+                    $group_html = '<select name="g_n[g]"><option value="">' . _("-- select --") . '</option>';
                     foreach ($new_groups as $groupId => $group) {
                         $group_html .= '<option value="' . $groupId . '"';
                         $group_html .= $g_n == $groupId ? ' selected="selected"' : '';
@@ -388,17 +440,23 @@ class Perms_UI {
             $info['creator'] = isset($info['creator'][0]) ? $info['creator'][0] : null;
         } else {
             $u_n = $this->_vars->get('u_n');
-            $info['u'] = array($u_n['u'] => $info['u_n']['v']);
+            $info['u'] = array();
+            if (!empty($u_n['u'])) {
+                $info['u'][$u_n['u']] = $info['u_n']['v'];
+            }
             unset($info['u_n']);
             if (isset($info['u_v'])) {
-                $info['u'] = array_merge($info['u'], $info['u_v']);
+                $info['u'] += $info['u_v'];
                 unset($info['u_v']);
             }
             $g_n = $this->_vars->get('g_n');
-            $info['g'] = array($g_n['g'] => $info['g_n']['v']);
+            $info['g'] = array();
+            if (!empty($g_n['g'])) {
+                $info['g'][$g_n['g']] = $info['g_n']['v'];
+            }
             unset($info['g_n']);
             if (isset($info['g_v'])) {
-                $info['g'] = array_merge($info['g'], $info['g_v']);
+                $info['g'] += $info['g_v'];
                 unset($info['g_v']);
             }
         }
@@ -474,7 +532,7 @@ class Perms_UI {
         if (!is_a($this->_form, 'Horde_Form')) {
             /* No existing valid form object set so set up a new one. */
             require_once 'Horde/Form.php';
-            $this->_form = &Horde_Form::singleton('', $this->_vars);
+            $this->_form = new Horde_Form($this->_vars);
         }
     }
 
