@@ -1,23 +1,18 @@
 <?php
-
 /**
- * Preference is administratively locked.
+ * $Horde: framework/Prefs/Prefs.php,v 1.137.10.37 2008/10/09 15:55:51 chuck Exp $
  */
+
+/** Preference is administratively locked. */
 define('_PREF_LOCKED', 1);
 
-/**
- * Preference is shared amongst applications.
- */
+/** Preference is shared amongst applications. */
 define('_PREF_SHARED', 2);
 
-/**
- * Preference value has been changed.
- */
+/** Preference value has been changed. */
 define('_PREF_DIRTY', 4);
 
-/**
- * Preference value is the application default.
- */
+/** Preference value is the application default. */
 define('_PREF_DEFAULT', 8);
 
 /**
@@ -48,9 +43,7 @@ define('_PREF_DEFAULT', 8);
  *     'help'   => *Name of the entry in the XML help file*
  * );
  *
- * $Horde: framework/Prefs/Prefs.php,v 1.137.10.30 2007/06/20 11:24:21 jan Exp $
- *
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -62,7 +55,7 @@ define('_PREF_DEFAULT', 8);
 class Prefs {
 
     /**
-     * Hash holding all of the user's preferences. Each preference is
+     * Hash holding the current set of preferences. Each preference is
      * itself a hash, so this will ultimately be multi-dimensional.
      *
      * [*pref name*] => Array(
@@ -71,21 +64,31 @@ class Prefs {
      *     [v]  =>  *pref value*
      * )
      *
-     * @access private
-     *
      * @var array
      */
     var $_prefs = array();
 
     /**
-     * String containing the name of this scope. This is used to
-     * maintain the application scope between sets of preferences. By
+     * String containing the name of the current scope. This is used
+     * to differentiate between sets of preferences (multiple
+     * applications can have a "sortby" preference, for example). By
      * default, all preferences belong to the "global" (Horde) scope.
      *
      * @var string
      */
     var $_scope = 'horde';
 
+    /**
+     * Array of loaded scopes. In order to only load what we need, and
+     * to not load things multiple times, we need to maintain a list
+     * of loaded scopes. $this->_prefs will always be the combination
+     * of the current scope and the 'horde' scope (or just the 'horde'
+     * scope).
+     *
+     * @var array
+     */
+    var $_scopes = array();
+
     /**
      * String containing the current username. This indicates the owner of the
      * preferences.
@@ -101,6 +104,14 @@ class Prefs {
      */
     var $_caching = false;
 
+    /**
+     * Array to cache in. Usually a reference to an array in $_SESSION, but
+     * could be overridden by a subclass for testing or other users.
+     *
+     * @var array
+     */
+    var $_cache;
+
     /**
      * Hash holding preferences with hook functions defined.
      *
@@ -108,13 +119,6 @@ class Prefs {
      */
     var $_hooks = array();
 
-    /**
-     * Have we run hook functions yet?
-     *
-     * @var boolean
-     */
-    var $_hooksCalled = null;
-
     /**
      * Default constructor (must be called from each extending class in their
      * constructors via parent::Prefs()).
@@ -122,6 +126,29 @@ class Prefs {
     function Prefs()
     {
         $this->_shutdown();
+
+        // Create a unique key that's safe to use for caching even if we want
+        // another user's preferences later, then register the cache array in
+        // $_SESSION.
+        if ($this->_caching) {
+            $cacheKey = 'horde_prefs_' . sha1(serialize(array($this->_user)));
+            if (!isset($_SESSION[$cacheKey])) {
+                $_SESSION[$cacheKey] = array();
+            }
+
+            // Store a reference to the $_SESSION array.
+            $this->_cache = &$_SESSION[$cacheKey];
+        }
+    }
+
+    /**
+     * Register the shutdown function.
+     *
+     * @access private
+     */
+    function _shutdown()
+    {
+        register_shutdown_function(array(&$this, 'store'));
     }
 
     /**
@@ -135,70 +162,37 @@ class Prefs {
     }
 
     /**
-     * Updates the session-based preferences cache (if available).
+     * Return the user who owns these preferences.
      *
-     * @param string $pref  The preference to update.  If empty, will update
-     *                      the entire cache with the current set of prefs.
+     * @return string  The user these preferences are for.
      */
-    function cacheUpdate($pref = null)
+    function getUser()
     {
-        /* Return immediately if caching is disabled. */
-        if (!$this->_caching) {
-            return;
-        }
-
-        if (isset($_SESSION['prefs_cache'])) {
-            if (is_null($pref)) {
-                $prefs = $this->_prefs;
-            } else {
-                $prefs = array($pref => $this->_prefs[$pref]);
-            }
-
-            /* Place each preference in the cache according to its
-             * scope. */
-            foreach ($prefs as $name => $pref) {
-                $_SESSION['prefs_cache'][$this->getScope($name)][$name] = $pref;
-            }
-        }
+        return $this->_user;
     }
 
     /**
-     * Tries to find the requested preferences in the cache. If they exist,
-     * update the $prefs hash with the cached values.
+     * Get the current scope
      *
-     * @return boolean  True on success, false on failure.
+     * @return string The current scope (application)
+     *
+     * @since Horde 3.2
      */
-    function cacheLookup()
+    function getScope()
     {
-        /* Return immediately if caching is disabled. */
-        if (!$this->_caching) {
-            return false;
-        }
-
-        if (isset($_SESSION['prefs_cache']['horde']) &&
-            isset($_SESSION['prefs_cache'][$this->_scope])) {
-
-            /* Restore global preferences. */
-            $this->_prefs = array_merge($this->_prefs, $_SESSION['prefs_cache']['horde'], $_SESSION['prefs_cache'][$this->_scope]);
-
-            return true;
-        }
-
-        return false;
+        return $this->_scope;
     }
 
     /**
-     * Adds a new preference entry to the $prefs hash.
+     * Change scope without explicitly retrieving preferences
      *
-     * @param string $pref   The name of the preference to add.
-     * @param string $val    The initial value of the preference.
-     * @param integer $mask  The initial bitmask of the preference.
+     * @param string $scope The new scope.
+     *
+     * @since Horde 3.2
      */
-    function add($pref, $val = '', $mask = 0)
+    function setScope($scope)
     {
-        if (is_array($this->_prefs)) {
-            $this->_prefs[$pref] = array('v' => $val, 'm' => $mask, 'd' => $val);
-        }
+        $this->_scope = $scope;
     }
 
     /**
@@ -208,11 +202,10 @@ class Prefs {
      */
     function remove($pref)
     {
-        if (is_array($this->_prefs)) {
-            $scope = $this->getScope($pref);
-            unset($this->_prefs[$pref]);
-            unset($_SESSION['prefs_cache'][$scope][$pref]);
-        }
+        // FIXME not updated yet.
+        $scope = $this->_getPreferenceScope($pref);
+        unset($this->_prefs[$pref]);
+        unset($this->_cache[$scope][$pref]);
     }
 
     /**
@@ -235,7 +228,15 @@ class Prefs {
             return false;
         }
 
-        return $this->_setValue($pref, $val, true, $convert);
+        $result = $this->_setValue($pref, $val, true, $convert);
+        if ($result && $this->isDirty($pref)) {
+            $this->_cacheUpdate($this->_getPreferenceScope($pref), array($pref));
+
+            /* If this preference has a change hook, call it now. */
+            Horde::callHook('_prefs_change_hook_' . $pref);
+        }
+
+        return $result;
     }
 
     function __set($name, $value)
@@ -269,18 +270,17 @@ class Prefs {
             $val = $this->convertToDriver($val, NLS::getCharset());
         }
 
-        /* If the preference's value is already equal to $val, don't
-         * bother changing it. Changing it would set the "dirty" bit,
-         * causing an unnecessary update later on in the storage
-         * routine. */
+        // If the preference's value is already equal to $val, don't
+        // bother changing it. Changing it would set the "dirty" bit,
+        // causing an unnecessary update later.
         if (isset($this->_prefs[$pref]) &&
             (($this->_prefs[$pref]['v'] == $val) &&
              !$this->isDefault($pref))) {
             return true;
         }
 
-        /* Check to see if the value exceeds the allowable storage
-         * limit. */
+        // Check to see if the value exceeds the allowable storage
+        // limit.
         if (isset($conf['prefs']['maxsize'])) {
             if (strlen($val) > $conf['prefs']['maxsize']) {
                 global $notification;
@@ -291,8 +291,8 @@ class Prefs {
             }
         }
 
-        /* Assign the new value, unset the "default" bit, and set the
-           "dirty" bit. */
+        // Assign the new value, unset the "default" bit, and set the
+        // "dirty" bit.
         if (empty($this->_prefs[$pref]['m'])) {
             $this->_prefs[$pref]['m'] = 0;
         }
@@ -302,7 +302,8 @@ class Prefs {
             $this->setDirty($pref, true);
         }
 
-        $this->cacheUpdate($pref);
+        // Finally, copy into the $_scopes array.
+        $this->_scopes[$this->_getPreferenceScope($pref)][$pref] = $this->_prefs[$pref];
 
         return true;
     }
@@ -433,7 +434,7 @@ class Prefs {
      *
      * @param string $pref  The name of the preference to get the default for.
      *
-     * @return string       The preference's default value.
+     * @return string  The preference's default value.
      */
     function getDefault($pref)
     {
@@ -490,29 +491,19 @@ class Prefs {
      */
     function _getMask($pref, $mask)
     {
-        return isset($this->_prefs[$pref]['m']) ? ($this->_prefs[$pref]['m'] & $mask) : false;
-    }
-
-    /**
-     * Determines whether the current preference is empty.
-     *
-     * @param string $pref  The name of the preference to check.
-     *
-     * @return boolean  True if the preference is empty.
-     */
-    function isEmpty($pref)
-    {
-        return empty($this->_prefs[$pref]['v']);
+        return isset($this->_prefs[$pref]['m']) ? (bool)($this->_prefs[$pref]['m'] & $mask) : false;
     }
 
     /**
      * Returns the scope of the given preference.
      *
+     * @access protected
+     *
      * @param string $pref  The name of the preference to examine.
      *
      * @return string  The scope of the $pref.
      */
-    function getScope($pref)
+    function _getPreferenceScope($pref)
     {
         if ($this->isShared($pref)) {
             return 'horde';
@@ -522,35 +513,58 @@ class Prefs {
     }
 
     /**
-     * Return a list of "dirty" preferences.
+     * Retrieves preferences for the current scope + the 'horde'
+     * scope.
      *
-     * @access private
-     *
-     * @return array  The list of "dirty" preferences in $this->_prefs.
+     * @param $scope Optional scope specifier - if not present the
+     * current scope will be used.
      */
-    function _dirtyPrefs()
+    function retrieve($scope = null)
     {
-        $dirty_prefs = array();
-
-        foreach (array_keys($this->_prefs) as $pref) {
-            if ($this->isDirty($pref)) {
-                $dirty_prefs[] = $pref;
-            }
+        if ($scope === null) {
+            $scope = $this->_scope;
+        } else {
+            $this->_scope = $scope;
         }
 
-        return $dirty_prefs;
+        $this->_loadScope('horde');
+        if ($scope != 'horde') {
+            $this->_loadScope($scope);
+        }
+
+        if ($scope == 'horde') {
+            $this->_prefs = $this->_scopes['horde'];
+        } else {
+            $this->_prefs = array_merge($this->_scopes['horde'], $this->_scopes[$scope]);
+        }
     }
 
     /**
-     * Retrieves the default preferences.
+     * Load a specific preference scope.
      */
-    function retrieve()
+    function _loadScope($scope)
     {
-        /* Load defaults to make sure we have all preferences. */
-        $this->_setDefaults('horde');
-        $this->_setDefaults($this->_scope);
+        // Return if we've already loaded these prefs.
+        if (!empty($this->_scopes[$scope])) {
+            return;
+        }
 
-        return true;
+        // Basic initialization so _something_ is always set.
+        $this->_scopes[$scope] = array();
+
+        // Always set defaults to pick up new default values, etc.
+        $this->_setDefaults($scope);
+
+        // Now check the prefs cache for existing values.
+        if ($this->_cacheLookup($scope)) {
+            return;
+        }
+
+        $this->_retrieve($scope);
+        $this->_callHooks($scope);
+
+        /* Update the session cache. */
+        $this->_cacheUpdate($scope, array_keys($this->_scopes[$scope]));
     }
 
     /**
@@ -564,7 +578,6 @@ class Prefs {
      */
     function store()
     {
-        return true;
     }
 
     /**
@@ -575,21 +588,16 @@ class Prefs {
      */
     function cleanup($all = false)
     {
-        /* Remove this scope from the preferences cache, if it
-           exists. */
-        if (isset($_SESSION['prefs_cache'][$this->_scope])) {
-            unset($_SESSION['prefs_cache'][$this->_scope]);
-        }
-
         /* Perform a Horde-wide cleanup? */
         if ($all) {
             /* Destroy the contents of the preferences hash. */
             $this->_prefs = array();
 
             /* Destroy the contents of the preferences cache. */
-            if (isset($_SESSION['prefs_cache'])) {
-                unset($_SESSION['prefs_cache']);
-            }
+            unset($this->_cache);
+        } else {
+            /* Remove this scope from the preferences cache, if it exists. */
+            unset($this->_cache[$this->_scope]);
         }
     }
 
@@ -628,98 +636,165 @@ class Prefs {
     }
 
     /**
-     * Populates the $prefs hash with new entries and externally defined
-     * default values.
+     * Return all "dirty" preferences across all scopes.
      *
-     * @param string $app  The application to load defaults for.
+     * @access protected
+     *
+     * @return array The values for all dirty preferences, in a
+     * multi-dimensional array of scope => pref name => pref values.
      */
-    function _setDefaults($app)
+    function _dirtyPrefs()
     {
-        global $registry;
-        $filename = $registry->get('fileroot', $app) . '/config/prefs.php';
+        $dirty_prefs = array();
 
-        /* Ensure that the defaults from this file are only read once.
-           Also, make sure we can read this file. */
-        if (!@is_readable($filename)) {
-            return;
-        }
-
-        /* Read the configuration file. The $_prefs array, which will be
-           in local scope, is assumed to hold the default values. */
-        include $filename;
-        foreach ($_prefs as $pref => $pvals) {
-            if (isset($pvals['value']) &&
-                isset($pvals['locked']) &&
-                isset($pvals['shared']) &&
-                ($pvals['type'] != 'link') &&
-                ($pvals['type'] != 'special')) {
-                $pref = str_replace('.', '_', $pref);
-                $mask = 0;
-                if ($pvals['locked']) {
-                    $mask |= _PREF_LOCKED;
-                }
-                if ($pvals['shared'] || $app == 'horde') {
-                    $mask |= _PREF_SHARED;
-                }
-                $mask &= ~_PREF_DIRTY;
-                $mask |= _PREF_DEFAULT;
-
-                $this->add($pref, $pvals['value'], $mask);
-                if (!empty($pvals['hook'])) {
-                    $this->_setHook($pref);
+        foreach ($this->_scopes as $scope => $prefs) {
+            foreach ($prefs as $pref_name => $pref) {
+                if (isset($pref['m']) && ($pref['m'] & _PREF_DIRTY)) {
+                    $dirty_prefs[$scope][$pref_name] = $pref;
                 }
             }
         }
 
-        /* Update the preferences cache with the defaults. */
-        $this->cacheUpdate();
+        return $dirty_prefs;
     }
 
     /**
-     * Performs shutdown activities.
+     * Updates the session-based preferences cache (if available).
+     *
+     * @access protected
+     *
+     * @param string $scope The scope of the prefs being updated.
+     * @param array $prefs  The preferences to update.
+     */
+    function _cacheUpdate($scope, $prefs)
+    {
+        /* Return immediately if caching is disabled. */
+        if (!$this->_caching) {
+            return;
+        }
+
+        if (isset($this->_cache)) {
+            /* Place each preference in the cache according to its
+             * scope. */
+            foreach ($prefs as $name) {
+                if (isset($this->_scopes[$scope][$name])) {
+                    $this->_cache[$scope][$name] = $this->_scopes[$scope][$name];
+                }
+            }
+        }
+    }
+
+    /**
+     * Tries to find the requested preferences in the cache. If they
+     * exist, update the $_scopes hash with the cached values.
+     *
+     * @access protected
+     *
+     * @return boolean  True on success, false on failure.
+     */
+    function _cacheLookup($scope)
+    {
+        /* Return immediately if caching is disabled. */
+        if (!$this->_caching) {
+            return false;
+        }
+
+        if (isset($this->_cache[$scope])) {
+            $this->_scopes[$scope] = $this->_cache[$scope];
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Populates the $_scopes hash with new entries and externally
+     * defined default values.
      *
      * @access private
-     */
-    function _shutdown()
-    {
-        register_shutdown_function(array(&$this, 'store'));
-    }
-
-    /**
-     * Add $pref to the list of preferences with hook functions.
      *
-     * @param string $pref  The preference with a hook.
+     * @param string $scope  The scope to load defaults for.
      */
-    function _setHook($pref)
+    function _setDefaults($scope)
     {
-        $this->_hooks[] = $pref;
+        /* Read the configuration file. The $_prefs array is assumed to hold
+         * the default values. */
+        $result = Horde::loadConfiguration('prefs.php', array('_prefs'), $scope);
+        if (is_a($result, 'PEAR_Error') || empty($result)) {
+            return;
+        }
+        extract($result);
+
+        foreach ($_prefs as $name => $pref) {
+            if (isset($pref['value']) &&
+                isset($pref['locked']) &&
+                isset($pref['shared']) &&
+                ($pref['type'] != 'link') &&
+                ($pref['type'] != 'special')) {
+                $name = str_replace('.', '_', $name);
+
+                $mask = 0;
+                $mask &= ~_PREF_DIRTY;
+                $mask |= _PREF_DEFAULT;
+
+                if ($pref['locked']) {
+                    $mask |= _PREF_LOCKED;
+                }
+
+                if ($pref['shared'] || ($scope == 'horde')) {
+                    $mask |= _PREF_SHARED;
+                    $pref_scope = 'horde';
+                } else {
+                    $pref_scope = $scope;
+                }
+
+                if ($pref['shared'] && isset($this->_scopes[$pref_scope][$name])) {
+                    // This is a shared preference that was already
+                    // retrieved.
+                    $this->_scopes[$pref_scope][$name]['m'] = $mask & ~_PREF_DEFAULT;
+                    $this->_scopes[$pref_scope][$name]['d'] = $pref['value'];
+                } else {
+                    $this->_scopes[$pref_scope][$name] = array('v' => $pref['value'], 'm' => $mask, 'd' => $pref['value']);
+                }
+
+                if (!empty($pref['hook'])) {
+                    if (!isset($this->_hooks[$pref_scope])) {
+                        $this->_hooks[$pref_scope] = array();
+                    }
+                    $this->_hooks[$pref_scope][] = $name;
+                }
+            }
+        }
     }
 
     /**
      * After preferences have been loaded, set any locked or empty
      * preferences that have hooks to the result of the hook.
+     *
+     * @param string $scope The preferences scope to call hooks for.
+     *
+     * @access private
      */
-    function _callHooks()
+    function _callHooks($scope)
     {
-        if (!is_null($this->_hooksCalled)) {
+        if (empty($this->_hooks[$scope])) {
             return;
         }
 
-        $this->_hooksCalled = true;
+        foreach ($this->_hooks[$scope] as $name) {
+            if ($this->_scopes[$scope][$name]['m'] & _PREF_LOCKED ||
+                empty($this->_scopes[$scope][$name]['v']) ||
+                $this->_scopes[$scope][$name]['m'] & _PREF_DEFAULT) {
 
-        if (!count($this->_hooks)) {
-            return;
-        }
+                $val = Horde::callHook('_prefs_hook_' . $name, array($this->_user), $scope, $name);
+                if (is_a($val, 'PEAR_Error')) {
+                    Horde::logMessage($val, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    continue;
+                }
 
-        global $registry;
-        include_once $registry->get('fileroot', 'horde') . '/config/hooks.php';
-        foreach ($this->_hooks as $pref) {
-            if ($this->isLocked($pref) ||
-                !$this->getValue($pref) ||
-                $this->isDefault($pref)) {
-                $func = '_prefs_hook_' . $pref;
-                if (function_exists($func)) {
-                    $this->_setValue($pref, $func($this->_user));
+                $this->_scopes[$scope][$name]['v'] = $this->convertToDriver($val, NLS::getCharset());
+                if ($this->_scopes[$scope][$name]['m'] & _PREF_LOCKED) {
+                    $this->_scopes[$scope][$name]['m'] |= _PREF_DIRTY;
                 }
             }
         }
@@ -729,9 +804,6 @@ class Prefs {
      * Attempts to return a concrete Prefs instance based on $driver.
      *
      * @param mixed $driver     The type of concrete Prefs subclass to return.
-     *                          If $driver is an array, then we will look in
-     *                          $driver[0]/lib/Prefs/ for the subclass
-     *                          implementation named $driver[1].php.
      * @param string $scope     The scope for this set of preferences.
      * @param string $user      The name of the user who owns this set of
      *                          preferences.
@@ -746,18 +818,6 @@ class Prefs {
     function &factory($driver, $scope = 'horde', $user = '', $password = '',
                       $params = null, $caching = true)
     {
-        if (is_array($driver)) {
-            $app = $driver[0];
-            $driver = $driver[1];
-        }
-
-        /* Attempt to register (cache) the $prefs hash in session storage. */
-        if ($caching) {
-            if (!isset($_SESSION['prefs_cache'])) {
-                $_SESSION['prefs_cache'] = array();
-            }
-        }
-
         $driver = basename($driver);
         if (empty($driver) || $driver == 'none') {
             $driver = 'session';
@@ -775,17 +835,13 @@ class Prefs {
             $user = call_user_func($params['user_hook'], $user);
         }
 
-        if (!empty($app)) {
-            require_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Prefs/' . $driver . '.php';
-        } elseif (file_exists(dirname(__FILE__) . '/Prefs/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/Prefs/' . $driver . '.php';
-        } else {
-            include_once 'Horde/Prefs/' . $driver . '.php';
-        }
-
         $class = 'Prefs_' . $driver;
+        if (!class_exists($class)) {
+            include 'Horde/Prefs/' . $driver . '.php';
+        }
         if (class_exists($class)) {
             $prefs = &new $class($user, $password, $scope, $params, $caching);
+            $prefs->retrieve($scope);
         } else {
             $prefs = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
@@ -831,6 +887,9 @@ class Prefs {
             $instances[$signature] = &Prefs::factory($driver, $scope, $user, $password, $params, $caching);
         }
 
+        /* Preferences may be cached with a different scope. */
+        $instances[$signature]->setScope($scope);
+
         return $instances[$signature];
     }
 
