@@ -3,9 +3,9 @@
  * Class for handling a list of categories stored in a user's
  * preferences.
  *
- * $Horde: framework/Prefs/Prefs/CategoryManager.php,v 1.11.10.6 2007/01/02 13:54:35 jan Exp $
+ * $Horde: framework/Prefs/Prefs/CategoryManager.php,v 1.11.10.10 2008/09/12 16:24:51 jan Exp $
  *
- * Copyright 2004-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -36,30 +36,38 @@ class Prefs_CategoryManager {
 
     function getSelect($id, $current = null)
     {
-        global $prefs;
-
-        $html = '<select id="' . htmlspecialchars($id) . '" name="' . htmlspecialchars($id) . '">';
-
         $categories = Prefs_CategoryManager::get();
+        $colors = Prefs_CategoryManager::colors();
+        $fgcolors = Prefs_CategoryManager::fgColors();
+
+        $html = '<select id="' . htmlspecialchars($id) . '" name="'
+            . htmlspecialchars($id) . '">';
+
         if (!in_array($current, $categories) && !empty($current)) {
-            $html .= '<option value="*new*' . htmlspecialchars($current) . '">' . sprintf(_("Use Current: %s"), htmlspecialchars($current)) . '</option>' .
-                '<option value="">----</option>';
+            $html .= '<option value="*new*' . htmlspecialchars($current) . '">'
+                . sprintf(_("Use Current: %s"), htmlspecialchars($current))
+                . '</option>'
+                . '<option value="" disabled="disabled">- - - - - - - - -</option>';
         }
 
-        if (!$prefs->isLocked('categories')) {
-            $html .= '<option value="*new*">' . _("New Category") . "</option>\n" .
-                '<option value="">----</option>';
+        if (!$GLOBALS['prefs']->isLocked('categories')) {
+            $html .= '<option value="*new*">' . _("New Category")
+                . "</option>\n"
+                . '<option value="" disabled="disabled">- - - - - - - - -</option>';
         }
 
         // Always add an Unfiled option.
-        $html .= '<option value=""' .
-            (empty($current) ? ' selected="selected">' : '>') .
-            htmlspecialchars(_("Unfiled")) . '</option>';
+        $html .= '<option value="" style="background:'
+            . $colors['_unfiled_'] . ';color:' . $fgcolors['_unfiled_'] . '"'
+            . (empty($current) ? ' selected="selected">' : '>')
+            . htmlspecialchars(_("Unfiled")) . '</option>';
 
         foreach ($categories as $name) {
-            $html .= '<option value="' . htmlspecialchars($name) . '"' .
-                ($name === $current ? ' selected="selected">' : '>') .
-                htmlspecialchars($name) . '</option>';
+            $html .= '<option value="' . htmlspecialchars($name)
+                . '" style="background:' . (isset($colors[$name]) ? $colors[$name] : '#fff')
+                . ';color:' . (isset($fgcolors[$name]) ? $fgcolors[$name] : '#000') . '"'
+                . ($name === $current ? ' selected="selected">' : '>')
+                . htmlspecialchars($name) . '</option>';
         }
 
         return $html . '</select>';
@@ -76,7 +84,7 @@ class Prefs_CategoryManager {
 <!--
 function checkCategory()
 {
-    if (document.$formname.$elementname.value == '*new*') {
+    if (document.${formname}['$elementname'].value == '*new*') {
         var category = window.prompt('$prompt', '');
         if (category != null && category != '') {
             document.$formname.new_category.value = category;
@@ -84,8 +92,8 @@ function checkCategory()
             window.alert('$error');
             return false;
         }
-    } else if (document.$formname.$elementname.value.indexOf('*new*') != -1) {
-        document.$formname.new_category.value = document.$formname.$elementname.value.substr(5, document.$formname.$elementname.value.length);
+    } else if (document.${formname}['$elementname'].value.indexOf('*new*') != -1) {
+        document.$formname.new_category.value = document.${formname}['$elementname'].value.substr(5, document.${formname}['$elementname'].value.length);
     }
 
     return true;
@@ -112,7 +120,7 @@ JAVASCRIPT;
 
         $categories = Prefs_CategoryManager::get();
         if (in_array($category, $categories)) {
-            return false;
+            return $category;
         }
 
         $categories[] = $category;
@@ -214,6 +222,9 @@ JAVASCRIPT;
 
         $pairs = array();
         foreach ($colors as $category => $color) {
+            if ($color[0] != '#') {
+                $color = '#' . $color;
+            }
             if (!empty($category)) {
                 $pairs[] = "$category:$color";
             }
