@@ -7,16 +7,19 @@
  * - Introspection documentation and method signatures.
  *
  * EXAMPLE:
+ * <code>
  * $response = Horde_RPC::request('xmlrpc',
- *                                '/horde/rpc.php', 'www.example.com', 80,
+ *                                'http://localhost:80/horde/rpc.php',
  *                                'contacts.search',
  *                                array(array('jan'), array('localsql'),
  *                                      array('name', 'email')),
- *                                Auth::getAuth(), Auth::getCredential('password'));
+ *                                array('user' => Auth::getAuth(),
+ *                                      'pass' => Auth::getCredential('password')));
+ * </code>
  *
- * $Horde: framework/RPC/RPC.php,v 1.7.10.4 2005/05/22 21:33:24 selsky Exp $
+ * $Horde: framework/RPC/RPC.php,v 1.7.10.16 2008/12/04 17:16:55 jwm Exp $
  *
- * Copyright 2002-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -28,31 +31,45 @@
 class Horde_RPC {
 
     /**
-     * Whether we need an authorized user or not.
+     * All driver-specific parameters.
+     *
+     * @var array
+     */
+    var $_params = array();
+
+    /**
+     * Do we need an authenticated user?
      *
-     * @access protected
      * @var boolean
      */
-    var $_authorize = true;
+    var $_requireAuthorization = true;
 
     /**
-     * RPC server constructor
+     * Whether we should exit if auth fails instead of requesting
+     * authorization credentials.
      *
-     * @access private
-     * @return object   An RPC server instance.
+     * @var boolean
      */
-    function Horde_RPC()
-    {
-        register_shutdown_function(array($this, 'shutdown'));
-    }
+    var $_requestMissingAuthorization = true;
 
     /**
-     * Cleans up the RPC server.
+     * RPC server constructor.
      *
-     * @abstract
+     * @param array $config  A hash containing any additional configuration or
+     *                       connection parameters a subclass might need.
+     *
+     * @return Horde_RPC  An RPC server instance.
      */
-    function shutdown()
+    function Horde_RPC($params = array())
     {
+        $this->_params = $params;
+
+        if (isset($params['requireAuthorization'])) {
+            $this->_requireAuthorization = $params['requireAuthorization'];
+        }
+        if (isset($params['requestMissingAuthorization'])) {
+            $this->_requestMissingAuthorization = $params['requestMissingAuthorization'];
+        }
     }
 
     /**
@@ -67,7 +84,7 @@ class Horde_RPC {
      */
     function authorize()
     {
-        if (!$this->_authorize) {
+        if (!$this->_requireAuthorization) {
             return true;
         }
 
@@ -76,11 +93,19 @@ class Horde_RPC {
         if (isset($_SERVER['PHP_AUTH_USER'])) {
             $user = $_SERVER['PHP_AUTH_USER'];
             $pass = $_SERVER['PHP_AUTH_PW'];
+        } elseif (isset($_SERVER['Authorization'])) {
+            $hash = str_replace('Basic ', '', $_SERVER['Authorization']);
+            $hash = base64_decode($hash);
+            if (strpos($hash, ':') !== false) {
+                list($user, $pass) = explode(':', $hash, 2);
+            }
         }
 
         if (!isset($user)
             || !$auth->authenticate($user, array('password' => $pass))) {
-            header('WWW-Authenticate: Basic realm="Horde RPC"');
+            if ($this->_requestMissingAuthorization) {
+                header('WWW-Authenticate: Basic realm="Horde RPC"');
+            }
             header('HTTP/1.0 401 Unauthorized');
             echo '401 Unauthorized';
             exit;
@@ -118,11 +143,11 @@ class Horde_RPC {
      */
     function getResponse($request)
     {
-        return _("not implemented");
+        return 'not implemented';
     }
 
     /**
-     * Get the Content-Type of the response.
+     * Returns the Content-Type of the response.
      *
      * @return string  The MIME Content-Type of the RPC response.
      */
@@ -136,8 +161,8 @@ class Horde_RPC {
      *
      * This statically called method is actually the RPC client.
      *
-     * @param string $driver    The protocol driver to use. Currently 'soap'
-     *                          and 'xmlrpc' are available.
+     * @param string $driver    The protocol driver to use. Currently 'soap',
+     *                          'xmlrpc' and 'jsonrpc' are available.
      * @param string $url       The path to the RPC server on the called host.
      * @param string $method    The method to call.
      * @param array $params     A hash containing any necessary parameters for
@@ -150,24 +175,16 @@ class Horde_RPC {
      */
     function request($driver, $url, $method, $params = null, $options = array())
     {
-        if (is_array($driver)) {
-            list($app, $driver) = $driver;
-        }
-
         $driver = basename($driver);
-
-        if (!empty($app)) {
-            require_once $app . '/lib/RPC/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/RPC/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/RPC/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/RPC/' . $driver . '.php';
-        }
         $class = 'Horde_RPC_' . $driver;
+        if (!class_exists($class)) {
+            include 'Horde/RPC/' . $driver . '.php';
+        }
+
         if (class_exists($class)) {
             return call_user_func(array($class, 'request'), $url, $method, $params, $options);
         } else {
-            require_once 'PEAR.php';
+            include_once 'PEAR.php';
             return PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
     }
@@ -184,64 +201,26 @@ class Horde_RPC {
      *                       connection parameters a subclass might need.
      *
      * @return Horde_RPC  The newly created concrete Horde_RPC server instance,
-     *                    or a PEAR_Error on an error.
+     *                    or PEAR_Error on error.
      */
-    function &factory($driver, $params = null)
+    function factory($driver, $params = null)
     {
-        if (is_array($driver)) {
-            list($app, $driver) = $driver;
-        }
-
         $driver = basename($driver);
-
-        if (!empty($app)) {
-            require_once $app . '/lib/RPC/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/RPC/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/RPC/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/RPC/' . $driver . '.php';
+        if ($driver == 'soap' && class_exists('SoapServer')) {
+            $driver = 'PhpSoap';
         }
+
         $class = 'Horde_RPC_' . $driver;
+        if (!class_exists($class)) {
+            include 'Horde/RPC/' . $driver . '.php';
+        }
+
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            return new $class($params);
         } else {
-            require_once 'PEAR.php';
+            include_once 'PEAR.php';
             return PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
     }
 
-    /**
-     * Attempts to return a reference to a concrete RPC server
-     * instance based on $driver. It will only create a new instance
-     * if no RPC server instance with the same parameters currently
-     * exists.
-     *
-     * This should be used if multiple RPC servers (and thus, multiple RPC
-     * server instances) are required.
-     *
-     * This method must be invoked as: $var = &Horde_RPC::singleton()
-     *
-     * @param string $driver  The type of concrete RPC subclass to return.
-     * @param array $params   A hash containing any additional configuration or
-     *                        connection parameters a subclass might need.
-     *
-     * @return Horde_RPC  The concrete Horde_RPC server reference, or a
-     *                    PEAR_Error on an error.
-     */
-    function &singleton($driver, $params = null)
-    {
-        static $instances;
-
-        if (!isset($instances)) {
-            $instances = array();
-        }
-
-        $signature = serialize(array($driver, $params));
-        if (!array_key_exists($signature, $instances)) {
-            $instances[$signature] = &Horde_RPC::factory($driver, $params);
-        }
-
-        return $instances[$signature];
-    }
-
 }
