@@ -3,9 +3,9 @@
  * The Horde_RPC_soap class provides an SOAP implementation of the
  * Horde RPC system.
  *
- * $Horde: framework/RPC/RPC/soap.php,v 1.13.10.10 2007/01/02 13:54:36 jan Exp $
+ * $Horde: framework/RPC/RPC/soap.php,v 1.13.10.16 2008/10/28 10:45:20 jan Exp $
  *
- * Copyright 2003-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -23,6 +23,27 @@ class Horde_RPC_soap extends Horde_RPC {
      */
     var $_server;
 
+    /**
+     * List of types to emit in the WSDL.
+     *
+     * @var array
+     */
+    var $_allowedTypes = array();
+
+    /**
+     * List of method names to allow.
+     *
+     * @var array
+     */
+    var $_allowedMethods = array();
+
+    /**
+     * Name of the SOAP service to use in the WSDL.
+     *
+     * @var string
+     */
+    var $_serviceName = null;
+
     /**
      * Hash holding all methods' signatures.
      *
@@ -37,24 +58,20 @@ class Horde_RPC_soap extends Horde_RPC {
      */
     function Horde_RPC_soap($params = null)
     {
-        if ($params !== null) {
-            // BC: $params should be an array, but until 3.1.4 $params required
-            //     a string here.
-            if (is_string($params)) {
-                if (($params == 'wsdl') || ($params == 'disco')) {
-                    $this->_authorize = false;
-                }
-            } else {
-                if (!empty($params['wsdl']) || !empty($params['disco'])) {
-                    $this->_authorize = false;
-                }
-            }
+        parent::Horde_RPC($params);
+
+        if (!empty($params['allowedTypes'])) {
+            $this->_allowedTypes = $params['allowedTypes'];
+        }
+        if (!empty($params['allowedMethods'])) {
+            $this->_allowedMethods = $params['allowedMethods'];
+        }
+        if (!empty($params['serviceName'])) {
+            $this->_serviceName = $params['serviceName'];
         }
 
-        parent::Horde_RPC();
-
         require_once 'SOAP/Server.php';
-        $this->_server = &new SOAP_Server();
+        $this->_server = new SOAP_Server();
         $this->_server->_auto_translation = true;
     }
 
@@ -72,6 +89,10 @@ class Horde_RPC_soap extends Horde_RPC {
             if (!is_array($signature)) {
                 continue;
             }
+            if (!empty($this->_allowedMethods) &&
+                !in_array($method, $this->_allowedMethods)) {
+                continue;
+            }
             $method = str_replace('/', '.', $method);
             $this->__dispatch_map[$method] = array(
                 'in' => $signature[0],
@@ -79,7 +100,15 @@ class Horde_RPC_soap extends Horde_RPC {
             );
         }
 
-        $this->__typedef = $registry->listTypes();
+        $this->__typedef = array();
+        foreach ($registry->listTypes() as $type => $params) {
+            if (!empty($this->_allowedTypes) &&
+                !in_array($type, $this->_allowedTypes)) {
+                continue;
+            }
+
+            $this->__typedef[$type] = $params;
+        }
     }
 
     /**
@@ -108,6 +137,13 @@ class Horde_RPC_soap extends Horde_RPC {
      * Will be registered as the handler for all methods called in the
      * SOAP server and will call the appropriate function through the registry.
      *
+     * @todo  PEAR SOAP operates on a copy of this object at some unknown
+     *        point and therefore doesn't have access to instance
+     *        variables if they're set here. Instead, globals are used
+     *        to track the method name and args for the logging code.
+     *        Once this is PHP 5-only, the globals can go in favor of
+     *        instance variables.
+     *
      * @access private
      *
      * @param string $method    The name of the method called by the RPC request.
@@ -121,6 +157,15 @@ class Horde_RPC_soap extends Horde_RPC {
         global $registry;
         $method = str_replace('.', '/', $method);
 
+        if (!empty($this->_params['allowedMethods']) &&
+            !in_array($method, $this->_params['allowedMethods'])) {
+            return sprintf(_("Method \"%s\" is not defined"), $method);
+        }
+
+        $GLOBALS['__horde_rpc_soap']['lastMethodCalled'] = $method;
+        $GLOBALS['__horde_rpc_soap']['lastMethodParams'] =
+            !empty($params) ? $params : array();
+
         if (!$registry->hasMethod($method)) {
             return sprintf(_("Method \"%s\" is not defined"), $method);
         }
@@ -133,38 +178,44 @@ class Horde_RPC_soap extends Horde_RPC {
      * Takes an RPC request and returns the result.
      *
      * @param string  The raw request string.
-     * @param string  String specifying what kind of data to return. Defaults
-     *                to SOAP request. Possible other values: "wsdl" and
-     *                "disco".
      *
      * @return string  The XML encoded response from the server.
      */
-    function getResponse($request, $params = null)
+    function getResponse($request)
     {
         $this->_server->addObjectMap($this, 'urn:horde');
 
-        if (!$params) {
-            $this->_server->setCallHandler(array($this, '_dispatcher'));
-            /* We can't use Util::bufferOutput() here for some reason. */
-            ob_start();
-            $this->_server->service($request);
-            $output = ob_get_contents();
-            ob_end_clean();
-        } else {
+        if ($request == 'disco' || $request == 'wsdl') {
             require_once 'SOAP/Disco.php';
-            $disco = new SOAP_DISCO_Server($this->_server, 'horde');
-            // BC: $params should be an array, but until 3.1.4 $params
-            //     required a string here.
-            if ((is_string($params) && $params == 'wsdl') ||
-                (is_array($params) && !empty($params['wsdl']))) {
+            $disco = new SOAP_DISCO_Server($this->_server,
+                !empty($this->_serviceName) ? $this->_serviceName : 'horde');
+            if ($request == 'wsdl') {
                 $this->_setupDispatchMap();
-                $output = $disco->getWSDL();
+                return $disco->getWSDL();
             } else {
-                $output = $disco->getDISCO();
+                return $disco->getDISCO();
             }
         }
 
-        return $output;
+        $this->_server->setCallHandler(array($this, '_dispatcher'));
+
+        /* We can't use Util::bufferOutput() here for some reason. */
+        $beginTime = time();
+        ob_start();
+        $this->_server->service($request);
+        Horde::logMessage(
+            sprintf('SOAP call: %s(%s) by %s serviced in %d seconds, sent %d bytes in response',
+                    $GLOBALS['__horde_rpc_soap']['lastMethodCalled'],
+                    is_array($GLOBALS['__horde_rpc_soap']['lastMethodParams'])
+                        ? implode(', ', array_map(create_function('$a', 'return is_array($a) ? "Array" : $a;'),
+                                                  $GLOBALS['__horde_rpc_soap']['lastMethodParams']))
+                        : '',
+                    Auth::getAuth(),
+                    time() - $beginTime,
+                    ob_get_length()),
+            __FILE__, __LINE__, PEAR_LOG_INFO
+        );
+        return ob_get_clean();
     }
 
     /**
@@ -209,7 +260,7 @@ class Horde_RPC_soap extends Horde_RPC {
         }
 
         require_once 'SOAP/Client.php';
-        $soap = &new SOAP_Client($url, false, false, $options);
+        $soap = new SOAP_Client($url, false, false, $options);
         return $soap->call($method, $params, $options['namespace']);
     }
 
