@@ -11,12 +11,12 @@ require_once 'Horde.php';
  * entities -- If true the html entity versions of the patterns will be used.
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/emoticons.php,v 1.17.10.4 2005/08/18 16:40:09 slusarz Exp $
+ * $Horde: framework/Text_Filter/Filter/emoticons.php,v 1.17.10.14 2008/01/02 11:30:54 jan Exp $
  *
- * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did not
- * receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Marko Djukic <marko@oblo.com>
  * @package Horde_Text
@@ -37,40 +37,68 @@ class Text_Filter_emoticons extends Text_Filter {
      */
     function getPatterns()
     {
-        $regexp = array();
-        $iconpath = $GLOBALS['registry']->getImageDir('horde') . '/emoticons';
-
-        /* Loop through possible string emoticons and convert to graphics. */
-        foreach ($this->getIcons() as $string => $icon) {
-            if ($this->_params['entities']) {
-                $string = htmlspecialchars($string);
-            }
-
-            /* Check for a smiley either immediately at the start of a line or
-             * following a space. Use {} as the preg delimiters as this is not
-             * found in any smiley. */
-            $regexp['{(^|\s)' . preg_quote($string) . '}'] = ' ' . Horde::img($icon . '.png', $string, 'align="middle"', $iconpath);
+        /* Build the patterns. */
+        $patterns = array_keys($this->getIcons());
+        $beg_pattern = '(^|\s)(';
+        $end_pattern = ')(?=\s)';
+        if ($this->_params['entities']) {
+            $patterns = array_map('htmlspecialchars', $patterns);
+            $beg_pattern = '(^|\s|<br />|&nbsp;)(';
+            $end_pattern = ')(?=\s|<br />|&nbsp;)';
         }
+        $patterns = array_map('preg_quote', $patterns);
+
+        /* Check for a smiley either immediately at the start of a line or
+         * following a space. Use {} as the preg delimiters as this is not
+         * found in any smiley. */
+        $regexp['{' . $beg_pattern . implode('|', $patterns) . $end_pattern . '}e'] = 'Text_Filter_emoticons::getImage(\'$2\', \'$1\', \'' . ($this->_params['entities'] ? '$3' : '') . '\')';
 
         return array('regexp' => $regexp);
     }
 
     /**
-     * Returns a hash with emoticons and names.
+     * Returns the img tag for an emoticon.
      *
-     * @return array  Patterns hash.
+     * @see Text_Filter_emoticons::getPatterns()
+     * @since Horde 3.1.4
+     *
+     * @param string $icon  The emoticon.
+     * @param string $prefix  A html prefix.
+     * @param string $postfix  A html postfix.
+     *
+     * @return string  HTML code with the image tag and any additional prefix
+     *                 or postfix.
      */
-    function getIcons()
+    function getImage($icon, $prefix, $postfix)
+    {
+        static $iconpath;
+        if (empty($iconpath)) {
+            $iconpath = $GLOBALS['registry']->getImageDir('horde') . '/emoticons';;
+        }
+
+        return $prefix . Horde::img(Text_Filter_emoticons::getIcons($icon) . '.png', $icon, array('align' => 'middle', 'title' => $icon), $iconpath) . $postfix;
+    }
+
+    /**
+     * Returns a hash with all emoticons and names or the name of a single
+     * emoticon.
+     *
+     * @param string $icon  If set, return the name for that emoticon only.
+     *                      @since Horde 3.1.4
+     *
+     * @return array|string  Patterns hash or icon name.
+     */
+    function getIcons($icon = null)
     {
         /* List complex strings before simpler ones, otherwise for example :((
          * would be matched against :( before :(( is found. */
-        return array(
+        static $icons = array(
             ':/' => 'frustrated', ':-/' => 'frustrated',
             // ':*>' => 'blush',
             ':e' => 'disappointed',
             '=:)$' => 'mrt',
             '#|' => 'hangover', '#-|' => 'hangover',
-            ':-@' => 'shout',
+            ':-@' => 'shout', ':@' => 'shout',
             ':((' => 'bigfrown', ':C' => 'bigfrown',
             ':S' => 'dazed', ':-S' => 'dazed',
             'X@' => 'angry',
@@ -89,8 +117,8 @@ class Text_Filter_emoticons extends Text_Filter {
             // ':&' => 'annoyed',
             '=(' => 'worried', '=-(' => 'worried',
             ':|=' => 'vampire',
-            ':-(' => 'frown',
-            ':D' => 'biggrin',
+            ':-(' => 'frown', ':(' => 'frown',
+            ':D' => 'biggrin', ':-D' => 'biggrin', ':d' => 'biggrin', ':-d' => 'biggrin',
             '8)' => 'cool',
             // In English, 8PM occurs sufficiently often to specifically
             // search for and exclude
@@ -107,14 +135,16 @@ class Text_Filter_emoticons extends Text_Filter {
             '|I' => 'asleep', '|-I' => 'asleep',
             'O:)' => 'angel', 'O:-)' => 'angel',
             'O;)' => 'angelwink',
-            ';-)' => 'wink',
+            ';-)' => 'wink', ';)' => 'wink',
             ':#)' => 'clown', ':o)' => 'clown',
-            ':)' => 'smile',
-            ';)' => 'wink',
-            ':-)' => 'smile',
-            ':@' => 'shout',
-            ':(' => 'frown',
+            ':)' => 'smile', ':-)' => 'smile',
         );
+
+        if ($icon) {
+            return isset($icons[$icon]) ? $icons[$icon] : null;
+        } else {
+            return $icons;
+        }
     }
 
 }
