@@ -1,6 +1,7 @@
 <?php
 
 require_once 'Horde.php';
+require_once 'Horde/String.php';
 
 /**
  * Turn text into HTML with varying levels of parsing.  For no html
@@ -33,13 +34,12 @@ require_once 'Horde.php';
  *                              Included for completeness.
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/text2html.php,v 1.4.2.7 2007/01/02 13:54:43 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/text2html.php,v 1.4.2.14 2008/12/09 04:25:59 slusarz Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2004-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -57,6 +57,21 @@ class Text_Filter_text2html extends Text_Filter {
                          'nofollow' => false,
                          'callback' => 'Horde::externalUrl');
 
+    /**
+     * Constructor.
+     *
+     * @param array $params  Any parameters that the filter instance needs.
+     */
+    function Text_Filter_text2html($params = array())
+    {
+        parent::Text_Filter($params);
+
+        // Use ISO-8859-1 instead of US-ASCII
+        if (String::lower($this->_params['charset']) == 'us-ascii') {
+            $this->_params['charset'] = 'iso-8859-1';
+        }
+    }
+
     /**
      * Executes any code necessary before applying the filter patterns.
      *
@@ -67,7 +82,9 @@ class Text_Filter_text2html extends Text_Filter {
     function preProcess($text)
     {
         if (is_null($this->_params['charset'])) {
-            $this->_params['charset'] = isset($GLOBALS['_HORDE_STRING_CHARSET']) ? $GLOBALS['_HORDE_STRING_CHARSET'] : 'ISO-8859-1';
+            $this->_params['charset'] = isset($GLOBALS['_HORDE_STRING_CHARSET'])
+                ? $GLOBALS['_HORDE_STRING_CHARSET']
+                : 'ISO-8859-1';
         }
 
         /* Abort out on simple cases. */
@@ -78,53 +95,44 @@ class Text_Filter_text2html extends Text_Filter {
             return @htmlspecialchars($text, ENT_COMPAT, $this->_params['charset']);
         }
 
-        /* Do in-lining of http://xxx.xxx to link, xxx@xxx.xxx to email, part
-         * one. */
         if ($this->_params['parselevel'] < TEXT_HTML_NOHTML) {
-            /* Make sure that the original message doesn't contain any capital
-             * </A> tags or open <A> tags , so we can assume we generated
-             * them. */
-            $text = str_replace(array('</A>', '<A'), array('</a>', '<a'), $text);
             $filter_array = array('linkurls');
-            $filter_params = array(array('callback' => $this->_params['callback'],
-                                         'nofollow' => $this->_params['nofollow'],
-                                         'capital' => true));
+            $filter_params = array(
+                array('callback' => $this->_params['callback'],
+                      'nofollow' => $this->_params['nofollow'],
+                      'encode' => true));
             if ($this->_params['parselevel'] < TEXT_HTML_MICRO_LINKURL) {
                 $filter_array[] = 'emails';
-                $filter_params[] = array('capital_tags' => true);
+                $filter_params[] = array('encode' => true);
             }
             $text = Text_Filter::filter($text, $filter_array, $filter_params);
         }
 
         /* For level TEXT_HTML_MICRO, TEXT_HTML_NOHTML, start with
          * htmlspecialchars(). */
-        $text = @htmlspecialchars($text, ENT_COMPAT, $this->_params['charset']);
+        $old_error = error_reporting(0);
+        $text2 = htmlspecialchars($text, ENT_COMPAT, $this->_params['charset']);
+        /* Bad charset input in may result in an empty string. If so, try
+         * using the default charset encoding instead. */
+        if (!$text2) {
+            $text2 = htmlspecialchars($text, ENT_COMPAT);
+        }
+        $text = $text2;
+        error_reporting($old_error);
 
-        /* Do in-lining of http://xxx.xxx to link, xxx@xxx.xxx to email, part
-         * two. */
+        /* Do in-lining of http://xxx.xxx to link, xxx@xxx.xxx to email. */
         if ($this->_params['parselevel'] < TEXT_HTML_NOHTML) {
-            $syntax = array(
-                '&lt;A href=&quot;' => '<a' . (empty($this->_params['class']) ? '' : ' class="' . $this->_params['class'] . '"') . ' href="',
-                '&quot; rel=&quot;' => '" rel="',
-                '&quot; title=&quot;' => '" title="',
-                '&quot; target=&quot;_blank&quot;&gt;'  => '" target="_blank">',
-                '&quot; onclick=&quot;' => '" onclick="',
-                '\');&quot;&gt;' => '\');">',
-                '&quot;&gt;' =>  '">',
-                /* Only reconvert capital /A tags - the ones we generated. */
-                '&lt;/A&gt;' => '</a>'
-                );
+            $text = Text_Filter_linkurls::decode($text);
+            if ($this->_params['parselevel'] < TEXT_HTML_MICRO_LINKURL) {
+                $text = Text_Filter_emails::decode($text);
+            }
 
-            $text = str_replace(array_keys($syntax), $syntax, $text);
             $text = Text_Filter::filter($text, 'space2html');
         }
 
-        /* Do the blank-line ---> <br /> substitution.  Everybody gets this;
-         * if you don't want even that, just save the htmlspecialchars()
-         * version of the input. */
-        $text = nl2br($text);
-
-        return $text;
+        /* Do the newline ---> <br /> substitution. Everybody gets this; if
+         * you don't want even that, just use htmlspecialchars(). */
+        return nl2br($text);
     }
 
 }
