@@ -9,12 +9,12 @@
  * replacement -- The replacement string.  Defaults to "*****".
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/words.php,v 1.2.10.6 2007/01/02 13:54:43 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/words.php,v 1.2.10.8 2008/01/02 11:30:54 jan Exp $
  *
- * Copyright 2004-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
  * @since   Horde 3.0
@@ -38,7 +38,7 @@ class Text_Filter_words extends Text_Filter {
     {
         $regexp = array();
 
-        if (@is_readable($this->_params['words_file'])) {
+        if (is_readable($this->_params['words_file'])) {
             /* Read the file and iterate through the lines. */
             $lines = file($this->_params['words_file']);
             foreach ($lines as $line) {
@@ -48,7 +48,7 @@ class Text_Filter_words extends Text_Filter {
 
                 /* Filter the text. */
                 if (!empty($line)) {
-                    $regexp["/(\b(\w*)$line\b|\b$line(\w*)\b)/i"] = $this->_params['replacement'];
+                    $regexp["/(\b(\w*)$line\b|\b$line(\w*)\b)/i"] = $this->_getReplacement($line);
                 }
             }
         }
@@ -56,4 +56,18 @@ class Text_Filter_words extends Text_Filter {
         return array('regexp' => $regexp);
     }
 
+    /**
+     * Returns a hash with replace patterns.
+     *
+     * @return array  Patterns hash.
+     */
+    function _getReplacement($line)
+    {
+        if ($this->_params['replacement']) {
+            return $this->_params['replacement'];
+        } else {
+            return substr($line, 0, 1) . str_repeat('*', strlen($line) - 1);
+        }
+    }
+
 }
