@@ -22,12 +22,12 @@ define('HORDE_TREE_TOGGLE', 'ht_toggle_');
  * can work together with the Horde_Tree javascript class to achieve this in
  * DHTML on supported browsers.
  *
- * Copyright 2003-2007 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Tree/Tree.php,v 1.46.6.13 2007/01/11 06:19:58 chuck Exp $
+ * $Horde: framework/Tree/Tree.php,v 1.46.6.17 2008/08/11 12:50:36 jan Exp $
  *
  * @author  Marko Djukic <marko@oblo.com>
  * @package Horde_Tree
@@ -166,7 +166,7 @@ class Horde_Tree {
 
         $class = 'Horde_Tree_' . $renderer;
         if (class_exists($class)) {
-            $tree = &new $class($name, $params);
+            $tree = new $class($name, $params);
         } else {
             $tree = PEAR::raiseError(sprintf(_("\"%s\" tree renderer not found."), $renderer));
         }
@@ -301,7 +301,7 @@ class Horde_Tree {
      * @param array   $extra_left   Any other columns to display to the left of
      *                              the tree.
      */
-    function addNode($id, $parent, $label, $indent, $expanded = true,
+    function addNode($id, $parent, $label, $indent = null, $expanded = true,
                      $params = array(), $extra_right = array(),
                      $extra_left = array())
     {
@@ -372,6 +372,7 @@ class Horde_Tree {
      * onclick   --  Onclick event attached to this node
      * url       --  URL to link the node to
      * urlclass  --  CSS class for the node's URL
+     * title     --  Link tooltip title
      * target    --  Target for the 'url' link
      * </pre>
      */
@@ -383,7 +384,7 @@ class Horde_Tree {
 
         $allowed = array(
             'class', 'icon', 'iconalt', 'icondir', 'iconopen',
-            'onclick', 'url', 'urlclass', 'target',
+            'onclick', 'url', 'urlclass', 'title', 'target',
         );
 
         foreach ($params as $param_id => $param_val) {
