@@ -1,15 +1,19 @@
 <?php
+
+include_once 'Horde/NLS.php';
+
 /**
  * The Horde_UI_VarRenderer:: class provides base functionality for
  * other Horde UI elements.
  *
- * $Horde: framework/UI/UI/VarRenderer.php,v 1.12.10.11 2007/01/02 13:54:46 jan Exp $
+ * $Horde: framework/UI/UI/VarRenderer.php,v 1.12.10.13 2008/01/02 11:30:58 jan Exp $
  *
- * Copyright 2003-2007 Jason M. Felice <jfelice@cronosys.com>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
+ * @author  Jason M. Felice <jason.m.felice@gmail.com>
  * @since   Horde_UI 0.0.1
  * @package Horde_UI
  */
@@ -61,18 +65,17 @@ class Horde_UI_VarRenderer {
         }
 
         $driver = basename($driver);
-
-        if (!empty($app)) {
-            require_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/UI/VarRenderer/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/VarRenderer/' . $driver . '.php')) {
-            require_once dirname(__FILE__) . '/VarRenderer/' . $driver . '.php';
-        } else {
-            @include_once 'Horde/UI/VarRenderer/' . $driver . '.php';
+        $class = 'Horde_UI_VarRenderer_' . $driver;
+        if (!class_exists($class)) {
+            if (!empty($app)) {
+                include $GLOBALS['registry']->get('fileroot', $app) . '/lib/UI/VarRenderer/' . $driver . '.php';
+            } else {
+                include 'Horde/UI/VarRenderer/' . $driver . '.php';
+            }
         }
 
-        $class = 'Horde_UI_VarRenderer_' . $driver;
         if (class_exists($class)) {
-            $vr = &new $class($params);
+            $vr = new $class($params);
         } else {
             $vr = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
@@ -80,41 +83,16 @@ class Horde_UI_VarRenderer {
         return $vr;
     }
 
-    /**
-     * Returns a Horde_UI_VarRenderer:: instance, constructing one with the
-     * specified parameters if necessary.
-     *
-     * @param string $driver  This is the renderer subclass we will
-     *                        instantiate.
-     * @param array $params   Parameters specific to the subclass.
-     *
-     * @return Horde_UI_VarRenderer  A Horde_UI_VarRenderer:: subclass
-     *                               instance.
-     */
-    function &singleton($driver, $params = array())
-    {
-        static $cache;
-
-        if (is_null($driver)) {
-            $class = 'Horde_UI_VarRenderer';
-        }
-        $key = serialize(array ($driver, $params));
-        if (!isset($cache[$key])) {
-            $cache[$key] = &Horde_UI_VarRenderer::factory($driver, $params);
-        }
-        return $cache[$key];
-    }
-
     /**
      * Renders a variable.
      *
-     * @param Horde_Form &$form           Reference to a Horde_Form instance,
+     * @param Horde_Form $form            A Horde_Form instance,
      *                                    or null if none is available.
      * @param Horde_Form_Variable &$var   Reference to a Horde_Form_Variable.
      * @param Variables &$vars            A Variables instance.
      * @param boolean $isInput            Whether this is an input field.
      */
-    function render(&$form, &$var, &$vars, $isInput = false)
+    function render($form, &$var, &$vars, $isInput = false)
     {
         if ($isInput) {
             $state = 'Input';
@@ -122,7 +100,7 @@ class Horde_UI_VarRenderer {
             $state = 'Display';
         }
         $method = "_renderVar${state}_" . $var->type->getTypeName();
-        if (!@method_exists($this, $method)) {
+        if (!method_exists($this, $method)) {
             $method = "_renderVar${state}Default";
         }
         return $this->$method($form, $var, $vars);
