@@ -13,10 +13,9 @@ define('HORDE_ERROR_DRIVER_CONFIG', 2);
 /**
  * The Util:: class provides generally useful methods of different kinds.
  *
- * $Horde: framework/Util/Util.php,v 1.384.6.9.2.1 2006/03/28 12:33:32 jan Exp $
+ * $Horde: framework/Util/Util.php,v 1.384.6.33 2008/10/30 14:35:03 chuck Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -37,19 +36,29 @@ class Util {
      */
     function &cloneObject(&$obj)
     {
-        /* Cloning does *not* currently work if using PHP 5.
-         * This is a known PHP issue: http://bugs.php.net/bug.php?id=27268
-         * Therefore, work around this issue by using the PHP 4
-         * serialize/unserialize clone hack. */
-        /*
-        if (version_compare(zend_version(), '2', '>')) {
-            return clone($obj);
-        } else {
-            $newObj = $obj;
-            return $newObj;
+        if (!is_object($obj)) {
+            $bt = debug_backtrace();
+            if (isset($bt[1])) {
+                $caller = $bt[1]['function'];
+                if (isset($bt[1]['class'])) {
+                    $caller = $bt[1]['class'].$bt[1]['type'].$caller;
+                }
+            } else {
+                $caller = 'main';
+            }
+            $caller .= ' on line ' . $bt[0]['line'] . ' of ' . $bt[0]['file'];
+            Horde::logMessage('Util::cloneObject called on variable of type ' . gettype($obj) . ' by ' . $caller, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $ret = $obj;
+            return $ret;
         }
-        */
-        $ret = unserialize(serialize($obj));
+
+        if (version_compare(zend_version(), '2', '>')) {
+            $ret = clone($obj);
+        } else {
+            $ret = unserialize(serialize($obj));
+        }
+
         return $ret;
     }
 
@@ -110,10 +119,8 @@ class Util {
         } else {
             call_user_func_array($function, $args);
         }
-        $output = ob_get_contents();
-        ob_end_clean();
 
-        return $output;
+        return ob_get_clean();
     }
 
     /**
@@ -122,18 +129,21 @@ class Util {
      * scope.
      *
      * @param string $varname  The variable name to check.
+     * @param mixed $default   Default value if the variable isn't present
+     *                         or was specified by the user. Defaults to null.
+     *                         @since Horde 3.1
      *
-     * @return mixed  Null if the var is in user input, the variable value
-     *                otherwise.
+     * @return mixed  $default if the var is in user input or not present,
+     *                the variable value otherwise.
      */
-    function nonInputVar($varname)
+    function nonInputVar($varname, $default = null)
     {
         if (isset($_GET[$varname]) ||
             isset($_POST[$varname]) ||
             isset($_COOKIE[$varname])) {
-            return null;
+            return $default;
         } else {
-            return isset($GLOBALS[$varname]) ? $GLOBALS[$varname] : null;
+            return isset($GLOBALS[$varname]) ? $GLOBALS[$varname] : $default;
         }
     }
 
@@ -149,12 +159,9 @@ class Util {
      *                          an array of name/value pairs.
      * @param string $value     If specified, the value part ($parameter is
      *                          then assumed to just be the parameter name).
-     * @param boolean $encode   If true, and we don't have argument separators
-     *                          yet, the argument separator gets encoded.
+     * @param boolean $encode   Encode the argument separator?
      *
      * @return string  The modified URL.
-     *
-     * @since Horde 2.1
      */
     function addParameter($url, $parameter, $value = null, $encode = true)
     {
@@ -162,53 +169,54 @@ class Util {
             return $url;
         }
 
-        if (!is_array($parameter)) {
-            /* This is deprecated should be removed in the future. */
-            if (is_null($value)) {
-                @list($parameter, $value) = explode('=', $parameter, 2);
-            }
-            $add = array($parameter => $value);
-        } else {
-            $add = $parameter;
-        }
-
+        $add = array();
         $arg = $encode ? '&amp;' : '&';
-        if (($pos = strpos($url, '?')) === false) {
-            $glue = '?';
-        } else {
+
+        if (strpos($url, '?') !== false) {
+            list($url, $query) = explode('?', $url);
+
             /* Check if the argument separator has been already
              * htmlentities-ized in the URL. */
-            $query = substr($url, $pos + 1);
             if (preg_match('/=.*?&amp;.*?=/', $query)) {
+                $query = html_entity_decode($query);
                 $arg = '&amp;';
-                $query = strtr($query, array_flip(get_html_translation_table(HTML_ENTITIES)));
             } elseif (preg_match('/=.*?&.*?=/', $query)) {
                 $arg = '&';
             }
-            $pairs = explode($arg, $query);
-            $params = array();
+            $pairs = explode('&', $query);
             foreach ($pairs as $pair) {
-                $pair = explode('=', $pair, 2);
-                $params[$pair[0]] = count($pair) == 2 ? $pair[1] : '';
+                $pair = explode('=', urldecode($pair), 2);
+                $pair_val = (count($pair) == 2) ? $pair[1] : '';
+                if (substr($pair[0], -2) == '[]') {
+                    $name = substr($pair[0], 0, -2);
+                    if (!isset($add[$name])) {
+                        $add[$name] = array();
+                    }
+                    $add[$name][] = $pair_val;
+                } else {
+                    $add[$pair[0]] = $pair_val;
+                }
             }
-            $glue = $arg;
+        }
+        if (is_array($parameter)) {
+            $add = array_merge($add, $parameter);
+        } else {
+            $add[$parameter] = $value;
         }
 
         $url_params = array();
         foreach ($add as $parameter => $value) {
-            if (!isset($params[$parameter])) {
-                if (is_array($value)) {
-                    foreach ($value as $val) {
-                        $url_params[] = urlencode($parameter) . '[]=' . urlencode($val);
-                    }
-                } else {
-                    $url_params[] = urlencode($parameter) . '=' . urlencode($value);
+            if (is_array($value)) {
+                foreach ($value as $val) {
+                    $url_params[] = urlencode($parameter) . '[]=' . urlencode($val);
                 }
+            } else {
+                $url_params[] = urlencode($parameter) . '=' . urlencode($value);
             }
         }
 
         if (count($url_params)) {
-            return $url . $glue . implode($arg, $url_params);
+            return $url . '?' . implode($arg, $url_params);
         } else {
             return $url;
         }
@@ -222,8 +230,6 @@ class Util {
      *                       of parameters to remove.
      *
      * @return string  The modified URL.
-     *
-     * @since Horde 2.2
      */
     function removeParameter($url, $remove)
     {
@@ -243,7 +249,7 @@ class Util {
          * htmlentities-ized in the URL. */
         if (preg_match('/=.*?&amp;.*?=/', $query)) {
             $entities = true;
-            $query = strtr($query, array_flip(get_html_translation_table(HTML_ENTITIES)));
+            $query = html_entity_decode($query);
         }
 
         /* Get the list of parameters. */
@@ -288,11 +294,13 @@ class Util {
      * Returns a url with the 'nocache' parameter added, if the browser is
      * buggy and caches old URLs.
      *
-     * @param string $url  The URL to modify.
+     * @param string $url       The URL to modify.
+     * @param boolean $encode   Encode the argument separator? (since
+     *                          Horde 3.2)
      *
      * @return string  The requested URI.
      */
-    function nocacheUrl($url)
+    function nocacheUrl($url, $encode = true)
     {
         static $rand_num;
 
@@ -305,7 +313,7 @@ class Util {
             if (!isset($rand_num)) {
                 $rand_num = base_convert(microtime(), 10, 36);
             }
-            return Util::addParameter($url, 'nocache', $rand_num);
+            return Util::addParameter($url, 'nocache', $rand_num, $encode);
         } else {
             return $url;
         }
@@ -391,8 +399,6 @@ class Util {
      *                         there.
      *
      * @return string  The cleaned form variable, or $default.
-     *
-     * @since Horde 2.2
      */
     function getGet($var, $default = null)
     {
@@ -410,8 +416,6 @@ class Util {
      *                         there.
      *
      * @return string  The cleaned form variable, or $default.
-     *
-     * @since Horde 2.2
      */
     function getPost($var, $default = null)
     {
@@ -481,15 +485,15 @@ class Util {
 
         $tmp_file = tempnam($tmp_dir, $prefix);
 
-        /* If the file was created, then register it for deletion and return */
+        /* If the file was created, then register it for deletion and return. */
         if (empty($tmp_file)) {
             return false;
-        } else {
-            if ($delete) {
-                Util::deleteAtShutdown($tmp_file, true, $secure);
-            }
-            return $tmp_file;
         }
+
+        if ($delete) {
+            Util::deleteAtShutdown($tmp_file, true, $secure);
+        }
+        return $tmp_file;
     }
 
     /**
@@ -516,18 +520,18 @@ class Util {
         /* Get the first 8 characters of a random string to use as a temporary
            directory name. */
         do {
-            $temp_dir .= '/' . substr(base_convert(mt_rand() . microtime(), 10, 36), 0, 8);
-        } while (file_exists($temp_dir));
+            $new_dir = $temp_dir . '/' . substr(base_convert(mt_rand() . microtime(), 10, 36), 0, 8);
+        } while (file_exists($new_dir));
 
         $old_umask = umask(0000);
-        if (!mkdir($temp_dir, 0700)) {
-            $temp_dir = false;
+        if (!mkdir($new_dir, 0700)) {
+            $new_dir = false;
         } elseif ($delete) {
-            Util::deleteAtShutdown($temp_dir);
+            Util::deleteAtShutdown($new_dir);
         }
         umask($old_umask);
 
-        return $temp_dir;
+        return $new_dir;
     }
 
     /**
@@ -546,7 +550,7 @@ class Util {
     function realPath($path)
     {
         /* Standardize on UNIX directory separators. */
-        if (substr(PHP_OS, 0, 3) == 'WIN') {
+        if (!strncasecmp(PHP_OS, 'WIN', 3)) {
             $path = str_replace('\\', '/', $path);
         }
 
@@ -655,16 +659,20 @@ class Util {
 
         foreach ($files as $file => $val) {
             /* Delete files */
-            if ($val && @file_exists($file)) {
-                /* Should we securely delete the file by overwriting the
-                   data with a random string? */
+            if ($val && file_exists($file)) {
+                /* Should we securely delete the file by overwriting the data
+                   with a random string? */
                 if (isset($secure[$file])) {
-                    $random_str = '';
-                    for ($i = 0; $i < filesize($file); $i++) {
-                        $random_str .= chr(mt_rand(0, 255));
-                    }
+                    $filesize = filesize($file);
+                    /* See http://www.cs.auckland.ac.nz/~pgut001/pubs/secure_del.html.
+                     * We save the random overwrites for efficiency reasons. */
+                    $patterns = array("\x55", "\xaa", "\x92\x49\x24", "\x49\x24\x92", "\x24\x92\x49", "\x00", "\x11", "\x22", "\x33", "\x44", "\x55", "\x66", "\x77", "\x88", "\x99", "\xaa", "\xbb", "\xcc", "\xdd", "\xee", "\xff", "\x92\x49\x24", "\x49\x24\x92", "\x24\x92\x49", "\x6d\xb6\xdb", "\xb6\xdb\x6d", "\xdb\x6d\xb6");
                     $fp = fopen($file, 'r+');
-                    fwrite($fp, $random_str);
+                    foreach ($patterns as $pattern) {
+                        $pattern = substr(str_repeat($pattern, floor($filesize / strlen($pattern)) + 1), 0, $filesize);
+                        fwrite($fp, $pattern);
+                        fseek($fp, 0);
+                    }
                     fclose($fp);
                 }
                 @unlink($file);
@@ -673,7 +681,7 @@ class Util {
 
         foreach ($dirs as $dir => $val) {
             /* Delete directories */
-            if ($val && @file_exists($dir)) {
+            if ($val && file_exists($dir)) {
                 /* Make sure directory is empty. */
                 $dir_class = dir($dir);
                 while (false !== ($entry = $dir_class->read())) {
@@ -690,13 +698,13 @@ class Util {
     /**
      * Outputs javascript code to close the current window.
      *
-     * @param string $code  Any addtional javascript code to run before
+     * @param string $code  Any additional javascript code to run before
      *                      closing the window.
      */
     function closeWindowJS($code = '')
     {
-        echo '<script language="JavaScript" type="text/javascript">' . $code .
-            'window.close();</script>';
+        echo '<script type="text/javascript">//<![CDATA[' . "\n"
+            . $code . 'window.close();' . "\n//]]></script>\n";
     }
 
     /**
@@ -741,7 +749,7 @@ class Util {
             return false;
         }
 
-        if (substr(PHP_OS, 0, 3) == 'WIN') {
+        if (!strncasecmp(PHP_OS, 'WIN', 3)) {
             $suffix = 'dll';
         } else {
             switch (PHP_OS) {
@@ -845,6 +853,100 @@ class Util {
         return str_replace($strftimeSymbols, $dateSymbols, $format);
     }
 
+    /**
+     * Utility function to obtain PATH_INFO information.
+     *
+     * @since Horde 3.2
+     *
+     * @return string  The PATH_INFO string.
+     */
+    function getPathInfo()
+    {
+        if (isset($_SERVER['PATH_INFO'])) {
+            return $_SERVER['PATH_INFO'];
+        } elseif (isset($_SERVER['REQUEST_URI']) &&
+                  isset($_SERVER['SCRIPT_NAME'])) {
+            $search = array($_SERVER['SCRIPT_NAME']);
+            $replace = array('');
+            if (!empty($_SERVER['QUERY_STRING'])) {
+                $search[] = '?' . $_SERVER['QUERY_STRING'];
+                $replace[] = '';
+            }
+            return str_replace($search, $replace, $_SERVER['REQUEST_URI']);
+        }
+
+        return '';
+    }
+
+    /**
+     * Calculate an HMAC for a given $data and secret $key using SHA-1.
+     *
+     * @param string $data  Data to sign
+     * @param string $key   Secret key
+     * @param boolean $raw_output  Return binary data? Default to hex.
+     *
+     * @return string | binary  HMAC
+     *
+     * @since Horde 3.3
+     */
+    function hmac($data, $key, $raw_output = false)
+    {
+        if (function_exists('hash_hmac')) {
+            return hash_hmac('sha1', $data, $key, $raw_output);
+        }
+
+        if (strlen($key) > 64) {
+            $key =  pack('H40', sha1($key));
+        }
+        if (strlen($key) < 64) {
+            $key = str_pad($key, 64, chr(0));
+        }
+
+        /* Calculate the padded keys and save them */
+        $ipad = substr($key, 0, 64) ^ str_repeat(chr(0x36), 64);
+        $opad = substr($key, 0, 64) ^ str_repeat(chr(0x5C), 64);
+
+        if ($raw_output) {
+            return pack('H*', sha1($opad . pack('H40', sha1($ipad . $data))));
+        }
+        return sha1($opad . pack('H40', sha1($ipad . $data)));
+    }
+
+    /**
+     * URL-safe base64 encoding, with trimmed =
+     *
+     * @since Horde 3.3
+     *
+     * @param string $string  String to encode.
+     *
+     * @return string  URL-safe, base64 encoded data.
+     */
+    function uriB64Encode($string)
+    {
+        $data = base64_encode($string);
+        $data = str_replace(array('+', '/', '='), array('-', '_', ''), $data);
+        return $data;
+    }
+
+    /**
+     * Decode URL-safe base64 data, dealing with missing =
+     *
+     * @since Horde 3.3
+     *
+     * @param string $string  Encoded data
+     *
+     * @return string  Decoded data.
+     */
+    function uriB64Decode($string)
+    {
+        $data = str_replace(array('-', '_'), array('+', '/'), $string);
+        $mod4 = strlen($data) % 4;
+        if ($mod4) {
+            $data .= substr('====', $mod4);
+        }
+        return base64_decode($data);
+    }
+
 }
 
 if (!function_exists('_')) {
@@ -853,6 +955,11 @@ if (!function_exists('_')) {
         return $string;
     }
 
+    function ngettext($msgid1, $msgid2, $n)
+    {
+        return $n > 1 ? $msgid2 : $msgid1;
+    }
+
     function bindtextdomain()
     {
     }
