@@ -11,12 +11,12 @@ define('VC_SORT_AUTHOR', 4);      // sort by author name
 define('VC_SORT_ASCENDING', 0);   // ascending order
 define('VC_SORT_DESCENDING', 1);  // descending order
 
-define('VC_WINDOWS', substr(PHP_OS, 0, 3) == 'WIN');
+define('VC_WINDOWS', !strncasecmp(PHP_OS, 'WIN', 3));
 
 /**
  * Version Control generalized library.
  *
- * $Horde: framework/VC/VC.php,v 1.12.8.12 2006/05/31 17:06:37 selsky Exp $
+ * $Horde: framework/VC/VC.php,v 1.12.8.14 2008/06/07 16:07:26 chuck Exp $
  *
  * @package VC
  */
@@ -63,11 +63,7 @@ class VC {
     function getPath($binary)
     {
         if (isset($this->_paths[$binary])) {
-            if (VC_WINDOWS) {
-                return $this->_paths[$binary];
-            } else {
-                return is_executable($this->_paths[$binary]) ? $this->_paths[$binary] : false;
-            }
+            return $this->_paths[$binary];
         }
 
         return false;
@@ -158,14 +154,11 @@ class VC {
      */
     function &singleton($driver, $params = array())
     {
-        static $instances;
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         $signature = serialize(array($driver, $params));
         if (!isset($instances[$signature])) {
-            $instances[$signature] = VC::factory($driver, $params);
+            $instances[$signature] = &VC::factory($driver, $params);
         }
 
         return $instances[$signature];
@@ -367,7 +360,7 @@ class VC_Revision {
      *
      * @return string  Stripped revision number
      */
-    function strip($val, $amount)
+    function strip($val, $amount = 1)
     {
         if (!VC_Revision::valid($val)) {
             return false;
@@ -397,32 +390,7 @@ class VC_Revision {
     }
 
     /**
-     * Given a valid revision number, this will return the revision
-     * number from which it branched. If it cannot be determined, then
-     * false is returned.
-     *
-     * @param string $val  Revision number.
-     *
-     * @return string|boolean  Branch point revision, or false.
-     */
-    function branchPoint($val)
-    {
-        /* Check if we have a valid revision number */
-        if (!VC_Revision::valid($val)) {
-            return false;
-        }
-
-        /* If its on the trunk, or is an odd size, ret false */
-        if (VC_Revision::sizeof($val) < 3 || (VC_Revision::sizeof($val) % 2)) {
-            return false;
-        }
-
-        /* Strip off two revision portions, and return it */
-        return VC_Revision::strip($val, 2);
-    }
-
-    /**
-     * Given two SVN revision numbers, this figures out which one is
+     * Given two revision numbers, this figures out which one is
      * greater than the other by stepping along the decimal points
      * until a difference is found, at which point a sign comparison
      * of the two is returned.
@@ -468,31 +436,4 @@ class VC_Revision {
         }
     }
 
-    /**
-     * Given a revision number of the form x.y.0.z, this remaps it
-     * into the appropriate branch number, which is x.y.z
-     *
-     * @param string $rev  Even-digit revision number of a branch
-     *
-     * @return string  Odd-digit Branch number
-     */
-    function toBranch($rev)
-    {
-        /* Check if we have a valid revision number */
-        if (!VC_Revision::valid($rev)) {
-            return false;
-        }
-
-        if (($end = strrpos($rev, '.')) === false) {
-            return false;
-        }
-
-        $rev[$end] = 0;
-        if (($end2 = strrpos($rev, '.')) === false) {
-            return substr($rev, ++$end);
-        }
-
-        return substr_replace($rev, '.', $end2, ($end-$end2+1));
-    }
-
 }
