@@ -5,9 +5,9 @@ require_once dirname(__FILE__) . '/rcs.php';
 /**
  * VC_cvs implementation.
  *
- * Copyright 2000-2006 Anil Madhavapeddy, <anil@recoil.org>
+ * Copyright 2000-2008 The Horde Project (http://www.horde.org/)
  *
- * $Horde: framework/VC/VC/cvs.php,v 1.32.2.24 2006/05/12 04:47:08 chuck Exp $
+ * $Horde: framework/VC/VC/cvs.php,v 1.32.2.27 2008/01/02 11:30:59 jan Exp $
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @package VC
@@ -36,7 +36,7 @@ class VC_cvs extends VC_rcs {
 
     function isFile($where)
     {
-        return @is_file($where . ',v');
+        return @is_file($where . ',v') || @is_file(dirname($where) . '/Attic/' . basename($where) . ',v');
     }
 
     function &queryDir($where)
@@ -588,13 +588,18 @@ class VC_File_cvs extends VC_File {
 
     function &getFileObject($rep, $filename, $cache = null, $quicklog = false)
     {
+        /* Assume file is in the Attic if it doesn't exist. */
+        if (!@is_file($filename . ',v')) {
+            $filename = dirname($filename) . '/Attic/' . basename($filename);
+        }
+
         /* The version of the cached data. Increment this whenever the
          * internal storage format changes, such that we must
          * invalidate prior cached data. */
         $cacheVersion = 2;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f' . (int)$quicklog . '_v' . $cacheVersion;
 
-        $ctime = time() - @filemtime($filename . ',v');
+        $ctime = time() - filemtime($filename . ',v');
         if ($cache &&
             $cache->exists($cacheId, $ctime)) {
             $fileOb = unserialize($cache->get($cacheId, $ctime));
@@ -781,14 +786,16 @@ class VC_File_cvs extends VC_File {
                     $this->revsym = $revsym;
                 } elseif (preg_match("/^\s+([^:]+):\s+([\d\.]+)/", $line, $regs)) {
                     // Check to see if this is a branch
-                    if (preg_match('/^(\d+(\.\d+)+)\.0\.(\d+)$/',$regs[2])) {
-                        $branchRev = VC_Revision::toBranch($regs[2]);
+                    if (preg_match('/^(\d+(\.\d+)+)\.0\.(\d+)$/', $regs[2])) {
+                        $branchRev = $this->toBranch($regs[2]);
                         if (!isset($this->branches[$branchRev])) {
                             $this->branches[$branchRev] = $regs[1];
                         }
                     } else {
                         $symrev[$regs[1]] = $regs[2];
-                        if (empty($revsym[$regs[2]])) $revsym[$regs[2]]=array();
+                        if (empty($revsym[$regs[2]])) {
+                            $revsym[$regs[2]] = array();
+                        }
                         $revsym[$regs[2]][] = $regs[1];
                     }
                 }
@@ -834,6 +841,33 @@ class VC_File_cvs extends VC_File {
         return preg_replace('|^'. $this->rep->sourceroot() . '/?(.*),v$|', '\1', $this->queryFullPath());
     }
 
+    /**
+     * Given a revision number of the form x.y.0.z, this remaps it
+     * into the appropriate branch number, which is x.y.z
+     *
+     * @param string $rev  Even-digit revision number of a branch
+     *
+     * @return string  Odd-digit Branch number
+     */
+    function toBranch($rev)
+    {
+        /* Check if we have a valid revision number */
+        if (!VC_Revision::valid($rev)) {
+            return false;
+        }
+
+        if (($end = strrpos($rev, '.')) === false) {
+            return false;
+        }
+
+        $rev[$end] = 0;
+        if (($end2 = strrpos($rev, '.')) === false) {
+            return substr($rev, ++$end);
+        }
+
+        return substr_replace($rev, '.', $end2, ($end - $end2 + 1));
+    }
+
 }
 
 /**
@@ -891,7 +925,8 @@ class VC_Log_cvs {
                 }
                 break;
 
-            /* Look for a branch point here - format is 'branches:  x.y.z;  a.b.c;' */
+            /* Look for a branch point here - format is 'branches:
+             * x.y.z; a.b.c;' */
             case 'branches':
                 /* If we find a branch tag, process and pop it,
                    otherwise leave input stream untouched */
@@ -905,7 +940,6 @@ class VC_Log_cvs {
                         }
                     }
                     array_shift($raw);
-
                 }
 
                 $state = 'done';
@@ -915,10 +949,9 @@ class VC_Log_cvs {
 
         /* Assume the rest of the lines are the log message */
         $this->log = implode("\n", $raw);
-        $this->tags = @$this->file->revsym[$this->rev];
-        if (empty($this->tags)) {
-            $this->tags = array();
-        }
+        $this->tags = isset($this->file->revsym[$this->rev]) ?
+            $this->file->revsym[$this->rev] :
+            array();
     }
 
     function queryDate()
@@ -1003,7 +1036,7 @@ class VC_Patchset_cvs extends VC_Patchset {
         $cacheVersion = 1;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f_v' . $cacheVersion;
 
-        $ctime = time() - @filemtime($filename . ',v');
+        $ctime = time() - filemtime($filename . ',v');
         if ($cache &&
             $cache->exists($cacheId, $ctime)) {
             $psOb = unserialize($cache->get($cacheId, $ctime));
