@@ -2,9 +2,9 @@
 /**
  * Class representing vAlarms.
  *
- * $Horde: framework/iCalendar/iCalendar/valarm.php,v 1.8.10.5 2007/01/02 13:54:51 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/valarm.php,v 1.8.10.8 2008/07/03 08:42:58 jan Exp $
  *
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -20,11 +20,6 @@ class Horde_iCalendar_valarm extends Horde_iCalendar {
         return 'vAlarm';
     }
 
-    function parsevCalendar($data)
-    {
-        parent::parsevCalendar($data, 'VALARM');
-    }
-
     function exportvCalendar()
     {
         return parent::_exportvData('VALARM');
