@@ -26,9 +26,9 @@ define('VCARD_GEO_LON',      1);
 /**
  * Class representing vCard entries.
  *
- * $Horde: framework/iCalendar/iCalendar/vcard.php,v 1.3.10.10 2007/01/02 13:54:51 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/vcard.php,v 1.3.10.16 2008/09/22 04:16:30 chuck Exp $
  *
- * Copyright 2003-2007 Karsten Fourmont (karsten@horde.org)
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -48,19 +48,17 @@ class Horde_iCalendar_vcard extends Horde_iCalendar {
         return 'vcard';
     }
 
-    function parsevCalendar($data)
-    {
-        return parent::parsevCalendar($data, 'vcard');
-    }
-
     /**
      * Unlike vevent and vtodo, a vcard is normally not enclosed in an
      * iCalendar container. (BEGIN..END)
      */
     function exportvCalendar()
     {
-        $requiredAttributes['BODY'] = '';
-        $requiredAttributes['VERSION'] = '2.1';
+        $requiredAttributes['VERSION'] = $this->_version;
+        $requiredAttributes['N'] = ';;;;;;';
+        if ($this->_version == '3.0') {
+            $requiredAttributes['FN'] = '';
+        }
 
         foreach ($requiredAttributes as $name => $default_value) {
             if (is_a($this->getAttribute($name), 'PEAR_Error')) {
@@ -131,7 +129,9 @@ class Horde_iCalendar_vcard extends Horde_iCalendar {
             $rfc822 = new Mail_RFC822();
         }
 
-        $rfc822->validateMailbox($address);
+        if (!$rfc822->validateMailbox($address)) {
+            return $address;
+        }
         return MIME::rfc822WriteAddress($address->mailbox, $address->host);
     }
 
