@@ -8,9 +8,9 @@
  *   'server'    The hostname of the IMSP server.
  *   'port'      The port of the IMSP server.</pre>
  *
- * $Horde: framework/Net_IMSP/IMSP/Auth/plaintext.php,v 1.7.10.12 2007/01/02 13:54:29 jan Exp $
+ * $Horde: framework/Net_IMSP/IMSP/Auth/plaintext.php,v 1.7.10.14 2008/01/02 11:30:45 jan Exp $
  *
- * Copyright 2003-2007 Michael Rubinsky <mrubinsk@horde.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -58,9 +58,10 @@ class Net_IMSP_Auth_plaintext extends Net_IMSP_Auth {
             if (!preg_match("/^\+/",
                             $imsp->imspReceive())) {
 
-                return $imsp->imspError('Did not receive expected command continuation response from IMSP server.',
+                $result = $imsp->imspError('Did not receive expected command continuation response from IMSP server.',
                                         __FILE__, __LINE__);
-            }
+                return $result;
+           }
         }
 
         $result = $imsp->imspSend($userId . ' ', false, false);
@@ -82,8 +83,9 @@ class Net_IMSP_Auth_plaintext extends Net_IMSP_Auth {
 
             if (!preg_match("/^\+/",
                             $imsp->imspReceive())) {
-                return $imsp->imspError('Did not receive expected command continuation response from IMSP server.',
+                $result = $imsp->imspError('Did not receive expected command continuation response from IMSP server.',
                                         __FILE__, __LINE__);
+                return $result;
             }
         }
 
@@ -101,7 +103,8 @@ class Net_IMSP_Auth_plaintext extends Net_IMSP_Auth {
         }
 
         if ($server_response != 'OK') {
-            return $imsp->imspError('Login to IMSP host failed.', __FILE__, __LINE__);
+            $result = $imsp->imspError('Login to IMSP host failed.', __FILE__, __LINE__);
+            return $result;
         }
 
         return $imsp;
