@@ -4,13 +4,13 @@
  * Vodafone Italy SMS gateway. Use of this gateway requires an email account
  * with Vodafone Italy (www.190.it).
  *
- * Copyright 2003-2007 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  * Copyright 2003-2007 Matteo Zambelli <mmzambe@hotmail.com>
  *
- * See the enclosed file COPYING for license information (LGPL). If you did not
- * receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Net_SMS/SMS/vodafoneitaly_smtp.php,v 1.11.10.10 2007/01/02 13:54:29 jan Exp $
+ * $Horde: framework/Net_SMS/SMS/vodafoneitaly_smtp.php,v 1.11.10.12 2008/01/02 11:30:46 jan Exp $
  *
  * @author Marko Djukic <marko@oblo.com>
  * @author Matteo Zambelli <mmzambe@hotmail.com>
@@ -54,15 +54,12 @@ class Net_SMS_vodafoneitaly_smtp extends Net_SMS {
      */
     function _send($message, $to)
     {
-        if (!@include_once 'Mail.php') {
-            return PEAR::raiseError(_("Missing required PEAR package Mail."));
-        }
-        $mailer = Mail::factory('mail');
-
         /* Since this only works for Italian numbers, this is hardcoded. */
         if (preg_match('/^.*?<?(\+?39)?(\d{10})>?/', $to, $matches)) {
             $headers['From'] = $this->_params['user'];
             $to = $matches[2] . '@sms.vodafone.it';
+
+            $mailer = Mail::factory('mail');
             $result = $mailer->send($to, $headers, $message['text']);
             if (is_a($result, 'PEAR_Error')) {
                 return array(0, $result->getMessage());
@@ -81,10 +78,10 @@ class Net_SMS_vodafoneitaly_smtp extends Net_SMS {
      */
     function getInfo()
     {
-        $info['name'] = _("Vodafone Italy via SMTP");
-        $info['desc'] = _("This driver allows sending of messages via SMTP through the Vodafone Italy gateway, only to Vodafone numbers. It requires an email account with Vodafone Italy (http://www.190.it).");
-
-        return $info;
+        return array(
+            'name' => _("Vodafone Italy via SMTP"),
+            'desc' => _("This driver allows sending of messages via SMTP through the Vodafone Italy gateway, only to Vodafone numbers. It requires an email account with Vodafone Italy (http://www.190.it)."),
+        );
     }
 
     /**
@@ -94,11 +91,8 @@ class Net_SMS_vodafoneitaly_smtp extends Net_SMS {
      */
     function getParams()
     {
-        $params = array();
-        $params['user']       = array('label'  => _("Username"),
-                                      'type'   => 'text');
-
-        return $params;
+        return array('user' => array('label' => _("Username"),
+                                     'type' => 'text'));
     }
 
     /**
