@@ -11,12 +11,12 @@ define('VFS_QUOTA_METRIC_GB', 4);
 /**
  * VFS API for abstracted file storage and access.
  *
- * $Horde: framework/VFS/VFS.php,v 1.69.4.17 2007/01/02 13:54:48 jan Exp $
+ * $Horde: framework/VFS/lib/VFS.php,v 1.1.2.7 2008/10/30 16:41:46 slusarz Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @package VFS
@@ -168,7 +168,7 @@ class VFS {
      */
     function setLogger(&$logger, $logLevel = null)
     {
-        if (!is_a($logger, 'Log')) {
+        if (!is_callable(array($logger, 'log'))) {
             return false;
         }
 
@@ -239,6 +239,80 @@ class VFS {
         return PEAR::raiseError(_("Not supported."));
     }
 
+    /**
+     * Retrieves a file from the VFS as an on-disk local file.
+     *
+     * This function provides a file on local disk with the data of a VFS file
+     * in it. This file <em>cannot</em> be modified! The behavior if you do
+     * modify it is undefined. It will be removed at the end of the request.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return string A local filename.
+     */
+    function readFile($path, $name)
+    {
+        // Create a temporary file and register it for deletion at the
+        // end of this request.
+        $localFile = $this->_getTempFile();
+        if (!$localFile) {
+            return PEAR::raiseError(_("Unable to create temporary file."));
+        }
+        register_shutdown_function(create_function('', 'unlink(\'' . addslashes($localFile) . '\');'));
+
+        if (is_callable(array($this, 'readStream'))) {
+            // Use a stream from the VFS if possible, to avoid reading all data
+            // into memory.
+            $stream = $this->readStream($path, $name);
+            if (is_a($stream, 'PEAR_Error')) {
+                return $stream;
+            }
+
+            $localStream = fopen($localFile, 'w');
+            if (!$localStream) {
+                return PEAR::raiseError(_("Unable to open temporary file."));
+            }
+
+            if (is_callable('stream_copy_to_stream')) {
+                // If we have stream_copy_to_stream, it can do the data transfer
+                // in one go.
+                stream_copy_to_stream($stream, $localStream);
+            } else {
+                // Otherwise loop through in chunks.
+                while ($buffer = fread($stream, 8192)) {
+                    fwrite($localStream, $buffer);
+                }
+            }
+
+            fclose($localStream);
+        } else {
+            // We have to read all of the data in one shot.
+            $data = $this->read($path, $name);
+            if (is_a($data, 'PEAR_Error')) {
+                return $data;
+            }
+
+            if (is_callable('file_put_contents')) {
+                // file_put_contents is more efficient if we have it.
+                file_put_contents($localFile, $data);
+            } else {
+                // Open the local file and write to it.
+                $localStream = fopen($localFile, 'w');
+                if (!$localStream) {
+                    return PEAR::raiseError(_("Unable to open temporary file."));
+                }
+                if (!fwrite($localStream, $data)) {
+                    return PEAR::raiseError(_("Unable to write temporary file."));
+                }
+                fclose($localStream);
+            }
+        }
+
+        // $localFile now has $path/$name's data in it.
+        return $localFile;
+    }
+
     /**
      * Retrieves a part of a file from the VFS. Particularly useful when
      * reading large files which would exceed the PHP memory limits if they
@@ -303,33 +377,93 @@ class VFS {
     /**
      * Moves a file through the backend.
      *
-     * @abstract
-     *
-     * @param string $path  The path of the original file.
-     * @param string $name  The name of the original file.
-     * @param string $dest  The destination file name.
+     * @param string $path         The path of the original file.
+     * @param string $name         The name of the original file.
+     * @param string $dest         The destination file name.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function move($path, $name, $dest)
+    function move($path, $name, $dest, $autocreate = false)
     {
-        return PEAR::raiseError(_("Not supported."));
+        if (is_a($result = $this->copy($path, $name, $dest, $autocreate), 'PEAR_Error')) {
+            return $result;
+        }
+        return $this->deleteFile($path, $name);
     }
 
     /**
      * Copies a file through the backend.
      *
-     * @abstract
-     *
-     * @param string $path  The path of the original file.
-     * @param string $name  The name of the original file.
-     * @param string $dest  The name of the destination directory.
+     * @param string $path         The path of the original file.
+     * @param string $name         The name of the original file.
+     * @param string $dest         The name of the destination directory.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function copy($path, $name, $dest)
+    function copy($path, $name, $dest, $autocreate = false)
     {
-        return PEAR::raiseError(_("Not supported."));
+        $orig = $this->_getPath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
+            return PEAR::raiseError(_("Cannot copy file(s) - source and destination are the same."));
+        }
+
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+        if ($this->isFolder($path, $name)) {
+            if (is_a($result = $this->_copyRecursive($path, $name, $dest), 'PEAR_Error')) {
+                return $result;
+            }
+        } else {
+            $data = $this->read($path, $name);
+            if (is_a($data, 'PEAR_Error')) {
+                return $data;
+            }
+            return $this->writeData($dest, $name, $data, $autocreate);
+        }
+        return true;
+    }
+
+    /**
+     * Recursively copies a directory through the backend.
+     *
+     * @access protected
+     *
+     * @param string $path         The path of the original file.
+     * @param string $name         The name of the original file.
+     * @param string $dest         The name of the destination directory.
+     */
+    function _copyRecursive($path, $name, $dest)
+    {
+        if (is_a($result = $this->createFolder($dest, $name), 'PEAR_Error')) {
+            return $result;
+        }
+
+        if (is_a($file_list = $this->listFolder($this->_getPath($path, $name)), 'PEAR_Error')) {
+            return $file_list;
+        }
+
+        foreach ($file_list as $file) {
+            $result = $this->copy($this->_getPath($path, $name),
+                                  $file['name'],
+                                  $this->_getPath($dest, $name));
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+    }
+
+    /**
+     * Alias to deleteFile()
+     */
+    function delete($path, $name)
+    {
+        return $this->deleteFile($path, $name);
     }
 
     /**
@@ -409,18 +543,19 @@ class VFS {
         if (is_array($dirs)) {
             $cur = '/';
             foreach ($dirs as $dir) {
-                if (!empty($dir)) {
-                    if (!$this->isFolder($cur, $dir)) {
-                        $result = $this->createFolder($cur, $dir);
-                        if (is_a($result, 'PEAR_Error')) {
-                            return $result;
-                        }
-                    }
-                    if ($cur != '/') {
-                        $cur .= '/';
-                    }
-                    $cur .= $dir;
+                if (!strlen($dir)) {
+                    continue;
                 }
+                if (!$this->isFolder($cur, $dir)) {
+                    $result = $this->createFolder($cur, $dir);
+                    if (is_a($result, 'PEAR_Error')) {
+                        return $result;
+                    }
+                }
+                if ($cur != '/') {
+                    $cur .= '/';
+                }
+                $cur .= $dir;
             }
         }
 
@@ -657,7 +792,7 @@ class VFS {
     function strtolower($string)
     {
         $language = setlocale(LC_CTYPE, 0);
-        setlocale(LC_CTYPE, 'en');
+        setlocale(LC_CTYPE, 'C');
         $string = strtolower($string);
         setlocale(LC_CTYPE, $language);
         return $string;
@@ -678,9 +813,9 @@ class VFS {
             if (is_null($charset)) {
                 $charset = 'ISO-8859-1';
             }
-            $ret = @mb_strlen($string, $charset);
-            if (!empty($ret)) {
-                return $ret;
+            $result = @mb_strlen($string, $charset);
+            if (!empty($result)) {
+                return $result;
             }
         }
         return strlen($string);
@@ -781,13 +916,13 @@ class VFS {
         $tmp = ini_get('upload_tmp_dir');
 
         /* Otherwise, try to determine the TMPDIR environment variable. */
-        if (empty($tmp)) {
+        if (!strlen($tmp)) {
             $tmp = getenv('TMPDIR');
         }
 
         /* If we still cannot determine a value, then cycle through a list of
          * preset possibilities. */
-        while (empty($tmp) && sizeof($tmp_locations)) {
+        while (!strlen($tmp) && count($tmp_locations)) {
             $tmp_check = array_shift($tmp_locations);
             if (@is_dir($tmp_check)) {
                 $tmp = $tmp_check;
@@ -796,7 +931,7 @@ class VFS {
 
         /* If it is still empty, we have failed, so return false; otherwise
          * return the directory determined. */
-        return empty($tmp) ? false : $tmp;
+        return strlen($tmp) ? $tmp : false;
     }
 
     /**
@@ -810,15 +945,12 @@ class VFS {
     function _getTempFile()
     {
         $tmp_dir = $this->_getTempDir();
-        if (empty($tmp_dir)) {
+        if (!strlen($tmp_dir)) {
             return false;
         }
 
         $tmp_file = tempnam($tmp_dir, 'vfs');
-
-        /* If the file was created, then register it for deletion and
-         * return. */
-        if (empty($tmp_file)) {
+        if (!strlen($tmp_file)) {
             return false;
         } else {
             return $tmp_file;
@@ -833,15 +965,15 @@ class VFS {
      * @param string $mode   Either 'string' or 'file'.  If 'string', $data is
      *                       the data to be written.  If 'file', $data is the
      *                       filename containing the data to be written.
-     * @param string &$data  Either the data or the filename to the data.
+     * @param string $data   Either the data or the filename to the data.
      *
      * @return mixed  PEAR_Error on error, true on success.
      */
-    function _checkQuotaWrite($mode, &$data)
+    function _checkQuotaWrite($mode, $data)
     {
         if ($this->_params['vfs_quotalimit'] != -1) {
             if ($mode == 'file') {
-                $filesize = @filesize($data);
+                $filesize = filesize($data);
                 if ($filesize === false) {
                     return PEAR::raiseError(_("Unable to read VFS file (filesize() failed)."));
                }
@@ -886,6 +1018,29 @@ class VFS {
         return true;
     }
 
+    /**
+     * Returns the full path of an item.
+     *
+     * @access protected
+     *
+     * @param string $path  The path of directory of the item.
+     * @param string $name  The name of the item.
+     *
+     * @return mixed  Full path when $path isset and just $name when not set.
+     */
+    function _getPath($path, $name)
+    {
+        if (strlen($path) > 0) {
+            if (substr($path, -1) == '/') {
+                return $path . $name;
+            } else {
+                return $path . '/' . $name;
+            }
+        } else {
+            return $name;
+        }
+    }
+
     /**
      * Attempts to return a concrete VFS instance based on $driver.
      *
@@ -900,10 +1055,14 @@ class VFS {
      */
     function &factory($driver, $params = array())
     {
-        include_once 'VFS/' . $driver . '.php';
+        $driver = basename($driver);
         $class = 'VFS_' . $driver;
+        if (!class_exists($class)) {
+            include_once 'VFS/' . $driver . '.php';
+        }
+
         if (class_exists($class)) {
-            $vfs = &new $class($params);
+            $vfs = new $class($params);
         } else {
             $vfs = PEAR::raiseError(sprintf(_("Class definition of %s not found."), $class));
         }
@@ -931,10 +1090,7 @@ class VFS {
      */
     function &singleton($driver, $params = array())
     {
-        static $instances;
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         $signature = serialize(array($driver, $params));
         if (!isset($instances[$signature])) {
