@@ -2,15 +2,14 @@
 /**
  * Class for providing garbage collection for any VFS instance.
  *
- * $Horde: framework/VFS/VFS/GC.php,v 1.4.12.6 2005/07/03 04:52:42 selsky Exp $
+ * $Horde: framework/VFS/lib/VFS/GC.php,v 1.1.2.2 2008/01/02 11:31:00 jan Exp $
  *
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package VFS
  */
 class VFS_GC {
@@ -26,15 +25,22 @@ class VFS_GC {
     function gc(&$vfs, $path, $secs = 345600)
     {
         /* A 1% chance we will run garbage collection during a call. */
-        if (rand(0, 99) == 0) {
-            /* Make sure cleaning is done recursively. */
-            $files = $vfs->listFolder($path, null, true, false, true);
-            if (!is_a($files, 'PEAR_Error') && is_array($files)) {
-                $modtime = time() - $secs;
-                foreach ($files as $val) {
-                    if ($val['date'] < $modtime) {
-                        $vfs->deleteFile($path, $val['name']);
-                    }
+        if (rand(0, 99) != 0) {
+            return;
+        }
+
+        /* Use a backend-specific method if one exists. */
+        if (is_callable(array($vfs, 'gc'))) {
+            return $vfs->gc($path, $secs);
+        }
+
+        /* Make sure cleaning is done recursively. */
+        $files = $vfs->listFolder($path, null, true, false, true);
+        if (!is_a($files, 'PEAR_Error') && is_array($files)) {
+            $modtime = time() - $secs;
+            foreach ($files as $val) {
+                if ($val['date'] < $modtime) {
+                    $vfs->deleteFile($path, $val['name']);
                 }
             }
         }
