@@ -1,15 +1,16 @@
 <?php
 /**
- * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.8 2007/01/02 13:54:50 jan Exp $
- *
- * Copyright 2003-2007 Anthony Mills <amills@pyramid6.com>
- *
- * See the enclosed file COPYING for license information (LGPL).  If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
- *
  * From Binary XML Content Format Specification Version 1.3, 25 July 2001
  * found at http://www.wapforum.org
  *
+ * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.11 2008/08/26 15:41:13 jan Exp $
+ *
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author  Anthony Mills <amills@pyramid6.com>
  * @package XML_WBXML
  */
 class XML_WBXML_ContentHandler {
@@ -32,12 +33,16 @@ class XML_WBXML_ContentHandler {
 
     function XML_WBXML_ContentHandler()
     {
-        $this->_currentUri = &new XML_WBXML_LifoQueue();
+        $this->_currentUri = new XML_WBXML_LifoQueue();
     }
 
+    /**
+     */
     function raiseError($error)
     {
-        include_once 'PEAR.php';
+        if (!class_exists('PEAR')) {
+            require 'PEAR.php';
+        }
         return PEAR::raiseError($error);
     }
 
@@ -71,7 +76,7 @@ class XML_WBXML_ContentHandler {
         return strlen($this->_output);
     }
 
-    function startElement($uri, $element, $attrs)
+    function startElement($uri, $element, $attrs = array())
     {
         $this->_output .= '<' . $element;
 
@@ -120,7 +125,7 @@ class XML_WBXML_ContentHandler {
     function createSubHandler()
     {
         $name = get_class($this); // clone current class
-        $sh = &new $name();
+        $sh = new $name();
         $sh->setCharset($this->getCharsetStr());
         $sh->setVersion($this->getVersion());
         return $sh;
