@@ -5,16 +5,17 @@ include_once 'XML/WBXML/DTDManager.php';
 include_once 'XML/WBXML/ContentHandler.php';
 
 /**
- * $Horde: framework/XML_WBXML/WBXML/Decoder.php,v 1.22.10.4 2005/07/07 14:59:34 chuck Exp $
+ * From Binary XML Content Format Specification Version 1.3, 25 July 2001
+ * found at http://www.wapforum.org
  *
- * Copyright 2003-2005 Anthony Mills <amills@pyramid6.com>
+ * $Horde: framework/XML_WBXML/WBXML/Decoder.php,v 1.22.10.11 2008/01/02 11:31:02 jan Exp $
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * From Binary XML Content Format Specification Version 1.3, 25 July
- * 2001 found at http://www.wapforum.org
- *
+ * @author  Anthony Mills <amills@pyramid6.com>
  * @package XML_WBXML
  */
 class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
@@ -54,6 +55,8 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
     var $_isAttribute;
     var $_isData = false;
 
+    var $_error = false;
+
     /**
      * The DTD Manager.
      *
@@ -68,30 +71,24 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
      */
     var $_strpos;
 
-    /**
-     * Use wbxml2xml from libwbxml.
-     *
-     * @var string
-     */
-    var $_wbxml2xml = '/usr/bin/wbxml2xml';
-
-    /**
-     * Arguments to pass to wbxml2xml.
-     *
-     * @var string
-     */
-    var $_wbxml2xml_args = '-m 0 -o - -';
-
     /**
      * Constructor.
      */
     function XML_WBXML_Decoder()
     {
-        if (empty($this->_wbxml2xml) || !is_executable($this->_wbxml2xml)) {
-            $this->_dtdManager = &new XML_WBXML_DTDManager();
-        }
+        $this->_dtdManager = &new XML_WBXML_DTDManager();
     }
 
+    /**
+     * Sets the contentHandler that will receive the output of the
+     * decoding.
+     *
+     * @param XML_WBXML_ContentHandler $ch The contentHandler
+     */
+    function setContentHandler(&$ch)
+    {
+        $this->_ch = &$ch;
+    }
     /**
      * Return one byte from the input stream.
      *
@@ -104,95 +101,95 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
 
     /**
      * Takes a WBXML input document and returns decoded XML.
+     * However the preferred and more effecient method is to
+     * use decode() rather than decodeToString() and have an
+     * appropriate contentHandler deal with the decoded data.
      *
      * @param string $wbxml  The WBXML document to decode.
      *
      * @return string  The decoded XML document.
      */
+    function decodeToString($wbxml)
+    {
+        $this->_ch = &new XML_WBXML_ContentHandler();
+
+        $r = $this->decode($wbxml);
+        if (is_a($r, 'PEAR_Error')) {
+            return $r;
+        }
+        return $this->_ch->getOutput();
+    }
+
+    /**
+     * Takes a WBXML input document and decodes it.
+     * Decoding result is directly passed to the contentHandler.
+     * A contenthandler must be set using setContentHandler
+     * prior to invocation of this method
+     *
+     * @param string $wbxml  The WBXML document to decode.
+     *
+     * @return mixed  True on success or PEAR_Error.
+     */
     function decode($wbxml)
     {
-        // Figure out if we're going to use wbxml2xml to do the
-        // conversion, or do it all in PHP code.
-        if (!empty($this->_wbxml2xml) && is_executable($this->_wbxml2xml)) {
-            require_once 'Horde/Util.php';
-            $inputFile = Util::getTempFile();
-            $errorFile = Util::getTempFile();
+        $this->_error = false; // reset state
 
-            $fp = fopen($inputFile, 'wb');
-            fwrite($fp, $wbxml);
-            fclose($fp);
+        $this->_strpos = 0;
 
-            $descriptorspec = array(
-                0 => array('file', $inputFile, 'rb'),
-                1 => array('pipe', 'w'),
-                2 => array('file', $errorFile, 'w'),
-            );
-
-            $wbxml2xml = proc_open($this->_wbxml2xml . ' ' . $this->_wbxml2xml_args,
-                                   $descriptorspec, $pipes);
-            if (is_resource($wbxml2xml)) {
-                $xml = '';
-                while (!feof($pipes[1])) {
-                    $xml .= fread($pipes[1], 8192);
-                }
-                fclose($pipes[1]);
-
-                proc_close($wbxml2xml);
-                return $xml;
-            } else {
-                return PEAR::raiseError('wbxml2xml failed');
-            }
-        } else {
-            $this->_strpos = 0;
-
-            // Get Version Number from Section 5.4
-            // version = u_int8
-            // currently 1, 2 or 3
-            $this->_wbxmlVersion = $this->getVersionNumber($wbxml);
-
-            // Get Document Public Idetifier from Section 5.5
-            // publicid = mb_u_int32 | (zero index)
-            // zero = u_int8
-            // Containing the value zero (0)
-            // The actual DPI is determined after the String Table is read.
-            $dpiStruct = $this->getDocumentPublicIdentifier($wbxml);
-
-            // Get Charset from 5.6
-            // charset = mb_u_int32
-            $this->_charset = $this->getCharset($wbxml);
-
-            // Get String Table from 5.7
-            // strb1 = length *byte
-            $this->_stringTable = $this->getStringTable($wbxml, $this->_charset);
-
-            // Get Document Public Idetifier from Section 5.5.
-            $this->_dpi = $this->getDocumentPublicIdentifierImpl($dpiStruct['dpiType'],
-                                                                 $dpiStruct['dpiNumber'],
-                                                                 $this->_stringTable);
-
-            // Now the real fun begins.
-            // From Sections 5.2 and 5.8
-
-            // Default content handler.
-            $this->_ch = &new XML_WBXML_ContentHandler();
-
-            // Default content handler.
-            $this->_dtdManager = &new XML_WBXML_DTDManager();
-
-            // Get the starting DTD.
-            $this->_tagDTD = $this->_dtdManager->getInstance($this->_dpi);
-            if (!$this->_tagDTD) {
-                return $this->raiseError('No DTD found for ' . $this->_dpi);
-            }
-
-            $this->_attributeDTD = $this->_tagDTD;
-
-            while ($this->_strpos < strlen($wbxml)) {
-                $this->_decode($wbxml);
-            }
-
-            return $this->_ch->getOutput();
+        if (empty($this->_ch)) {
+            return $this->raiseError('No Contenthandler defined.');
         }
+
+        // Get Version Number from Section 5.4
+        // version = u_int8
+        // currently 1, 2 or 3
+        $this->_wbxmlVersion = $this->getVersionNumber($wbxml);
+
+        // Get Document Public Idetifier from Section 5.5
+        // publicid = mb_u_int32 | (zero index)
+        // zero = u_int8
+        // Containing the value zero (0)
+        // The actual DPI is determined after the String Table is read.
+        $dpiStruct = $this->getDocumentPublicIdentifier($wbxml);
+
+        // Get Charset from 5.6
+        // charset = mb_u_int32
+        $this->_charset = $this->getCharset($wbxml);
+
+        // Get String Table from 5.7
+        // strb1 = length *byte
+        $this->retrieveStringTable($wbxml);
+
+        // Get Document Public Idetifier from Section 5.5.
+        $this->_dpi = $this->getDocumentPublicIdentifierImpl($dpiStruct['dpiType'],
+                                                             $dpiStruct['dpiNumber'],
+                                                             $this->_stringTable);
+
+        // Now the real fun begins.
+        // From Sections 5.2 and 5.8
+
+
+        // Default content handler.
+        $this->_dtdManager = &new XML_WBXML_DTDManager();
+
+        // Get the starting DTD.
+        $this->_tagDTD = $this->_dtdManager->getInstance($this->_dpi);
+
+        if (!$this->_tagDTD) {
+            return $this->raiseError('No DTD found for '
+                             . $this->_dpi . '/'
+                             . $dpiStruct['dpiNumber']);
+        }
+
+        $this->_attributeDTD = $this->_tagDTD;
+
+        while (empty($this->_error) && $this->_strpos < strlen($wbxml)) {
+            $this->_decode($wbxml);
+        }
+        if (!empty($this->_error)) {
+            return $this->_error;
+        }
+        return true;
     }
 
     function getVersionNumber($input)
@@ -212,12 +209,12 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
         }
     }
 
-    function getDocumentPublicIdentifierImpl($dpiType, $dpiNumber, $st)
+    function getDocumentPublicIdentifierImpl($dpiType, $dpiNumber)
     {
         if ($dpiType == 1) {
             return XML_WBXML::getDPIString($dpiNumber);
         } else {
-            return isset($st[$dpiNumber]) ? $st[$dpiNumber] : null;
+            return $this->getStringTableEntry($dpiNumber);
         }
     }
 
@@ -234,39 +231,52 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
     }
 
     /**
-     * @TODO needs to be fixed. Does this still really need to be
-     * fixed?
+     * Retrieves the string table.
+     * The string table consists of an mb_u_int32 length
+     * and then length bytes forming the table.
+     * References to the string table refer to the
+     * starting position of the (null terminated)
+     * string in this table.
      */
-    function getStringTable($input, $cs)
+    function retrieveStringTable($input)
     {
-        $stringTable = array();
         $size = XML_WBXML::MBUInt32ToInt($input, $this->_strpos);
+        $this->_stringTable = substr($input, $this->_strpos, $size);
+        $this->_strpos += $size;
+        // print "stringtable($size):" . $this->_stringTable ."\n";
+    }
 
-        // A hack to make it work with arrays.
-        // How/why is this necessary?
-        $str = 'j';
+    function getStringTableEntry($index)
+    {
+        if ($index >= strlen($this->_stringTable)) {
+            $this->_error =
+                $this->raiseError('Invalid offset ' . $index
+                                  . ' value encountered around position '
+                                  . $this->_strpos
+                                  . '. Broken wbxml?');
+            return '';
+        }
 
-        $numstr = 0;
-        $start = 0;
-        $j = 0;
-        for ($i = 0; $i < $size; $i++ ) {
-            /* May need to fix the null detector for more than single
-             * byte charsets like ASCII, UTF-8, etc. */
-            $ch = $input[$this->_strpos++];
-            if (ord($ch) == 0) {
-                $stringTable[$numstr++] = $str;
-                $str = '#';
-                $start = $i + 1;
-            } else {
-                $str[$j++] = $ch;
+        // copy of method termstr but without modification of this->_strpos
+
+        $str = '#'; // must start with nonempty string to allow array access
+
+        $i = 0;
+        $ch = $this->_stringTable[$index++];
+        if (ord($ch) == 0) {
+            return ''; // don't return '#'
+        }
+
+        while (ord($ch) != 0) {
+            $str[$i++] = $ch;
+            if ($index >= strlen($this->_stringTable)) {
+                break;
             }
+            $ch = $this->_stringTable[$index++];
         }
+        // print "string table entry: $str\n";
+        return $str;
 
-        if ($start < $size) {
-            $stringTable[$numstr++] = $str;
-        }
-
-        return $stringTable;
     }
 
     function _decode($input)
@@ -274,16 +284,20 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
         $token = $this->getByte($input);
         $str = '';
 
+        // print "position: " . $this->_strpos . " token: " . $token . " str10: " . substr($input, $this->_strpos, 10) . "\n"; // @todo: remove debug output
+
         switch ($token) {
         case XML_WBXML_GLOBAL_TOKEN_STR_I:
             // Section 5.8.4.1
             $str = $this->termstr($input);
             $this->_ch->characters($str);
+            // print "str:$str\n"; // @TODO Remove debug code
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_STR_T:
             // Section 5.8.4.1
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $x = XML_WBXML::MBUInt32ToInt($input, $this->_strpos);
+            $str = $this->getStringTableEntry($x);
             $this->_ch->characters($str);
             break;
 
@@ -299,7 +313,7 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
         case XML_WBXML_GLOBAL_TOKEN_EXT_T_1:
         case XML_WBXML_GLOBAL_TOKEN_EXT_T_2:
             // Section 5.8.4.2
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $str = $this->getStringTableEnty(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
             $this->_ch->characters($str);
             break;
 
@@ -321,40 +335,66 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
 
         case XML_WBXML_GLOBAL_TOKEN_PI:
             // Section 5.8.4.4
-            // throw new IOException("WBXML global token processing instruction(PI, " + token + ") is unsupported!");
+            // throw new IOException
+            // die("WBXML global token processing instruction(PI, " + token + ") is unsupported!\n");
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_LITERAL:
             // Section 5.8.4.5
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $str = $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
             $this->parseTag($input, $str, false, false);
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_LITERAL_A:
             // Section 5.8.4.5
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $str = $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
             $this->parseTag($input, $str, true, false);
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_LITERAL_AC:
             // Section 5.8.4.5
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $str = $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
             $this->parseTag($input, $string, true, true);
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_LITERAL_C:
             // Section 5.8.4.5
-            $str = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+            $str = $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
             $this->parseTag($input, $str, false, true);
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_OPAQUE:
             // Section 5.8.4.6
             $size = XML_WBXML::MBUInt32ToInt($input, $this->_strpos);
-            $b = substr($input, $this->_strpos, $this->_strpos + $size);
-            $this->_strpos += $size;
-            $this->_ch->opaque($b);
+            if ($size>0) {
+                $b = substr($input, $this->_strpos, $size);
+                // print "opaque of size $size: ($b)\n"; // @todo remove debug
+                $this->_strpos += $size;
+                // opaque data inside a <data> element may or may not be
+                // a nested wbxml document (for example devinf data).
+                // We find out by checking the first byte of the data: if it's
+                // 1, 2 or 3 we expect it to be the version number of a wbxml
+                // document and thus start a new wbxml decoder instance on it.
 
+                if ($size > 0 && $this->_isData && ord($b) <= 10) {
+                    $decoder = &new XML_WBXML_Decoder(true);
+                    $decoder->setContentHandler($this->_ch);
+                    $s = $decoder->decode($b);
+            //                /* // @todo: FIXME currently we can't decode Nokia
+                    // DevInf data. So ignore error for the time beeing.
+                    if (is_a($s, 'PEAR_Error')) {
+                        $this->_error = $s;
+                        return;
+                    }
+                    // */
+                    // $this->_ch->characters($s);
+                } else {
+                    /* normal opaque behaviour: just copy the raw data: */
+                    // print "opaque handled as string=$b\n"; // @todo remove debug
+                    $this->_ch->characters($b);
+                }
+            }
+            // old approach to deal with opaque data inside ContentHandler:
             // FIXME Opaque is used by SYNCML.  Opaque data that depends on the context
             // if (contentHandler instanceof OpaqueContentHandler) {
             //     ((OpaqueContentHandler)contentHandler).opaque(b);
@@ -365,8 +405,6 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
             //     contentHandler.characters(chars, 0, chars.length);
             // }
 
-            // This can cause some problems. We may have to use a
-            // event based decoder.
             break;
 
         case XML_WBXML_GLOBAL_TOKEN_END:
@@ -377,6 +415,7 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
         case XML_WBXML_GLOBAL_TOKEN_SWITCH_PAGE:
             // Section 5.8.4.7.2
             $codePage = $this->getByte($input);
+            // print "switch to codepage $codePage\n"; // @todo: remove debug code
             $this->switchElementCodePage($codePage);
             break;
 
@@ -388,11 +427,16 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
             $realToken = $token & 0x3F;
             $str = $this->getTag($realToken);
 
+            // print "element:$str\n"; // @TODO Remove debug code
             $this->parseTag($input, $str, $hasAttributes, $hasContent);
 
             if ($realToken == 0x0f) {
-                // FIXME Don't remember this one.
+                // store if we're inside a Data tag. This may contain
+                // an additional enclosed wbxml document on which we have
+                // to run a seperate encoder
                 $this->_isData = true;
+            } else {
+                $this->_isData = false;
             }
             break;
         }
@@ -424,10 +468,6 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
             $tag = 'Unknown';
         }
 
-        if ($tag == 'Data') {
-            $this->_isData = false;
-        }
-
         $this->_ch->endElement($this->getCurrentURI(), $tag);
 
         return $tag;
@@ -455,7 +495,7 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
                                      'value' => $value);
                 }
 
-                $attr = $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+                $attr = $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
                 break;
 
             // Value specified.
@@ -470,7 +510,7 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
             case XML_WBXML_GLOBAL_TOKEN_EXT_T_1:
             case XML_WBXML_GLOBAL_TOKEN_EXT_T_2:
                 // Section 5.8.4.2
-                $value .= $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+                $value .= $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
                 break;
 
             case XML_WBXML_GLOBAL_TOKEN_EXT_0:
@@ -492,7 +532,7 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
 
             case XML_WBXML_GLOBAL_TOKEN_STR_T:
                 // Section 5.8.4.1
-                $value .= $this->_stringTable[XML_WBXML::MBUInt32ToInt($input, $this->_strpos)];
+                $value .= $this->getStringTableEntry(XML_WBXML::MBUInt32ToInt($input, $this->_strpos));
                 break;
 
             case XML_WBXML_GLOBAL_TOKEN_OPAQUE:
@@ -602,13 +642,16 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
     }
 
     /**
-     * @TODO FIXME reads a null terminated string.
+     * Reads a null terminated string.
      */
     function termstr($input)
     {
-        $str = '#';
+        $str = '#'; // must start with nonempty string to allow array access
         $i = 0;
         $ch = $input[$this->_strpos++];
+        if (ord($ch) == 0) {
+            return ''; // don't return '#'
+        }
         while (ord($ch) != 0) {
             $str[$i++] = $ch;
             $ch = $input[$this->_strpos++];
@@ -618,3 +661,4 @@ class XML_WBXML_Decoder extends XML_WBXML_ContentHandler {
     }
 
 }
+
