@@ -3,19 +3,19 @@
 include_once 'XML/WBXML.php';
 include_once 'XML/WBXML/ContentHandler.php';
 include_once 'XML/WBXML/DTDManager.php';
-include_once 'Horde/String.php';
 
 /**
- * $Horde: framework/XML_WBXML/WBXML/Encoder.php,v 1.25.10.14 2007/01/02 13:54:50 jan Exp $
+ * From Binary XML Content Format Specification Version 1.3, 25 July 2001
+ * found at http://www.wapforum.org
  *
- * Copyright 2003-2007 Anthony Mills <amills@pyramid6.com>
+ * $Horde: framework/XML_WBXML/WBXML/Encoder.php,v 1.25.10.17 2008/08/26 15:41:21 jan Exp $
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * From Binary XML Content Format Specification Version 1.3, 25 July
- * 2001 found at http://www.wapforum.org
- *
+ * @author  Anthony Mills <amills@pyramid6.com>
  * @package XML_WBXML
  */
 class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
@@ -135,7 +135,11 @@ class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
 
     function writeDocumentPublicIdentifier($dpiString, &$strings)
     {
-        $i = XML_WBXML::getDPIInt($dpiString);
+        $i = 0;
+
+        // The OMA test suite doesn't like DPI as integer code.
+        // So don't try lookup and always send full DPI string.
+        // $i = XML_WBXML::getDPIInt($dpiString);
 
         if ($i == 0) {
             $strings[0] = $dpiString;
@@ -194,7 +198,6 @@ class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
 
     function _getBytes($string, $cs)
     {
-        $string = String::convertCharset($string, $cs, 'utf-8');
         $nbytes = strlen($string);
 
         $bytes = array();
@@ -213,7 +216,7 @@ class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
         return array($uri, $name);
     }
 
-    function startElement($uri, $name, $attributes)
+    function startElement($uri, $name, $attributes = array())
     {
         if ($this->_subParser == null) {
             if (!$this->_hasWrittenHeader) {
@@ -225,11 +228,11 @@ class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
             if ($this->_subParser == null) {
                 $this->writeTag($name, $attributes, true, $this->_charset);
             } else {
-                $this->_subParser->startElement($uri,$name, $attributes);
+                $this->_subParser->startElement($uri, $name, $attributes);
             }
         } else {
             $this->_subParserStack++;
-            $this->_subParser->startElement($uri,$name,$attributes);
+            $this->_subParser->startElement($uri, $name, $attributes);
         }
     }
 
@@ -377,13 +380,16 @@ class XML_WBXML_Encoder extends XML_WBXML_ContentHandler {
     function changecodepage($uri)
     {
         // @todo: this is a hack!
+        if ($this->_dtd->getVersion() == 2 && !preg_match('/1\.2$/', $uri)) {
+            $uri .= '1.2';
+        }
         if ($this->_dtd->getVersion() == 1 && !preg_match('/1\.1$/', $uri)) {
             $uri .= '1.1';
         }
         if ($this->_dtd->getVersion() == 0 && !preg_match('/1\.0$/', $uri)) {
             $uri .= '1.0';
         }
-        
+
         $cp = $this->_dtd->toCodePageURI($uri);
         if (strlen($cp)) {
             $this->_dtd = &$this->_dtdManager->getInstanceURI($uri);
