@@ -1,11 +1,10 @@
 <?php
 /**
- * $Horde: horde/lib/prefs.php,v 1.19.4.7.2.1 2005/12/27 17:02:02 chuck Exp $
+ * $Horde: horde/lib/prefs.php,v 1.19.4.19 2008/10/24 21:52:17 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  */
 
@@ -38,16 +37,22 @@ function handle_themeselect($updated)
 function handle_categorymanagement($updated)
 {
     require_once 'Horde/Prefs/CategoryManager.php';
-    $cManager = &new Prefs_CategoryManager();
+    $cManager = new Prefs_CategoryManager();
 
     /* Always save colors of all categories. */
     $colors = array();
     $categories = $cManager->get();
     foreach ($categories as $category) {
-        if ($color = Util::getFormData('color_' . base64_encode($category))) {
+        if ($color = Util::getFormData('color_' . md5($category))) {
             $colors[$category] = $color;
         }
     }
+    if ($color = Util::getFormData('color_' . md5('_default_'))) {
+        $colors['_default_'] = $color;
+    }
+    if ($color = Util::getFormData('color_' . md5('_unfiled_'))) {
+        $colors['_unfiled_'] = $color;
+    }
     $cManager->setColors($colors);
 
     $action = Util::getFormData('cAction');
@@ -70,37 +75,56 @@ function handle_categorymanagement($updated)
     return $updated;
 }
 
+function handle_credentialsui($updated)
+{
+    global $prefs;
+
+    $credentials = Util::getFormData('credentials');
+    if (!is_null($credentials)) {
+        $prefs->setValue('credentials', serialize($credentials));
+        $updated = true;
+    }
+
+    return $updated;
+}
+
 /**
  * Do anything that we need to do as a result of certain preferences
  * changing.
  */
 function prefs_callback()
 {
-    global $prefs, $registry, $notification;
-
-    if ($prefs->isDirty('language')) {
-        NLS::setLang($prefs->getValue('language'));
-        NLS::setTextdomain($registry->getApp(), $registry->get('fileroot') . '/locale', NLS::getCharset());
-        String::setDefaultCharset(NLS::getCharset());
-    }
+    global $prefs, $registry, $notification, $nls;
 
+    $reloaded = false;
     if ($prefs->isDirty('language') ||
         $prefs->isDirty('show_sidebar')) {
+        if ($prefs->isDirty('language')) {
+            NLS::setLanguageEnvironment($prefs->getValue('language'));
+            foreach ($registry->listAPIs() as $api) {
+                if ($registry->hasMethod($api . '/changeLanguage')) {
+                    $registry->call($api . '/changeLanguage');
+                }
+            }
+        }
+
         $url = $registry->get('webroot', 'horde');
         if (substr($url, -1) != '/') {
             $url .= '/';
         }
-        $url = Util::addParameter($url . 'index.php', 'url', Horde::selfUrl(true));
-        $notification->push('if (window.parent.frames) window.parent.frames.location = \'' . addslashes(Horde::url($url, true)) . '\';', 'javascript');
+        $url = addslashes(Horde::url(Util::addParameter($url . 'index.php', 'url', Horde::selfUrl(true, false, true)), true));
+        $notification->push("if (window.parent.frames && window.parent.frames.horde_menu) window.parent.frames.location = '$url'; else window.location = '$url';", 'javascript');
+        $reloaded = true;
     }
 
-    if ($prefs->isDirty('sidebar_width')) {
-        $notification->push('if (window.parent && window.parent.document.getElementById(\'hf\') && window.parent.horde_menu && window.parent.horde_menu.expandedSidebar.shown()) window.parent.document.getElementById(\'hf\').cols = window.parent.horde_menu.rtl ? \'*,' . $prefs->getValue('sidebar_width') . '\' : \'' . $prefs->getValue('sidebar_width') . ',*\';', 'javascript');
+    if (!$reloaded && $prefs->isDirty('sidebar_width')) {
+        $notification->push('if (window.parent && window.parent.document.getElementById(\'hf\') && window.parent.horde_menu && window.parent.horde_menu.document.getElementById(\'expandedSidebar\').style.display != \'hidden\') window.parent.document.getElementById(\'hf\').cols = window.parent.horde_menu.rtl ? \'*,' . $prefs->getValue('sidebar_width') . '\' : \'' . $prefs->getValue('sidebar_width') . ',*\';', 'javascript');
     }
 
-    if ($prefs->isDirty('theme') ||
-        $prefs->isDirty('menu_view') ||
-        $prefs->isDirty('menu_refresh_time')) {
+    if (!$reloaded &&
+        ($prefs->isDirty('theme') ||
+         $prefs->isDirty('menu_view') ||
+         $prefs->isDirty('menu_refresh_time'))) {
         $notification->push('if (window.parent.frames.horde_menu) window.parent.frames.horde_menu.location.reload();', 'javascript');
     }
 }
@@ -137,7 +161,9 @@ if (!$prefs->isLocked('theme')) {
             }
 
             $theme_name = null;
-            @include $theme_base . '/' . $dir . '/info.php';
+            if (is_readable($theme_base . '/' . $dir . '/info.php')) {
+                include $theme_base . '/' . $dir . '/info.php';
+            }
             if (!empty($theme_name)) {
                 $theme_options[$dir] = $theme_name;
             }
