@@ -1,4 +1,4 @@
--- $Horde: horde/scripts/sql/horde_sessionhandler.mssql.sql,v 1.1.2.1 2006/01/18 19:59:10 ben Exp $
+-- $Horde: horde/scripts/sql/horde_sessionhandler.mssql.sql,v 1.1.2.2 2007/12/20 15:03:03 jan Exp $
 
 CREATE TABLE horde_sessionhandler (
     session_id             VARCHAR(32) NOT NULL,
@@ -7,5 +7,3 @@ CREATE TABLE horde_sessionhandler (
 
     PRIMARY KEY (session_id)
 );
-
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_sessionhandler TO horde;
