@@ -4,24 +4,33 @@
  * referrer data being passed to the remote server and potentially exposing
  * any session IDs.
  *
- * Copyright 2003-2006 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: horde/services/go.php,v 1.6.2.3.2.6 2006/06/22 02:28:40 chuck Exp $
+ * $Horde: horde/services/go.php,v 1.6.2.20 2008/09/12 04:17:48 chuck Exp $
  *
  * @author Marko Djukic <marko@oblo.com>
  */
 
+$session_control = 'none';
+@define('AUTH_HANDLER', true);
+require_once dirname(__FILE__) . '/../lib/base.php';
+
 if (empty($_GET['url'])) {
     exit;
 }
 
 $url = trim($_GET['url']);
 if (preg_match('/;\s*url\s*=/i', $url)) {
-    /* IE will process the last ;URL= string, not the first, allowing
-     * protocols that shouldn't be let through. */
+    // IE will process the last ;URL= string, not the first, allowing
+    // protocols that shouldn't be let through.
+    exit;
+}
+
+// Check the HMAC
+if (!Horde::verifySignedQueryString($_SERVER['QUERY_STRING'])) {
     exit;
 }
 
@@ -41,7 +50,7 @@ if (empty($parsed_url['path'])) {
 // Do a little due diligence on the target URL. If it's on the same server
 // that we're already on, display an intermediate page asking people if
 // they're sure they want to click through.
-if (substr(php_sapi_name(), 0, 3) == 'cgi') {
+if (!strncmp(PHP_SAPI, 'cgi', 3)) {
     // When using CGI PHP, SCRIPT_NAME may contain the path to the PHP binary
     // instead of the script being run; use PHP_SELF instead.
     $myurl = $_SERVER['PHP_SELF'];
@@ -50,14 +59,27 @@ if (substr(php_sapi_name(), 0, 3) == 'cgi') {
         $_SERVER['SCRIPT_NAME'] :
         $_SERVER['PHP_SELF'];
 }
+// 16 is the length of "/services/go.php".
 $webroot = substr($myurl, 0, -16);
 
+// Build a list of hosts considered dangerous (local hosts, the user's
+// host, etc).
+$dangerous_hosts = array('localhost', 'localhost.localdomain', '127.0.0.1');
+if (!empty($_SERVER['SERVER_NAME'])) {
+    $dangerous_hosts[] = $_SERVER['SERVER_NAME'];
+}
+if (!empty($_SERVER['HTTP_HOST'])) {
+    $dangerous_hosts[] = $_SERVER['HTTP_HOST'];
+}
+
+// List of allowed services.
+$allowed_uris = array($webroot . '/services/confirm.php');
+
+// Check against our lists.
 if ((empty($webroot) || strpos($parsed_url['path'], $webroot) === 0) &&
     !empty($parsed_url['query']) &&
-    ((!empty($_SERVER['SERVER_NAME']) &&
-      $_SERVER['SERVER_NAME'] == $parsed_url['host']) ||
-     (!empty($_SERVER['HTTP_HOST']) &&
-      $_SERVER['HTTP_HOST'] == $parsed_url['host']))) {
+    !in_array($parsed_url['path'], $allowed_uris) &&
+    in_array($parsed_url['host'], $dangerous_hosts)) {
 ?>
 <html>
 <head>
