@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: horde/services/portal/mobile.php,v 2.15.10.3 2006/01/01 21:29:14 jan Exp $
+ * $Horde: horde/services/portal/mobile.php,v 2.15.10.7 2008/05/21 22:39:23 jan Exp $
  *
- * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/../..');
@@ -28,10 +30,9 @@ if (empty($fullname)) {
 $m = &new Horde_Mobile(_("Welcome"));
 $m->add(new Horde_Mobile_text(sprintf(_("Welcome, %s"), $fullname)));
 
-// Messy way of linking to active apps that support mobile
-// devices. Should be made more elegant at some point.
-if (!empty($registry->applications['mimp']['status']) &&
-    $registry->applications['mimp']['status'] != 'inactive') {
+// Messy way of linking to active apps that support mobile devices. Should be
+// made more elegant at some point.
+if (in_array('mimp', $registry->listApps())) {
     $m->add(new Horde_Mobile_link($registry->get('name', 'mimp'),
                                   Horde::url($registry->get('webroot', 'mimp') . '/'),
                                   $registry->get('name', 'mimp')));
