@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: horde/services/resetpassword.php,v 1.5.10.8 2007/01/02 13:55:15 jan Exp $
+ * $Horde: horde/services/resetpassword.php,v 1.5.10.10 2008/01/02 11:32:34 jan Exp $
  *
- * Copyright 2004-2007 Marko Djukic <marko@oblo.com>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Marko Djukic <marko@oblo.com>
  */
 
 @define('AUTH_HANDLER', true);
@@ -23,10 +25,10 @@ if (!$auth->hasCapability('resetpassword')) {
     exit;
 }
 
-$vars = &Variables::getDefaultVariables();
+$vars = Variables::getDefaultVariables();
 
 $title = _("Reset Your Password");
-$form = &Horde_Form::singleton('HordeSignupForm', $vars, $title);
+$form = new Horde_Form($vars, $title);
 $form->setButtons(_("Continue"));
 
 /* Set up the fields for the username and alternate email. */
@@ -62,20 +64,18 @@ if ($can_validate && $form->validate($vars)) {
     $answer = $prefs->getValue('security_answer');
 
     /* Check the given values witht the prefs stored ones. */
-    if ($email == $info['email'] && String::lower($answer) == String::lower($info['answer'])) {
+    if ($email == $info['email'] &&
+        strtolower($answer) == strtolower($info['answer'])) {
         /* Info matches, so reset the password. */
         $password = $auth->resetPassword($info['username']);
 
-        require_once 'Mail.php';
-        $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
-
-        /* Set up the email headers and body. */
-        $headers['From'] = $email;
-        $headers['To'] = $email;
-        $recipients[] = $headers['To'];
-        $headers['Subject'] = _("Your password has been reset");
-        $body = sprintf(_("Your new password for %s is: %s"), $registry->get('name', 'horde'), $password);
-        $mailer->send($recipients, $headers, $body);
+        require_once 'Horde/MIME/Mail.php';
+        $mail = new MIME_Mail(_("Your password has been reset"),
+                              sprintf(_("Your new password for %s is: %s"),
+                                      $registry->get('name', 'horde'),
+                                      $password),
+                              $email, $email, NLS::getCharset());
+        $mail->send($conf['mailer']['type'], $conf['mailer']['params']);
 
         $notification->push(_("Your password has been reset, check your email and log in with your new password."), 'horde.success');
         header('Location: ' . Auth::getLoginScreen('', $info['url']));
@@ -90,6 +90,6 @@ if ($can_validate && $form->validate($vars)) {
 require HORDE_TEMPLATES . '/common-header.inc';
 $notification->notify(array('listeners' => 'status'));
 require_once 'Horde/Form/Renderer.php';
-$renderer = &new Horde_Form_Renderer();
+$renderer = new Horde_Form_Renderer();
 $form->renderActive($renderer, $vars, 'resetpassword.php', 'post');
 require HORDE_TEMPLATES . '/common-footer.inc';
