@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: horde/services/shares/edit.php,v 1.26.10.5 2005/08/24 13:42:20 selsky Exp $
+ * $Horde: horde/services/shares/edit.php,v 1.26.10.16 2008/04/22 16:06:53 chuck Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 $fieldsList['show'] = 0;
@@ -15,14 +17,24 @@ $fieldsList['delete'] = 3;
 
 @define('HORDE_BASE', dirname(__FILE__) . '/../..');
 require_once HORDE_BASE . '/lib/base.php';
-require_once 'Horde/Menu.php';
 require_once 'Horde/Share.php';
 require_once 'Horde/Group.php';
 
+// Exit if the user shouldn't be able to change share permissions.
+if (!empty($conf['share']['no_sharing'])) {
+    exit('permission denied');
+}
+
 $app = Util::getFormData('app');
 $shares = &Horde_Share::singleton($app);
 $groups = &Group::singleton();
 $auth = &Auth::singleton($conf['auth']['driver']);
+if ($registry->hasMethod('shareHelp', $app)) {
+    require_once 'Horde/Help.php';
+    $help = $registry->callByPackage($app, 'shareHelp');
+} else {
+    $help = null;
+}
 
 $form = null;
 $reload = false;
@@ -59,33 +71,13 @@ case 'editform':
 
         // Process owner and owner permissions.
         $old_owner = $share->get('owner');
-        $new_owner = Util::getFormData('owner', $old_owner);
+        $new_owner = Auth::addHook(Util::getFormData('owner', $old_owner));
         if ($old_owner !== $new_owner && !empty($new_owner)) {
             if ($old_owner != Auth::getAuth() && !Auth::isAdmin()) {
                 $notification->push(_("Only the owner or system administrator may change ownership or owner permissions for a share"), 'horde.error');
             } else {
                 $share->set('owner', $new_owner);
                 $share->save();
-                if (Util::getFormData('owner_show')) {
-                    $perm->addUserPermission($new_owner, PERMS_SHOW, false);
-                } else {
-                    $perm->removeUserPermission($new_owner, PERMS_SHOW, false);
-                }                            
-                if (Util::getFormData('owner_read')) {
-                    $perm->addUserPermission($new_owner, PERMS_READ, false);
-                } else {
-                    $perm->removeUserPermission($new_owner, PERMS_READ, false);
-                }                            
-                if (Util::getFormData('owner_edit')) {
-                    $perm->addUserPermission($new_owner, PERMS_EDIT, false);
-                } else {
-                    $perm->removeUserPermission($new_owner, PERMS_EDIT, false);
-                }                            
-                if (Util::getFormData('owner_delete')) {
-                    $perm->addUserPermission($new_owner, PERMS_DELETE, false);
-                } else {
-                    $perm->removeUserPermission($new_owner, PERMS_DELETE, false);
-                }                            
             }
         }
 
@@ -163,6 +155,8 @@ case 'editform':
         $u_delete = Util::getFormData('u_delete');
 
         foreach ($u_names as $key => $user) {
+            // Apply backend hooks
+            $user = Auth::addHook($user);
             // If the user is empty, or we've already set permissions
             // via the owner_ options, don't do anything here.
             if (empty($user) || $user == $new_owner) {
@@ -225,18 +219,31 @@ case 'editform':
             }
         }
 
-        $share->setPermission($perm, false);
-        $share->save();
-        $notification->push(sprintf(_("Updated \"%s\"."), $share->get('name')), 'horde.success');
+        $result = $share->setPermission($perm, false);
+        if (is_a($result, 'PEAR_Error')) {
+            $notification->push($result, 'horde.error');
+        } else {
+            $result = $share->save();
+            if (is_a($result, 'PEAR_Error')) {
+                $notification->push($result, 'horde.error');
+            } else {
+                if (Util::getFormData('save_and_finish')) {
+                    Util::closeWindowJS();
+                    exit;
+                }
+                $notification->push(sprintf(_("Updated \"%s\"."), $share->get('name')), 'horde.success');
+            }
+        }
+
         $form = 'edit.inc';
     }
     break;
 }
 
 if (is_a($share, 'PEAR_Error')) {
-    $title = _("Edit Permissions");
+    $title = _("Edit permissions");
 } else {
-    $title = sprintf(_("Edit Permissions for %s"), $share->get('name'));
+    $title = sprintf(_("Edit permissions for \"%s\""), $share->get('name'));
 }
 
 if ($auth->hasCapability('list')) {
@@ -250,7 +257,11 @@ if ($auth->hasCapability('list')) {
     $userList = array();
 }
 
-$groupList = $groups->listGroups();
+if (!empty($conf['share']['any_group'])) {
+    $groupList = $groups->listGroups();
+} else {
+    $groupList = $groups->getGroupMemberships(Auth::getAuth(), true);
+}
 if (is_a($groupList, 'PEAR_Error')) {
     Horde::logMessage($groupList, __FILE__, __LINE__, PEAR_LOG_NOTICE);
     $groupList = array();
