@@ -3,12 +3,12 @@
  *
  * Provides the javascript class to create dynamic trees.
  *
- * Copyright 2003-2007 Marko Djukic <marko@oblo.com>
+ * $Horde: horde/templates/javascript/tree.js,v 1.62.2.15 2008/01/02 11:32:38 jan Exp $
  *
- * See the enclosed file COPYING for license information (GPL). If you did not
- * receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
- * $Horde: horde/templates/javascript/tree.js,v 1.62.2.12 2007/01/02 13:55:18 jan Exp $
+ * See the enclosed file COPYING for license information (GPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Marko Djukic <marko@oblo.com>
  * @package Horde_Tree
@@ -57,12 +57,11 @@ function Horde_Tree(instanceName)
     /* Tree building variables. */
     this.renderStatic   = false;
     this.target         = '';
-    this.header         = new Array();
-    this.rootNodes      = new Array();
-    this.nodes          = new Array();
-    this.node_pos       = new Array();
-    this.dropline       = new Array();
-    this.output         = '';
+    this.header         = [];
+    this.rootNodes      = [];
+    this.nodes          = [];
+    this.node_pos       = [];
+    this.dropline       = [];
 }
 
 Horde_Tree.prototype.setImgDir = function(imgDir)
@@ -83,19 +82,22 @@ Horde_Tree.prototype.renderTree = function(rootNodes, renderStatic)
 
 Horde_Tree.prototype._renderTree = function()
 {
-    this.output = '';
+    this.output = [];
     if (!this.options['hideHeaders']) {
-        this.output = this._buildHeader();
+        this.output.push(this._buildHeader());
     }
-    for (var i = 0; i < this.rootNodes.length; i++) {
+    for (var i = 0; i < this.rootNodes.length; ++i) {
         this.buildTree(this.rootNodes[i]);
     }
-    document.getElementById(this.target).innerHTML = this.output;
+    document.getElementById(this.target).innerHTML = this.output.join('');
     this._correctWidthForScrollbar();
     // If using alternating row shading, work out correct shade.
     if (this.options['alternate']) {
         this.stripe();
     }
+    if (typeof ToolTips == 'object') {
+        ToolTips.attachBehavior();
+    }
 }
 
 /**
@@ -111,28 +113,37 @@ Horde_Tree.prototype._buildHeader = function()
         return '';
     }
 
-    var html = '<div>';
-    for (var i = 0; i < this.header.length; i++) {
-        html += '<div';
+    var html = [];
+    html.push('<div>');
+    for (var i = 0; i < this.header.length; ++i) {
+        html.push('<div');
         if (this.header[i]['class']) {
-            html += ' class="' + this.header[i]['class'] + '"';
+            html.push(' class="');
+            html.push(this.header[i]['class']);
+            html.push('"');
         }
 
-        var style = this.floatDir;
+        html.push(' style="');
+        html.push(this.floatDir);
+
         if (this.header[i]['width']) {
-            style += 'width:' + this.header[i]['width'] + ';';
+            html.push('width:');
+            html.push(this.header[i]['width']);
+            html.push(';');
         }
         if (this.header[i]['align']) {
-            style += 'text-align:' + this.header[i]['align'] + ';';
+            html.push('text-align:');
+            html.push(this.header[i]['align']);
+            html.push(';');
         }
 
-        html += ' style="' + style + '"';
-        html += '>';
-        html += this.header[i]['html'] ? this.header[i]['html'] : '&nbsp;';
-        html += '</div>';
+        html.push('">');
+        html.push(this.header[i]['html'] ? this.header[i]['html'] : '&nbsp;');
+        html.push('</div>');
     }
 
-    return html + '</div>';
+    html.push('</div>');
+    return html.join('');
 }
 
 /**
@@ -141,7 +152,7 @@ Horde_Tree.prototype._buildHeader = function()
  */
 Horde_Tree.prototype.buildTree = function(nodeId)
 {
-    this.output += this.buildLine(nodeId);
+    this.buildLine(nodeId);
 
     if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
         var numSubnodes = this.nodes[nodeId]['children'].length;
@@ -151,17 +162,17 @@ Horde_Tree.prototype.buildTree = function(nodeId)
             } else {
                 rowStyle = 'display:none;';
             }
-            this.output += '<div id="nodeChildren_' + nodeId + '" style="' + rowStyle + '">';
+            this.output.push('<div id="nodeChildren_' + nodeId + '" style="' + rowStyle + '">');
 
-            for (var c = 0; c < numSubnodes; c++) {
+            for (var c = 0; c < numSubnodes; ++c) {
                 var childNodeId = this.nodes[nodeId]['children'][c];
-                this.node_pos[childNodeId] = new Array();
+                this.node_pos[childNodeId] = [];
                 this.node_pos[childNodeId]['pos'] = c + 1;
                 this.node_pos[childNodeId]['count'] = numSubnodes;
                 this.buildTree(childNodeId);
             }
 
-            this.output += '</div>';
+            this.output.push('</div>');
         }
     }
 
@@ -170,174 +181,239 @@ Horde_Tree.prototype.buildTree = function(nodeId)
 
 Horde_Tree.prototype.buildLine = function(nodeId)
 {
+    var node = this.nodes[nodeId];
+    var o = this.output;
     var style = '';
-    var rowClass = 'treeRow';
-    if (this.nodes[nodeId]['class']) {
-        rowClass += ' ' + this.nodes[nodeId]['class'];
-    }
 
-    var line = '<div class="' + rowClass + '">';
+    o.push('<div class="');
+    o.push('treeRow');
+    if (node['class']) {
+        o.push(' ');
+        o.push(node['class']);
+    }
+    o.push('">');
 
     // If we have headers, track which logical "column" we're in for
     // any given cell of content.
     var column = 0;
 
-    if (typeof(this.nodes[nodeId]['extra']) != 'undefined' &&
-        typeof(this.nodes[nodeId]['extra'][0]) != 'undefined') {
-        var extra = this.nodes[nodeId]['extra'][0];
-        for (var c = 0; c < extra.length; c++) {
-            style = this.floatDir;
+    if (typeof(node['extra']) != 'undefined' &&
+        typeof(node['extra'][0]) != 'undefined') {
+        var extra = node['extra'][0];
+        for (var c = 0; c < extra.length; ++c) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">' + extra[c] + '</div>';
-            column++;
+            o.push('">');
+            o.push(extra[c]);
+            o.push('</div>');
+            ++column;
         }
-        for (var d = c; d < extraColsLeft; d++) {
-            style = this.floatDir;
+        for (var d = c; d < extraColsLeft; ++d) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">&nbsp;</div>';
-            column++;
+            o.push('">&nbsp;</div>');
+            ++column;
         }
     } else {
-        for (var c = 0; c < extraColsLeft; c++) {
-            style = this.floatDir;
+        for (var c = 0; c < extraColsLeft; ++c) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">&nbsp;</div>';
-            column++;
+            o.push('">&nbsp;</div>');
+            ++column;
         }
     }
 
-    style = this.floatDir;
+    o.push('<div style="');
+    o.push(this.floatDir);
     if (this.header[column] && this.header[column]['width']) {
-        style += 'width:' + this.header[column]['width'] + ';';
+        o.push('width:');
+        o.push(this.header[column]['width']);
+        o.push(';');
     }
-    line += '<div style="' + style + '">';
+    o.push('">');
 
     if (this.options['multiline']) {
-        line += '<table cellspacing="0"><tr><td>';
+        o.push('<table cellspacing="0"><tr><td>');
     }
 
-    for (var i = this.renderStatic ? 1 : 0; i < this.nodes[nodeId]['indent']; i++) {
+    for (var i = this.renderStatic ? 1 : 0; i < node['indent']; ++i) {
+        o.push('<img src="');
+        o.push(this.imgDir);
+        o.push('/');
         if (this.dropline[i] && this.options['lines']) {
-            line += '<img src="' + this.imgDir + '/' + this.imgLine + '" alt="|&nbsp;&nbsp;&nbsp;" height="20" width="20" />';
+            o.push(this.imgLine);
+            o.push('" alt="|&nbsp;&nbsp;&nbsp;" height="20" width="20" />');
         } else {
-            line += '<img src="' + this.imgDir + '/' + this.imgBlank + '" alt="&nbsp;&nbsp;&nbsp;" height="20" width="20" />';
+            o.push(this.imgBlank);
+            o.push('" alt="&nbsp;&nbsp;&nbsp;" height="20" width="20" />');
         }
     }
-    line += this._setNodeToggle(nodeId);
+    o.push(this._setNodeToggle(nodeId));
     if (this.options['multiline']) {
-        line += '</td><td>';
+        o.push('</td><td>');
     }
-    line += this._setLabel(nodeId);
+    o.push(this._setLabel(nodeId));
 
     if (this.options['multiline']) {
-        line += '</td></tr></table>';
+        o.push('</td></tr></table>');
     }
 
-    line += '</div>';
-    column++;
+    o.push('</div>');
+    ++column;
 
-    if (typeof(this.nodes[nodeId]['extra']) != 'undefined' &&
-        typeof(this.nodes[nodeId]['extra'][1]) != 'undefined') {
-        var extra = this.nodes[nodeId]['extra'][1];
-        for (var c = 0; c < extra.length; c++) {
-            style = this.floatDir;
+    if (typeof(node['extra']) != 'undefined' &&
+        typeof(node['extra'][1]) != 'undefined') {
+        var extra = node['extra'][1];
+        for (var c = 0; c < extra.length; ++c) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">' + extra[c] + '</div>';
-            column++;
+            o.push('">');
+            o.push(extra[c]);
+            o.push('</div>');
+            ++column;
         }
-        for (var d = c; d < extraColsRight; d++) {
-            style = this.floatDir;
+        for (var d = c; d < extraColsRight; ++d) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">&nbsp;</div>';
-            column++;
+            o.push('">&nbsp;</div>');
+            ++column;
         }
     } else {
-        for (var c = 0; c < extraColsRight; c++) {
-            style = this.floatDir;
+        for (var c = 0; c < extraColsRight; ++c) {
+            o.push('<div style="');
+            o.push(this.floatDir);
             if (this.header[column] && this.header[column]['width']) {
-                style += 'width:' + this.header[column]['width'] + ';';
+                o.push('width:');
+                o.push(this.header[column]['width']);
+                o.push(';');
             }
-            line += '<div style="' + style + '">&nbsp;</div>';
-            column++;
+            o.push('">&nbsp;</div>');
+            ++column;
         }
     }
-    line += '</div>';
-
-    return line;
+    o.push('</div>');
 }
 
 Horde_Tree.prototype._setLabel = function(nodeId)
 {
-    label = this.nodes[nodeId]['label'];
+    var node = this.nodes[nodeId];
+    var label = [];
 
-    if (this.nodes[nodeId]['url']) {
-        var urlClass = '';
-        if (this.nodes[nodeId]['urlclass']) {
-            urlClass = ' class="' + this.nodes[nodeId]['urlclass'] + '"';
+    if (node['url']) {
+        label.push('<a');
+
+        if (node['urlclass']) {
+            label.push(' class="');
+            label.push(node['urlclass']);
+            label.push('"');
         } else if (this.options['urlclass']) {
-            urlClass = ' class="' + this.options['urlclass'] + '"';
+            label.push(' class="');
+            label.push(this.options['urlclass']);
+            label.push('"');
         }
 
-        var target = '';
-        if (this.nodes[nodeId]['target']) {
-            target = ' target="' + this.nodes[nodeId]['target'] + '"';
+        label.push(' href="');
+        label.push(node['url']);
+        label.push('"');
+
+        if (node['title']) {
+            label.push(' title="');
+            label.push(node['title']);
+            label.push('"');
+        }
+
+        if (node['target']) {
+            label.push(' target="');
+            label.push(node['target']);
+            label.push('"');
         } else if (this.options['target']) {
-            target = ' target="' + this.options['target'] + '"';
+            label.push(' target="');
+            label.push(this.options['target']);
+            label.push('"');
         }
 
-        var onclick = '';
-        if (this.nodes[nodeId]['onclick']) {
-            onclick = ' onclick="' + this.nodes[nodeId]['onclick'] + '"';
+        if (node['onclick']) {
+            label.push(' onclick="');
+            label.push(node['onclick']);
+            label.push('"');
         }
 
-        return '<a' + urlClass + ' href="' + this.nodes[nodeId]['url'] + '"' + target + onclick + '>' + this._setNodeIcon(nodeId) + label + '</a>';
+        label.push('>');
+        label.push(this._setNodeIcon(nodeId));
+        label.push(node['label']);
+        label.push('</a>');
     } else {
-        return '<span class="toggle" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')">' + this._setNodeIcon(nodeId) + label + '</span>';
+        label.push('<span class="toggle" onclick="');
+        label.push(this._instanceName);
+        label.push('.toggle(\'');
+        label.push(nodeId.toString().replace(/'/, "\\'"));
+        label.push('\')">');
+        label.push(this._setNodeIcon(nodeId));
+        label.push(node['label']);
+        label.push('</span>');
     }
+
+    return label.join('');
 }
 
 Horde_Tree.prototype._setNodeToggle = function(nodeId)
 {
+    var node = this.nodes[nodeId];
     var attrib = '';
-    if (this.nodes[nodeId]['indent'] == '0' &&
-        typeof(this.nodes[nodeId]['children']) != 'undefined') {
+    if (node['indent'] == '0' &&
+        typeof(node['children']) != 'undefined') {
         // Top level with children.
         this.dropline[0] = false;
         if (this.renderStatic) {
             return '';
         } else {
-            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
+            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId.toString().replace(/'/, "\\'") + '\')"';
         }
-    } else if (this.nodes[nodeId]['indent'] != '0' &&
-               typeof(this.nodes[nodeId]['children']) == 'undefined') {
+    } else if (node['indent'] != '0' &&
+               typeof(node['children']) == 'undefined') {
         // Node no children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
             // Not last node.
-            this.dropline[this.nodes[nodeId]['indent']] = true;
+            this.dropline[node['indent']] = true;
         } else {
-            this.dropline[this.nodes[nodeId]['indent']] = false;
+            this.dropline[node['indent']] = false;
         }
-    } else if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
+    } else if (typeof(node['children']) != 'undefined') {
         // Node with children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
             // Not last node.
-            this.dropline[this.nodes[nodeId]['indent']] = true;
+            this.dropline[node['indent']] = true;
         } else {
             // Last node.
-            this.dropline[this.nodes[nodeId]['indent']] = false;
+            this.dropline[node['indent']] = false;
         }
         if (!this.renderStatic) {
-            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId + '\')"';
+            attrib = ' style="cursor:pointer" onclick="' + this._instanceName + '.toggle(\'' + nodeId.toString().replace(/'/, "\\'") + '\')"';
         }
     } else {
         // Top level node with no children.
@@ -347,35 +423,46 @@ Horde_Tree.prototype._setNodeToggle = function(nodeId)
         this.dropline[0] = false;
     }
 
-    nodeToggle = this._getNodeToggle(nodeId);
-    img = '<img id="nodeToggle_' + nodeId + '" src="' + this.imgDir + '/' + nodeToggle[0] + '" ';
+    var nodeToggle = this._getNodeToggle(nodeId);
+    var img = [];
+    img.push('<img id="nodeToggle_');
+    img.push(nodeId);
+    img.push('" src="');
+    img.push(this.imgDir);
+    img.push('/');
+    img.push(nodeToggle[0]);
+    img.push('" ');
     if (nodeToggle[1]) {
-        img += 'alt="' + nodeToggle[1] + '" ';
+        img.push('alt="');
+        img.push(nodeToggle[1]);
+        img.push('" ');
     }
-    img += attrib + ' height="20" width="20" />';
-    return img;
+    img.push(attrib);
+    img.push(' height="20" width="20" />');
+    return img.join('');
 }
 
 Horde_Tree.prototype._getNodeToggle = function(nodeId)
 {
-    var nodeToggle = new Array('', '');
-    if (this.nodes[nodeId]['indent'] == '0' &&
-        typeof(this.nodes[nodeId]['children']) != 'undefined') {
+    var node = this.nodes[nodeId];
+    var nodeToggle = ['', ''];
+    if (node['indent'] == '0' &&
+        typeof(node['children']) != 'undefined') {
         // Top level with children.
         if (this.renderStatic) {
             return nodeToggle;
         } else if (!this.options['lines']) {
             nodeToggle[0] = this.imgBlank;
             nodeToggle[1] = '&nbsp;&nbsp;&nbsp;'
-        } else if (this.nodes[nodeId]['expanded']) {
+        } else if (node['expanded']) {
             nodeToggle[0] = this.imgMinusOnly;
             nodeToggle[1] = '-';
         } else {
             nodeToggle[0] = this.imgPlusOnly;
             nodeToggle[1] = '+';
         }
-    } else if (this.nodes[nodeId]['indent'] != '0' &&
-        typeof(this.nodes[nodeId]['children']) == 'undefined') {
+    } else if (node['indent'] != '0' &&
+        typeof(node['children']) == 'undefined') {
         // Node no children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
             // Not last node.
@@ -396,7 +483,7 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
                 nodeToggle[1] = '&nbsp;&nbsp;&nbsp;';
             }
         }
-    } else if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
+    } else if (typeof(node['children']) != 'undefined') {
         // Node with children.
         if (this.node_pos[nodeId]['pos'] < this.node_pos[nodeId]['count']) {
             // Not last node.
@@ -406,7 +493,7 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
             } else if (this.renderStatic) {
                 nodeToggle[0] = this.imgJoin;
                 nodeToggle[1] = '|-';
-            } else if (this.nodes[nodeId]['expanded']) {
+            } else if (node['expanded']) {
                 nodeToggle[0] = this.imgMinus;
                 nodeToggle[1] = '-';
             } else {
@@ -421,7 +508,7 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
             } else if (this.renderStatic) {
                 nodeToggle[0] = this.imgJoinBottom;
                 nodeToggle[1] = '`-';
-            } else if (this.nodes[nodeId]['expanded']) {
+            } else if (node['expanded']) {
                 nodeToggle[0] = this.imgMinusBottom;
                 nodeToggle[1] = '-';
             } else {
@@ -448,54 +535,103 @@ Horde_Tree.prototype._getNodeToggle = function(nodeId)
 
 Horde_Tree.prototype._setNodeIcon = function(nodeId)
 {
-    var imgDir = (typeof(this.nodes[nodeId]['icondir']) != 'undefined') ?
-        this.nodes[nodeId]['icondir'] :
-        this.imgDir;
-    if (imgDir) {
-        imgDir += '/';
+    var node = this.nodes[nodeId];
+    var img = []
+
+    img.push('<img id="nodeIcon_');
+    img.push(nodeId);
+    img.push('" src="');
+
+    // Image directory.
+    if (typeof(node['icondir']) != 'undefined') {
+        if (node['icondir']) {
+            img.push(node['icondir']);
+            img.push('/');
+        }
+    } else if (this.imgDir) {
+        img.push(this.imgDir);
+        img.push('/');
     }
 
-    if (typeof(this.nodes[nodeId]['icon']) != 'undefined') {
+    // Image.
+    if (typeof(node['icon']) != 'undefined') {
         // Node has a user defined icon.
-        if (!this.nodes[nodeId]['icon']) {
+        if (!node['icon']) {
             return '';
         }
-        if (typeof(this.nodes[nodeId]['iconopen']) != 'undefined' && this.nodes[nodeId]['expanded']) {
-            img = this.nodes[nodeId]['iconopen'];
+        if (typeof(node['iconopen']) != 'undefined' && node['expanded']) {
+            img.push(node['iconopen']);
         } else {
-            img = this.nodes[nodeId]['icon'];
+            img.push(node['icon']);
         }
     } else {
         // Use standard icon set.
-        if (typeof(this.nodes[nodeId]['children']) != 'undefined') {
+        if (typeof(node['children']) != 'undefined') {
             // Node with children.
-            img = (this.nodes[nodeId]['expanded']) ? this.imgFolderOpen
-                                                   : this.imgFolder;
+            img.push((node['expanded']) ? this.imgFolderOpen : this.imgFolder);
         } else {
-            // Node no children.
-            img = this.imgLeaf;
+            // Node, no children.
+            img.push(this.imgLeaf);
         }
     }
 
-    var imgtxt = '<img src="' + imgDir + img + '"';
+    img.push('"');
 
-    if (typeof(this.nodes[nodeId]['iconalt']) != 'undefined') {
-        imgtxt += ' alt="' + this.nodes[nodeId]['iconalt'] + '"';
+    if (typeof(node['iconalt']) != 'undefined') {
+        img.push(' alt="');
+        img.push(node['iconalt']);
+        img.push('"');
     }
 
-    return imgtxt + ' /> ';
+    img.push(' /> ');
+    return img.join('');
 }
 
 Horde_Tree.prototype.toggle = function(nodeId)
 {
-    this.nodes[nodeId]['expanded'] = !this.nodes[nodeId]['expanded'];
-    if (this.nodes[nodeId]['expanded']) {
-        if (node = document.getElementById('nodeChildren_' + nodeId)) {
-            node.style.display = 'block';
+    var node = this.nodes[nodeId];
+    node['expanded'] = !node['expanded'];
+    if (node['expanded']) {
+        if (children = document.getElementById('nodeChildren_' + nodeId)) {
+            children.style.display = 'block';
         }
     } else {
-        if (node = document.getElementById('nodeChildren_' + nodeId)) {
-            node.style.display = 'none';
+        if (children = document.getElementById('nodeChildren_' + nodeId)) {
+            children.style.display = 'none';
+        }
+    }
+
+    // Toggle the node's icon if it has seperate open and closed
+    // icons.
+    if (typeof node['iconopen'] != 'undefined') {
+        var icon = document.getElementById('nodeIcon_' + nodeId);
+        if (icon) {
+            var src = [];
+
+            // Image directory.
+            if (typeof(node['icondir']) != 'undefined') {
+                if (node['icondir']) {
+                    src.push(node['icondir']);
+                    src.push('/');
+                }
+            } else if (this.imgDir) {
+                src.push(this.imgDir);
+                src.push('/');
+            }
+
+            // Image.
+            if (typeof(node['icon']) != 'undefined') {
+                if (node['expanded']) {
+                    src.push(node['iconopen']);
+                } else {
+                    src.push(node['icon']);
+                }
+            } else {
+                // Use standard icon set.
+                src.push((node['expanded']) ? this.imgFolderOpen : this.imgFolder);
+            }
+
+            icon.src = src.join('');
         }
     }
 
@@ -510,7 +646,7 @@ Horde_Tree.prototype.toggle = function(nodeId)
         toggle.alt = nodeToggle[1];
     }
 
-    this.saveState(nodeId, this.nodes[nodeId]['expanded'])
+    this.saveState(nodeId, node['expanded'])
 }
 
 Horde_Tree.prototype.stripe = function()
@@ -529,14 +665,20 @@ Horde_Tree.prototype.stripe = function()
     }
 
     // Iterate over each child div.
-    for (var i = 0; i < tree.childNodes.length; i++) {
-        if (tree.childNodes[i].id.indexOf('nodeChildren') != -1) {
-            if (this.nodes[tree.childNodes[i].id.replace('nodeChildren_', '')]['expanded']) {
-                even = this.stripe(tree.childNodes[i].id, even);
+    for (var i = 0; i < tree.childNodes.length; ++i) {
+        var _node = tree.childNodes[i];
+        if (_node.id.indexOf('nodeChildren') != -1) {
+            if (this.nodes[_node.id.toString().replace('nodeChildren_', '')]['expanded']) {
+                even = this.stripe(_node.id, even);
             }
         } else {
-            tree.childNodes[i].className = tree.childNodes[i].className.replace(' rowEven', '').replace(' rowOdd', '');
-            tree.childNodes[i].className += even ? ' rowEven' : ' rowOdd';
+            _node.className = _node.className.replace(new RegExp(' ?rowEven ?'), '');
+            _node.className = _node.className.replace(new RegExp(' ?rowOdd ?'), '');
+            if (_node.className) {
+                _node.className += even ? ' rowEven' : ' rowOdd';
+            } else {
+                _node.className = even ? 'rowEven' : 'rowOdd';
+            }
 
             // Flip from odd to even, or vice-versa.
             even = !even;
@@ -557,8 +699,8 @@ Horde_Tree.prototype.saveState = function(nodeId, expanded)
     } else {
         // Collapse requested so remove from cookie.
         var nodes = oldCookie.split(',');
-        var newNodes = new Array();
-        for (var i = 0; i < nodes.length; i++) {
+        var newNodes = [];
+        for (var i = 0; i < nodes.length; ++i) {
             if (nodes[i] != nodeId) {
                 newNodes[newNodes.length] = nodes[i];
             }
